/* LASE - Logiciel d'Analyse des spectres EXAFS
   (C) 1997 - 2002, Emmanuel CURIS
   
   Modles structuraux
   Aller  un objet (atome, molcule, absorbeur,...)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h> /* Pour isspace */
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "atomes.h"
#include "chemins.h" /* Pour affichage des chemins */
#include "couches.h" /* Pour affichage / couches des modles FEFF */
#include "cristallo.h" /* Pour coordonnes fractionnaires */
#include "feff.h"    /* Pour affichage / modles FEFF */
#include "modeles.h" /* Pour affichage / absorbeur */
#include "tracer.h"  /* Pour redessin des molcules */
#include "science.h" /* Pour M_PI */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Aller  un atome ~-~-~-~-~-~-~-~- */

void ATM_aller(Atome *atome)
{
  if (NULL == mol_courante) return; /* On va _forcment_  un atome de la molcule courante */
  if (atome == NULL) return;

  ATM_courant = atome;

  /* En tout cas, il faut refaire l'affichage de la liste des atomes ! */
  ATM_PrepareListe();

  if (fen_vue_3d != NULL) { /* On a la vue 3d ouverte */
    char infos[LG_MAX_NOM_FRAGMENT + 200];

    sprintf(infos, "Atome courant : %s en (%f, %f, %f)", ATM_Z_vers_nom( atome->num_atomique ),
	    atome->position.x, atome->position.y, atome->position.z);
    if ( mol_courante->maille != NULL ) { /* Il y a une maille : coordonnes fractionnaires */
      Vecteur3D fractionnaire;
      char chaine[50];

      CRIS_AbsoluVersFrac( mol_courante->maille, &(atome->position), &fractionnaire );

      sprintf(chaine, " = (%f, %f, %f)", fractionnaire.x, fractionnaire.y, fractionnaire.z);
      strcat(infos, chaine);
    }
    if (atome->marques.absorbeur == VRAI)
      strcat(infos, " - Absorbeur");
    if (atome->fragment != NULL) {
      strcat(infos, " - fragment : ");
      strcat(infos, atome->fragment->nom);
    }
    FEN_infos(fen_vue_3d, infos);

#ifdef AVEC_OPEN_GL_
    /* Il faudra amliorer, avec une partie non-OpenGL, mais bon... */
    if (mode_vue3d & V3D_VOIR_ATOMES) {
      if (liste_molecule) {
	glNewList(liste_molecule, GL_COMPILE);
	dessine_molecule(mol_courante, mode_vue3d);
	glEndList(); /* On a fini le dessin */
      }
      FEN_redessin(fen_vue_3d, BI_RDS_GL); /* N'a pas d'intrt sinon ! */
    }
#endif
  }
}

/* ~-~-~-~-~-~-~-~- Aller  une molcule ~-~-~-~-~-~-~-~- */

int MOL_aller(Molecule *molecule)
{
  Molecule *m;

  if ( NULL == molecule ) return -1; /* Rien  faire */
/*   if ( mol_courante == molecule ) return 0; */ /* On est dj  cette molcule
						     Pas de contrle car complique les mises  jour... */

#ifdef _DEVELOPPE_
  printf("-- On demande d'aller  la molcule %s.\n", molecule->nom);
#endif
  for ( m = premiere_molecule ; m != molecule ; m = m->suivante ) {
    if ( NULL == m ) { /* La molcule est hors chanage : inutilisable (en cours d'effacement probablement) */
#ifdef _DEVELOPPE_
      printf("   Cette molcule est hors chanage, action annule !\n");
#endif
      return -2; /* Pas besoin de mettre  jour, sera fait en fin d'effacement - si ncessaire... */
    }
  }

  mol_courante = molecule; /* Nouvelle molcule courante */

  /* Gestion des atomes absorbeurs */
  if ( mol_courante->nbr_absorbeurs > 0 ) {
    Absorbeur *absorbeur = mol_courante->absorbeurs + mol_courante->absorbeur_courant;

    ABS_aller( absorbeur, FAUX );

    OBJ_active(AFF_STRUCTURE, AST_ABS_AVANT, mol_courante->absorbeur_courant > 0);
    OBJ_active(AFF_STRUCTURE, AST_ABS_APRES, mol_courante->absorbeur_courant < mol_courante->nbr_absorbeurs - 1);
  } else {
    ABS_aller( NULL, FAUX );
  }

  /* On dfinit le nouvel atome courant */
  ATM_PrepareListe(); /* Nouvelle liste d'atomes */
  ATM_aller(mol_courante->premier);

  /* Affichage de la vue en 3D */
  if ( fen_vue_3d != NULL ) { /* L'affichage est actif */
    unsigned long n_mol = 1;
    float position;

    for ( m = premiere_molecule ; m != molecule ; m = m->suivante, n_mol++ );

#ifdef AVEC_OPEN_GL_ /* Affichage de la vue 3d seulement avec Open GL */
    if ( liste_molecule != 0 ) {
      glNewList(liste_molecule, GL_COMPILE);
      dessine_molecule(mol_courante, mode_vue3d);
      glEndList();
    }
    if ( liste_liaisons != 0 ) {
      glNewList(liste_liaisons, GL_COMPILE);
      dessine_liaisons(mol_courante, mode_vue3d);
      glEndList();
    }
#endif

    FEN_titre( fen_vue_3d, mol_courante->nom );

    if ( nombre_molecules > 1 )
      position = 1000.0 * ((float) (n_mol - 1)) / (float) (nombre_molecules - 1);
    else
      position = 0.0;
    FEN_ascenseur( fen_vue_3d, ASC_HORIZONTAL, (int) position, -1 );
    FEN_redessin( fen_vue_3d, BI_RDS_TOUT );
  }

  MOL_PrepareListe(); /* Liste des molcules */
  return 0; /* Tout s'est bien pass */
}

/* ~-~-~-~-~-~-~-~- Aller  un atome absorbeur ~-~-~-~-~-~-~-~- */

void ABS_aller(Absorbeur *absorbeur, boolean redessin)
{
  OBJ_active( AFF_STRUCTURE, AST_CHEM_AVANT, FORCE_NON ); /* De toute faon... */

  if ( NULL == absorbeur ) { /* <=> Aucun absorbeur dans la molcule courante */
    OBJ_active( AFF_STRUCTURE, AST_ABS_AVANT, FORCE_NON );
    OBJ_active( AFF_STRUCTURE, AST_ABS_APRES, FORCE_NON );

    met_chaine(AFF_STRUCTURE, AST_NOM_ABSORB, LG_MAX_TXT_ABSORBEUR, "Aucun absorbeur", TXT_COPIER);
    OBJ_active(AFF_STRUCTURE, AST_EXPERIENCE, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_THEORIE, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEMINS, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEM_APRES, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEMIN_NOM, FORCE_NON);
    met_chaine(AFF_STRUCTURE, AST_CHEMIN_NOM, LG_MAX_TXT_CHEMIN, "Aucun absorbeur", TXT_COPIER);
    return;
  }

  /* Affichage dans la fentre de la vue structurale */
  met_chaine(AFF_STRUCTURE, AST_NOM_ABSORB, LG_MAX_TXT_ABSORBEUR, absorbeur->nom, TXT_COPIER);

  /* On va au modle par dfaut de l'atome absorbeur */
  if ( absorbeur->nbr_modeles > 0 ) {
    ABS_AllerModele( absorbeur->modele_actif, FAUX);
  } else { /* Aucun modle */
    OBJ_choisit(AFF_STRUCTURE, AST_EXPERIENCE, FORCE_NON);
    OBJ_choisit(AFF_STRUCTURE, AST_THEORIE, FORCE_NON);
    OBJ_choisit(AFF_STRUCTURE, AST_CHEMINS, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_EXPERIENCE, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_THEORIE, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEMINS, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEM_APRES, FORCE_NON);
    OBJ_active(AFF_STRUCTURE, AST_CHEMIN_NOM, FORCE_NON);
    met_chaine(AFF_STRUCTURE, AST_CHEMIN_NOM, LG_MAX_TXT_CHEMIN, "Aucun mod"e_GRAVE"le", TXT_COPIER);
  }

  if ( VRAI == redessin ) {
#ifdef AVEC_OPEN_GL_ /* Affichage de la vue 3d seulement avec Open GL */
    if (liste_molecule != 0) { /* Retrace la molcule car absorbeur en orange... */
      glNewList(liste_molecule, GL_COMPILE);
      dessine_molecule(mol_courante, mode_vue3d);
      glEndList();
    }
#endif
    FEN_redessin(fen_vue_3d, BI_RDS_TOUT);

    ABS_PrepareListe(); /* Liste des absorbeurs, dans la liste des molcules */
  }
}

/* ~-~-~-~-~-~-~-~- Aller au k-me absorbeur d'une molcule ~-~-~-~-~-~-~-~- */

void MOL_AllerAbsorbeur(Molecule *molecule, unsigned int num_absorbeur)
{
  Absorbeur *absorbeur;

  if ( NULL == molecule ) return;
  if ( NULL == molecule->absorbeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : molcule sans absorbeur !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    ABS_aller( NULL, VRAI );
    return;
  }
  if ( num_absorbeur >= molecule->nbr_absorbeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : la molcule n'a que %u absorbeur(s) mais on demande le n%u !\n",
		   __FILE__, __FUNCTION__, __LINE__, molecule->nbr_absorbeurs, num_absorbeur + 1);
    num_absorbeur = 0;
  }

  absorbeur = molecule->absorbeurs + num_absorbeur;
  molecule->absorbeur_courant = num_absorbeur;
  ABS_aller(absorbeur, VRAI);
}

/* ~-~-~-~-~-~-~-~- Aller  un modle de l'atome absorbeur ~-~-~-~-~-~-~-~-
 * Ne change pas le modele dans le selecteur de modele,
 *   car deja appele quand on le change ==> ferait une pseudo-boucle sans fin !
 */

void ABS_AllerModele(struct modele_ajuste *modele, boolean redessin)
{
  if ( NULL == modele ) return;

  if ( NULL == modele->experience ) {
    OBJ_choisit( AFF_STRUCTURE, AST_EXPERIENCE, FORCE_NON );
    OBJ_active ( AFF_STRUCTURE, AST_EXPERIENCE, FORCE_NON );
  } else {
    OBJ_active( AFF_STRUCTURE, AST_EXPERIENCE, FORCE_OUI );
  }

  if ( NULL == modele->modele ) {
    OBJ_choisit( AFF_STRUCTURE, AST_THEORIE, FORCE_NON );
    OBJ_active ( AFF_STRUCTURE, AST_THEORIE, FORCE_NON );
  } else {
    OBJ_active( AFF_STRUCTURE, AST_THEORIE, FORCE_OUI );
  }

  if ( MODELE_EXAFS_CHEMINS == modele->type ) {
    Modele_FEFF *feff;

    feff = modele->infos_modele.feff;
    if ( 0 == feff->nbr_couches ) { /* Aucune couche dans l'ajustement */
      OBJ_choisit(AFF_STRUCTURE, AST_CHEMINS, FORCE_NON);
      OBJ_active(AFF_STRUCTURE, AST_CHEMINS, FORCE_NON);
      OBJ_active(AFF_STRUCTURE, AST_CHEM_APRES, FORCE_NON);
      OBJ_active(AFF_STRUCTURE, AST_CHEMIN_NOM, FORCE_NON);
      met_chaine(AFF_STRUCTURE, AST_CHEMIN_NOM, LG_MAX_TXT_CHEMIN, "Aucun chemin", TXT_COPIER);
    } else {
      OBJ_active(AFF_STRUCTURE, AST_CHEMINS, FORCE_OUI);
      OBJ_active(AFF_STRUCTURE, AST_CHEM_APRES, feff->nbr_couches > 1);
      OBJ_active(AFF_STRUCTURE, AST_CHEMIN_NOM, FORCE_OUI);

      if (OBJ_choisit(AFF_STRUCTURE, AST_CHEMINS, QUESTION) == VRAI) {
	VUE3D_ChoisirChemin();
      }
    }

    /* Arbre d'dition des couches */
    if ( VRAI == arbres[ AJ_EXAFS_CHEMINS ].drapeaux.ouvert ) {
      COUCHE_OuvrirEditeur( feff );
    }
  }

  if ( VRAI == redessin ) {
    FEN_redessin( fen_vue_3d, BI_RDS_TOUT );
  }
}
