/* - Logiciel d'Analyse des spectres EXAFS
   (C) 1997 - 1999, Emmanuel CURIS
   Cration : 22 novembre 1997

   Tout ce qui est gestion des atomes (briques de base d'une entit chimique pour LASE)

   tat : il faudrait travailler sur l'effacement -> remodelage des liaisons si besoin...

    6, 9 janvier ; 5 fvrier 1998
     * Inclus "gem_gene.h" = pour les dfinitions GEM...
     * Fonction gnrique d'import des atomes : ATM_import()
     * Continuation de ATM_num_atomique() : devrait marcher jusqu' l'lment 103...

   22 & 23 novembre ; 10, 18 dcembre 1997
     * Cration de ATM_num_atomique() : numro atomique associ au symbole donn
     * On continue le paramtrage des infos atomiques
     * Entre des paramtres de base - de H  Zn
     * Routine de cration d'un nouvel atome
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h> /* Pour isspace */
#include <string.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h"
#include "feff.h"    /* Pour vrification du modle FEFF si on efface un atome */
#include "modeles.h" /* Pour vrification des modles d'ajustement */
#include "science.h" /* Pour M_PI, si ncessaire */
#include "tracer.h"  /* Pour le redessin en cas d'effacement */

#include "atm_atomes.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean ATM_DefaitLiens(Molecule *molecule, Atome *atome);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~~~~~~~~~~~~~~~~ Cration, effacement, ... des atomes ~~~~~~~~~~~~~~~~~~~~~~ */

Atome *ATM_creer(Molecule *molecule, Atome *avant, int numero_atomique, Vecteur3D *position, double charge, double rayon)
{
  Atome *atome = NULL;

  if ( NULL == molecule ) {
    fprintf(stderr, "ATM_creer : molcule NULL\n");
    return NULL;
  }

  atome = (Atome *) malloc( (size_t) sizeof(Atome) );
  if (atome == NULL) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "atomes.c", "ATM_creer", "atome");
    return NULL;
  }

  /* On ralise l'enchanement des atomes, dans la molcule indique */
  molecule->nbr_atomes++;
  if ( NULL == molecule->premier ) { /* Aucun atome dans la molcule */
    molecule->premier = atome;
    atome->suivant = NULL;
  } else if (avant == NULL) { /* Pas d'atome avant = on place aprs le dernier atome */
    avant = molecule->premier;
    while ( avant->suivant != NULL )
      avant = avant->suivant;
    avant->suivant = atome;
    atome->suivant = NULL;
  } else { /* On place juste aprs l'atome indiqu */
    /* On vrifie que l'atome est bien dans la molcule indique */
    Atome *atm;

    atm = molecule->premier;
    while ( atm != avant ) {
      atm = atm->suivant;
      if ( NULL == atm ) {
	(void) fprintf(stderr, "ATM_creer : l'atome %p n'est pas dans la molcule %p !\n",
		       (void *) avant, (void *) molecule);
	return NULL;
      }
    }

    atome->suivant = avant->suivant;
    avant->suivant = atome;
  }

  /* On remplit avec les donnes transmises */
  atome->num_atomique = numero_atomique;
  if ( position != NULL ) {
    atome->position = *position;
  } else {
    atome->position.x = 0.0; atome->position.y = 0.0; atome->position.z = 0.0;
  }
  atome->charge = (float) charge;
  if (rayon <= 0.0) { /* Valeur absurde --> on prend le rayon de covalence des tables */
    atome->rayon = ATM_Z_vers_rayon( numero_atomique, ATM_RAYON_COVALENT);
  } else
    atome->rayon = (float) rayon;

  atome->nbr_liaisons = 0;
  atome->nbr_coval = 0;
  atome->type_atome = 0;

  /* Pas d'erreur sur la position, a priori */
  atome->err_position = NULL;

  /* Pas de modle ajust -> plac  l'origine arbitrairement */
  atome->position_ajustee.x = 0.0; atome->position_ajustee.y = 0.0; atome->position_ajustee.z = 0.0;

  atome->fragment = NULL; /* Contenu dans aucun fragment */
  atome->marques.visible = VRAI; /* Par dfaut, marqu visible */
  atome->marques.absorbeur = FAUX;
  atome->marques.multi_frg = FAUX;
  atome->marques.avec_modele = FAUX; /* x_aj... n'ont aucun sens */
  atome->nbr_modeles = 0; /* Dans aucun modle  la cration */
  atome->index = 0;

  /* Mise  jour de la molcule */
  molecule->masse_molaire += ATM_Z_vers_masse( numero_atomique, ATM_MASSE_MOLAIRE );
  /* Voir pour l'adaptation du barycentre */

  if (molecule == mol_courante)
    ATM_PrepareListe();

  return atome; /* L'atome nouvellement cr */
}

/* -------- Crer des erreurs sur un atome -------- */

int ATM_CreerErreurs(Atome *atome)
{
  if ( NULL == atome ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'atome indiqu\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( atome->err_position != NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : L'atome a dj des erreurs dfinies\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return +1;
  }

  /* On rserve pour les erreurs */
  atome->err_position = (struct err_coord_3d *) malloc( sizeof( struct err_coord_3d ) );
  if ( NULL == atome->err_position ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver les erreurs.\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -10;
  }

  /* On initialise  0 */
  atome->err_position->var_x  = 0.0; atome->err_position->var_y  = 0.0; atome->err_position->var_z  = 0.0;
  atome->err_position->cov_xy = 0.0; atome->err_position->cov_xz = 0.0; atome->err_position->cov_yz = 0.0;

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Copier un atome avec des informations complmentaires -------- */

Atome *ATM_copier(Atome *original, Molecule *cible)
{
  Atome *copie;

  if (original == NULL) return NULL;
  if ( NULL == cible ) return NULL;

  copie = ATM_creer(cible, NULL, original->num_atomique, &(original->position), original->charge, original->rayon);
  if (copie == NULL) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la copie\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On s'occupe des erreurs */
  if ( original->err_position != NULL ) {
    int retour;

    retour = ATM_CreerErreurs( copie );
    if ( retour < 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans ATM_CreerErreurs() [code %d] : impossible de copier les erreurs\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return copie;
    }

    *(copie->err_position) = *(original->err_position);
  }


  return copie;
}

/* -------- Dfaire les divers liens qui peuvent exister vers un atome -------- */

static boolean ATM_DefaitLiens(Molecule *molecule, Atome *atome)
{
  if (NULL == atome) return FAUX;
  if (NULL == molecule) return VRAI; /* Aucune molcule: l'atome ne peut avoir de lien au-dehors de cette molcule */

  /* Premire tape : on l'limine des fragments */
  while (atome->fragment != NULL)
    FRG_efface_atome(atome->fragment, atome, VRAI);

  /* Deuxime tape : les liaisons chimiques
     En l'tant des fragments, les liaisons sont toutes devenues inter-fragments */
  while (atome->nbr_liaisons > 0) { /* Tant qu'il reste des liaisons */
    Liaison *liaison;

#ifdef _DEVELOPPE_
    printf("        il lui reste %d liaisons...\n", (int) (atome->nbr_liaisons));
#endif
    /* Dans la molcule  laquelle il appartient ? */
    if ( molecule->liaisons != NULL ) {
      boolean trouve = FAUX;

      liaison = molecule->liaisons;
      while (liaison != NULL) {
	if ( VRAI == ATM_EstDansLiaison(atome, liaison) ) {
	  trouve = VRAI;
	  effacer_liaison(liaison, VRAI);
#ifdef _DEVELOPPE_
	  printf("          dont une trouve dans la molcule %s\n", molecule->nom);
#endif
	  break; /* Car suite de liaisons modifie, a priori */
	}
	liaison = liaison->suivante;
      }

      if (trouve == VRAI)
	continue;
    }
#ifdef _DEVELOPPE_
    printf("          non trouve dans sa molcule (%p) ni son fragment (%p)\n", (void *) molecule, (void *) atome->fragment);
#endif

    /* A FAIRE : liaisons inter-molculaires */
  }

  if ( VRAI == atome->marques.absorbeur ) { /* C'est un absorbeur ! */
    unsigned int i;

    for (i = 0 ; i < molecule->nbr_absorbeurs ; i++) {
      Absorbeur *absorbeur = molecule->absorbeurs + i;

      if (absorbeur->absorbeur == atome) { /* On efface donc cet absorbeur */
	ABS_oter(molecule, i, FAUX); /* On n'efface pas l'atome associ */
	break;
      }
    }
  }

  while ( atome->nbr_modeles > 0 ) { /* Il appartient au moins  un modle */ 
    unsigned int i;

    for (i = 0 ; i < molecule->nbr_absorbeurs ; i++) {
      Absorbeur *absorbeur = molecule->absorbeurs + i;
      struct modele_ajuste *modele;

      if (NULL == absorbeur->modeles) { /* Aucun modle */
	continue;
      }

      for ( modele = absorbeur->modeles ; modele != NULL ; modele = modele->suivant ) {
	switch (modele->type)
	{
	case MODELE_EXAFS_CHEMINS:
	  FEFF_EnleverAtome(modele->infos_modele.feff, atome, FAUX);
	  break;

	default:
	  (void) fprintf(stderr, "ATM_effacer / Dfaire liens : type de modle inconnu !\n");
	}
      }
    }
  }

  return VRAI;
}

/* ------------------ Effacement d'un atome -----------------------------------------------------\
 |   Renvoie un int : <0 en cas d'erreur, FAUX si on n'a pas effac, VRAI si on a effac         |
 |
 | atome : l'atome qu'il faut effacer
 | forcer : VRAI si on ne demande pas confirmation
 \---------------------------------------------------------------------------------------------- */

int ATM_efface(Molecule *molecule, Atome *atome, boolean forcer)
{
  if (atome == NULL) return FAUX; /* Atome inexistant */
  if ( NULL == molecule ) { /* On cherche la molcule associe */
    molecule = quelle_molecule(atome, NULL);
    if ( NULL == molecule ) {
      (void) fprintf(stderr, "ATM_efface : atome hors d'une molcule !\n");
      return FAUX;
    }
  }

  if (forcer == FAUX) { /* On demande confirmation */
    int reponse;

    reponse = BI_ERR_alerte( NULL, &Q_EffacerAtome, VRAI, 
			     ATM_Z_vers_nom( atome->num_atomique ), atome->position.x, atome->position.y, atome->position.z);
    if ( 2 == reponse ) return FAUX;
  }

#ifdef _DEVELOPPE_
  printf("      atome : on efface le %s en (%f, %f, %f)\n", ATM_donnees[atome->num_atomique - 1].nom,
	 atome->position.x, atome->position.y, atome->position.z);
#endif

  /* On commence par dfaire le chanage */
  molecule->nbr_atomes--;
  if ( atome == molecule->premier ) { /* On efface le premier atome */
    if ( atome->suivant == NULL ) { /* C'est aussi le dernier */
      molecule->premier = NULL;
      molecule->nbr_atomes = 0;
      /* Molcule sans atome : on l'efface */
      MOL_effacer( molecule, VRAI, VRAI);
      molecule = NULL;
    } else {
      molecule->premier = atome->suivant;
      if (ATM_courant == atome) 
	ATM_aller(molecule->premier);
    }
  } else {
    Atome *avant;

    avant = molecule->premier;
    while (avant->suivant != atome) {
      avant = avant->suivant;
      if (NULL == avant) {
	(void) fprintf(stderr, "ATM_efface : pas d'atome avant celui indiqu, dans la molcule !\n");
	break;
      }
    }
    if (NULL != avant)
      avant->suivant = atome->suivant;
    if (atome == ATM_courant) {
      ATM_aller(avant);
    }
  }
  ATM_PrepareListe();

  /* La srie de vrifications... */
  (void) ATM_DefaitLiens(molecule, atome);

  /* On libre les erreurs */
  ATM_EffacerErreurs( atome );

  /* On peut le librer */
  free(atome); atome = NULL;

  /* Faut-il redessiner ? */
  if ( (fen_vue_3d != NULL) && (mol_courante == molecule) ) {
#ifdef AVEC_OPEN_GL_
    if (liste_molecule) {
      /* Les atomes */
      glNewList(liste_molecule, GL_COMPILE);
        dessine_molecule(mol_courante, mode_vue3d);
      glEndList(); 
      /* Les liaisons ont pu changer aussi */
      glNewList(liste_liaisons, GL_COMPILE);
        dessine_liaisons(mol_courante, mode_vue3d);
      glEndList(); 
    }
#endif
    FEN_redessin(fen_vue_3d, BI_RDS_GL);
  }

  return VRAI; /* Pas de probleme et l'effacement a eu lieu */
}

/* ~~~~~~~~~~~~~~~~~~ Quelques fonctions d'interface sympathiques ~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Renvoie la (premire) molcule  laquelle appartient l'atome indiqu --------\
 |   Utilise pour a le fragment ou (en priorit) les atomes absorbeurs                  |
 |   S'il existe, renvoie aussi l'absorbeur qui correspond  cet atome                   |
 \-------------------------------------------------------------------------------------- */

Molecule *quelle_molecule(Atome *atome, Absorbeur **absorbeur)
{
  Molecule *molecule;

  if (atome == NULL) {
#ifdef _DEVELOPPE_
    fprintf(stderr, "quelle_molecule : atome NULL !\n");
#endif
    return NULL;
  }
  if (premiere_molecule == NULL) return NULL;

  if (atome->marques.absorbeur == VRAI) { /* Il est utilis quelque part comme absorbeur */
    for (molecule = premiere_molecule ; molecule ; molecule = molecule->suivante) {
      if (molecule->nbr_absorbeurs > 0) { /* La molcule a des absorbeurs */
	unsigned int i;

	for (i = 0 ; i < molecule->nbr_absorbeurs ; i++) {
	  if ( molecule->absorbeurs[i].absorbeur == atome ) {
	    if (absorbeur != NULL) /* On s'intresse aussi  l'absorbeur */
	      *absorbeur = molecule->absorbeurs + i;
	    return molecule;
	  }
	}
      }
    }

#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "Atome marqu comme absorbeur, mais aucune molcule ne le contient comme absorbeur !\n");
#endif
  }

  if (absorbeur != NULL)
    *absorbeur = NULL; /* Pas d'absorbeur associ */

  if (atome->fragment) { /* Il appartient  une entit chimique prcise */
    /* Comme il y a moins de fragments que d'atomes, a priori plus efficace de chercher le fragment */
    for (molecule = premiere_molecule ; molecule ; molecule = molecule->suivante) {
      Fragment *fragment;

      for (fragment = molecule->fragments ; fragment ; fragment = fragment->suivant) {
	if (fragment == atome->fragment)
	  return molecule;
      }
    }

#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "Atome appartenant  un fragment, mais aucune molcule ne contient ce fragment !\n");
#endif
  }

  /* On cherche dans les chanages */
  for (molecule = premiere_molecule ; molecule != NULL ; molecule = molecule->suivante) {
    Atome *atm;

    if (NULL == molecule->premier) continue; /* Pas d'atome dans cette molcule */

    for (atm = molecule->premier ; atm != NULL ; atm = atm->suivant) {
      if ( atm == atome ) {
	return molecule; /* L'atome appartient  cette molcule */
      }
    }
  }

  (void) fprintf(stderr, "quelle_molecule : l'atome indiqu n'appartient  aucune molcule !\n");

  return NULL; /* Atome n'appartenant  rellement aucune molcule... */
}

/* -------- Mettre  zro tous les index d'atomes -------- */

void ATM_zero(Molecule *molecule)
{
  Atome *atome;

  if (NULL == molecule) return;

  for (atome = molecule->premier ; atome ; atome = atome->suivant)
    atome->index = 0;
}

/* ~~~~~~~~ Quelques fonctions gomtriques simples ~~~~~~~~ */

/* -------- Distance entre deux atomes AB ------------------ */

double ATM_distance(Atome *A, Atome *B)
{
  double distance;

  if ( (A == NULL) || (B == NULL) ) return 0.0;

  distance = U3D_Distance( &(A->position), &(B->position) );
  OPTION_A_FAIRE( "Prendre en compte les erreurs ?" );

  return distance;
}

/* -------- Angle entre trois atome ABC (en degrs)
   L'angle est positif, compris entre 0 et 180, avec l'atome B au sommet
 -------- */

double ATM_angle3(Atome *A, Atome *B, Atome *C)
{
  double cosinus, angle, err_cosinus = 0.0;
  Vecteur3D v_AB, v_BC;
  Vecteur3D_Erreurs var_AB, var_BC;

  if ( (A == NULL) || (B == NULL) || (C == NULL) )
    return 0.0;

  /* Vecteur AB normalis */
  (void) U3D_VecteurDirecteur( &(A->position), &(B->position), &v_AB, A->err_position, B->err_position, &var_AB );
  (void) U3D_Normalise( &v_AB, &var_AB );

  /* Vecteur BC normalis */
  (void) U3D_VecteurDirecteur( &(B->position), &(C->position), &v_BC, B->err_position, C->err_position, &var_BC );
  (void) U3D_Normalise( &v_BC, &var_BC );

  /* Cosinus donn par le produit scalaire, puisque les deux vecteurs sont norms */
  cosinus = U3D_ProduitScalaire( &v_AB, &v_BC, &var_AB, &var_BC, &err_cosinus );
  angle = acos(cosinus) * 180.0 / M_PI;

  return angle;
}
