/* Bibliothèque GEM / X_windows étendue
   (C) Emmanuel CURIS, janvier 2003

   Dessin des objets « GEM », à l'origine
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"
#include "txt_utils.h"

Pixmap PIX_inactif;

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ------ Portions communes ------- */

static void OBJ_DessineFond   (Window fenetre, unsigned int larg, unsigned haut, int couleur_fond, int couleur_hg, int couleur_bd);
static void OBJ_DessineFleche (Window fenetre, int x, int y, int couleur, int type);
static void OBJ_DessineLigne3D(Window fenetre, int x, int y, unsigned longueur, int sens);
static void OBJ_DessineTexte  (Window fenetre, int x, int y, const char *texte, int couleur, XFontStruct *fonte);

static unsigned int OBJ_LargeurTexte  (const char *texte, XFontStruct *fonte);
static char *       OBJ_TedinfoVChaine(TEDINFO *tedinfo);

/* ------ Contenu des objets ------ */

static void DOBJ_AppelMenu    (Window fenetre, OBJECT *objet);
static void DOBJ_Boite        (Window fenetre, OBJECT *objet);
static void DOBJ_Bouton       (Window fenetre, OBJECT *objet);
static void DOBJ_Boxchar      (Window fenetre, OBJECT *objet);
static void DOBJ_Chaine       (Window fenetre, OBJECT *objet);
static void DOBJ_Interne      (Window fenetre, OBJECT *objet);
static void DOBJ_Nombre       (Window fenetre, OBJECT *objet);
static void DOBJ_Separateur   (Window fenetre, OBJECT *objet);
static void DOBJ_StyleLigne   (Window fenetre, OBJECT *objet);
static void DOBJ_StyleMarque  (Window fenetre, OBJECT *objet);
static void DOBJ_Texte        (Window fenetre, OBJECT *objet);
static void DOBJ_TexteEditable(Window fenetre, OBJECT *objet);
static void DOBJ_TexteInfini  (Window fenetre, OBJECT *objet);
static void DOBJ_Titre        (Window fenetre, OBJECT *objet);
static void DOBJ_Utilisateur  (Window fenetre, OBJECT *objet);

/* ~~~~~~~~ Réalisation pratique des fonctions ~~~~~~~~ */

/* -------- Routine de redessin d'un objet (et de ses fils, éventuellement) : conventions GEM -------- */

void BI_DessinerObjet(unsigned int num_arbre, unsigned int objet)
{
  int reponse;

  /* 1. Vérification des valeurs */
  reponse = OBJ_ControlerValeurs( num_arbre, objet );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : valeurs incorrectes [arbre = %u, objet = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, objet);
    return;
  }

  /* Inutile de redessiner si l'arbre est fermé ! */
  if ( FAUX == arbres[num_arbre].drapeaux.ouvert ) {
#ifdef _DEVELOPPE_
    (void) printf("BI_DessinerObjet : appel avec un arbre fermé [arbre : %u, objet : %u]\n", num_arbre, objet);
#endif
    return;
  }

  /* On redessine l'objet lui-même */
#ifdef _DEVELOPPE_
  if ( NULL == arbres[ num_arbre ].objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : arbre ouvert, mais pas de fenêtres X associée [arbre = %u, objet = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, objet);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
#endif
  OBJ_dessine( arbres[ num_arbre ].objets[ objet ], arbres[num_arbre].adresse + objet );

  if ( arbres[ num_arbre ].adresse[ objet ].ob_head != -1 ) { /* L'objet a des fils */
    unsigned int i, dernier;

    i = (unsigned int) arbres[ num_arbre ].adresse[ objet ].ob_head;
    dernier = (unsigned int) arbres[ num_arbre ].adresse[ objet ].ob_tail;
    for ( ; i <= dernier ; i++ ) {
      /* Remarque : si un enfant a aussi des enfants, ils ne sont pas redessinés par cette méthode... */
      OBJ_dessine( arbres[ num_arbre ].objets[ i ], arbres[ num_arbre ].adresse + objet);
    }
  }

  /* Si c'est l'éditable courant, il faut le curseur ! */
  if ( objet == arbres[ num_arbre ].edit_courant ) {
    BI_DessinerCurseur( arbres + num_arbre );
  }
}

/* -------- Redessin d'un objet : conventions X --------------\
 |   Renvoie 0 si tout s'est bien passé                       |
 |           <0 en cas d'erreur grave (+ message sur stderr)  |
 |           >0 en cas d'erreur mineure                       |
 |                                                            |
 | fenetre : la fenêtre correspondant au bouton               |
 | objet   : pointeur sur _l'objet_ lui-même                  |
 \----------------------------------------------------------- */

int OBJ_dessine(Window fenetre, OBJECT *objet)
{
  int x_obj = 0, y_obj = 0; /* Position de l'objet */

  /* 1. Vérifications préliminaires */
  if ( None == fenetre ) { /* Redessin d'un objet non ouvert ! */
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s, %s l.%u : objet visible, mais sans fenêtre X associée (arbre fermé ?)\n",
		    __FILE__, __FUNCTION__, __LINE__ );
#endif
    return +1;
  }
  if ( NULL == objet ) {
    (void) fprintf( stderr, "%s, %s l.%u : objet inexistant\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( objet->ob_flags & HIDETREE ) /* Objet caché : rien à faire ! */
    return 0;

  /* 2. Préparation des dessins X */
  XClearWindow( BI_Display, fenetre ); /* On remet le fond à son origine ! */
  XSetLineAttributes( BI_Display, BI_GC_objets, 1, LineSolid, CapButt, JoinMiter );
  if ( objet->ob_state & DISABLED ) {
    XGCValues valeurs;

    valeurs.fill_style = FillOpaqueStippled;
    valeurs.stipple = PIX_inactif;

    XChangeGC( BI_Display, BI_GC_objets, GCStipple | GCFillStyle, &valeurs );
  } else {
    XSetFillStyle( BI_Display, BI_GC_objets, FillSolid );
  }

  /* 3. En fonction du type, dessin du contenu */
  switch ( objet->ob_type )
  {
  case G_SSMENU:
    DOBJ_AppelMenu( fenetre, objet );
    break;

  case G_TXT_INFINI:
    DOBJ_TexteInfini( fenetre, objet );
    break;

  case G_NOMBRE:
    DOBJ_Nombre( fenetre, objet );
    break;

  case G_SEPARATEUR:
    DOBJ_Separateur( fenetre, objet );
    break;

  case G_STYLELIGNE:
    DOBJ_StyleLigne( fenetre, objet );
    break;

  case G_STYLEMARQUE:
    DOBJ_StyleMarque( fenetre, objet );
    break;

  case G_CLOSER: /* Objets internes quasi-identiques */
  case G_INFOS:
  case G_FULLER:
  case G_FLECHE:
  case G_ASCENSEUR:
    DOBJ_Interne(fenetre, objet);
    break;

  case G_BUTTON: /* Boutons proprement dits */
    DOBJ_Bouton(fenetre, objet);
    break;

  case G_TITLE:  /* Titre des menus : comme des boutons... */
    DOBJ_Titre(fenetre, objet);
    break;
    
  case G_BOX:
  case G_IBOX:
    DOBJ_Boite(fenetre, objet);
    break;

  case G_STRING:
    DOBJ_Chaine(fenetre, objet);
    break;

  case G_BOXCHAR: /* Boîte contenant _1_ caractère */
    DOBJ_Boxchar(fenetre, objet);
    break;

  case G_TEXT:
  case G_BOXTEXT:
    DOBJ_Texte(fenetre, objet);
    break;

  case G_FTEXT:
  case G_FBOXTEXT:
    DOBJ_TexteEditable(fenetre, objet);
    break;

  case G_IMAGE:
    OPTION_A_FAIRE( "Redessin d'un objet G_IMAGE" );
    break;

  case G_USERDEF: /* Objet utilisateur */
    if ( NULL == objet->ob_spec.util->ub_code ) {
      (void) fprintf( stderr, "%s,%s l.%u : pas de fonction de redessin pour l'objet utilisateur\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      DOBJ_Separateur( fenetre, objet );
    } else {
      DOBJ_Utilisateur( fenetre, objet );
    }
    break;

  case G_ICON: /* Rien à faire normalement : le XPM est en fond de fenêtre -> dessiné par X comme il faut... */
    break;

  default:
    AVERTISSEMENT( "Objet de type inconnu : %d",  objet->ob_type );
  }

  /* Parties du dessin communes à tous, mais à faire après */
  if ( objet->ob_state & CROSSED ) {
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );

    XDrawLine( BI_Display, fenetre, BI_GC_objets, x_obj, y_obj, (int) objet->ob_width, (int) objet->ob_height );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, x_obj, (int) objet->ob_height, (int) objet->ob_width, y_obj );
  }

  if ( objet->ob_state & CHECKED ) {
    if ( objet->ob_state & SELECTED )
      XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    else
      XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR  ].pixel );
    XSetLineAttributes( BI_Display, BI_GC_objets, 2, LineSolid, CapButt, JoinMiter );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, x_obj + (int) (0.25 * BI_larg_car), y_obj + (int) (BI_haut_car >> 1),
	       x_obj + (int) BI_larg_car, y_obj + (int) (0.75 * BI_haut_car) );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, x_obj + (int) BI_larg_car, y_obj + (int) (0.75 * BI_haut_car),
	       x_obj + (int) (1.75 * BI_larg_car), y_obj + (int) (0.25 * BI_haut_car) );
  }

  /* Corrections des bords, pour avoir un résultat plus joli */
  if ( 0 == objet->ob_x ) {
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, 0, (int) objet->ob_height );
  }
  if ( 0 == objet->ob_y ) {
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, (int) objet->ob_width, 0 );
  }

  return 0; /* Pas d'erreur */
}

/* ~-~-~-~-~-~-~-~- Parties communes à tous les objets ~-~-~-~-~-~-~-~- */

/* -------- Une flèche, de direction variable, faisant 1x1 caractère -------- */

static void OBJ_DessineFleche(Window fenetre, int x, int y, int couleur, int sens)
{
  int nbr_sommets = 0; /* unsigned int serait plus logique, mais XFillPolygon demande un int... */
  unsigned int largeur;
  XPoint points[ 7 ];

  /* Quelle est la largeur d'un caractère ? */
  largeur = ( BI_larg_car > BI_haut_car ) ? ( BI_larg_car ) : ( BI_haut_car );

  /* On choisit la couleur demandée */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur ].pixel );

  /* On prépare les points de la flèche */
  switch ( sens )
  {
  case BI_OBJ_FLECHE_HAUTE:
    nbr_sommets = 7;
    points[ 0 ].x = x + (short) ( largeur * 0.3 );
    points[ 0 ].y = y + (short) ( largeur * 0.9 );
    points[ 1 ].x = points[ 0 ].x;
    points[ 1 ].y = y + (short) ( largeur * 0.5 );
    points[ 2 ].x = x + (short) ( largeur * 0.1 );
    points[ 2 ].y = points[ 1 ].y;
    points[ 3 ].x = x + (short) ( largeur * 0.5 );
    points[ 3 ].y = y + (short) ( largeur * 0.1 );
    points[ 4 ].x = x + (short) ( largeur * 0.9 );
    points[ 4 ].y = points[ 1 ].y;
    points[ 5 ].x = x + (short) ( largeur * 0.7 );
    points[ 5 ].y = points[ 1 ].y;
    points[ 6 ].x = points[ 5 ].x;
    points[ 6 ].y = points[ 0 ].y;
    break;

  case BI_OBJ_FLECHE_BASSE:
    nbr_sommets = 7;
    points[ 0 ].x = x + (short) ( largeur * 0.7 );
    points[ 0 ].y = y + (short) ( largeur * 0.1 );
    points[ 1 ].x = points[ 0 ].x;
    points[ 1 ].y = y + (short) ( largeur * 0.5 );
    points[ 2 ].x = x + (short) ( largeur * 0.9 );
    points[ 2 ].y = points[ 1 ].y;
    points[ 3 ].x = x + (short) ( largeur * 0.5 );
    points[ 3 ].y = y + (short) ( largeur * 0.9 );
    points[ 4 ].x = x + (short) ( largeur * 0.1 );
    points[ 4 ].y = points[ 1 ].y;
    points[ 5 ].x = x + (short) ( largeur * 0.3 );
    points[ 5 ].y = points[ 1 ].y;
    points[ 6 ].x = points[ 5 ].x;
    points[ 6 ].y = points[ 0 ].y;
    break;

  case BI_OBJ_FLECHE_DROITE:
    nbr_sommets = 7;
    points[ 0 ].x = x + (short) ( largeur * 0.1 );
    points[ 0 ].y = y + (short) ( largeur * 0.3 );
    points[ 1 ].x = x + (short) ( largeur * 0.5 );
    points[ 1 ].y = points[ 0 ].y;
    points[ 2 ].x = points[ 1 ].x;
    points[ 2 ].y = y + (short) ( largeur * 0.1 );
    points[ 3 ].x = x + (short) ( largeur * 0.9 );
    points[ 3 ].y = y + (short) ( largeur * 0.5 );
    points[ 4 ].x = points[ 1 ].x;
    points[ 4 ].y = y + (short) ( largeur * 0.9 );
    points[ 5 ].x = points[ 1 ].x;
    points[ 5 ].y = y + (short) ( largeur * 0.7 );
    points[ 6 ].x = points[ 0 ].x;
    points[ 6 ].y = points[ 5 ].y;
    break;

  case BI_OBJ_FLECHE_GAUCHE:
    nbr_sommets = 7;
    points[ 0 ].x = x + (short) ( largeur * 0.9 );
    points[ 0 ].y = y + (short) ( largeur * 0.3 );
    points[ 1 ].x = x + (short) ( largeur * 0.5 );
    points[ 1 ].y = points[ 0 ].y;
    points[ 2 ].x = points[ 1 ].x;
    points[ 2 ].y = y + (short) ( largeur * 0.1 );
    points[ 3 ].x = x + (short) ( largeur * 0.1 );
    points[ 3 ].y = y + (short) ( largeur * 0.5 );
    points[ 4 ].x = points[ 1 ].x;
    points[ 4 ].y = y + (short) ( largeur * 0.9 );
    points[ 5 ].x = points[ 1 ].x;
    points[ 5 ].y = y + (short) ( largeur * 0.7 );
    points[ 6 ].x = points[ 0 ].x;
    points[ 6 ].y = points[ 5 ].y;
    break;

  default:
    (void) fprintf( stderr, "%s,%s l.%u : type de flèche inconnu [%d]\n",
		    __FILE__, __FUNCTION__, __LINE__, sens );
    return;
  }

  /* On dessine */
  XFillPolygon( BI_Display, fenetre, BI_GC_objets, points, nbr_sommets, Convex, CoordModeOrigin );
}

/* -------- Le fond en 3D, avec des couleurs paramétrables -------- */

static void OBJ_DessineFond(Window fenetre, unsigned int larg, unsigned haut,
			    int couleur_fond, int couleur_hg, int couleur_bd)
{
  /* Le fond */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur_fond ].pixel );
  XFillRectangle( BI_Display, fenetre, BI_GC_objets, 0, 0, larg, haut );

  /* Bords haut & gauche, pour effets 3D */
  if ( couleur_hg != couleur_fond ) { /* Sinon, ne sert à rien ! */
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur_hg ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, larg, 0    );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, 0   , haut );
  }

  /* Bords droit & bas, pour effets 3D */
  if ( couleur_bd != couleur_fond ) { /* Sinon, ne sert à rien ! */
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur_bd ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 0       , haut - 1, larg    , haut - 1);
    XDrawLine( BI_Display, fenetre, BI_GC_objets, larg - 1, 0       , larg - 1, haut    );
  }
}

/* -------- Une ligne creuse "en 3D", horizontale ou verticale -------- */

static void OBJ_DessineLigne3D(Window fenetre, int x, int y, unsigned int longueur, int sens)
{
  XPoint points[ 2 ];

  /* Les extrémités */
  points[ 0 ].x = (short) x;
  points[ 0 ].y = (short) y;
  if ( ASC_VERTICAL == sens ) { /* Ligne verticale */
    points[ 1 ].x = points[ 0 ].x;
    points[ 1 ].y = (short) ( y + longueur );

    /* Tracé proprement dit */
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x + 1, points[ 0 ].y, points[ 1 ].x + 1, points[ 1 ].y );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 1 ].x - 1, points[ 1 ].y, points[ 1 ].x + 1, points[ 1 ].y );

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS_FONCE ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x - 1, points[ 0 ].y, points[ 1 ].x - 1, points[ 1 ].y );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x - 1, points[ 0 ].y, points[ 0 ].x + 1, points[ 0 ].y );
  } else {
    points[ 1 ].x = (short) ( x + longueur );
    points[ 1 ].y = points[ 0 ].y;

    /* Tracé proprement dit */
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS_FONCE ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x, points[ 0 ].y - 1, points[ 1 ].x, points[ 1 ].y - 1 );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 1 ].x, points[ 0 ].y - 1, points[ 1 ].x, points[ 1 ].y + 1 );

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC      ].pixel );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x, points[ 0 ].y + 1, points[ 1 ].x, points[ 1 ].y + 1 );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, points[ 0 ].x, points[ 0 ].y - 1, points[ 0 ].x, points[ 0 ].y + 1 );
  }
}

/* -------- Affiche une chaîne de caractères, avec caractères spéciaux éventuellement -------- */

static void OBJ_DessineTexte(Window fenetre, int x, int y, const char *texte, int couleur, XFontStruct *fonte)
{
  const char *debut, *pos_special;
  unsigned int larg_car;
  int lg_texte; /* unsigned serait + logique, mais XDrawString demande un int... */

  if ( NULL == fonte )
    fonte = BI_Fonte; /* Police par défaut ! */

#ifdef _DEVELOPPE_
  /* (void) fprintf( stderr, "%s,%s l.%u : Texte à afficher = *%s*\n", */
  /* 		  __FILE__, __FUNCTION__, __LINE__, texte ); */
#endif

  /* Largeur d'un caractère : pour les codes spéciaux... */
  larg_car = ( BI_larg_car > BI_haut_car ) ? ( BI_larg_car ) : ( BI_haut_car );

  /* Couleur & police du texte */
  XSetFont( BI_Display, BI_GC_objets, fonte->fid );
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur ].pixel );

  debut = texte;
  do {
    pos_special = debut;

    /* On cherche un bloc de texte « normal » */
    while ( ( *pos_special >= ' ' ) || ( *pos_special < 0 ) ) {
      pos_special++;
    }

    /* Longueur de ce bloc */
    lg_texte = pos_special - debut;

    /* On affiche ce bloc normal, s'il n'est pas vide */
    if ( lg_texte > 0 ) {
      if ( BI_FonteUTF8 != NULL ) {
	XRectangle dim_texte;

#ifdef X_HAVE_UTF8_STRING
	Xutf8DrawString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, x, y, debut, lg_texte );
	Xutf8TextExtents( BI_FonteUTF8, debut, lg_texte, NULL, &dim_texte );
#else
	XmbDrawString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, x, y, debut, lg_texte );
	XmbTextExtents( BI_FonteUTF8, debut, lg_texte, NULL, &dim_texte );
#endif
	x += dim_texte.width;
      } else {
	XDrawString( BI_Display, fenetre, BI_GC_objets, x, y, debut, lg_texte );

	/* On prévoit la position de la suite de la chaîne */
	x += XTextWidth( fonte, debut, lg_texte );
      }
    }

    if ( 0 == *pos_special ) { /* Dessin terminé */
      return;
    } else if ( 1 == *pos_special ) { /* Flèche vers le haut */
      OBJ_DessineFleche( fenetre, x, y - larg_car + 1, couleur, BI_OBJ_FLECHE_HAUTE );
      x += larg_car;
    } else if ( 2 == *pos_special ) { /* Flèche vers le bas */
      OBJ_DessineFleche( fenetre, x, y - larg_car + 2, couleur, BI_OBJ_FLECHE_BASSE );
      x += larg_car;
    } else if ( 3 == *pos_special ) { /* Flèche vers la droite */
      OBJ_DessineFleche( fenetre, x + 1, y - larg_car + 2, couleur, BI_OBJ_FLECHE_DROITE );
      x += larg_car;
    } else if ( 4 == *pos_special ) { /* Flèche vers la gauche */
      OBJ_DessineFleche( fenetre, x, y - larg_car + 2, couleur, BI_OBJ_FLECHE_GAUCHE );
      x += larg_car;
    } else if ( 5 == *pos_special ) { /* Croix de fermeture */
      XPoint points[ 4 ];

      points[ 0 ].x = x + (short) ( 0.2 * larg_car );
      points[ 0 ].y = (y - larg_car) + (short) ( 0.2 * larg_car );
      points[ 1 ].x = x + (short) ( 0.8 * larg_car );
      points[ 1 ].y = (y - larg_car) + (short) ( 0.8 * larg_car );
      points[ 2 ].x = points[ 0 ].x;
      points[ 2 ].y = points[ 1 ].y;
      points[ 3 ].x = points[ 1 ].x;
      points[ 3 ].y = points[ 0 ].y;

      XDrawSegments( BI_Display, fenetre, BI_GC_objets, (XSegment *) points, 2 );

      x += larg_car;
    } else {
      (void) fprintf( stderr, "%s,%s l.%u : code spécial inconnu [%hhX / texte=%s] — ignoré\n",
		      __FILE__, __FUNCTION__, __LINE__, *pos_special, texte );
    }

    debut = pos_special + 1;
  } while ( *debut != 0 );
}

/* -------- Calcul de la place occupée par une chaîne, avec modifications effectuées */

static unsigned int OBJ_LargeurTexte(const char *texte, XFontStruct *fonte)
{
  const char *debut, *pos_special;
  unsigned int larg_car, larg_texte = 0;

  /* Contrôles préliminaires */
  if ( NULL == texte ) return 0;
  if ( NULL == fonte )
    fonte = BI_Fonte;

  /* Largeur d'un caractère : pour les codes spéciaux... */
  larg_car = ( BI_larg_car > BI_haut_car ) ? ( BI_larg_car ) : ( BI_haut_car );

  /* Estimation de la largeur occupée */
  debut = texte;
  do {
    unsigned long lg_texte;

    pos_special = debut;
    while ( ( *pos_special >= ' ' ) || ( *pos_special < 0 ) ) {
      pos_special++;
    }

    /* Bloc de caractères normaux */
    lg_texte = pos_special - debut;
    if ( lg_texte > 0 ) {
      if ( BI_FonteUTF8 != NULL ) {
	XRectangle dim_texte;

#ifdef X_HAVE_UTF8_STRING
	Xutf8TextExtents( BI_FonteUTF8, debut, lg_texte, NULL, &dim_texte );
#else
	XmbTextExtents( BI_FonteUTF8, debut, lg_texte, NULL, &dim_texte );
#endif
	larg_texte += dim_texte.width;
      } else {
	larg_texte += XTextWidth( fonte, debut, lg_texte );
      }
    }

    /* Traitement du caractère spécial */
    if ( 0 == *pos_special ) {
#ifdef _DEVELOPPE_
/*       printf("OBJ_LargeurTexte : texte = \"%s\", partie : \"%s\" : %lu cars [%u]\n", */
/* 	     texte, debut, lg_texte, strlen( debut )); */
#endif
      return larg_texte; /* Chaîne finie */
    }
    if ( ( *pos_special > 0 ) && ( *pos_special <= 5 ) ) { /* Flèches, croix de fermeture */
      larg_texte += larg_car;
    }

    debut = pos_special + 1; /* On saute le caractère spécial */
  } while ( *debut != 0 );

  return larg_texte;
}

/* ~-~-~-~-~-~-~-~- Contenu spécifique de chaque objet ~-~-~-~-~-~-~-~- */

static void DOBJ_AppelMenu(Window fenetre, OBJECT *objet)
{
  const char *texte;
  int x_texte, y_texte;
  unsigned int larg, haut;
  unsigned int larg_texte;

  larg = objet->ob_width;
  if ( objet->ob_x > 0 ) larg++;
  haut = objet->ob_height;
  if ( objet->ob_y > 0 ) haut++;

  /* On met l'aspect 3D */
  if ( objet->ob_state & SELECTED) { /* Objet choisi */
    OBJ_DessineFond( fenetre, larg, haut, GRIS_FONCE, NOIR, BLANC );
  } else if ( objet->ob_state & DISABLED ) { /* Objet inactif */
    OBJ_DessineFond( fenetre, larg, haut, BLANC, BLANC, GRIS_FONCE );
  } else {
    OBJ_DessineFond( fenetre, larg, haut, GRIS, BLANC, GRIS_FONCE );
  }

  /* Position verticale du texte */
  y_texte  = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* Contrôle préliminaire */
  if ( NULL == objet->ob_spec.menu ) {
    (void) fprintf( stderr, "%s, %s l.%u : Pas d'infos de sous-menu !\n",
		    __FILE__, __FUNCTION__, __LINE__ );

    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> Menu = NULL <<", 10 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* On décide quel texte doit être affiché */
  texte = objet->ob_spec.menu->texte;
  if ( NULL == texte ) {
    /* Pas de texte particulier : on affiche le texte de l'option active */
    if ( ( 0 == objet->ob_spec.menu->nombre_options ) || ( NULL == objet->ob_spec.menu->options ) ) {
      (void) fprintf( stderr, "%s, %s l.%u : Pas de table d'options !\n",
		      __FILE__, __FUNCTION__, __LINE__ );
      return;
    }
    if ( objet->ob_spec.menu->option_choisie >= objet->ob_spec.menu->nombre_options ) {
      (void) fprintf( stderr, "%s, %s l.%u : Option choisie absurde [numéro %lu, max %lu]\n",
		      __FILE__, __FUNCTION__, __LINE__,
		      objet->ob_spec.menu->option_choisie, objet->ob_spec.menu->nombre_options - 1 );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return;
    }

    /* Contrôles faits : on prend le texte... */
    texte = objet->ob_spec.menu->options[ objet->ob_spec.menu->option_choisie ];
  }
  if ( NULL == texte ) {
    (void) fprintf( stderr, "%s, %s l.%u : Aucun texte trouvé pour le menu !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    texte = "Aucune option disponible";
  }

  /* Largeur occupée par le texte [dépend des caractères spéciaux présents] */
  larg_texte = OBJ_LargeurTexte( texte, BI_Fonte );

  /* On centre le texte dans le bouton */
  x_texte = ( larg - larg_texte ) >> 1;

#ifdef _DEVELOPPE_
  if ( x_texte < 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : chaîne trop longue [%s]\n",
		    __FILE__, __FUNCTION__, __LINE__, texte );
  }
#endif

  /* On affiche le texte */
  if ( objet->ob_state & SELECTED ) 
    OBJ_DessineTexte( fenetre, x_texte, y_texte, texte, BLANC, BI_Fonte );
  else
    OBJ_DessineTexte( fenetre, x_texte, y_texte, texte, NOIR , BI_Fonte );
}

static void DOBJ_Boite(Window fenetre, OBJECT *objet)
{
  int larg_obj, haut_obj;
  int couleur_fond, motif;

  larg_obj = objet->ob_width + 1;
  haut_obj = objet->ob_height + 1;

  couleur_fond = objet->ob_spec.gbox.couleur_fond;
  if ( BLANC == couleur_fond )
    couleur_fond = GRIS; /* Pour enjoliver l'aspect 3D */

  motif = objet->ob_spec.gbox.motif; /* Trame de remplissage... */
  if ( 0 == motif ) /* Pas de trame -> "blanc" */
    couleur_fond = GRIS; /* Donc 3D */

/*   printf("DOBJ_boite... couleur = %d, motif = %d\n", couleur_fond, motif); */
  /* Petit essai pour voir ce que donne la 3D : rend très bien */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ couleur_fond ].pixel ); /* Fond de la couleur */
  XFillRectangle( BI_Display, fenetre, BI_GC_objets, 0, 0, (unsigned) larg_obj, (unsigned) haut_obj );

  if ( ( motif < 7 ) && ( motif > 0 ) ) return; /* Pas de 3D si tramé */

  XSetLineAttributes( BI_Display, BI_GC_objets, 1, LineSolid, CapButt, JoinMiter);
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );
  XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, larg_obj, 0 );
  XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, 0, 0, haut_obj );

  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS_FONCE ].pixel ); /* Côté en gris --> 3D */
  XDrawLine( BI_Display, fenetre, BI_GC_objets, 0, haut_obj - 1, larg_obj, haut_obj - 1 );
  XDrawLine( BI_Display, fenetre, BI_GC_objets, larg_obj - 1, 0, larg_obj - 1, haut_obj );
}

static void DOBJ_Bouton(Window fenetre, OBJECT *objet)
{
  int x_texte, y_texte;
  unsigned int larg, haut;
  unsigned int larg_texte;

  larg = objet->ob_width;
  if ( objet->ob_x > 0 ) larg++;
  haut = objet->ob_height;
  if ( objet->ob_y > 0 ) haut++;

  /* On met l'aspect 3D */
  if ( objet->ob_state & SELECTED) { /* Objet choisi */
    OBJ_DessineFond( fenetre, larg, haut, GRIS_FONCE, NOIR, BLANC );
  } else if ( objet->ob_state & DISABLED ) { /* Objet inactif */
    OBJ_DessineFond( fenetre, larg, haut, BLANC, BLANC, GRIS_FONCE );
  } else {
    OBJ_DessineFond( fenetre, larg, haut, GRIS, BLANC, GRIS_FONCE );
  }

  /* Position verticale du texte */
  y_texte  = (haut + BI_Fonte->max_bounds.ascent) / 2;

  if ( NULL == objet->ob_spec.texte ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : chaîne NULL. Objet : %p\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet );
#endif
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL <<", 10 );
    return; /* Pas de texte associé */
  }

  /* Largeur occupée par le texte [dépend des caractères spéciaux présents] */
  larg_texte = OBJ_LargeurTexte( objet->ob_spec.texte, BI_Fonte );

  /* On centre le texte dans le bouton */
  x_texte = ( larg - larg_texte ) >> 1;

#ifdef _DEVELOPPE_
  if ( x_texte < 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : chaîne trop longue [%s]\n",
		    __FILE__, __FUNCTION__, __LINE__, objet->ob_spec.texte );
  }
#endif

  /* On affiche le texte */
  if ( objet->ob_state & SELECTED ) 
    OBJ_DessineTexte( fenetre, x_texte, y_texte, objet->ob_spec.texte, BLANC, BI_Fonte );
  else
    OBJ_DessineTexte( fenetre, x_texte, y_texte, objet->ob_spec.texte, NOIR , BI_Fonte );
}

static void DOBJ_Boxchar(Window fenetre, OBJECT *objet)
{
  unsigned int larg_obj, haut_obj;
  int x_texte, y_texte;
  unsigned int larg_texte;
  char code[ 2 ] = { 0, 0 };

  larg_obj = objet->ob_width;
  haut_obj = objet->ob_height;
  DOBJ_Boite(fenetre, objet); /* Pour le fond et tout et tout */

  /* Affichage du caractère */
  code[ 0 ] = objet->ob_spec.gbox.caractere;
  larg_texte = OBJ_LargeurTexte( code, BI_Fonte );
  x_texte  = (larg_obj - larg_texte) / 2; /* On centre */    
  y_texte  = (haut_obj + BI_Fonte->max_bounds.ascent) / 2;
  OBJ_DessineTexte( fenetre, x_texte, y_texte, code, NOIR, BI_Fonte );
}

/* -------- Chaîne de caractère -------- */

static void DOBJ_Chaine(Window fenetre, OBJECT *objet)
{
  unsigned int larg, haut;
  int y_texte;

  larg = objet->ob_width;
  haut = objet->ob_height;
  if ( objet->ob_y > 0 ) haut++; /* On a légèrement étendu l'objet pour améliorer le dessin... */

  /* On dessine le fond */
  if ( objet->ob_state & DRAW3D ) {
    if ( objet->ob_state & SELECTED ) {
      OBJ_DessineFond( fenetre, larg, haut, GRIS_FONCE, NOIR , BLANC      );
    } else {
      OBJ_DessineFond( fenetre, larg, haut, GRIS      , BLANC, GRIS_FONCE );
    }
  } else {
    if ( objet->ob_state & SELECTED ) { /* Fond noir */
      XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );  /* Fond noir   */
      XFillRectangle( BI_Display, fenetre, BI_GC_objets, 0, 0, larg, haut );
    } else {
      if ( objet->ob_state & DISABLED )
	XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ BLANC ].pixel );  /* Fond noir */
      else
	XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ GRIS  ].pixel );  /* Fond gris */
      XFillRectangle( BI_Display, fenetre, BI_GC_objets, 0, 0, 1U + larg, 1U + haut );
    }
  }

  /* Position du texte */
  y_texte = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* On met le texte */
  if ( NULL == objet->ob_spec.texte ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : chaîne NULL. Objet : %p\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet );
#endif
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL <<", 10 );
  } else {
    if ( objet->ob_state & SELECTED )
      OBJ_DessineTexte( fenetre, 1, y_texte, objet->ob_spec.texte, BLANC, NULL );
    else
      OBJ_DessineTexte( fenetre, 1, y_texte, objet->ob_spec.texte, NOIR , NULL );
  }
}

/* -------- Gadgets de fenêtre -------- */

static void DOBJ_Interne(Window fenetre, OBJECT *objet)
{
  const char *texte;
  int y_texte;
  unsigned int larg, haut;

  if ( ( objet->ob_type == G_INFOS ) || ( objet->ob_type == G_ASCENSEUR ) ) {
    int x, y;
    unsigned int i;
    Window racine;

    XGetGeometry(BI_Display, fenetre, &racine, &x, &y, &larg, &haut, &i, &i);
  } else { /* Objet carré, 1x1 */
    larg = ( BI_larg_car > BI_haut_car ) ? ( BI_larg_car ) : ( BI_haut_car );
    haut = larg;
  }

  /* Le fond 3D */
  OBJ_DessineFond( fenetre, larg, haut, GRIS, BLANC, GRIS_FONCE );

  /* Tous les dessins sont en noir, a priori... */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );

  /* Position du texte, s'il y en a un */
  y_texte  = (haut + BI_Fonte->max_bounds.ascent) / 2 - 1;

  /* La partie spécifique du dessin */
  switch ( objet->ob_type )
  {
  case G_CLOSER:
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 2, 2, (int) larg - 2, (int) haut - 2 );
    XDrawLine( BI_Display, fenetre, BI_GC_objets, 2, (int) haut - 2, (int) larg - 2, 2 );
    return;

  case G_INFOS:
    texte = objet->ob_spec.texte;
    if ( texte != NULL ) { /* Il y a bien un texte associé */
      if ( BI_FonteUTF8 != NULL ) {
	Xutf8DrawString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, 5, y_texte, texte, strlen( texte ) );
      } else {
	XDrawString( BI_Display, fenetre, BI_GC_objets, 5, y_texte, texte, strlen( texte ) );
      }
#ifdef _DEVELOPPE_
    } else {
      XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
      XDrawString( BI_Display, fenetre, BI_GC_objets, 5, y_texte, ">> NULL <<", 10 );
#endif
    }
    return;

  case G_FLECHE:
    OBJ_DessineFleche( fenetre, 0, 0, NOIR, objet->ob_spec.autre );
    return;
  }
}

/* -------- Éditable contenant un nombre -------- */

static void DOBJ_Nombre(Window fenetre, OBJECT *objet)
{
  struct infos_editable *nombre;
  unsigned int larg, haut;
  int y_texte, nbr_caracteres;
  char *vrai_texte;
  size_t lg_texte;

  larg = objet->ob_width;
  haut = objet->ob_height;
  y_texte = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* On met le même type qu'un objet éditable FTEXT */
  if ( objet->ob_flags & EDITABLE ) {
    OBJ_DessineFond( fenetre, larg, haut, BLANC, GRIS_FONCE, BLANC );
  }

  nombre = objet->ob_spec.nombre;
  if ( NULL == nombre ) {
    (void) fprintf( stderr, "%s, %s l.%u : Pas de structure d'infos associée !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
  if ( NULL == nombre->texte.texte ) {
#ifdef _DEVELOPPE_
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL << [G_NOMBRE]", 21 );
#endif
    return;
  }
  lg_texte = strlen( nombre->texte.texte );
  if ( 0 == lg_texte ) {
#ifdef _DEVELOPPE_
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ VERT ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, "lg=0 [G_NOMBRE]", 15 );
#endif
    return;
  }

  /* S'agit-il d'un message d'erreur ? */
  if ( nombre->proprietes & BI_EDITABLE_PROBLEME ) {
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ VERT ].pixel );
    vrai_texte = nombre->texte.texte;
    nbr_caracteres = lg_texte;
  } else {
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );
    vrai_texte = nombre->texte.texte + nombre->texte.prem_car;

    lg_texte -= nombre->texte.prem_car;
    if ( lg_texte > nombre->texte.max_visibles )
      nbr_caracteres = nombre->texte.max_visibles;
    else
      nbr_caracteres = lg_texte;
/*     printf("%s : texte = %s, vrai = %s [%d caractères]\n", __FUNCTION__, nombre->texte.texte, vrai_texte, nbr_caracteres); */
  }

  if ( BI_FonteUTF8 != NULL ) {
#ifdef X_HAVE_UTF8_STRING
    Xutf8DrawString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, 5, y_texte, vrai_texte, nbr_caracteres );
#else
    XmbDrawString  ( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, 5, y_texte, vrai_texte, nbr_caracteres );
#endif
  } else {
    XDrawString( BI_Display, fenetre, BI_GC_objets, 2, y_texte, vrai_texte, nbr_caracteres );
  }
}

/* -------- Séparateur -------- */

static void DOBJ_Separateur(Window fenetre, OBJECT *objet)
{
  unsigned int larg, haut;

#ifdef a_controler
  /* À contrôler : pourquoi demander la taille de la fenêtre ?? */
  XGetGeometry(BI_Display, fenetre, &racine, &x_obj, &y_obj, &larg, &haut, &i, &i);
#else
  larg = objet->ob_width;
  haut = objet->ob_height;
#endif

  if ( NULL == objet->ob_spec.texte ) { /* Séparateur simple */
    if ( larg > haut ) { /* Séparateur horizontal */
      OBJ_DessineLigne3D( fenetre, 3, (int) (haut >> 1), larg - 6, ASC_HORIZONTAL );
    } else { /* Séparateur vertical */
      OBJ_DessineLigne3D( fenetre, (int) (larg >> 1), 3, haut - 6, ASC_VERTICAL );
    }
  } else {
    const char *texte;
    int x_texte, y_texte;
    unsigned int larg_texte;
    unsigned long lg_texte;

    /* Affichage du texte lui-même */
    texte = objet->ob_spec.texte;
    lg_texte = strlen(texte);
    y_texte = (haut + BI_Fonte->max_bounds.ascent) >> 1;

    if ( BI_FonteUTF8 != NULL ) {
      XRectangle cadre_texte;

      larg_texte = Xutf8TextExtents( BI_FonteUTF8, texte, lg_texte, NULL, &cadre_texte );
      larg_texte = cadre_texte.width;
      x_texte = (larg - larg_texte) >> 1;

#ifdef X_HAVE_UTF8_STRING
      Xutf8DrawString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, x_texte, y_texte, texte, lg_texte );
#else
      XmbDrawString  ( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets, x_texte, y_texte, texte, strlen( texte ) );
#endif
    } else {
      larg_texte = XTextWidth( BI_Fonte, texte, lg_texte );
      x_texte = (larg - larg_texte) >> 1;

      XDrawString( BI_Display, fenetre, BI_GC_objets, x_texte, y_texte, texte, lg_texte );
    }

    /* Les lignes qui l'encadrent */ 
    OBJ_DessineLigne3D( fenetre, 3, (int) (haut >> 1), x_texte - 6, ASC_HORIZONTAL );
    OBJ_DessineLigne3D( fenetre, x_texte + larg_texte + 3, (int) (haut >> 1), x_texte - 6, ASC_HORIZONTAL );
  }
}

/* -------- Objet affichant une ligne pointillée -------- */

static void DOBJ_StyleLigne(Window fenetre, OBJECT *objet)
{
  int retour;
  unsigned int style;
  XGCValues parametres;

  /* On met un fond blanc pseudo-3D */
  OBJ_DessineFond( fenetre, objet->ob_width, objet->ob_height, BLANC, GRIS_FONCE, NOIR );

  /* On installe les pointillés, en fonction de ce style, dans le GC de dessin dans les objets */
  style = objet->ob_spec.style_ligne;
  if ( BI_AUCUNE_LIGNE == style ) return;

  if ( ( BI_CONTINU == style ) || ( style > BI_NOMBRE_STYLES_LIGNES ) ) {
    parametres.line_style = LineSolid;
  } else {
    retour = BIX_InstallerPointilles( BI_GC_objets, objet->ob_spec.style_ligne );
    if ( retour != 0 ) {
      /* Erreur dans les pointillés */
      OBJ_DessineTexte( fenetre, 0, 0, "Pointillé inconnu", NOIR, NULL );
      return;
    }
    parametres.line_style = LineOnOffDash;
  }
  XChangeGC( BI_Display, BI_GC_objets, GCLineStyle, &parametres );

  /* On trace la ligne, en noir, avec ces pointillés */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );
  XDrawLine( BI_Display, fenetre, BI_GC_objets, 3, objet->ob_height / 2, objet->ob_width - 6, objet->ob_height / 2 );

  /* On repasse en continu, pour être tranquille */
  parametres.line_style = LineSolid;
  XChangeGC( BI_Display, BI_GC_objets, GCLineStyle, &parametres );
}

static void DOBJ_StyleMarque(Window fenetre, OBJECT *objet)
{
  int x_marque, y_marque;
  unsigned int taille;

  /* On met un fond blanc pseudo-3D */
  OBJ_DessineFond( fenetre, objet->ob_width, objet->ob_height, BLANC, GRIS_FONCE, NOIR );

  /* Taille des marques : 2 pixels libres de chaque côté */
  taille = objet->ob_height - 4;

  /* On trace la ligne de marques, en noir, avec ces pointillés */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );

  x_marque = 3 + taille/2;
  y_marque = objet->ob_height / 2;
  while ( ( x_marque + taille/2 ) < objet->ob_width ) {
    BIX_TracerMarque( fenetre, BI_GC_objets, x_marque, y_marque, objet->ob_spec.style_marque, taille );
    x_marque += 2 * taille;	/* Une marque vide sur 2 */
  }
}

/* -------- Objet contenant un texte « simple » -------- */

static void DOBJ_Texte(Window fenetre, OBJECT *objet)
{
  unsigned int larg, haut;
  int x_texte, y_texte, couleur;
  const char *texte = NULL;

  /* Position du texte */
  larg = objet->ob_width;
  haut = objet->ob_height;
  y_texte = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* Fond : gris, pas d'effet 3D : rien à faire... */
  /* On affiche le texte dans l'objet */
  texte = objet->ob_spec.tedinfo->te_ptext.constant;
  if ( NULL == texte ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : chaîne NULL. Objet : %p\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet );
#endif
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL <<", 10 );
  }

  if ( TE_CNTR == objet->ob_spec.tedinfo->te_just ) { /* Texte centré */
    x_texte = larg - XTextWidth( BI_Fonte, texte, strlen( texte ) );
    x_texte /= 2;
  } else if ( TE_RIGHT == objet->ob_spec.tedinfo->te_just ) { /* Texte fer à droite */
    x_texte = larg - XTextWidth( BI_Fonte, texte, strlen( texte ) ) - 2;
  } else {
    x_texte = 2;
  }

  /* De quelle couleur est le texte ? */
  couleur = (objet->ob_spec.tedinfo->te_color & 0x0F00) >> 8;

  /* Dessin proprement dit */
  OBJ_DessineTexte( fenetre, x_texte, y_texte, objet->ob_spec.tedinfo->te_ptext.constant, couleur, BI_Fonte );
}

/* -------- Objet contenant un texte éditable -------- */

static void DOBJ_TexteEditable(Window fenetre, OBJECT *objet)
{
  unsigned int larg, haut;
  int x_texte = 2, y_texte, couleur;
  char *texte = NULL;

  /* Position du texte */
  larg = objet->ob_width;
  haut = objet->ob_height;
  y_texte = (haut + BI_Fonte->max_bounds.ascent) / 2;

  if ( objet->ob_flags & EDITABLE ) {
    OBJ_DessineFond( fenetre, larg, haut, BLANC, GRIS_FONCE, BLANC );
  } else {
/*     OBJ_DessineFond( fenetre, larg, haut, ROUGE, VERT_FONCE, BLEU ); */
    /* Rien à faire de spécial... */
  }

  texte = OBJ_TedinfoVChaine( objet->ob_spec.tedinfo );
  if ( NULL == texte ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : chaîne NULL. Objet : %p\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet );
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL <<", 10 );
#endif
    return;
  }

  /* De quelle couleur est le texte ? */
  couleur = (objet->ob_spec.tedinfo->te_color & 0x0F00) >> 8;

  if ( objet->ob_spec.tedinfo->te_just == TE_CNTR ) { /* Texte centré */
    unsigned int larg_texte;

    larg_texte = XTextWidth(BI_Fonte, texte, (signed) strlen(texte));
    x_texte = ( objet->ob_width - larg_texte ) >> 1;
  }
  OBJ_DessineTexte( fenetre, x_texte, y_texte, texte, couleur, BI_Fonte );

  /* Plus besoin du texte */
  free( texte );
}

/* -------- Objet contenant un texte infini -------- */

static void DOBJ_TexteInfini(Window fenetre, OBJECT *objet)
{
  Texte_Infini *txt_infini;
  unsigned int larg, haut;
  int y_texte;
  char *vrai_texte;
  size_t lg_texte, nombre_affichables;

  larg = objet->ob_width;
  haut = objet->ob_height;

  /* On met le même type qu'un objet éditable FTEXT */
  OBJ_DessineFond( fenetre, larg, haut, BLANC, GRIS_FONCE, BLANC );

  /* Couleur du texte : noir */
  XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel );

  y_texte = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* On affiche le texte dans l'objet */
  txt_infini = objet->ob_spec.txt_infini;
  if ( txt_infini->texte == NULL ) {
#ifdef _DEVELOPPE_
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL << [txt_infini]", 23 );
#endif
    return;
  }

  lg_texte = strlen( txt_infini->texte );
  if ( 0 == lg_texte ) {
#ifdef _DEVELOPPE_
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ VERT ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, "lg=0 [txt_infini]", 17 );
#endif
    return;
  }

  vrai_texte = txt_infini->texte + txt_infini->prem_car;
  nombre_affichables = TXT_CompterCaracteres( vrai_texte );
  if ( nombre_affichables > txt_infini->max_visibles ) {
    nombre_affichables = txt_infini->max_visibles;
  }
  if ( BI_FonteUTF8 != NULL ) {
#ifdef _DEVELOPPE_
    /* (void) fprintf( stderr, "%s,%s l.%u : Texte à afficher = [%s], soit %lu octets et %lu caractères pour %lu affichables\n", */
    /* 		    __FILE__, __FUNCTION__, __LINE__, */
    /* 		    vrai_texte, strlen( vrai_texte ), nombre_affichables, txt_infini->max_visibles ); */
#endif
#ifdef X_HAVE_UTF8_STRING
    Xutf8DrawImageString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets,
			  2, y_texte, vrai_texte, strlen( vrai_texte ) );
#else
    XmbDrawImageString( BI_Display, fenetre, BI_FonteUTF8, BI_GC_objets,
			2, y_texte, vrai_texte, strlen( vrai_texte ) );
#endif
  } else {
    XDrawString( BI_Display, fenetre, BI_GC_objets, 2, y_texte, vrai_texte, nombre_affichables );
  }
}

/* -------- Titre d'un menu dans une barre de menus -------- */

static void DOBJ_Titre(Window fenetre, OBJECT *objet)
{
  const char *texte;
  int x_texte, y_texte;
  unsigned int larg, haut;
  unsigned int larg_texte;

  larg = objet->ob_width;
  if ( objet->ob_x > 0 ) larg++;
  haut = objet->ob_height;
  if ( objet->ob_y > 0 ) haut++;

  /* On met l'aspect 3D */
  if ( objet->ob_state & SELECTED) { /* Objet choisi */
    OBJ_DessineFond( fenetre, larg, haut, GRIS_FONCE, NOIR, BLANC );
  } else if ( objet->ob_state & DISABLED ) { /* Objet inactif */
    OBJ_DessineFond( fenetre, larg, haut, BLANC, BLANC, GRIS_FONCE );
  } else {
    OBJ_DessineFond( fenetre, larg, haut, GRIS, BLANC, GRIS_FONCE );
  }

  /* Position verticale du texte */
  y_texte  = (haut + BI_Fonte->max_bounds.ascent) / 2;

  /* Construction du texte */
  texte = objet->ob_spec.texte;
  if ( NULL == texte ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : chaîne NULL. Objet : %p\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet );
#endif
    XSetForeground( BI_Display, BI_GC_objets, BI_Couleurs[ ROUGE ].pixel );
    XDrawString( BI_Display, fenetre, BI_GC_objets, 1, y_texte, ">> NULL <<", 10 );
    return; /* Pas de texte associé */
  }

  /* On saute les espaces initiaux */
  while ( ' ' == *texte )
    texte++;

  /* Largeur occupée par le texte [dépend des caractères spéciaux présents] */
  larg_texte = OBJ_LargeurTexte( texte, BI_Fonte );

  /* On centre le texte dans le titre */
  x_texte = ( larg - larg_texte ) >> 1;

  /* On affiche le texte */
  if ( objet->ob_type & SELECTED ) 
    OBJ_DessineTexte( fenetre, x_texte, y_texte, texte, BLANC, BI_Fonte );
  else
    OBJ_DessineTexte( fenetre, x_texte, y_texte, texte, NOIR, BI_Fonte );
}

/* -------- Objet défini par l'utilisateur -------- */

static void DOBJ_Utilisateur(Window fenetre, OBJECT *objet)
{
  Window vx_Fen_redessin;
  OBJECT *adresse = NULL;
  PARMBLK parametres;
  unsigned int i;

  /* Adresse de l'arbre contenant l'objet ? */
  for ( i = 0 ; i < BI_nbr_arbres ; i++ ) {
    if ( ( objet >= arbres[ i ].adresse ) &&
	 ( objet <= arbres[ i ].adresse + arbres[ i ].nbr_objets ) ) {
      adresse = arbres[ i ].adresse;
      break;
    }
  }

  if ( NULL == adresse ) {
    (void) fprintf( stderr, "%s,%s l.%u : impossible de trouver l'arbre [objet : %p]\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) objet);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* On initialise les infos de redessin */
  parametres.pb_tree = adresse;
  parametres.pb_obj  = (int) (objet - adresse); /* Index de l'objet */
  parametres.pb_prevstate = objet->ob_state;
  parametres.pb_currstate = objet->ob_state; /* C'est juste et uniquement du redessin... */
  parametres.pb_x    = 0; /* objet->ob_x; */
  parametres.pb_y    = 0; /* objet->ob_y; */
  parametres.pb_w    = objet->ob_width;
  parametres.pb_h    = objet->ob_height;
  parametres.pb_xc   = 0; /* objet->ob_x; */
  parametres.pb_yc   = 0; /* objet->ob_y; */
  parametres.pb_wc   = objet->ob_width;
  parametres.pb_hc   = objet->ob_height;
  parametres.pb_parm = (objet->ob_spec.util)->ub_parm;

  /* On mémorise la fenêtre « normale » de redessin */
  vx_Fen_redessin = BI_Fen_redessin;

  /* On travaille dans la fenêtre de l'objet */
  BI_Fen_redessin = fenetre;
  ((objet->ob_spec.util)->ub_code)( &parametres );
  XSetClipMask( BI_Display, _PRG_station->gr_contexte, None ); /* Pour ne pas conserver de clip. inutile */

  /* On remet l'ancienne fenêtre de redessin */
  BI_Fen_redessin = vx_Fen_redessin;
}

/* -~-~-~-~-~-~-~-~ Quelques fonctions utilitaires -~-~-~-~-~-~-~-~ */

/* -------- Dessin du curseur dans les champs éditables -------- */


/* -------- Création de la chaîne visible à partir du masque & du texte déjà entré -------- */

static char *OBJ_TedinfoVChaine(TEDINFO *tedinfo)
{
  unsigned int i_chaine, i_texte = 0;

  char *chaine;

  if ( NULL == tedinfo ) {
    (void) fprintf( stderr, "%s,%s l.%u : tedinfo NULLE\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  /* Contrôle : le template est forcément de longueur inférieure à la largeur de l'objet */
  if ( tedinfo->te_tmplen > 100 ) {
    AVERTISSEMENT( "Longueur de template étrange - %u\n", tedinfo->te_tmplen );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
  }

  chaine = (char *) malloc( (unsigned) (tedinfo->te_tmplen + 1));
  if ( NULL == chaine ) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante -- chaîne résultat : %u caractères\n",
		    __FILE__, __FUNCTION__, __LINE__, tedinfo->te_tmplen + 1 );
    return NULL;
  }

  for ( i_chaine = 0 ; i_chaine < tedinfo->te_tmplen ; i_chaine++ ) {
    if ( '_' == (tedinfo->te_ptmplt)[ i_chaine ] ) { /* Caractère éditable */
      if ( 0 == (tedinfo->te_ptext.editable)[ i_texte ]) /* Fin du texte */
	chaine[ i_chaine ] = '_';
      else {
	chaine[ i_chaine ] = (tedinfo->te_ptext.editable)[ i_texte ];
	i_texte++; /* Caractère suivant de l'entrée */
      }
    } else
      chaine[ i_chaine ] = (tedinfo->te_ptmplt)[ i_chaine ];
  }

  /* On a fini */
  return chaine;
}
