/**** Bibliothèque mixte GEM / X_windows
  (C) 1998-2001 Emmanuel CURIS
  Dernière modification : 25 novembre 2001

  Sélecteur de fichiers - compatible gem_fen.c [référence]

  30 octobre 1998
    * Ajout des corrections de position des objets : plus joli (en particulier ascenseur);
    * Corrigé : quand on faisait glisser le poussoir, les flèches n'étaient pas désactivées si nécessaire
*/

#define _LIBRAIRIE_X_ /* Pour accéder aux fonctions internes aussi */

#include <errno.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#ifdef _UNIX_
# include <dirent.h> /* Fichier à inclure pour Unix, System V - ex : SunOS, Solaris */
# include <sys/param.h>
# include <sys/stat.h>
# ifdef _SOLARIS_
#  include <unistd.h>
# endif /* _SOLARIS_ */
# ifdef _IRIX_
#  include <unistd.h>
# endif /* _IRIX_ */
# ifdef _LINUX_
#  include <unistd.h>
# endif /* _LINUX_ */
#endif

#ifdef _BSD_
# include <sys/dir.h> /* Fichier à inclure pour Unix, BSD 4.3 */
#endif

#include "bib_interface.h"
#include "txt_utils.h"

/* ~~~~~~~~ Structures et variables internes ************************************************** */

/* Numéros d'objets dans l'arbre */
#define FSEL_TXT_CHEMIN        1
#define FSEL_CHEMIN            2
#define FSEL_VALIDER_CHEMIN    3
#define FSEL_TXT_PREDEFINI     4
#define FSEL_PREDEFINIS        5
#define FSEL_AJOUTER_PREDEFINI 6
#define FSEL_TXT_FILTRE        7
#define FSEL_FILTRE            8
#define FSEL_FILTRE_LIBRE      9
#define FSEL_TXT_OPTIONS      10
#define FSEL_OPTIONS          11
#define FSEL_APERCU           12
#define FSEL_TXT_APERCU       13
#define FSEL_OK               14
#define FSEL_ANNULER          15
#define FSEL_TXT_NOM          16
#define FSEL_NOM              17
#define FSEL_REMONTE          18
#define FSEL_AVANT            19
#define FSEL_FOND_ASCENSEUR   20
#define FSEL_BOUTON_ASCENSEUR 21
#define FSEL_APRES            22
#define FSEL_FOND_DEFILEUR    23
#define FSEL_PREMIER_NOM      24

static unsigned int FSEL_NOMBRE_NOMS = 0;

#define FSEL_DERNIER_NOM (FSEL_PREMIER_NOM + FSEL_NOMBRE_NOMS - 1)

/* Dimensions des objets */
#define FSEL_LARGEUR_TEXTES_CHEMIN  12  /* "Prédéfinis :", "Chemin :", "Nom :" */
#define FSEL_LARGEUR_BOUTONS_CHEMIN 10  /* "Changer", "Ajouter" */
#define FSEL_LARGEUR_ASCENSEUR       2
#define FSEL_LARGEUR_TEXTES_FILTRE   9  /* Filtre :", "Options :" */
#define FSEL_LARGEUR_BOUTONS_FILTRE  8

#define FSEL_LARGEUR_OPTIONS       (2 * FSEL_LARGEUR_BOUTONS_FILTRE + 1)
#define FSEL_LARGEUR_APERCU        (FSEL_LARGEUR_TEXTES_FILTRE + 1 + FSEL_LARGEUR_OPTIONS + 1)
#define FSEL_LARGEUR_OK            ( (FSEL_LARGEUR_APERCU - 1) / 2 )

#define FSEL_LARGEUR_MIN_DEFILEUR  (FSEL_LARGEUR_TEXTES_CHEMIN + 1)
#define FSEL_LARGEUR_MAX_DEFILEUR  50
#define FSEL_LARGEUR_MIN_BOITE     (1 + FSEL_LARGEUR_MIN_DEFILEUR + FSEL_LARGEUR_ASCENSEUR + 1 + FSEL_LARGEUR_APERCU + 1)
#define FSEL_LARGEUR_MAX_BOITE     (1 + FSEL_LARGEUR_MAX_DEFILEUR + FSEL_LARGEUR_ASCENSEUR + 1 + FSEL_LARGEUR_APERCU + 1)

#define FSEL_HAUTEUR_MIN_APERCU     4
#define FSEL_HAUTEUR_MIN_DEFILEUR  (4 + FSEL_HAUTEUR_MIN_APERCU + 2)
#define FSEL_HAUTEUR_MAX_DEFILEUR  50
#define FSEL_HAUTEUR_MIN_BOITE     (5 + FSEL_HAUTEUR_MIN_DEFILEUR + 3)
#define FSEL_HAUTEUR_MAX_BOITE     (5 + FSEL_HAUTEUR_MAX_DEFILEUR + 3)

/* Positions des objets */
#define FSEL_X_CHEMINS  (1 + FSEL_LARGEUR_TEXTES_CHEMIN + 1)  /* Des éditables/sous-menus */
#define FSEL_X_DEFILEUR 1

#define FSEL_Y_DEFILEUR  5
#define FSEL_Y_ASCENSEUR (FSEL_Y_DEFILEUR + 1)

#define FSEL_NOMBRE_MAX_ENTREES    40

typedef struct entree_fsel {
  struct entree_fsel *apres; /* Celui d'après dans le tri */
  char *nom_fichier;
  boolean dossier;  /* VRAI si c'est un dossier, FAUX sinon */
  boolean choisit;  /* VRAI si le fichier est sélectionné   */
  boolean afficher; /* VRAI si le fichier doit être affiché */
} Entree_selecteur;

struct infos_selecteur {
  char *chemin;			 /* Le chemin de travail */

  unsigned long nbr_fichiers;	/* Nombre de fichiers parmi ces entrées */
  unsigned long nbr_dossiers;	/* Nombre de dossiers parmi ces entrées */
  Entree_selecteur *dossiers;	/* Pointeur vers la table de dossiers */
  Entree_selecteur *fichiers;	/* Pointeur vers la table de fichiers */

  unsigned long premier_affiche; /* Numero de la première entrée affichée [0 = premier dossier] */
  unsigned long nombre_affichables; /* Nombre d'entrées affichables dans le dossier courant [selon le filtre en cours...] */
  int           numero_choisi;   /* -1 : aucun choisi ; dans les objets ! */

  unsigned int  numero_filtre; /* Numéro du filtre sélectionné, si applicable ! */

  BI_InfosSelecteur *parametres; /* Pour avoir accès aux propriétés, à la table des filtres, des chemins,... */
};

static char *FSEL_chemin_initial = NULL; /* Le chemin avec lequel on travaille */

/* ******** Fonctions à usage interne seulement *********************************************** */

static boolean       FSEL_masque_convient(char *nom_fichier, const char *masque);

static int           FSEL_InitialiserDefileur(struct infos_selecteur *infos);
static void          FSEL_LibererDefileur    (void);

static void          FSEL_AfficherListe  (struct infos_selecteur *infos);
static int           FSEL_ConstruireListe(struct infos_selecteur *infos);
static int           FSEL_FiltrerListe   (struct infos_selecteur *infos);
static int           FSEL_LibererListe   (struct infos_selecteur *infos);
static int           FSEL_TrierListe     (const void *obj_1, const void *obj_2);

static char         *FSEL_CheminCourant(void);

static int           FSEL_EliminerDoublons(char **noms_choisis, unsigned long *nombre_noms);
static char         *FSEL_NomRetenu    (struct infos_selecteur *infos_selection);
static unsigned long FSEL_NomsChoisis  (struct infos_selecteur *infos_selection, char ***noms_choisis );
static unsigned long FSEL_NomsEntres   (struct infos_selecteur *infos_selection, char ***noms_choisis );
static int           FSEL_TrierNoms    (const void *obj_1, const void *obj_2);

static int         FSEL_GererSelecteur  (unsigned int num_objet, int x_souris, int y_souris);
static int         FSEL_ApercuParDefaut (USR_FONCTION *parametres);
static int         FSEL_AnalyserReponse (struct infos_selecteur *infos_selection, int reponse, char ***noms_choisis, unsigned long *nombre_choisis);

static void        FSEL_GererChoixFiltre(unsigned int arbre, unsigned int objet, unsigned long filtre);
static const char *FSEL_QuelFiltre      (struct infos_selecteur *infos, unsigned long numero_filtre);

static void        FSEL_GererChoixChemin(unsigned int arbre, unsigned int objet, unsigned long chemin);

static char *X_strdup(const char *nom);
static char *X_strcat(const char *chemin, const char *nom);

/* ******** Réalisation effective des fonctions *********************************************** */

/* ---------- Renvoie la liste des noms des dossiers et des fichiers dans le rep. chemin
   Cette liste est un arbre binaire trié au vol et "filtré" au vol : tous les fichiers sont présents
   seuls les fichiers correspondant au masque sont marqués affichables

   Renvoi 0    si tout s'est bien passé
          != 0 en cas de problème

   Dans infos, modifie : nbr_entrees, nbr_fichiers, nbr_dossiers
                         dossiers, fichiers
			 premier_affiche, numero_choisi
 ---- */

static int FSEL_ConstruireListe(struct infos_selecteur *infos)
{
  DIR *repertoire;
  Entree_selecteur *nouveau = NULL;
  const char *masque;
  struct dirent *fichier;
  unsigned long affichables = 0;

  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'informations associées au sélecteur !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( NULL == infos->chemin ) {
    AVERTISSEMENT( "Pas de chemin indiqué [%s]", infos->chemin );
    return -2;
  }
  if ( ( infos->dossiers != NULL ) || ( infos->fichiers != NULL ) ) {
    AVERTISSEMENT( "Table des dossiers ou fichiers déjà existante ! [%p, %p]", infos->dossiers, infos->fichiers );
    infos->dossiers = NULL;
    infos->fichiers = NULL;
  }

  /* On initialise tout pour être tranquille */
  infos->nbr_dossiers = 0;
  infos->nbr_fichiers = 0;

  /* On ouvre le répertoire courant */
  repertoire = opendir( infos->chemin );
  if ( NULL == repertoire ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de lire le dossier [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, infos->chemin );
    OPTION_A_FAIRE( "Message d'erreur plus clair" );
    return +1;
  }

  /* On récupère le masque */
  masque = FSEL_QuelFiltre( infos, infos->numero_filtre );

  for ( fichier = readdir( repertoire ) ; fichier ; fichier = readdir( repertoire ) ) {
    char *complet;
    struct stat infos_fichier;

    if ( ( strcmp(fichier->d_name, "." ) == 0 ) ||
	 ( strcmp(fichier->d_name, "..") == 0 ) ) { /*. et .. ne sont jamais affichés */
      continue;
    }

    complet = X_strcat( infos->chemin, fichier->d_name );
    if ( NULL == complet ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans X_strcat [%s/%s]\n",
		     __FILE__, __FUNCTION__, __LINE__, infos->chemin, fichier->d_name);
      continue;
    }
    stat( complet, &infos_fichier );

    if ( S_ISCHR(infos_fichier.st_mode) ) { /* Fichier spécial : caractères */
#ifdef _DEVELOPPE_
      printf("Fichier spécial / caractères : %s\n", complet);
#endif
      free( complet ); complet = NULL;
      continue;
    }
    if ( S_ISBLK(infos_fichier.st_mode) ) { /* Fichier spécial : bloc */
#ifdef _DEVELOPPE_
      printf("Fichier spécial / blocs : %s\n", complet);
#endif
      free( complet ); complet = NULL;
      continue;
    }
    if ( S_ISFIFO(infos_fichier.st_mode) ) { /* Fichier spécial : fifo */
#ifdef _DEVELOPPE_
      printf("Fichier spécial / FIFO : %s\n", complet);
#endif
      free( complet ); complet = NULL;
      continue;
    }

    /* Plus besoin de lui */
    free( complet ); complet = NULL;

    /* Répertoire, fichier ou lien : il faut une entrée dans le sélecteur */
    nouveau = (Entree_selecteur *) malloc( sizeof(Entree_selecteur) );
    if ( NULL == nouveau ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de réserver l'entrée [%s]\n",
		     __FILE__, __FUNCTION__, __LINE__, fichier->d_name);
      break;
    }
    /* On initialise pour ce fichier */
    nouveau->apres = NULL;
    nouveau->choisit = FAUX; /* Aucun fichier ou dossier sélectionné par défaut */

    nouveau->nom_fichier = NULL;
    nouveau->nom_fichier = X_strdup( fichier->d_name );

    if ( S_ISDIR(infos_fichier.st_mode) ) { /* C'est un dossier */
      nouveau->dossier = VRAI;
      nouveau->nom_fichier[ 0 ] = 'D';
      infos->nbr_dossiers++;

      if ( fichier->d_name[ 0 ] == '.' ) {
	if ( ( masque != NULL ) && ( masque[ 0 ] == '.' ) ) {
	  nouveau->afficher = VRAI;
	  affichables++; /* Tous les dossiers sont affichables, de toute façon */
	} else {
	  nouveau->afficher = FAUX; /* Les dossiers cachés sont affichés seulement si le masque le demande */
	}
      } else {
	nouveau->afficher = VRAI;
	affichables++; /* Tous les autres dossiers sont toujours affichés */
      }

      /* On le place dans la liste chaînée, directement à sa place car on suppose qu'il y a peu de dossiers... */
      if ( NULL == infos->dossiers ) {
	infos->dossiers = nouveau;
      } else {
	if ( strcmp( nouveau->nom_fichier + 2, infos->dossiers->nom_fichier + 2 ) < 0 ) { /* On est avant le premier */
	  nouveau->apres = infos->dossiers;
	  infos->dossiers = nouveau;
	} else if ( NULL == infos->dossiers->apres ) { /* C'est le second */
	  infos->dossiers->apres = nouveau;
	} else {
	  Entree_selecteur *avant = infos->dossiers;

	  while ( strcmp( avant->apres->nom_fichier + 2, nouveau->nom_fichier + 2 ) < 0) { /* On est après avant->apres */
	    avant = avant->apres;
	    if ( NULL == avant->apres ) { /* On est après le dernier */
	      break;
	    }
	  }
	  nouveau->apres = avant->apres;
	  avant->apres = nouveau;
	}
      }
    } else {
      infos->nbr_fichiers++;
      nouveau->dossier = FAUX;

      /* On place en début de liste, on fera le tri après plus efficacement */
      nouveau->apres = infos->fichiers;
      infos->fichiers = nouveau;

      if ( VRAI == FSEL_masque_convient( fichier->d_name, masque ) ) {
	nouveau->afficher = VRAI;
	affichables++;
      } else {
	nouveau->afficher = FAUX;
      }
    }    
  }
  closedir( repertoire ); repertoire = NULL;

  /* Il faut trier la liste simplement chaînée des fichiers */
  if ( infos->nbr_fichiers > 0 ) {
    Entree_selecteur **noms_tries;

    noms_tries = (Entree_selecteur **) calloc( infos->nbr_fichiers, sizeof(Entree_selecteur *) );
    if ( NULL == noms_tries ) {
      (void) fprintf(stderr, "%s, %s l.%u : Pas assez de mémoire pour trier les noms de fichiers\n",
		     __FILE__, __FUNCTION__, __LINE__);
    } else {
      Entree_selecteur *entree, **nt;
      unsigned long i;

      /* On fait la liste des entrées */
      for ( i = 0, entree = infos->fichiers, nt = noms_tries ; i < infos->nbr_fichiers ; i++, entree = entree->apres, nt++ )
	*nt = entree;

      /* On la trie */
      qsort( (void *) noms_tries, (size_t) infos->nbr_fichiers, sizeof(Entree_selecteur *), FSEL_TrierListe );

      /* On fait la liste triée des entrées */
      infos->fichiers = *noms_tries;
      for ( i = 1, entree = infos->fichiers, nt = noms_tries + 1 ; i < infos->nbr_fichiers ; i++, entree = entree->apres, nt++) {
	entree->apres = *nt;
      }
      entree->apres = NULL;
    }
    free( noms_tries ); noms_tries = NULL;
  }

  /* On fusionne la liste des fichiers et celle des dossiers */
  if ( infos->dossiers != NULL) {
    for ( nouveau = infos->dossiers ; nouveau->apres ; nouveau = nouveau->apres);
    nouveau->apres = infos->fichiers;
  } else {
    infos->dossiers = infos->fichiers;
  }

  infos->nombre_affichables = affichables;
  return 0; /* Pas d'erreur */
}

/* -------- Modifie la liste des fichiers affichés, en fonction du masque -------- */

static int FSEL_FiltrerListe(struct infos_selecteur *infos)
{
  Entree_selecteur *entree;
  const char *masque;
  unsigned long affichables = 0;

  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'informations associées au sélecteur !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* On récupère le masque */
  masque = FSEL_QuelFiltre( infos, infos->numero_filtre );
  if ( NULL == masque ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de masque : on affiche tout, sauf les cachés\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }
  
  for ( entree = infos->dossiers ; entree != NULL ; entree = entree->apres) {
    char *nom = entree->nom_fichier + 2;

    if ( VRAI == entree->dossier ) {
      if ( '.' == nom[ 0 ] ) {
	if ( ( masque != NULL ) && ( masque[ 0 ] == '.' ) ) {
	  entree->afficher = VRAI;
	  affichables++; /* Tous les dossiers sont affichables, de toute façon */
	} else {
	  entree->afficher = FAUX; /* Les dossiers cachés sont affichés seulement si le masque le demande */
	}
      } else {
	entree->afficher = VRAI;
	affichables++; /* Tous les autres dossiers sont toujours affichés */
      }
    } else {
      if ( VRAI == FSEL_masque_convient( nom, masque ) ) {
	entree->afficher = VRAI;
	affichables++;
      } else {
	entree->afficher = FAUX;
	entree->choisit = FAUX; /* Un fichier non affiché n'est pas choisi */
/*	printf("Fichier non affiché : %s\n", nom); */
      }
    }
  }

  /* Nombre d'affichables */
  infos->nombre_affichables = affichables;

  /* Pas d'erreur */
  return 0;
}

/* -------- Comparer deux noms de fichiers (appelé par qsort)
     Renvoie une valeur <0 si 1 est avant 2, >0 sinon
 ------ */

static int FSEL_TrierListe(const void *objet1, const void *objet2)
{
  const Entree_selecteur *e1, *e2;
  char *entree1, *entree2;

  e1 = *(const Entree_selecteur * const *) objet1;
  e2 = *(const Entree_selecteur * const *) objet2;
  entree1 = e1->nom_fichier;
  entree2 = e2->nom_fichier;
#ifdef _DEVELOPPE_
  if ( ( entree1 == NULL ) || ( entree2 == NULL ) )
    AVERTISSEMENT( "Appel avec NULL comme paramètre [e1 = %p, e2 = %p]", entree1, entree2 );
#endif

  /* Les fichiers cachés sont toujours classés en premier */
  if ( ( *entree1 == '.' ) && ( *entree2 != '.' ) )
    return -2;
  if ( ( *entree1 != '.' ) && ( *entree2 == '.' ) )
    return +2;

  /* Par défaut, ordre alphabétique */
  return strcmp( entree1, entree2 );
}

/* ----------------------- Effacement de toutes les structures (sauf le chemin) --------------- */

static int FSEL_LibererListe(struct infos_selecteur *infos)
{
  /* Supprimer les entrées dans le défileur */
  FSEL_LibererDefileur();

  /* Effacement de la liste des noms de fichier */
  while ( infos->dossiers ) {
    Entree_selecteur *tmp = infos->dossiers->apres;

    /* Libérer le nom du fichier */
    free( infos->dossiers->nom_fichier );
    infos->dossiers->nom_fichier = NULL;

    free( infos->dossiers );
    infos->dossiers = tmp;
  } /* Efface aussi les fichiers, car listes enchaînées... */
  infos->fichiers = NULL;

  infos->nbr_fichiers = 0;
  infos->nbr_dossiers = 0;

  infos->nombre_affichables = 0;
  infos->premier_affiche = 0;
  infos->numero_choisi = -1; /* Aucun choisi, forcément */

  return 0; /* Pas de problème */
}

/* ----------------------- Affichage de la liste des fichiers dans le défileur ---------------- */

#define MAX_VISIBLES (FSEL_DERNIER_NOM - FSEL_PREMIER_NOM + 1) /* Nombre max. de fichiers affichable */

static void FSEL_AfficherListe(struct infos_selecteur *infos)
{
  Entree_selecteur *entree;
  unsigned long i;

  /* On met NULL dans tous les textes du défileur, pour ne pas libérer de mémoire par accident */
  FSEL_LibererDefileur();

  /* On saute les premiers éléments */
  for ( i = 0, entree = infos->dossiers ; i < infos->premier_affiche ; entree = entree->apres ) {
    if ( NULL == entree ) {
      (void) fprintf(stderr, "%s, %s l.%u : erreur grave, liste faussée...\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      entree = infos->dossiers;
      break;
    } else if ( VRAI == entree->afficher ) {
/*      printf("On saute %s\n", entree->nom_fichier); */
      i++;
    } else {
/*      printf("Fichier ignoré : %s\n", entree->nom_fichier); */
    }
  }

  /* On affiche les textes */
  for ( i = 0 ; entree ; entree = entree->apres) {
    if ( entree->afficher == VRAI ) {
      int j;

      j = (int) (FSEL_PREMIER_NOM + i);
      OBJ_RemplacerTexte( BI_ArbreSelecteur, j, entree->nom_fichier, FAUX );
      OBJ_active ( BI_ArbreSelecteur, j, FORCE_OUI );
      OBJ_choisit( BI_ArbreSelecteur, j, (entree->choisit == VRAI) );
      i++;
      if ( i >= MAX_VISIBLES )
	return; /* Le sélecteur est plein... */
    } else {
/*      printf("Fichier ignoré : %s\n", entree->nom_fichier); */
    }

  }
  for ( ; i < MAX_VISIBLES ; i++) { /* S'il reste des entrées libres */
    int j;

    j = (int) ( FSEL_PREMIER_NOM + i );
    /* On peut mettre une chaine statique, puisque à la fin on remplace tout par NULL */
    OBJ_RemplacerTexte( BI_ArbreSelecteur, j, "", FAUX );
    OBJ_active ( BI_ArbreSelecteur, j, FORCE_NON );
    OBJ_choisit( BI_ArbreSelecteur, j, FORCE_NON );
  }
}

static int FSEL_InitialiserDefileur(struct infos_selecteur *infos)
{
  infos->premier_affiche = 0;
  infos->numero_choisi = -1;

  /* L'ascenseur */
  ARB_ascenseur( BI_ArbreSelecteur, FSEL_AVANT, FSEL_FOND_ASCENSEUR, FSEL_BOUTON_ASCENSEUR, FSEL_APRES, ASC_VERTICAL,
		 infos->nombre_affichables, MAX_VISIBLES, 0, ROOT, 0, 0, METTRE );

  /* Les textes */
  FSEL_AfficherListe( infos );

  return 0;
}

/* -------- Libérer les anciennes entrées du défileur -------- */

static void FSEL_LibererDefileur(void)
{
  unsigned int i;

  for ( i = FSEL_PREMIER_NOM ; i <= FSEL_DERNIER_NOM ; i++ ) {
    /* La libération sera faite avec les autres noms de fichier, dans FSEL_LibererListe() */
    BI_ARB_fselect->adresse[ i ].ob_spec.txt_malloc = NULL;
  }
}

/* ----------------------- Création d'un nom de fichier complet à partir de chemin + nom ------ */

static char *X_strcat(const char *chemin, const char *nom)
{
  char *resultat;
  unsigned long j, debut;

  if ( (nom == NULL) || (chemin == NULL) ) return NULL;

  resultat = (char *) malloc(strlen(nom) + strlen(chemin) + 2); /* 0 final et / de séparation */
  if (resultat == NULL)
    return NULL;

  strcpy(resultat, chemin); /* On met le chemin */
  if ( resultat[strlen(resultat) - 1] != '/' ) {
    j = strlen(resultat);
    resultat[j] = '/'; /* On ajoute un '/' pour marquer le changement */
    resultat[j + 1] = 0;
  }

  debut = strlen(resultat);
  for (j = 0 ; j <= strlen(nom) ; j++) { /* On recopie aussi le 0 final */
    resultat[debut + j] = nom[j];
  }

  return resultat;
}

/* -------- Conversion nom de fichier -> entrée du sélecteur ------------------- */

static char *X_strdup(const char *nom)
{
  char *copie;
  unsigned long lg_nom;

  if (nom == NULL) return NULL;
  lg_nom = strlen(nom);

  copie = (char *) malloc( lg_nom + 3 ); /* Deux espaces avant + 0 final */
  if (copie) {
    memcpy(copie + 2, nom, lg_nom);
    copie[0] = ' ';
    copie[1] = ' ';
    copie[lg_nom + 2] = 0;
  }

  return copie;
}

/* -------- Le nom de fichier indiqué convient-t-il étant donné le masque précisé ?
      Renvoie VRAI si le masque convient

  Conventions du masque : unix, sauf que , sépare plusieurs masques (ou inclusif)
  -------- */

static boolean FSEL_masque_convient( char *nom_fichier, const char *masque )
{
  boolean retour;
  const char *pos_virgule;
  char car_ref;
  unsigned long i = 0, j = 0; /* Index dans le masque [i], le nom de fichier [j] */

  if (masque == NULL) return VRAI; /* Pas de masque */
  if (nom_fichier == NULL) return FAUX; /* Pas de fichier ! */

  pos_virgule = strchr(masque, ',');
  if (pos_virgule != NULL) {
    /* On ignore d'éventuels espaces */
    pos_virgule++;
    while ( *pos_virgule == ' ')
      pos_virgule++;
    if (*pos_virgule == 0) {
      fprintf(stderr, "FSEL_masque_convient : masque incorrect, fini par une virgule : %s\n", masque);
      return FAUX; /* Un masque incorrect ne colle à aucun fichier */
    }
    retour = FSEL_masque_convient(nom_fichier, pos_virgule);
    if (VRAI == retour)
      return VRAI; /* Pas besoin de chercher plus loin ! */
  }

  /* Premier motif de la série : on l'analyse donc... */
  if ( nom_fichier[0] == '.' ) { /* Fichier "caché" */
    if ( masque[0] != '.' )
      return FAUX; /* Sauf demande expresse, on ne les affiche pas */
    else {
      nom_fichier++; /* Affiché que s'il colle */
      masque++;
    }
  }

  if ( ('*' == masque[0]) && (0 == masque[1]) )
    return VRAI; /* Masque = * : convient à tout */

  /* Le travail sérieux commence ici
     Remarque : actuellement, probablement pas encore idéal */
  do {
    car_ref = masque[i];
    if ( (car_ref == ',') || (car_ref == 0) ) { /* On a terminé la 1re variante du masque */
      return VRAI; /* Le nom de fichier est donc correct */
    } else if (car_ref == '*') {
      char car_lu;

      car_ref = masque[++i]; /* Caractère suivant */
      if (car_ref == 0) /* Masque finit par * => le reste est valable de toute façon */
	return VRAI;

      car_lu = nom_fichier[j];
      while ( car_lu != car_ref ) {
	j++;
	car_lu = nom_fichier[j];
	if (car_lu == 0)
	  return FAUX; /* On a atteint la fin du nom sans trouver le caractère cherché */
      }
      i++; j++;
/*      printf("Masque restant : %s, texte restant : %s\n", masque + i, nom_fichier + j); */
    } else { /* On cherche un caractère identique dans le nom de fichier */
      if ( nom_fichier[j] != car_ref )
	return FAUX; /* Le nom ne convient pas */
      i++; j++; /* Caractère suivant du masque et du nom de fichier */
    }
  } while (car_ref != 0);

  return FAUX; /* Pour l'instant, pas de masque... */
}

/* ----------------------- Gestion du sélecteur de fichier ------------------------------------ */

static int FSEL_GererSelecteur(unsigned int objet, int x_souris, int y_souris)
{
  struct infos_selecteur *infos;

  /* On récupère les infos associées à l'arbre */
  infos = ARB_LireComplement( BI_ArbreSelecteur );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune info associée au sélecteur !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  /* On gère la mollette, éventuellement */
  if ( BI_bouton == Button4 ) {
    objet = FSEL_AVANT;
    if ( OBJ_active( BI_ArbreSelecteur, FSEL_AVANT, QUESTION ) == FAUX ) {
      return BI_EVENEMENT_TRAITE; /* Pas d'affichage avant */
    }
  } else if ( BI_bouton == Button5 ) {
    objet = FSEL_APRES;
    if ( OBJ_active( BI_ArbreSelecteur, FSEL_APRES, QUESTION ) == FAUX ) {
      return BI_EVENEMENT_TRAITE; /* Pas d'affichage après */
    }
  }

  /* On gère les objets du défileur */
  if ( ( objet >= FSEL_PREMIER_NOM ) && ( objet <= FSEL_DERNIER_NOM ) ) {
    Entree_selecteur *entree;
    unsigned long i, clic = infos->premier_affiche + ( objet - FSEL_PREMIER_NOM ) + 1;

    for ( entree = infos->dossiers, i = 0 ; entree != NULL ; entree = entree->apres ) {
      if ( NULL == entree ) {
	(void) fprintf(stderr, "X_gere_selecteur : erreur grave dans la liste des fichiers ! On annule\n");
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return BI_EVENEMENT_ERREUR;
      }
      if ( VRAI == entree->afficher ) {
	i++;
	if ( i == clic )
	  break;
      }
    }
    
    if ( entree->dossier == VRAI ) { /* C'est un dossier */
      char *nv_chemin;

      nv_chemin = X_strcat( infos->chemin, entree->nom_fichier + 2 );
      if ( nv_chemin  != NULL ) {
	/* On change de dossier */
	OBJ_choisit( BI_ArbreSelecteur, objet, FORCE_OUI );
	free( infos->chemin );
	infos->chemin = nv_chemin;

	/* On efface donc l'ancienne liste de fichiers */
	FSEL_LibererListe( infos );

	/* On construit la nouvelle liste des fichiers & dossiers */
	FSEL_ConstruireListe( infos );

	/* On affiche le tout */
	OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_CHEMIN, infos->chemin, 0 );
	FSEL_InitialiserDefileur( infos );
      }

      return BI_EVENEMENT_TRAITE;
    }

    if ( infos->parametres->proprietes & BI_SELECTEUR_MULTIPLE ) { /* Sélection multiple */
      entree->choisit = OBJ_choisit( BI_ArbreSelecteur, objet, ECHANGE );
    } else { /* Sélection simple */
      if ( infos->numero_choisi >= 0 ) /* Il y avait déjà un fichier choisi */
	OBJ_choisit( BI_ArbreSelecteur, infos->numero_choisi, FORCE_NON );

      infos->numero_choisi = objet;
      OBJ_choisit( BI_ArbreSelecteur, objet, FORCE_OUI );
    }

    /* Le dernier nom cliqué se place dans la liste des noms */
    OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_NOM, entree->nom_fichier + 2, 0 );
    return BI_EVENEMENT_TRAITE;
  }

  switch ( objet )
  {
  case FSEL_REMONTE: /* On change le chemin du sélecteur */
  {
    if ( strlen( infos->chemin ) > 1 ) { /* Plus long que '/' -> on n'est pas à la racine */
      unsigned long j;
      const char *filtre __attribute__((__unused__));

      j = strlen( infos->chemin ) - 1;

      if ( infos->chemin[ j ] == '/' )
	infos->chemin[ j ] = 0;

      while ( infos->chemin[ j ] != '/' ) /* On rogne jusqu'au dossier précédent */
	infos->chemin[ j-- ] = 0;

      /* On efface l'ancienne liste de fichiers */
      FSEL_LibererListe( infos );

      /* On construit la nouvelle liste des fichiers & dossiers */
      FSEL_ConstruireListe( infos );

      /* On affiche le tout */
      OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_CHEMIN, infos->chemin, 0 );
      FSEL_InitialiserDefileur( infos );

      /* On récupère le filtre actif */
      filtre = FSEL_QuelFiltre( infos, infos->numero_filtre );
    }
    return BI_EVENEMENT_TRAITE;
  }

  case FSEL_AVANT: /* Clic sur la flèche vers le haut */
  case FSEL_APRES: /* Clic sur la flèche vers le bas */
  case FSEL_FOND_ASCENSEUR:   /* Clic sur le fond de l'ascenseur */
  case FSEL_BOUTON_ASCENSEUR:/* On déplace le poussoir */
    infos->premier_affiche = ARB_ascenseur( BI_ArbreSelecteur, FSEL_AVANT, FSEL_FOND_ASCENSEUR, FSEL_BOUTON_ASCENSEUR, FSEL_APRES, ASC_VERTICAL,
					    infos->nombre_affichables, MAX_VISIBLES, infos->premier_affiche,
					    objet, x_souris, y_souris, ECHANGE );
    FSEL_AfficherListe( infos );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE; /* Bouton avec fonctionnement par défaut [OK, Annuler]... */
}

/* -------- Aperçu par défaut : rien de spécial... -------- */

static int FSEL_ApercuParDefaut(USR_FONCTION *parametres)
{
  unsigned int largeur, hauteur;
  struct infos_selecteur *infos;

  /* On récupère les coordonnées de l'objet */
  largeur = parametres->pb_w;
  hauteur = parametres->pb_h;

  /* On passe en mode blanc */
  XSetForeground( BI_Display, BI_GC_programme, BI_Couleurs[ BLANC ].pixel );

  /* On trace un rectangle plein, blanc */
  XFillRectangle( BI_Display, arbres[ BI_ArbreSelecteur ].objets[ FSEL_APERCU ], BI_GC_programme, 0, 0, largeur, hauteur );
  
  /* On récupère les infos associées à l'arbre */
  infos = (struct infos_selecteur *) ARB_LireComplement( BI_ArbreSelecteur );
  if ( NULL == infos ) return 0;
  if ( NULL == infos->parametres ) return 0;

  /* S'il y a une routine d'aperçu, on l'appelle */
  if ( infos->parametres->apercu != ( void (*)(unsigned int, unsigned int, const char *) ) NULL ) {
    OPTION_A_FAIRE( "Trouver le nom du fichier dont on veut l'aperçu" );
    infos->parametres->apercu( BI_ArbreSelecteur, FSEL_APERCU, NULL );
  }

  /* Terminé */
  return 0;
}

/* -------- Changer de filtre dans la liste -------- */

static void FSEL_GererChoixFiltre(unsigned int numero_arbre, unsigned int numero_objet __attribute__((__unused__)), unsigned long numero_filtre)
{
  struct infos_selecteur *infos;

  /* On récupère les infos du sélecteur */
  infos = ARB_LireComplement( numero_arbre );
  if ( NULL == infos ) return;

  /* Est-ce que l'on a choisi le filtre utilisateur ? */
  if ( infos->parametres->proprietes & BI_SELECTEUR_FILTRE_UTILISATEUR ) {
    if ( 1 == numero_filtre ) {
      /* Oui : on active donc le filtre */
      OBJ_active( BI_ArbreSelecteur, FSEL_FILTRE_LIBRE, FORCE_OUI );
    } else {
      /* Non : on le désactive donc */
      OBJ_active( BI_ArbreSelecteur, FSEL_FILTRE_LIBRE, FORCE_NON );
    }
  }

  /* On mémorise le filtre demandé */
  infos->numero_filtre = numero_filtre;

  /* On l'utilise pour adapter l'affichage */
  FSEL_FiltrerListe( infos );
  FSEL_InitialiserDefileur( infos );
}

/* -------- Trouver le filtre en fonction de l'option choisie... -------- */

static const char *FSEL_QuelFiltre(struct infos_selecteur *infos, unsigned long numero_filtre)
{
  unsigned long premier_filtre_utilisateur = 2;

  if ( 0 == numero_filtre ) return "*";	 /* Toujours présent, toujours le premier */
  if ( infos->parametres->proprietes & BI_SELECTEUR_FILTRE_UTILISATEUR ) {
    premier_filtre_utilisateur++;

    if ( 1 == numero_filtre ) {
      /* On veut le filtre utilisateur */
      return lit_chaine( BI_ArbreSelecteur, FSEL_FILTRE_LIBRE, 0, NULL, TXT_REMPLACE );
    }
  }

  if ( numero_filtre < premier_filtre_utilisateur ) {
    /* Séparateur ou inconnu... */
    return NULL;
  }

  /* Si on est là : filtre prédéfini, après le séparateur */
  /* On passe donc à 0 pour le premier */
  numero_filtre -= premier_filtre_utilisateur;

  /* On a ainsi l'index dans la table indiquée */
  if ( numero_filtre >= infos->parametres->nbr_filtres ) {
    (void) fprintf(stderr, "%s, %s l.%u : Numéro de filtre incorrect : %lu\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_filtre);
    return NULL;
  }
  return infos->parametres->filtres[ numero_filtre ];
}

/* -------- Changer de chemin, en en prenant un dans la liste des prédéfinis -------- */

static void FSEL_GererChoixChemin(unsigned int numero_arbre, unsigned int numero_objet __attribute__((__unused__)), unsigned long numero_chemin)
{
  struct infos_selecteur *infos;
  char **table_chemins, *nouveau_chemin;

  /* On récupère les infos du sélecteur */
  infos = ARB_LireComplement( numero_arbre );
  if ( NULL == infos ) return;
  if ( NULL == infos->parametres ) return;
  if ( numero_chemin > infos->parametres->nbr_predefinis ) return;

  /* La table des chemins */
  table_chemins = infos->parametres->predefinis;

  /* On récupère le nouveau chemin */
  nouveau_chemin = table_chemins[ numero_chemin ];
  if ( NULL == nouveau_chemin ) return;	/* Rien à faire... */
  nouveau_chemin = strdup( nouveau_chemin );
  if ( NULL == nouveau_chemin ) return; /* Impossible de changer de chemin... */

  /* On change ce chemin... */
  free( infos->chemin );
  infos->chemin = nouveau_chemin;

  /* On efface donc l'ancienne liste de fichiers */
  FSEL_LibererListe( infos );

  /* On construit la nouvelle liste des fichiers & dossiers */
  FSEL_ConstruireListe( infos );

  /* On affiche le tout */
  OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_CHEMIN, infos->chemin, 0 );
  FSEL_InitialiserDefileur( infos );
}

/* -------- Initialise le sélecteur (une fois pour toute) -------- */

int BIX_FSEL_Initialiser(void)
{
  int retour;
  unsigned int largeur, hauteur, largeur_defileur, largeur_champs, hauteur_apercu;
  unsigned int nombre_objets;
  unsigned int x_apercu, x_ascenseur, x_boutons_chemin __attribute__((__unused__));
  unsigned int y_fin_defileur;

  /* Contrôle grossier, mais efficace */
  if ( FSEL_NOMBRE_NOMS != 0 ) {
    AVERTISSEMENT( "Sélecteur déjà initialisé [pour %u noms]", FSEL_NOMBRE_NOMS );
    return +1;
  }

  BI_ARB_fselect = arbres + BI_ArbreSelecteur;

  /* Calcul du nombre d'objets dans l'arbre */
  nombre_objets = FSEL_FOND_DEFILEUR + 1;  /* ROOT à ne pas oublier ! */

  /* Combien de noms dans la hauteur de l'écran ? */
  FSEL_NOMBRE_NOMS = (BI_haut_ecran - 50) / BI_haut_car;  /* Nombre de lignes au maximum [50 : barre de titre d'une fenêtre, estimée] */
  if ( FSEL_NOMBRE_NOMS <= FSEL_HAUTEUR_MIN_BOITE ) {
    (void) fprintf(stderr, "%s, %s l.%u : Taille de police trop grande, sélecteur inaffichable !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  } else {
    FSEL_NOMBRE_NOMS -= (FSEL_HAUTEUR_MIN_BOITE - FSEL_HAUTEUR_MIN_DEFILEUR);
  }
  if ( FSEL_NOMBRE_NOMS < 6 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Taille de police limite, moins de 6 noms affichables !\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }
  if ( FSEL_NOMBRE_NOMS > FSEL_NOMBRE_MAX_ENTREES ) FSEL_NOMBRE_NOMS = FSEL_NOMBRE_MAX_ENTREES;

  nombre_objets += FSEL_NOMBRE_NOMS;

  /* On en déduit la hauteur de l'arbre [en caractères] et de l'aperçu */
  hauteur        = FSEL_Y_DEFILEUR + FSEL_NOMBRE_NOMS + 3;
  hauteur_apercu = hauteur - (9 + 5);

  /* Puis les positions des objets */
  y_fin_defileur = FSEL_Y_DEFILEUR + FSEL_NOMBRE_NOMS;

  /* On regarde la largeur maximale possible */
  largeur_defileur  = BI_larg_ecran / BI_larg_car;
  if ( largeur_defileur <= FSEL_LARGEUR_MIN_BOITE ) {
    (void) fprintf(stderr, "%s, %s l.%u : Police trop large, le sélecteur ne tiendrait pas à l'écran !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  largeur_defileur -= FSEL_LARGEUR_APERCU - 3 - FSEL_LARGEUR_ASCENSEUR;
  if ( largeur_defileur > FSEL_LARGEUR_MAX_DEFILEUR ) largeur_defileur = FSEL_LARGEUR_MAX_DEFILEUR;
  if ( largeur_defileur < FSEL_LARGEUR_MIN_DEFILEUR ) {
    (void) fprintf(stderr, "%s, %s l.%u : Police trop large, le sélecteur serait inutilisable !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -3;
  }

  /* On en déduit la largeur de l'arbre, en caractères */
  largeur = 1 + largeur_defileur + FSEL_LARGEUR_ASCENSEUR + 1 + FSEL_LARGEUR_APERCU + 1;

  /* Puis les positions des objets */
  x_boutons_chemin = largeur - ( 1 + FSEL_LARGEUR_BOUTONS_CHEMIN );
  x_ascenseur      = 1 + largeur_defileur;
  x_apercu         = x_ascenseur + FSEL_LARGEUR_ASCENSEUR + 1;
  largeur_champs   = largeur - ( 1 + FSEL_LARGEUR_TEXTES_CHEMIN + 1 + 1 + FSEL_LARGEUR_BOUTONS_CHEMIN + 1 );

  /* On construit l'arbre du sélecteur de fichiers */
  /* a. La racine et la structure de gestion */
  retour = BI_CreerArbre( BI_ArbreSelecteur, nombre_objets, largeur, hauteur );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR %d dans la création du sélecteur de fichiers !\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }
  arbres[ BI_ArbreSelecteur ].drapeaux.interne = VRAI;  /* Arbre interne : chaînes en statique non libérées en fin de programme */
#ifdef _DEVELOPPE_
/*   (void) fprintf(stderr, "%s, %s l.%u : Arbre réservé pour %u objets [soit %u entrées]\n", */
/* 		 __FILE__, __FUNCTION__, __LINE__, nombre_objets, FSEL_NOMBRE_NOMS ); */
#endif


  /* b. Les chemins */
  retour  = ARB_CreerEditable( BI_ArbreSelecteur, FSEL_TXT_CHEMIN   , 1, 1, FSEL_LARGEUR_TEXTES_CHEMIN, largeur_champs, ARB_OPTIONS_EDIT_INFINI, "Chemin    :" );
  retour |= ARB_CreerBouton  ( BI_ArbreSelecteur, FSEL_VALIDER_CHEMIN, largeur - FSEL_LARGEUR_BOUTONS_CHEMIN - 1, 1, FSEL_LARGEUR_BOUTONS_CHEMIN, "Changer" );
  retour |= ARB_CreerTexte   ( BI_ArbreSelecteur, FSEL_TXT_PREDEFINI, 1, 3, FSEL_LARGEUR_TEXTES_CHEMIN, "Prédéfinis :" );
  retour |= ARB_CreerObjet   ( BI_ArbreSelecteur, FSEL_PREDEFINIS, G_SSMENU,
			       1 + FSEL_LARGEUR_TEXTES_CHEMIN + 1, 3, largeur_champs, 1 );
  retour |= ARB_CreerBouton  ( BI_ArbreSelecteur, FSEL_AJOUTER_PREDEFINI, largeur - FSEL_LARGEUR_BOUTONS_CHEMIN - 1, 3, FSEL_LARGEUR_BOUTONS_CHEMIN, "Ajouter" );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/chemins [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }

  /* c. Les filtres */
  retour  = ARB_CreerTexte   ( BI_ArbreSelecteur, FSEL_TXT_FILTRE,
			       x_apercu, FSEL_Y_DEFILEUR, FSEL_LARGEUR_TEXTES_FILTRE , "Filtre  :" );
  retour |= ARB_CreerObjet   ( BI_ArbreSelecteur, FSEL_FILTRE, G_SSMENU,
			       x_apercu + FSEL_LARGEUR_TEXTES_FILTRE + 1, FSEL_Y_DEFILEUR, FSEL_LARGEUR_BOUTONS_FILTRE, 1 );
  retour |= ARB_CreerEditable( BI_ArbreSelecteur, FSEL_FILTRE_LIBRE,
			       x_apercu + FSEL_LARGEUR_TEXTES_FILTRE + 1 + FSEL_LARGEUR_BOUTONS_FILTRE + 1, FSEL_Y_DEFILEUR, 0, FSEL_LARGEUR_BOUTONS_FILTRE,
			       ARB_OPTIONS_EDIT_INFINI, NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/filtres [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }

  /* d. Les options */
  retour  = ARB_CreerTexte   ( BI_ArbreSelecteur, FSEL_TXT_OPTIONS,
			       x_apercu, FSEL_Y_DEFILEUR + 2, FSEL_LARGEUR_TEXTES_FILTRE , "Options :" );
  retour |= ARB_CreerObjet   ( BI_ArbreSelecteur, FSEL_OPTIONS, G_SSMENU,
			       x_apercu + FSEL_LARGEUR_TEXTES_FILTRE + 1, FSEL_Y_DEFILEUR + 2, FSEL_LARGEUR_OPTIONS, 1 );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/options [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }

  /* e. L'aperçu */
  retour  = ARB_CreerObjet   ( BI_ArbreSelecteur, FSEL_APERCU, G_USERDEF,
			       x_apercu, FSEL_Y_DEFILEUR + 4, FSEL_LARGEUR_APERCU, hauteur_apercu );
  retour |= ARB_CreerTexte   ( BI_ArbreSelecteur, FSEL_TXT_APERCU,
			       x_apercu + 1, FSEL_Y_DEFILEUR + 3, FSEL_LARGEUR_TEXTES_FILTRE , " Aperçu " );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/aperçu [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }
  OBJ_utilisateur( BI_ArbreSelecteur, FSEL_APERCU, FSEL_ApercuParDefaut, 0, METTRE );

  /* f. Les boutons pour quitter */
  retour  = ARB_CreerBouton( BI_ArbreSelecteur, FSEL_OK     , x_apercu , y_fin_defileur - 1, FSEL_LARGEUR_OK, "OK" );
  retour |= ARB_CreerBouton( BI_ArbreSelecteur, FSEL_ANNULER, largeur - FSEL_LARGEUR_OK - 1, y_fin_defileur - 1, FSEL_LARGEUR_OK, "Annuler" );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/sortie [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }
  arbres[ BI_ArbreSelecteur ].adresse[ FSEL_OK ].ob_flags |= DEFAULT|EXIT;  /* OK : bouton par défaut */
  arbres[ BI_ArbreSelecteur ].adresse[ FSEL_ANNULER ].ob_flags |= EXIT;

  /* g. Le nom du fichier */
  retour = ARB_CreerEditable( BI_ArbreSelecteur, FSEL_TXT_NOM,
			      1, y_fin_defileur + 1, FSEL_LARGEUR_TEXTES_CHEMIN, largeur_champs + 1 + FSEL_LARGEUR_BOUTONS_CHEMIN,
			      ARB_OPTIONS_EDIT_INFINI, "Nom :" );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/nom de fichier [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }

  /* h. Le défileur */
  retour  = ARB_CreerBouton( BI_ArbreSelecteur, FSEL_REMONTE, FSEL_X_DEFILEUR + largeur_defileur, FSEL_Y_DEFILEUR, FSEL_LARGEUR_ASCENSEUR, "\005" );
  retour |= ARB_CreerAscenseur( BI_ArbreSelecteur, FSEL_AVANT, FSEL_X_DEFILEUR + largeur_defileur, FSEL_Y_DEFILEUR + 1, ASC_VERTICAL, FSEL_NOMBRE_NOMS - 1 );
  retour |= ARB_CreerDefileur( BI_ArbreSelecteur, FSEL_FOND_DEFILEUR, FSEL_X_DEFILEUR, FSEL_Y_DEFILEUR, largeur_defileur, FSEL_NOMBRE_NOMS, FAUX, 0, 0 );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans la création des objets/défileur [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -30;
  }

  /* Adaptation des coordonnées */
  retour = ARB_Initialiser( BI_ArbreSelecteur, NULL, FSEL_GererSelecteur, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR %d dans l'initialisation du sélecteur de fichiers !\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -25;
  }

  /* Enjolivements */
  ARB_EnjoliverDefileur( BI_ArbreSelecteur, FSEL_FOND_DEFILEUR, FSEL_PREMIER_NOM, FSEL_DERNIER_NOM,
			 FSEL_AVANT, FSEL_FOND_ASCENSEUR, FSEL_BOUTON_ASCENSEUR, FSEL_APRES, ARB_ASCENSEUR_DROITE );
  OBJ_deplace( BI_ArbreSelecteur, FSEL_REMONTE, +1, -(int) (BI_haut_car / 2), DEPLACE_PIXELS );
  if ( FSEL_NOMBRE_NOMS > BI_haut_car ) {
    OBJ_deplace( BI_ArbreSelecteur, FSEL_TXT_NOM, 0, 2 * BI_haut_car, DEPLACE_PIXELS );
    OBJ_deplace( BI_ArbreSelecteur, FSEL_NOM    , 0, 2 * BI_haut_car, DEPLACE_PIXELS );

    OBJ_deplace( BI_ArbreSelecteur, FSEL_OK     , 0, 2 * BI_haut_car, DEPLACE_PIXELS );
    OBJ_deplace( BI_ArbreSelecteur, FSEL_ANNULER, 0, 2 * BI_haut_car, DEPLACE_PIXELS );

    OBJ_agrandit( BI_ArbreSelecteur, FSEL_APERCU, 0, 2 * BI_haut_car );

    OBJ_agrandit( BI_ArbreSelecteur, ROOT, 0, 2 * BI_haut_car );
  }

  /* Choix de la fonction et fin */
  arbres[ BI_ArbreSelecteur ].drapeaux.prepare = VRAI;

  /* Fin correcte, le sélecteur a pu être initialisé */
  return 0;
}

/* -------- Obtenir le chemin initial -------- */

static char *FSEL_CheminCourant(void)
{
  if (  NULL == FSEL_chemin_initial ) {
    /* Là, c'est le premier appel : il faut récupérer le chemin pour l'application */
    FSEL_chemin_initial = (char *) malloc( MAXPATHLEN + 1 ); /* Pour pouvoir faire des changements de chemin... +1 pour le 0 final */
    if ( NULL == FSEL_chemin_initial ) {
      (void) fprintf( stderr, "%s, %s l.%u : Impossible de mémoriser le chemin initial...\n",
		    __FILE__, __FUNCTION__, __LINE__ );
      return NULL;
    }
    
    if ( getcwd( FSEL_chemin_initial, (size_t) ( MAXPATHLEN + 1 ) ) == NULL ) {
      (void) fprintf( stderr, "%s, %s l.%u : Impossible d'obtenir le chemin initial, errno = %d\n",
		      __FILE__, __FUNCTION__, __LINE__, errno );
      return NULL;
    }
  }

  return strdup( FSEL_chemin_initial );
}

/* -------- Libère tout ce qui a pu être réservé pour le sélecteur de fichiers -------- */

void BIX_FSEL_Terminer(void)
{
  if ( NULL == arbres ) return;

  /* On libère les entrées dans le défileur */
  FSEL_LibererDefileur( );

  /* On libère les chaines et tout et tout */
  BI_LibererArbre( BI_ArbreSelecteur );

  /* Plus besoin du chemin du sélecteur */
  free( FSEL_chemin_initial ); FSEL_chemin_initial = NULL;
}

/* ******** Interface avec l'extérieur ******************************************************** */

int BI_SelecteurDeFichiers(BI_InfosSelecteur *infos, unsigned long *nombre_choisis, char ***fichiers_choisis, Fenetre *fenetre_mere)
{
  int reponse;
  unsigned long nombre_filtres = 1; /* Toujours * au minimum */
  const char **filtres = NULL;	/* Table des filtres */
  struct infos_selecteur infos_arbre = { NULL,
					 0, 0, NULL, NULL,
					 0, 0, 0, 0, NULL };

  /* Contrôles préliminaires */
  if ( ( NULL == infos ) || ( NULL == nombre_choisis ) || ( NULL == fichiers_choisis ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune information indiquée, ou autre paramètre NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( *fichiers_choisis != NULL ) {
    AVERTISSEMENT( "Liste de fichiers préinitialisée indiquée [%p], ignorée !", (void *) *fichiers_choisis );
    free( *fichiers_choisis );
  }

  /* Initialisations */
  *nombre_choisis = 0;
  *fichiers_choisis = NULL;

  infos_arbre.chemin     = (infos->chemin != NULL) ? ( strdup( infos->chemin ) ) : ( NULL ) ;
  infos_arbre.parametres = infos;

  /* On récupère le chemin de travail, si besoin */
  if ( NULL == infos_arbre.chemin ) {
    infos_arbre.chemin = FSEL_CheminCourant();
    if ( NULL == infos_arbre.chemin ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de déterminer le chemin initial ! Appel annulé\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -2;
    }
  }

  /* On prépare les filtres */
  /* a. Combien ? */
  if ( infos->proprietes & BI_SELECTEUR_FILTRE_UTILISATEUR ) {
    nombre_filtres++;		/* [Libre] */
  }
  if ( ( infos->filtres != NULL ) && ( infos->nbr_filtres > 0 ) ) {
    nombre_filtres++;		/* Séparateur */
    nombre_filtres += infos->nbr_filtres;
  }

  /* b. Réservation */
  filtres = (const char **) malloc( sizeof( char * ) * nombre_filtres );
  if ( NULL == filtres ) {
    (void) fprintf(stderr, "%s, %s l.%u : Mémoire insuffisante pour réserver les filtres, seul * sera présent\n",
		   __FILE__, __FUNCTION__, __LINE__);
    nombre_filtres = 1;
  } else {
    unsigned long i_separateur = 1;

    filtres[ 0 ] = "*";
    if ( infos->proprietes & BI_SELECTEUR_FILTRE_UTILISATEUR ) {
      filtres[ 1 ] = "[Libre]";
      i_separateur++;
    }
    if ( ( infos->filtres != NULL ) && ( infos->nbr_filtres > 0 ) ) {
      unsigned long i;

      filtres[ i_separateur++ ] = "-- Pr"e_AIGU"d"e_AIGU"finis";
      for ( i = 0 ; i < infos->nbr_filtres ; i++ ) {
	filtres[ i_separateur + i ] = infos->filtres[ i ];
      }
    }
  }

  /* Liste des chemins prédéfinis */
  if ( ( infos->nbr_predefinis > 0 ) && ( infos->predefinis != NULL ) ) {
    OBJ_active( BI_ArbreSelecteur, FSEL_PREDEFINIS, FORCE_OUI );
    OBJ_MettreMenu( BI_ArbreSelecteur, FSEL_PREDEFINIS, infos->nbr_predefinis, (const char **) infos->predefinis, "Choisir un chemin pr"e_AIGU"enregistr"e_AIGU, FSEL_GererChoixChemin );
  } else {
    OBJ_active( BI_ArbreSelecteur, FSEL_PREDEFINIS, FORCE_NON );
  }

  /* Affichages dans le sélecteur */
  OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_CHEMIN, infos_arbre.chemin, 0 );
  OBJ_CopierTexte( BI_ArbreSelecteur, FSEL_NOM, "", 0 );
  if ( nombre_filtres > 1 ) {
    OBJ_MettreMenu( BI_ArbreSelecteur, FSEL_FILTRE, nombre_filtres, filtres, NULL, FSEL_GererChoixFiltre );
    OBJ_active( BI_ArbreSelecteur, FSEL_FILTRE, FORCE_OUI );
  } else {
    OBJ_RemplacerTexte( BI_ArbreSelecteur, FSEL_FILTRE, "*", FAUX );
    OBJ_active( BI_ArbreSelecteur, FSEL_FILTRE, FORCE_NON );
  }
  OBJ_active( BI_ArbreSelecteur, FSEL_FILTRE_LIBRE, FORCE_NON );

  /* Association des infos */
  ARB_MettreComplement( BI_ArbreSelecteur, &infos_arbre );

  /* Préparation de la liste des fichiers & dossiers du chemin initial */
  FSEL_ConstruireListe( &infos_arbre );
  FSEL_InitialiserDefileur( &infos_arbre );

  /* Appel du sélecteur en tant qu'arbre */
  reponse = ARB_Bloquant( BI_ArbreSelecteur, infos->titre, fenetre_mere, BI_ArbreSelecteur );

  /* Plus besoin de la liste des filtres */
  free( filtres ); filtres = NULL;
  ARB_MettreComplement( BI_ArbreSelecteur, NULL );

  /* Analyse de la réponse de ext_formdo */
  reponse = FSEL_AnalyserReponse( &infos_arbre, reponse, fichiers_choisis, nombre_choisis );

  /* Travail complémentaire en fonction de cette réponse */
  if ( 0 == reponse ) {
    /* On a bien validé quelque chose... */
    if ( infos->proprietes & BI_SELECTEUR_MEMORISER_CHEMIN ) {
      /* On veut que l'appelant sache quel est le nouveau chemin courant */
      free( infos->chemin ); infos->chemin = NULL;
      infos->chemin = strdup( infos_arbre.chemin );
    }

    if ( *nombre_choisis > 1 ) {
      /* On trie la liste des fichiers par ordre alphabétique */
      qsort( *fichiers_choisis, *nombre_choisis, sizeof(char *), FSEL_TrierNoms );

      /* On élimine les doublons */
      FSEL_EliminerDoublons( *fichiers_choisis, nombre_choisis );
    }
  } else {
    /* On contrôle qu'il n'y a pas une sélection de fichiers qui traine... */
    if ( *nombre_choisis > 0 ) {
      unsigned long i;

      AVERTISSEMENT( "Erreur ou Annulation, mais liste de %lu fichiers !", *nombre_choisis );
      for ( i = 0 ; i < *nombre_choisis ; i++ ) {
	free( (*fichiers_choisis)[ i ] ); (*fichiers_choisis)[ i ] = NULL;
      }
      free( *fichiers_choisis ); *fichiers_choisis = NULL;
    }
  }

  /* Plus besoin de la liste de fichiers */
  FSEL_LibererListe( &infos_arbre );

  /* On n'a plus besoin de la copie interne du chemin */
  free( infos_arbre.chemin ); infos_arbre.chemin = NULL;

  /* Terminé, sans problème */
  return reponse;
}

/* -------- Analyser le retour du sélecteur pour savoir que répondre -------- */

static int FSEL_AnalyserReponse(struct infos_selecteur *infos_selection, int reponse, char ***fichiers_choisis, unsigned long *nombre_choisis)
{
  /* Code négatif : erreur */
  if ( reponse < 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : Erreur survenue dans l'arbre du sélecteur de fichiers [code %d]...\n",
		    __FILE__, __FUNCTION__, __LINE__, reponse );
    return reponse;  /* Une erreur est survenue : on la renvoie telle qu'elle */
  }

  /* Bouton d'annulation */
  if ( FSEL_ANNULER == reponse ) {
    return BI_SELECTEUR_ANNULER;
  }

  /* Autre code positif : on accepte... */
  if ( infos_selection->parametres->proprietes & BI_SELECTEUR_CHEMIN_SEULEMENT ) {
    /* On ne veut que le chemin final ==> pas besoin de construire une liste de noms ! */
    return 0;
  }

  /* Si on est là, il faut construire la liste des noms. */
  if ( infos_selection->parametres->proprietes & BI_SELECTEUR_MULTIPLE ) {
    /* La sélection multiple est autorisée : la liste des noms comprend tous les noms sélectionnés + le(s) nom(s) du champ éditable */
    char **noms_choisis = NULL, **noms_entres = NULL;
    unsigned long nbr_noms_choisis = 0, nbr_noms_entres = 0, i;

    /* 1. On récupère les noms sélectionnés dans la liste */
    nbr_noms_choisis = FSEL_NomsChoisis( infos_selection, &noms_choisis );

    /* 2. On récupère les noms tapés dans le champ correspondant */
    nbr_noms_entres = FSEL_NomsEntres( infos_selection, &noms_entres );

    if ( 0 == ( nbr_noms_choisis + nbr_noms_entres ) ) {
      /* C'est comme si l'on annulait */
      return BI_SELECTEUR_ANNULER;
    }

    /* La table va contenir pour chaque entrée le chemin complet vers un fichier */
    *fichiers_choisis = (char **) malloc( sizeof(char *) * ( nbr_noms_choisis + nbr_noms_entres ) );
    if ( NULL == *fichiers_choisis ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de réserver la place pour la table des %lu noms de fichiers !\n",
		     __FILE__, __FUNCTION__, __LINE__, nbr_noms_choisis + nbr_noms_entres );
    } else {
      unsigned long j;

      for ( i = 0 ; i < nbr_noms_choisis ; i++ ) {
	(*fichiers_choisis)[ i ] = X_strcat( infos_selection->chemin, noms_choisis[ i ] );
      }
      for ( j = 0 ; j < nbr_noms_entres ; i++, j++ ) {
	(*fichiers_choisis)[ i ] = X_strcat( infos_selection->chemin, noms_entres[ j ] );
      }
      *nombre_choisis = nbr_noms_choisis + nbr_noms_entres;
    }

    /* On peut libérer les noms d'origine */
    /* 1. Noms choisis : pas de réservation de mémoire spécifique, figure déjà dans les entrées -> ne pas libérer ! */
    free( noms_choisis ); noms_choisis = NULL;

    /* 2. Noms entrés : plus délicat... */
    free( noms_entres ); noms_entres = NULL;
  } else {
    char *nom_retenu;

    /* Un seul nom est demandé :  */
    nom_retenu = FSEL_NomRetenu( infos_selection );
    if ( NULL == nom_retenu ) {
      /* C'est comme si l'on annulait ! */
      return BI_SELECTEUR_ANNULER;
    }

    /* La table va contenir une seule entrée, le chemin complet vers ce fichier */
    *fichiers_choisis = (char **) malloc( sizeof(char *) );
    if ( NULL == *fichiers_choisis ) {
      (void) fprintf(stderr, "%s, %s l.%u : Impossible de réserver la place pour la table des noms de fichiers !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -15;
    }

    (*fichiers_choisis)[ 0 ] = X_strcat( infos_selection->chemin, nom_retenu );
    *nombre_choisis = 1;
  }

  /* Ici tout va bien */
  return 0;
}

/* -------- Éliminer les doublons de la liste des fichiers TRIÉÉ -------- */

static int FSEL_EliminerDoublons(char **noms_choisis, unsigned long *nombre_noms)
{
  unsigned long i_lu, i_ecrit, nv_nombre_noms;

  i_lu = 0; i_ecrit = 0;
  nv_nombre_noms = 0;
  do {
    printf( "i_lu = %lu, i_ecrit = %lu, noms = %s et %s\n", i_lu, i_ecrit, noms_choisis[ i_lu ], noms_choisis[ i_ecrit ] );
    while ( 0 == strcmp( noms_choisis[ i_lu ], noms_choisis[ i_ecrit ] ) ) {
      /* On a le même nom : inutile de l'écrire donc */
      i_lu++;
      if ( i_lu >= (*nombre_noms) ) {
	/* On termine sur un doublon */
	nv_nombre_noms++;

	*nombre_noms = nv_nombre_noms;
	return 0;
      }
    }

    /* Ici : i_lu et i_ecrit sont donc deux noms différents => on augmente i_ecrit de 1 */
    i_ecrit++;

    /* Et on écrit le nouveau lu comme nouvelle référence */
    noms_choisis[ i_ecrit ] = noms_choisis[ i_lu ];
    nv_nombre_noms++;
  } while ( i_lu < (*nombre_noms) - 1 );

  /* Nouveau nombre de fichiers lus */
   *nombre_noms = nv_nombre_noms;

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Analyser le sélecteur pour en déduire les noms des fichiers sélectionnés, en cas de sélection multiple -------- */

static unsigned long FSEL_NomsChoisis(struct infos_selecteur *infos_selection, char ***noms_choisis)
{
  unsigned long i, nombre_choisis = 0;
  Entree_selecteur *entree;

  for ( entree = infos_selection->dossiers ; entree != NULL ; entree = entree->apres ) {
    if ( ( FAUX == entree->dossier ) && ( VRAI == entree->choisit ) ) {
      nombre_choisis++;
    }
  }

  if ( 0 == nombre_choisis ) {
    /* Aucun nom choisi dans le sélecteur */
    *noms_choisis = NULL;
    return 0;
  }

  /* On réserve la table */
  *noms_choisis = (char **) malloc( sizeof( char * ) * nombre_choisis );
  if ( NULL == *noms_choisis ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR en réservant la table des noms choisis\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0;
  }

  /* On la remplit */
  for ( i = 0, entree = infos_selection->dossiers ; entree != NULL ; entree = entree->apres ) {
    if ( ( FAUX == entree->dossier ) && ( VRAI == entree->choisit ) ) {
      if ( i >= nombre_choisis ) {
	(void) fprintf(stderr, "%s, %s l.%u : Incohérence, le nombre de fichiers choisi a augmenté !\n",
		       __FILE__, __FUNCTION__, __LINE__);
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	break;
      }
      (*noms_choisis)[ i ] = entree->nom_fichier + 2;
      i++;
    }
  }

  /* Terminé */
  return nombre_choisis;
}

/* -------- Analyser le champ texte du sélecteur pour avoir la liste des noms de fichiers à construire, sélection multiple -------- */

static unsigned long FSEL_NomsEntres(struct infos_selecteur *infos_selection, char ***noms_choisis)
{
  char *chaine_a_liberer;
  char *texte_entre = NULL;
  char **table_temporaire = NULL;
  unsigned long i, nombre_entres = 0;

  /* 1. On récupère le texte entré */
  texte_entre = lit_chaine( BI_ArbreSelecteur, FSEL_NOM, 0, NULL, TXT_REMPLACE );
  if ( ( NULL == texte_entre ) || ( 0 == *texte_entre ) ) {
    /* Aucun nom tapé : inutile de se fatiguer ! */
    *noms_choisis = NULL;
    return 0;
  }

  /* 2. On le décompose : le ; sert à séparer les fichiers */
  table_temporaire = TXT_DecomposerChaine( texte_entre, ';', &nombre_entres );
  if ( NULL == table_temporaire ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problème en décomposant la chaîne\n",
		   __FILE__, __FUNCTION__, __LINE__);
    *noms_choisis = NULL;
    return 0;
  }

  /* Si l'on néglige les jokers, il suffit de recopier cette table */
  if ( FAUX == ( infos_selection->parametres->proprietes & BI_SELECTEUR_AVEC_JOKERS ) ) {
    char *chaine_a_liberer = table_temporaire[ 0 ];
    unsigned long i;

    for ( i = 0 ; i < nombre_entres ; i++ ) {
      table_temporaire[ i ] = strdup( table_temporaire[ i ] );
    }
    free( chaine_a_liberer ); chaine_a_liberer = NULL;

    *noms_choisis = table_temporaire;
    return nombre_entres;
  }

  OPTION_A_FAIRE( "Prendre en compte les jokers" );

  /* Ce qui suit n'est qu'un pis-aller en attendant mieux ! */
  chaine_a_liberer = table_temporaire[ 0 ];
  for ( i = 0 ; i < nombre_entres ; i++ ) {
    table_temporaire[ i ] = strdup( table_temporaire[ i ] );
  }
  free( chaine_a_liberer ); chaine_a_liberer = NULL;

  *noms_choisis = table_temporaire;
  return nombre_entres;
}

/* -------- Analyser le sélecteur pour en déduire le nom du fichier, en cas de sélection simple -------- */

static char *FSEL_NomRetenu(struct infos_selecteur *infos_selection)
{
  char *nom = NULL;

  /* 1. Un nom a-t-il été entré dans le champ éditable ? */
  nom = lit_chaine( BI_ArbreSelecteur, FSEL_NOM, 0, NULL, TXT_REMPLACE );
  if ( ( nom != NULL ) && ( *nom != 0 ) ) {
    return nom;
  }

  /* 2. Une entrée a-t-elle été sélectionnée ? */
  if ( infos_selection->numero_choisi >= 0 ) {
    unsigned long i;
    Entree_selecteur *entree;

    /* Oui : on récupère le nom associé */
    for ( i = 0,  entree = infos_selection->dossiers ;
	  i < infos_selection->premier_affiche + ( infos_selection->numero_choisi - FSEL_PREMIER_NOM ) ;
	  entree = entree->apres ) {
      if ( VRAI == entree->afficher ) i++;  /* On ne compte que les fichiers visibles dans le sélecteur ! */
    }
    nom = entree->nom_fichier + 2;
  }

  /* Si le nom est vide, c'est comme s'il n'y avait pas de nom ! */
  if ( 0 == *nom ) nom = NULL;

  /* On renvoie le nom ainsi trouvé */
  return nom;
}

/* -------- Trier les noms par ordre alphabétique [routine appelée par qsort()] -------- */

static int FSEL_TrierNoms(const void *objet1, const void *objet2)
{
  char *nom_1, *nom_2;

  nom_1 = *(char **) objet1;
  nom_2 = *(char **) objet2;
#ifdef _DEVELOPPE_
  if ( ( NULL == nom_1 ) || ( NULL == nom_2 ) ) {
    AVERTISSEMENT( "Appel avec NULL comme paramètre [e1 = %p, e2 = %p]", nom_1, nom_2 );
    return 0;
  }
#endif

  /* Par défaut, ordre alphabétique */
  return strcmp( nom_1, nom_2 );
}

