/* Bibliothèque X-Window pour les interfaces
   Routines pour les menus (barre de menu et sous-menus "pop-up")

   (C) 1998, Emmanuel CURIS - Création le 11 septembre 1998
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"

/* ********** Variables globales limitées à ce module ************** */

static XSizeHints *taille_menu = NULL,     /* Infos X assurant que la taille des fenêtre est fixe   */
                  *taille_fenetre = NULL;

#define MENU_CHANGER_MENU 0
#define MENU_CHOIX_ANNULE 1
#define MENU_CHOIX_OPTION 2
#define MENU_IGNORER      3

/**************** Fonctions internes à ce module *****************/

/* Barre de menus */
static int   MENU_GenererBarre (Menu *menu);
static void  MENU_LibererMenu  (Menu *menu);
static void  MENU_PreparerArbre(unsigned int num_arbre);
static Menu *MENU_ReserverMenu (unsigned int num_arbre, boolean avec_fenetre);

/* Gestion des options : menus déroulés */
static int  MENU_ActiveOption   (Menu *menu, unsigned int num_menu, unsigned int option);
static int  MENU_ControleMenu   (Menu *menu, unsigned int num_menu);
static int  MENU_DesactiveOption(Menu *menu, unsigned int num_menu);
static int  MENU_GererClavier   (Menu *menu, unsigned int num_menu, XKeyEvent *clavier, unsigned int *option);
static int  MENU_GererOptions   (Menu *menu, unsigned int num_menu, unsigned int *option);
static int  MENU_LibererOptions (Menu *menu, unsigned int num_menu);
static void MENU_PreparerMenu   (OBJECT *adresse, unsigned int pere, boolean interne);
static int  MENU_PreparerOptions(Menu *menu, unsigned int num_menu, int x, int y, const char *titre);

/* Cas des sous-menus */
static int  SSMenu_AvecAscenseur(const char *textes[], unsigned int nbr_txt, unsigned int nbr_visibles, const char *titre,
				 int arbre_pere, unsigned int objet_pere);
static int  SSMenu_SansAscenseur(const char *textes[], unsigned int nbr_txt, const char *titre,
				 int arbre_pere, unsigned int objet_pere);

static int  SSMenu_Placer(int arbre_pere, unsigned int objet_pere, unsigned int nbr_textes, int *x, int *y);

/* ********** Barre de menu principale *********** */

/* -~-~-~-~~~~~ Travail sur les barres de menu -~-~-~-~~~~~ */

/* ---------------- Installation d'une barre de menu -----------------\
 |    Remplace la barre existante s'il y en a déjà une                |
 |                                                                    |
 | num_arbre : arbre GEM à installer comme menu                       |
 | gestion   : routine de gestion du menu                             |
 \------------------------------------------------------------------- */

int BI_InstallerMenu(unsigned int num_arbre, const char *titre, boolean (*gestion)(unsigned int option) )
{
  int retour;

  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "BI_InstallerMenu : numéro d'arbre incorrect [%u, max %u]\n", num_arbre + 1, BI_nbr_arbres);
    return -1;
  }
  if ( FAUX == arbres[ num_arbre ].drapeaux.prepare ) {
    MENU_PreparerArbre( num_arbre );
  }

  if (BI_Menu != NULL) { /* Il y a déjà un menu */
    if ( BI_Menu->adresse == arbres[ num_arbre ].adresse ) {
      return 0; /* Pas de problème */
    }

    free(BI_Menu->titres); BI_Menu->titres = NULL;
    free(BI_Menu->nbr_options); BI_Menu->nbr_options = NULL;
    free(BI_Menu->prem_option); BI_Menu->prem_option = NULL;

    XDestroySubwindows(BI_Display, BI_Menu->racine); /* On détruit les sous-fenêtres */

    /* Le nouveau menu */
    BI_Menu->adresse = arbres[ num_arbre ].adresse;
  } else {
    BI_Menu = MENU_ReserverMenu( num_arbre, VRAI );
    if (BI_Menu == NULL) {
      (void) fprintf(stderr, "BI_InstallerMenu : erreur dans MENU_ReserverMenu()\n");
      return -2;
    }

    taille_menu = XAllocSizeHints();
  }

  /* Le nouveau titre de la barre de menu, s'il existe */
  if ( NULL == titre ) {
    XStoreName( BI_Display, BI_Menu->racine, titre );
  }

  /* On créé les sous-fenêtres du menu */
  retour = MENU_GenererBarre(BI_Menu);
  if (retour != 0) {
    (void) fprintf(stderr, "BI_InstallerMenu : erreur dans MENU_GenererBarre()\n");
    MENU_LibererMenu(BI_Menu);
    BI_Menu = NULL;
    return -5;
  }

  BI_Menu->gestion = gestion; /* Fonction de gestion du menu */
  BI_PRG_menu = num_arbre; /* Menu correspondant conservé ! */

  /* On affiche le menu */
  XMapWindow(BI_Display, BI_Menu->racine);
  XMapSubwindows(BI_Display, BI_Menu->racine);

  return 0; /* Pas d'erreur */
}

/* ---------------- Suppression de la barre de menu ---------------- */

int BI_SupprimerMenu(void)
{
  if ( BI_Menu == NULL ) { /* Aucun menu */
    return 0; /* donc aucun problème... */
  }

  MENU_LibererMenu( BI_Menu );

  if (taille_menu) /* On avait pu bloquer la taille avec les XSizeHints */
    XFree(taille_menu);

  BI_Menu = NULL;

  return 0; /* Tout s'est bien passé */
}

/* -------- Ouvrir une fenêtre pour la barre de menus -------- */

static int MENU_GenererBarre(Menu *menu)
{
  XSetWindowAttributes parametres;
  int x, y;
  unsigned int largeur;
  unsigned int premier_titre, i, pere_menus, num_menu;

  if (NULL == menu) return -1;
  if (None == menu->racine) {
    (void) fprintf(stderr, "MENU_GenererBarre : menu sans fenêtre racine.\n");
    return -2;
  }
  if (NULL == menu->adresse) {
    (void) fprintf(stderr, "MENU_GenererBarre : menu sans arbre associé.\n");
    return -3;
  }
  if ( NULL != menu->titres ) {
    (void) fprintf(stderr, "MENU_GenererBarre : le menu contient déjà une barre de menus réalisée\n");
    return -4;
  }

  /* Combien de titres dans le menu ? */
  premier_titre = OB_HEAD(menu->adresse, ROOT); /* Pointe sur une boîte contenant tout */
  premier_titre = OB_HEAD(menu->adresse, premier_titre); /* Boîte contenant les titres */
  menu->nbr_titres = OB_TAIL(menu->adresse, premier_titre) - OB_HEAD(menu->adresse, premier_titre) + 1;
 
  menu->titres = (Window *) calloc( menu->nbr_titres, sizeof(Window) );
  if (menu->titres == NULL) {
    (void) fprintf(stderr, "MENU_GenererBarre : mémoire insuffisante -- table des titres des menus\n");
    return -5; /* Erreur : pas assez de fenêtres */
  }
  menu->nbr_options = (unsigned int *) calloc( menu->nbr_titres, sizeof(unsigned int) );
  if (menu->nbr_options == NULL) {
    (void) fprintf(stderr, "MENU_GenererBarre : mémoire insuffisante -- table des nombres d'options par menu\n");
    return -6; /* Erreur : pas assez de fenêtres */    
  }
  menu->prem_option = (unsigned int *) calloc( menu->nbr_titres, sizeof(unsigned int) );
  if (menu->prem_option == NULL) {
    (void) fprintf(stderr, "MENU_GenererBarre : mémoire insuffisante -- table des débuts des menus\n");
    return -7; /* Erreur : pas assez de fenêtres */    
  }

  premier_titre = OB_HEAD(menu->adresse, premier_titre); /* On se place sur le premier titre */
  menu->prem_titre = premier_titre;

  parametres.background_pixel = BI_Couleurs[GRIS].pixel;
  parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    | ButtonReleaseMask | ExposureMask | PointerMotionMask
    | EnterWindowMask | LeaveWindowMask; /* Tous événements transmis */

  x = 0; y = 0;
  for (i = 0 ; i < menu->nbr_titres ; i++) {
    largeur = OB_WIDTH(menu->adresse, premier_titre + i);
    OB_HEIGHT(menu->adresse, premier_titre + i) = (int) (1.5 * BI_haut_car);

    menu->titres[i] = XCreateWindow(BI_Display, menu->racine, x, y, largeur, (unsigned int) (1.5 * BI_haut_car),
				    1, CopyFromParent, InputOutput, BI_Visual,
				    CWBackPixel | CWEventMask, &parametres);

    x += largeur + 1;
  }

  /* On met à la bonne largeur */
  largeur = x;
  if ( taille_menu ) { /* A pu être créé */
    taille_menu->flags = PMinSize | PMaxSize; /* On ne peut pas en changer la taille */
    taille_menu->min_width  = largeur;
    taille_menu->min_height = (int) (1.5 * BI_haut_car); /* +2 = pour les bords... */
    taille_menu->max_width  = taille_menu->min_width;
    taille_menu->max_height = taille_menu->min_height;
    XSetWMNormalHints( BI_Display, menu->racine, taille_menu );
  }
  XResizeWindow(BI_Display, menu->racine, largeur, 2 + (unsigned int) (1.5 * BI_haut_car) ); /* Nouvelle taille */

  /* On repère, pour chaque menu, la première option et le nombre */

  premier_titre += i; /* G_IBOX après le dernier titre */
  pere_menus = premier_titre;

  for (i = OB_HEAD(menu->adresse, premier_titre), num_menu = 0 ;
       i != pere_menus ; i = OB_NEXT(menu->adresse, i), num_menu++) {
    unsigned int premiere_option, derniere_option;

#ifdef _DEVELOPPE_
/*     (void) printf("MENU_GenererBarre : options / titre %u = objet %u\n", num_menu + 1, i); */
#endif
    premiere_option = (unsigned int) OB_HEAD(menu->adresse, i); /* Première option pointée */
    derniere_option = (unsigned int) OB_TAIL(menu->adresse, i); /* Dernière option pointée */
    (menu->prem_option)[ num_menu ] = premiere_option;
    (menu->nbr_options)[ num_menu ] = derniere_option - premiere_option + 1;
  }

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Libérer la barre de menus -------- */

static void MENU_LibererMenu(Menu *menu)
{
  if (NULL == menu) return;

  if ( menu->fond_option != None ) {
    (void) fprintf(stderr, "MENU_LibererMenu : appel avec un menu ouvert !\n");
    MENU_LibererOptions(menu, 0);
  }

  /* Libération de la fenêtre racine */
  if ( menu->racine != None ) {
    XDestroyWindow(BI_Display, menu->racine);
  }

  /* Libération de la mémoire -- normalement, NULL si non utilisé donc pas de problème */
  free(menu->prem_option);
  free(menu->nbr_options);
  free(menu->titres);
  free(menu->options);

  free(menu);
}

/* -------- Amélioration de l'aspect d'un arbre de menus -------- */

static void MENU_PreparerArbre(unsigned int num_arbre)
{
  OBJECT *adresse;
  unsigned int premier_titre, nbr_titres, premier_menu, i;
  int retour;

  retour = ARB_Initialiser( num_arbre, NULL, ( boolean (*)(unsigned int, int, int) ) NULL, arbres[ num_arbre ].clavier );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR %d en voulant initialiser le menu %u\n",
		    __FILE__, __FUNCTION__, __LINE__, retour, num_arbre );
    return;
  }

  adresse = arbres[ num_arbre ].adresse;

/*   printf("Racine : next = %d, tail = %d, head = %d\n", OB_NEXT(adresse, ROOT), OB_TAIL(adresse, ROOT), OB_HEAD(adresse, ROOT)); */

  /* On cherche le nombre de titres */
  premier_titre = OB_HEAD(adresse, ROOT); /* Pointe sur une boîte contenant tout */
/*   printf("Head   : next = %d, tail = %d, head = %d\n", */
/* 	 OB_NEXT(adresse, premier_titre), OB_TAIL(adresse, premier_titre), OB_HEAD(adresse, premier_titre)); */

  premier_titre = OB_HEAD(adresse, premier_titre); /* Boîte contenant les titres */
/*   printf("Head2  : next = %d, tail = %d, head = %d\n", */
/* 	 OB_NEXT(adresse, premier_titre), OB_TAIL(adresse, premier_titre), OB_HEAD(adresse, premier_titre)); */
  nbr_titres    = OB_TAIL(adresse, premier_titre) - OB_HEAD(adresse, premier_titre) + 1;
/*   premier_titre = OB_HEAD(adresse, premier_titre); */
/*   printf("Titre  : next = %d, tail = %d, head = %d\n", */
/* 	 OB_NEXT(adresse, premier_titre), OB_TAIL(adresse, premier_titre), OB_HEAD(adresse, premier_titre)); */

  /* On cherche le premier menu
     Ils sont présents, mais pas liés aux précédents ! */
  premier_menu = OB_TAIL(adresse, ROOT) + 1;
#ifdef _DEVELOPPE_
/*   (void) printf("MENU_PreparerArbre : menu n°%u - premier titre (sur %u) : objet %u ; premier menu : objet %u\n", */
/* 		num_arbre, nbr_titres, premier_titre, premier_menu); */
#endif
  for (i = 0 ; i < nbr_titres ; i++, premier_menu = OB_NEXT(adresse, premier_menu) ) {
    MENU_PreparerMenu( adresse, premier_menu, arbres[ num_arbre ].drapeaux.interne );
  }

  /* L'arbre est un menu et a été traité */
  arbres[ num_arbre ].drapeaux.menu = VRAI;
  arbres[ num_arbre ].drapeaux.prepare = VRAI;
}

/* -------- Réservation des infos pour une barre de menu -------- */

static Menu *MENU_ReserverMenu(unsigned int num_arbre, boolean avec_fenetre)
{
  Menu *menu;

  /* Réservation */
  menu = (Menu *) malloc( sizeof(Menu) );
  if (NULL == menu) {
    (void) fprintf(stderr, "MENU_ReserverMenu : mémoire insuffisante -- menu (%lu octets)\n",
		   (unsigned long) sizeof(Menu) );
    return NULL;
  }

  /* Initialisation */
  menu->adresse = arbres[ num_arbre ].adresse;
  menu->racine = None;
  menu->nbr_titres = 0;
  menu->prem_titre = 0;
  menu->titres = NULL;
  menu->prem_option = NULL;
  menu->nbr_options = NULL;
  menu->fond_option = None;
  menu->options = NULL;
  menu->titre = NULL;
  menu->option_active = 0;
  menu->gestion = ( boolean (*)(unsigned int option) ) NULL;

  if ( VRAI == avec_fenetre ) { /* Ce sera une vraie barre de menus */
    XSetWindowAttributes parametres;

    /* On crée donc une fenêtre de fond */
    parametres.background_pixel = BI_Couleurs[BLANC].pixel; /* Fond en blanc */
    parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
      | ButtonReleaseMask | ExposureMask | PointerMotionMask
      | EnterWindowMask | LeaveWindowMask; /* Tous événements transmis */

    menu->racine = XCreateWindow(BI_Display, BI_Fen_racine, 0, 0, 100, (unsigned int) (1.5 * BI_haut_car),
				 0, CopyFromParent, InputOutput, BI_Visual,
				 CWBackPixel | CWEventMask, &parametres);
    if ( None == menu->racine ) {
      (void) fprintf(stderr, "MENU_ReserverMenu : impossible d'ouvrir la fenêtre de fond\n");
      free(menu);
      return NULL;
    }

    /* On la protège contre une fermeture intempestive par le WM */
    if ( XSetWMProtocols(BI_Display, menu->racine, &BI_ProtocoleFermerFenetre, 1) == 0 ) {
      (void) fprintf(stderr, "MENU_ReserverMenu : impossible de définir WM_DELETE_WINDOW, barre de menus\n");
    }

    /* Titre par défaut */
    XStoreName(BI_Display, menu->racine, "Barre de menus"); 
  }

  return menu;
}

/* ~-~-~-~-~-~-~-~- Travail sur les options ~-~-~-~-~-~-~-~- */

/* -------- Activer une option -------- */

static int MENU_ActiveOption(Menu *menu, unsigned int num_menu, unsigned int option)
{
  int retour;
  unsigned int objet;

  /* Contrôles préliminaires */
  retour = MENU_ControleMenu(menu, num_menu);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "MENU_ActiveOption : erreur dans MENU_ControleMenu [code %d]\n", retour);
    return -1;
  }
  if ( option >= menu->nbr_options[ num_menu ] ) {
    (void) fprintf(stderr, "MENU_ActiveOption : numéro d'option incorrect [%u / %u max --- menu n°%u]\n",
		   option + 1, menu->nbr_options[ num_menu ], num_menu);
    return -2;
  }

  /* On repère l'objet associé à l'option choisie */
  objet = menu->prem_option[ num_menu ] + option;
  if ( FAUX == (OB_STATE(menu->adresse, objet) & DISABLED ) ) {
    if ( G_STRING == OB_TYPE(menu->adresse, objet) ) {
      OB_STATE(menu->adresse, objet) |= DRAW3D;
    } else if ( OB_TYPE(menu->adresse, objet) != G_SEPARATEUR ) {
      OB_STATE(menu->adresse, objet) |= CHECKED;
    }
    OBJ_dessine(menu->options[ option ], menu->adresse + objet);
  }
  menu->option_active = option;

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Contrôle de la validité d'un menu -------- */

static int MENU_ControleMenu(Menu *menu, unsigned int num_menu)
{
  if (NULL == menu) return -1;
  if ( (NULL == menu->prem_option) || (NULL == menu->nbr_options) ) {
    (void) fprintf(stderr, "MENU_ControleMenu : menu sans table d'options !\n");
    return -2;
  }
  if ( (num_menu > menu->nbr_titres) || ( (num_menu == menu->nbr_titres) && (num_menu > 0) ) ) {
    (void) fprintf(stderr, "MENU_ControleMenu : numéro de menu incorrect [%u / %u max]\n",
		   num_menu + 1, menu->nbr_titres);
    return -3;
  }
  if ( NULL == menu->adresse ) {
    (void) fprintf(stderr, "MENU_ControleMenu : menu sans arbre associé !\n");
    return -4;
  }

  /* Rien à signaler */
  return 0;
}

/* -------- Désactivation de l'option actuellement active -------- */

static int MENU_DesactiveOption(Menu *menu, unsigned int num_menu)
{
  int retour;
  unsigned int objet;

  /* Contrôles préliminaires */
  retour = MENU_ControleMenu(menu, num_menu);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "MENU_DesactiveOption : erreur dans MENU_ControleMenu [code %d]\n", retour);
    return -1;
  }
  if ( menu->option_active >= menu->nbr_options[ num_menu ] ) {
    (void) fprintf(stderr, "MENU_DesactiveOption : numéro d'option incorrect [%u / %u max --- menu n°%u]\n",
		   menu->option_active + 1, menu->nbr_options[ num_menu ], num_menu);
    return -2;
  }

  /* On repère l'objet associé à l'option choisie */
  objet = menu->prem_option[ num_menu ] + menu->option_active;

  if ( G_STRING == OB_TYPE(menu->adresse, objet) ) {
    /* Cette option redevient une chaîne normale */
    OB_STATE(menu->adresse, objet) &= ~DRAW3D;
  } else {
    OB_STATE(menu->adresse, objet) &= ~CHECKED;
  }

  /* On la redessine donc */
  OBJ_dessine(menu->options[ menu->option_active ], menu->adresse + objet);

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Gestion du clavier dans un menu -------- */

static int MENU_GererClavier(Menu *menu, unsigned int num_menu, XKeyEvent *clavier, unsigned int *option)
{
  KeySym touche;
  char tampon[128];
  unsigned int nbr_options;

  nbr_options = menu->nbr_options[ num_menu ];

  BIBX_verifie_modificateurs(clavier->state);
  XLookupString(clavier, tampon, 128, &touche, (XComposeStatus *) NULL);

  if ( (touche >= XK_F1) && (touche <= XK_F35) ) { /* Touches de fonction */
    unsigned int nv_menu;

    nv_menu = (unsigned int) (touche - XK_F1); /* Numéro du menu à déroulé */
    if ( nv_menu < menu->nbr_titres ) {
      if ( VRAI == ( menu->adresse[ menu->prem_titre + nv_menu].ob_state & DISABLED ) )
	return MENU_IGNORER; /* Comme si on n'avait appuyé sur aucune touche */

      (void) MENU_DesactiveOption(menu, num_menu);
      if ( nv_menu == num_menu ) /* On referme le menu */
	return MENU_CHOIX_ANNULE;

      /* On change de menu */
      *option = nv_menu;
      return MENU_CHANGER_MENU;
    }
  }

  if (menu->nbr_titres > 1) {
    if ( touche == XK_Left ) { /* <- = on appelle le menu précédent */
      unsigned int nv_menu;

      nv_menu = num_menu;
      do {
	if (nv_menu > 0) { /* Il y a des titres avant */
	  nv_menu--;
	} else { /* On fait un truc cyclique : appel du dernier menu */
	  nv_menu = menu->nbr_titres - 1;
	}
      } while ( OB_STATE( menu->adresse, menu->prem_titre + nv_menu ) & DISABLED );

      if ( nv_menu != num_menu ) {
	(void) MENU_DesactiveOption(menu, num_menu);
	*option = nv_menu;
	return MENU_CHANGER_MENU;
      }
    }

    if ( touche == XK_Right ) { /* -> = on appelle le menu suivant */
      unsigned int nv_menu;

      nv_menu = num_menu;
      do {
	nv_menu++;
	if (nv_menu >= menu->nbr_titres)
	  nv_menu = 0;
      } while ( OB_STATE( menu->adresse, menu->prem_titre + nv_menu ) & DISABLED );

      if ( nv_menu != num_menu ) {
	(void) MENU_DesactiveOption(menu, num_menu);
	*option = nv_menu;
	return MENU_CHANGER_MENU;
      }
    }
  }

  if ( touche == XK_Down ) { /* v = on passe à l'option active suivante */
    unsigned int nv_option;

    /* On annule l'ancienne option active */
    (void) MENU_DesactiveOption(menu, num_menu);

    /* On cherche la nouvelle option active */
    nv_option = menu->option_active;
    do {
      nv_option++;
      if ( nv_option >= nbr_options )
	nv_option = 0;
    } while ( ( menu->adresse[ menu->prem_option[num_menu] + nv_option ].ob_state & DISABLED ) ||
	      ( menu->adresse[ menu->prem_option[num_menu] + nv_option ].ob_type < 0 ) );

    (void) MENU_ActiveOption(menu, num_menu, nv_option);
    return MENU_IGNORER;
  }

  if ( touche == XK_Up ) { /* ^ = option précédente dans le menu */
    unsigned int nv_option = menu->option_active;

    /* On annule l'ancienne option active */
    (void) MENU_DesactiveOption(menu, num_menu);

    do {
      if (nv_option > 0)
	nv_option--;
      else
	nv_option = nbr_options - 1;
    } while ( ( menu->adresse[ menu->prem_option[num_menu] + nv_option ].ob_state & DISABLED ) ||
	      ( menu->adresse[ menu->prem_option[num_menu] + nv_option ].ob_type < 0 ) );

    (void) MENU_ActiveOption(menu, num_menu, nv_option);
    return MENU_IGNORER;
  }

  if ( (touche == XK_space) || (touche == XK_Return) || (touche == XK_KP_Enter) ) {
    *option = menu->prem_option[ num_menu ] + menu->option_active;

    /* On annule l'ancienne option active */
    (void) MENU_DesactiveOption(menu, num_menu);
    return MENU_CHOIX_OPTION; /* On a sélectionné cette option */
  }

  return MENU_IGNORER; /* Rien de spécial */
}

/* -------- Gestion des options, une fois le menu ouvert -------- */

static int MENU_GererOptions(Menu *menu, unsigned int num_menu, unsigned int *option)
{
  unsigned int nbr_options, i;
  boolean sortie = FAUX;
  XEvent evenement;
  Fenetre *fenetre;

  if ( (NULL == menu) || (NULL == option) ) return -1;
  if ( None == menu->fond_option ) {
    (void) fprintf(stderr, "MENU_GererOptions : appel avec le menu fermé !\n");
    return -2;
  }
  if ( NULL == menu->options ) {
    (void) fprintf(stderr, "MENU_GererOptions : pas de table des fenêtres d'options !\n");
    return -3;
  }

  nbr_options = menu->nbr_options[ num_menu ];
  do {
    XNextEvent(BI_Display, &evenement);

    switch (evenement.type)
    {
    case KeyPress:
      {
	int retour;

	/* On s'assure que la fenêtre est visible */
	XRaiseWindow( BI_Display, menu->fond_option );

	/* Et on traite le clavier */
	retour = MENU_GererClavier(menu, num_menu, &(evenement.xkey), option);
	if ( retour != MENU_IGNORER ) {
	  if ( retour < 0 ) {
	    (void) fprintf(stderr, "MENU_GererOptions : erreur dans MENU_GererClavier [code %d]\n", retour);
	  }
	  return retour;
	}
      }
      break;

    case KeyRelease:
      break;

    case Expose:
      for (i = 0 ; i < nbr_options ; i++) {
	if (evenement.xexpose.window == menu->options[i]) {
	  if ( 0 == evenement.xexpose.count ) {
	    OBJ_dessine(evenement.xexpose.window, menu->adresse + menu->prem_option[ num_menu ] + i);
	  }
	  break;
	}
      }

      if ( ( i >= nbr_options ) /* Non traité */
	   && ( evenement.xexpose.window != menu->fond_option ) ) {
	BI_Fen_redessin = evenement.xexpose.window;
	EVNT_redessin( BIX_QuelleFenetre( BI_Fen_redessin ), (XExposeEvent *) &evenement );
      }
      break;

    case ButtonPress:
      /* On s'assure que la fenêtre est visible */
      XRaiseWindow( BI_Display, menu->fond_option );

      /* Dans tous les cas, on annule l'option active */
      (void) MENU_DesactiveOption(menu, num_menu);

      /* On cherche sur quelle option on a cliqué */
      for (i = 0 ; i < nbr_options ; i++) {
	if ( evenement.xbutton.window == menu->options[i] ) { /* Clic sur l'option numéro i (en _relatif_) */
	  int code_sortie;
	  int type_objet;

	  /* Type de l'objet cliqué */
	  type_objet = OB_TYPE(menu->adresse, menu->prem_option[ num_menu ] + i);

	  if ( OB_STATE(menu->adresse, menu->prem_option[ num_menu ] + i) & DISABLED ) { /* Option inactive */
	    code_sortie = MENU_CHOIX_ANNULE;
	  } else if ( ( type_objet > 0 ) || ( G_STYLELIGNE == type_objet ) || ( G_STYLEMARQUE == type_objet ) ) {
	    /* L'objet est autre chose qu'un séparateur ou un objet très interne (gadget de fenêtre) */
	    *option = menu->prem_option[ num_menu ] + i;
	    code_sortie = MENU_CHOIX_OPTION;
	  } else { /* Clic sur un séparateur : on annule */
	    code_sortie = MENU_CHOIX_ANNULE;
	  }

	  return code_sortie;
	}
      }

      /* Si on est là : clic sur autre chose que le menu déroulé
         1. On vérifie que le clic n'est pas sur un autre titre */
      for (i = 0 ; i < menu->nbr_titres ; i++) {
	if (evenement.xbutton.window == menu->titres[i]) {
	  if ( OB_STATE(menu->adresse, menu->prem_titre + i) & DISABLED ) {
	    sortie = FAUX; /* Menu désactivé = il ne se passe rien; c'est l'un des deux seuls cas
			      où le sous-menu déroulé n'est pas refermé */
	  } else if (i != num_menu) {
	    *option = i; /* Nouveau titre à appeler */
	    return MENU_CHANGER_MENU;
	  } else { /* On clique sur le même titre... */
	    sortie = FAUX; /* Autre cas où le sous-menu déroulé n'est pas refermé */
	    XRaiseWindow(BI_Display, menu->fond_option);
	    (void) MENU_ActiveOption(menu, num_menu, 0);
	  }

	  break;
	}
      }

      /* 2. Clic hors des titres et du menu */
      break;

    case MotionNotify: /* Déplacement du curseur */
      fenetre = BIX_QuelleFenetre( evenement.xmotion.window );
      EVNT_pointeur( fenetre, &(evenement.xmotion) );
      break;

    case EnterNotify:
      fenetre = BIX_QuelleFenetre( evenement.xcrossing.window );
      if (fenetre == NULL) { /* Fenêtre du menu, probablement */
	for (i = 0 ; i < nbr_options ; i++) {
	  if ( evenement.xexpose.window == menu->options[i] ) {
	    (void) MENU_DesactiveOption(menu, num_menu);
	    (void) MENU_ActiveOption(menu, num_menu, i);
	    break;
	  }
	}
      } else
	EVNT_entree(fenetre, &(evenement.xcrossing));
      break;

    case LeaveNotify:
      fenetre = BIX_QuelleFenetre(evenement.xcrossing.window);
      EVNT_sortie(fenetre, &(evenement.xcrossing));
      break;

    case ClientMessage:
      if (evenement.xclient.window == menu->fond_option) {
	if ( (Atom) (evenement.xclient.data.l[0]) == BI_ProtocoleFermerFenetre ) {
	  (void) MENU_DesactiveOption(menu, num_menu);
	  return MENU_CHOIX_ANNULE;
	} else {
	  (void) printf("MENU_GererOptions : événement utilisateur de code inconnu [code %ld]\n", evenement.xclient.data.l[0]);
	}
      }
      break;

#ifdef _DEVELOPPE_
    default:
      (void) printf("MENU_GererOptions : autre événement reçu, code %d\n", evenement.type);
#endif
    }
  } while (sortie == FAUX);

  (void) fprintf(stderr, "MENU_GererOptions : on ne devrait jamais arriver là !\n");
  return -10;
}

/* -------- Fermeture d'un menu : libération et tout et tout -------- */

static int MENU_LibererOptions(Menu *menu, unsigned int num_menu)
{
  int retour;

  retour = MENU_ControleMenu(menu, num_menu);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "MENU_LibererOptions : erreur dans MENU_ControleMenu [code %d]\n", retour);
    return -1;
  }
  if (NULL == menu->options) {
    (void) fprintf(stderr, "MENU_LibererOptions : aucun menu ouvert actuellement !\n");
    return -1;
  }

  /* On élimine d'éventuels événements associés */
  FEN_ote_evenement( menu->fond_option );

  /* On ferme toutes les fenêtres */
  XDestroyWindow(BI_Display, menu->fond_option);
  menu->fond_option = None;
  XFree(taille_fenetre); /* donc plus besoin de la contrainte de taille */
  taille_fenetre = NULL;

  /* Plus besoin de la table des options */
  free(menu->options);
  menu->options = NULL;

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Adaptation du menu --------------------\
 |   Renvoie 0 si tout va bien, <0 si erreur       |
 |                                                 |
 | Parcours l'arbre et convertit les délimiteurs   |
 | de façon à avoir des dessins plus jolis         |
 |                                                 |
 | Attention, les objets GR_SEPARATEUR ont une     |
 | chaîne spécifique : ne pas oublier de la libérer|
 | après utilisation !                             |
 \------------------------------------------------ */

static void MENU_PreparerMenu(OBJECT *adresse, unsigned int racine, boolean interne)
{
  OBJECT *objet;
  unsigned int premiere_option, nbr_options, i;

  if (-1 == OB_HEAD(adresse, racine)) {
    (void) fprintf(stderr, "MENU_PreparerMenu : menu sans aucune option [racine : %u] !\n", racine);
    return;
  }

  premiere_option = (unsigned int) OB_HEAD(adresse, racine);
  nbr_options = (unsigned int) OB_TAIL(adresse, racine) - premiere_option + 1;

#ifdef _DEVELOPPE_
/*   (void) printf("MENU_PreparerMenu : objet racine = %u, première option = %u, %u options\n", */
/* 		racine, premiere_option, nbr_options); */
#endif

  OB_HEIGHT(adresse, racine) += nbr_options;

  /* Suppose les objets numérotés à la suite, correctement, dans la boîte ! */
  for ( i = 0, objet = adresse + premiere_option ; i < nbr_options ; i++, objet++ ) {
    const char *chaine;

    objet->ob_y += i;

    if ( objet->ob_type != G_STRING ) {
#ifdef _DEVELOPPE_
      (void) printf("MENU_PreparerMenu : objet de type autre que G_STRING dans le menu [objet %u].\n",
		    premiere_option + i);
#endif
      continue;
    }

    chaine = objet->ob_spec.texte;
    if ( NULL == chaine ) {
#ifdef _DEVELOPPE_
      (void) printf("%s : objet avec une chaîne NULL [objet %u]\n", __FUNCTION__, premiere_option + i);
#endif
    } else if ( chaine[0] == '-' ) { /* Les séparateurs commencent pas le signe - par hypothèse */
      unsigned long i, lg_chaine;
      boolean simple = VRAI;
      const char *debut_texte = NULL;

      lg_chaine = (unsigned long) strlen( chaine );
      for ( i = 0 ; i < lg_chaine ; i++ ) {
	if ( chaine[ i ] != '-' ) { /* Il n'y a pas que des - */
	  simple = FAUX;
	  debut_texte = chaine + i;
	  break;
	}
      }

      objet->ob_type = G_SEPARATEUR;
      if ( VRAI == simple ) {
	if ( FAUX == interne ) {
	  /* On libère l'ancien texte, car arbre lu depuis un RSC -> texte réservé avec malloc() */
	  free( objet->ob_spec.txt_malloc );
	}
	objet->ob_spec.txt_malloc = NULL;
      } else {
	char *txt_separateur;

	txt_separateur = strdup( debut_texte );
	if ( NULL == txt_separateur ) {
	  /* Au moins, on garde l'ancien texte ! */
	  (void) fprintf(stderr, "MENU_PreparerMenu : mémoire insuffisante -- texte du séparateur [%s]\n", debut_texte);
	} else {
	  if ( FAUX == interne ) {
	    /* On libère l'ancien texte, car arbre lu depuis un RSC -> texte réservé avec malloc() */
	    free( objet->ob_spec.txt_malloc );
	  }

	  lg_chaine = strlen( txt_separateur );
	  while ( txt_separateur[ lg_chaine - 1 ] == '-' ) {
	    lg_chaine--;
	    txt_separateur[ lg_chaine ] = 0;
	  }

	  objet->ob_spec.txt_malloc = txt_separateur;
	}
      }
      objet->ob_state &= ~DISABLED; /* Pas de sens d'inactiver un séparateur... */ 
    }
  }
}

/* -------- Création de la fenêtre affichant les options -------- */

static int MENU_PreparerOptions(Menu *menu, unsigned int num_menu, int x, int y, const char *titre)
{
  XSetWindowAttributes parametres;
  int retour;
  unsigned int premiere_option, contenant, i;
  unsigned int larg, haut;

  retour = MENU_ControleMenu(menu, num_menu);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "MENU_PrepareOptions : erreur dans MENU_ControleMenu [code %d]\n", retour);
    return -1;
  }
  if ( NULL != menu->options ) {
    (void) fprintf(stderr, "MENU_PrepareOptions : il y a déjà un menu ouvert pour cette barre de menus !\n");
    return -2;
  }

  premiere_option = menu->prem_option[ num_menu ];
  contenant = premiere_option - 1;

#ifdef _DEVELOPPE_
  (void) printf("MENU_PrepareOptions : menu %u en (%d, %d) [%s] ; fond : %u ; option 1 : %u\n",
		num_menu, x, y, titre, contenant, premiere_option);
#endif

  /* Préparation de la table des fenêtres pour chaque option du menu */
  menu->options = (Window *) calloc(menu->nbr_options[ num_menu ], sizeof(Window));
  if (NULL == menu->options) {
    (void) fprintf(stderr, "MENU_PrepareOptions : mémoire insuffisante --- table des options du menu\n");
    return -5;
  }

  /* On calcule les dimensions que doit avoir la fenêtre de fond
     Suppose que les coordonnées aient été converties en coordonnées écran,
     avec corrections éventuelles pour éviter un recouvrement */
  larg = OB_WIDTH(menu->adresse, contenant);
  haut = OB_HEIGHT(menu->adresse, contenant);

  /* Contrôle : ne déborde-t-on pas ? */
  if ( ( y + haut ) > ( BI_haut_ecran - BI_haut_car ) ) {
    y  = BI_haut_ecran - haut;
    y -= 2*BI_haut_car; /* Un peu de marge pour tenir compte du titre ajouté par le WM ! */
  }

  /* Création de la fenêtre de fond, contenant chaque option */
  parametres.background_pixel  = BI_Couleurs[GRIS].pixel;
  parametres.border_pixel      = BI_Couleurs[NOIR].pixel;
  parametres.override_redirect = False /* True */;
  parametres.event_mask        = ButtonPressMask | ButtonReleaseMask | ExposureMask | KeyPressMask;

  menu->fond_option = XCreateWindow( BI_Display, BI_Fen_racine, x, y, larg, haut + 2,
				     2, CopyFromParent, InputOutput, BI_Visual,
				     CWBackPixel | CWEventMask | CWOverrideRedirect | CWBorderPixel,
				     &parametres );
  if ( None == menu->fond_option ) {
    (void) fprintf(stderr, "MENU_PrepareOptions : impossible de créer la fenêtre X\n");
    free( menu->options ); menu->options = NULL;
    return -5;
  }

  /* Décoration minimale par le WM */
  if ( menu->titres != NULL ) {
    XSetTransientForHint( BI_Display, menu->fond_option, menu->titres[ num_menu ] );
  }

  /* Protection contre une fermeture intempestive dans le WM */
  if ( XSetWMProtocols( BI_Display, menu->fond_option, &BI_ProtocoleFermerFenetre, 1 ) == 0 ) {
    (void) fprintf(stderr, "MENU_PrepareOptions : impossible de définir WM_DELETE_WINDOW pour le menu\n");
  }

  /* Taille imposée */
  taille_fenetre = XAllocSizeHints(); /* À libérer à la fin avec Xfree() */
  if ( taille_fenetre ) { /* A pu être créé */
    taille_fenetre->flags = USPosition | PPosition | PMinSize | PMaxSize;
    taille_fenetre->x = x;
    taille_fenetre->y = y;
    taille_fenetre->min_width  = larg;
    taille_fenetre->min_height = haut;
    taille_fenetre->max_width  = larg;
    taille_fenetre->max_height = haut;
    XSetWMNormalHints( BI_Display, menu->fond_option, taille_fenetre );
  }

  /* Le titre du menu */
  if ( NULL == titre )
    titre = "Options...";
  XStoreName( BI_Display, menu->fond_option, titre );

  /* Maintenant, on crée une fenêtre (ie un objet) pour chaque option du menu */
  parametres.event_mask |= EnterWindowMask | LeaveWindowMask;
  for (i = 0 ;
       i < menu->nbr_options[ num_menu ] ; i++) {
    menu->options[ i ] = XCreateWindow(BI_Display, menu->fond_option, 
				       OB_X(menu->adresse, premiere_option + i), /* en général, 0 */
				       OB_Y(menu->adresse, premiere_option + i),
				       (unsigned) OB_WIDTH(menu->adresse, premiere_option + i),
				       (unsigned) OB_HEIGHT(menu->adresse, premiere_option + i) + 1,
				       0, CopyFromParent, InputOutput, BI_Visual,
				       CWBackPixel | CWEventMask, &parametres);
  }

  /* Maintenant que tout est prêt, on peut ouvrir la fenêtre */
  XMapSubwindows(BI_Display, menu->fond_option);
  XMapWindow(BI_Display, menu->fond_option);

  /* Option active par défaut : la première */
  (void) MENU_ActiveOption(menu, num_menu, 0);

  /* Tout s'est bien passé */
  return 0;
}

/* ~~~~~~~~ Gestion d'un menu : déroulement et tout et tout ~~~~~~~~~~
   Renvoie un code BI_EVENEMENT_* en fonction de ce qu'il s'est passé
 */

int CLIC_menu(Menu *menu, unsigned int num_menu)
{
  int retour;
  int x_menu, y_menu;
  unsigned int option_choisie;
  Window titre_appelant;

  /* Contrôles préliminaires */
  if ( NULL == menu ) {
    AVERTISSEMENT( "Menu %u demandé, mais pas de menu !", num_menu );
    return BI_EVENEMENT_ERREUR;
  }
  if ( num_menu >= menu->nbr_titres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numéro de menu incorrect [%u / %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_menu + 1, menu->nbr_titres );
    return BI_EVENEMENT_ERREUR;
  }

  /* Infos sur la fenêtre de barre de menu - pour bien positionner le menu "déroulant"
     Gardées identiques tant que l'on n'a pas fini d'utiliser le menu
     Suppose donc qu'en général, l'utilisateur ne s'amuse pas à déplacer la barre de
     titres pendant son utilisation - sinon, ce n'est pas très beau, mais pas gênant */

  titre_appelant = menu->titres[ num_menu ]; /* Fenêtre du titre */
  x_menu = 0; y_menu = 0;
  while ( titre_appelant != BI_Fen_racine ) {
    int x, y;
    unsigned int larg_menu, haut_menu, _rien_;
    Window w, *fils;

    XGetGeometry( BI_Display, titre_appelant, &w, &x, &y, &larg_menu, &haut_menu,
		  &_rien_, &_rien_ );

    x_menu += x;
    y_menu += y;
    if ( titre_appelant == menu->titres[ num_menu ] )
      y_menu += haut_menu;

    XQueryTree( BI_Display, titre_appelant, &w, &titre_appelant, &fils, &_rien_ );
    XFree( fils );
  }

  if ( menu->fond_option != None ) { /* Déjà un menu ouvert */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : appel avec un menu déjà ouvert dans cette barre de menus !\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    retour = MENU_LibererOptions( menu, 0 );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : erreur dans MENU_LibererOptions()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return BI_EVENEMENT_ERREUR;
    }
  }

  /* On montre que l'on a choisi ce menu */
  OB_STATE( menu->adresse, menu->prem_titre + num_menu ) |= SELECTED;
  OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );

  /* On ouvre le menu déroulant associé */
  retour = MENU_PreparerOptions( menu, num_menu, x_menu, y_menu, menu->adresse[ menu->prem_titre + num_menu ].ob_spec.texte );
  if (retour != 0) {
    (void) fprintf(stderr, "%s,%s l.%u : erreur dans MENU_PreparerOptions [code %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour );
    OB_STATE( menu->adresse, menu->prem_titre + num_menu ) &= ~SELECTED;
    OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );
    return BI_EVENEMENT_ERREUR;
  }

  /* On gère le menu ainsi ouvert */
  do {
    retour = MENU_GererOptions(menu, num_menu, &option_choisie);
    if ( retour < 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : erreur dans MENU_GererOptions [code %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour );
      retour = MENU_LibererOptions( menu, num_menu );
      OB_STATE( menu->adresse, menu->prem_titre + num_menu ) &= ~SELECTED;
      OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : erreur dans MENU_LibererOptions\n",
		       __FILE__, __FUNCTION__, __LINE__);
      }
      return BI_EVENEMENT_ERREUR;
    }

    if ( MENU_CHOIX_ANNULE == retour ) { /* On annule le choix d'une option dans le menu */
#ifdef _DEVELOPPE_
      printf("%s : on annule l'appel du menu\n", __FUNCTION__ );
#endif
      (void) MENU_LibererOptions( menu, num_menu );
      OB_STATE( menu->adresse, menu->prem_titre + num_menu ) &= ~SELECTED;
      OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );
      return BI_EVENEMENT_TRAITE;
    }

    if ( MENU_CHANGER_MENU == retour ) { /* On change de menu */
      int x, y;
      unsigned int l, h, r;
      Window w;

      /* On cherche la position du nouveau menu */
      XGetGeometry( BI_Display, menu->titres[ num_menu ], &w, &x, &y, &l, &h, &r, &r );
      x_menu -= x;
      XGetGeometry( BI_Display, menu->titres[ option_choisie ], &w, &x, &y, &l, &h, &r, &r );
      x_menu += x;

      /* On libère l'ancien menu */
      (void) MENU_LibererOptions( menu, num_menu );
      OB_STATE( menu->adresse, menu->prem_titre + num_menu ) &= ~SELECTED;
      OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );

      /* On met le nouveau */
      num_menu = option_choisie;
      OB_STATE( menu->adresse, menu->prem_titre + num_menu ) |= SELECTED;
      OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );
      (void) MENU_PreparerOptions( menu, num_menu, x_menu, y_menu, menu->adresse[ menu->prem_titre + num_menu ].ob_spec.texte );
    }
  } while ( retour != MENU_CHOIX_OPTION );

  /* On a fini : on referme le menu */
#ifdef _DEVELOPPE_
  printf("%s : option choisie = %u\n", __FUNCTION__, option_choisie);
#endif
  retour = MENU_LibererOptions( menu, num_menu );
  OB_STATE( menu->adresse, menu->prem_titre + num_menu ) &= ~SELECTED;
  OBJ_dessine( menu->titres[ num_menu ], menu->adresse + menu->prem_titre + num_menu );
  if (retour != 0) {
    (void) fprintf(stderr, "%s,%s l.%u : erreur dans MENU_LibererOptions\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  if ( menu->gestion != ( int (*)(unsigned int) ) NULL ) { /* Il y a une routine de gestion du menu */
    return (menu->gestion)( option_choisie );
  } else { /* On s'envoie un message en interne */
    buffer[ 0 ] = MN_SELECTED; /* Message : menu appelé */
    buffer[ 1 ] = 0;
    buffer[ 2 ] = 0;
    buffer[ 3 ] = menu->prem_titre + num_menu;
    buffer[ 4 ] = (int) option_choisie;
    OPTION_A_FAIRE( "Gestion par messages GEM" );

    return BI_EVENEMENT_MESSAGE; /* Non traité : il faut lire le message ! */
  }

  return BI_EVENEMENT_TRAITE; /* Traité */
}

/* ~~~~~~~~~~ Gestion d'un menu : clavier ~~~~~~~~~~~~~~~~ */

int CLAV_menu(Menu *menu, XKeyEvent *clavier)
{
  Arbre *arbre_menu;
  KeySym touche;
  unsigned int touche_gem;

  BIBX_verifie_modificateurs(clavier->state);

  if ( NULL == menu ) {
    (void) fprintf(stderr, "%s,%s l.%u : pas de menu indiqué !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_NON_TRAITE; /* Pas de menu ! */
  }

#ifdef _DEVELOPPE_
/*   (void) printf("CLAV_menu : menu = %p ; clavier = %p\n", (void *) menu, (void *) clavier); */
#endif

  /* On récupère le code de la touche utilisée */

  touche_gem = XToucheVersGEM(clavier, TOUCHE_PRESSEE, &touche);
  if (touche_gem == TOUCHE_TRAITEE) {
    (void) fprintf(stderr, "%s,%s l.%u : touche déjà traitée [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, (unsigned int) touche);
    return BI_EVENEMENT_TRAITE;
  }

  arbre_menu = quel_arbre( menu->adresse ); /* Arbre associé au menu */
  if ( (arbre_menu != NULL) && (arbre_menu->clavier) ) {   /* Si fonction de clavier associée au menu */
    int retour;
    int renvoi; /* Bouton utilisé pour sortir */

    retour = (arbre_menu->clavier)(touche_gem, BI_etat, &renvoi);
    if ( retour != BI_EVENEMENT_NON_TRAITE ) /* A été traité */
      return retour;
  }

  if ( (touche >= XK_F1) && (touche <= XK_F35) ) { /* Touches de fonction */
    int retour = BI_EVENEMENT_TRAITE;
    unsigned int num_menu;

    num_menu = (unsigned int) (touche - XK_F1); /* Numéro du menu à déroulé */
    if (  ( num_menu < menu->nbr_titres ) && /* Ce menu existe bien */
	 !( menu->adresse[ menu->prem_titre + num_menu ].ob_state & DISABLED ) ) { /* et il est actif */
      retour = CLIC_menu(menu, num_menu);
    }

    /* Quoi qu'il arrive, les touches de fonction sont considérées gérées ! */
    return retour;
  }

  return BI_EVENEMENT_NON_TRAITE; /* N'a pas pu être traité comme un raccourci général */
}

/* -------- Traitement du clavier quand un menu est déroulé -------- */

/* ********** Sous-menu créé à partir d'un tableau de chaînes de caractère ********** */

/* ---------- Création du sous-menu à partir du tableau de textes --------------------\
 |    Renvoie le numéro du texte choisi dans le tableau (0 = premier,...)             |
 |            -1 si aucun n'a été choisi (erreur, sortie avec Esc,...)                |
 |                                                                                    |
 | textes  = les textes des différentes options utilisables                           |
 | nbr_txt = nombre de textes dans le tableau (= nombre d'options)                    |
 | titre   = le titre à afficher au-dessus du sous-menu                               |
 | _pere   = infos sur l'objet appelant, pour mieux positionner le sous-menu au début |
 |   (si <0 = aucun, donc on centre sur l'écran)                                      |
 \----------------------------------------------------------------------------------- */

int SS_menu_texte(const char *textes[], unsigned int nbr_txt, const char *titre,
		  int arbre_pere, unsigned int objet_pere)
{
  int retour;
  unsigned int max_textes;

  if ( (textes == NULL) || (nbr_txt == 0) ) return -1;
  if (titre == NULL)
    titre = "Sous-menu - clic pour sortir";

  /* Nombre maximal de textes visibles simultanément - fonction de la taille de l'écran */
  max_textes = BI_haut_ecran / (BI_haut_car + 1);
  max_textes -= 3; /* Place du titre et un peu de marge... pour les agrandissements après */

  if ( nbr_txt <= max_textes ) {
    retour = SSMenu_SansAscenseur(textes, nbr_txt, titre, arbre_pere, objet_pere);
  } else {
    retour = SSMenu_AvecAscenseur(textes, nbr_txt, max_textes, titre, arbre_pere, objet_pere);
  }

  if (retour >= 0) {
    /* À faire quand lg_max_txt sera passé en paramètre
       if ( (arbre_pere >= 0) && (objet_pere >= ROOT) )
       {
       met_chaine(arbre_pere, objet_pere, lg_max_txt, textes[obj_choisi], TXT_COPIER);
       }
    */

    return retour;
  } else
    return -1;
}

static int SSMenu_AvecAscenseur(const char *textes[], unsigned int nbr_txt, unsigned int nbr_visibles, const char *titre,
				int arbre_pere, unsigned int objet_pere)
{
  Fenetre *fenetre, *fen_evenement = NULL;
  XEvent evenement;
  Window vx_fenetre = None, fenetre_mere = None;
  boolean sortie = FAUX;
  int x_menu = 0, y_menu = 0;
  int retour;
  unsigned int i;
  unsigned int largeur = 0, larg_max;
  unsigned int nbr_objets, premier_texte __attribute__((__unused__)), dernier_texte __attribute__((__unused__));

  int obj_choisi = -1;
  unsigned int prem_affiche = 0;

  nbr_objets = nbr_visibles + 5; /* textes + l'ascenseur (4 objets) + la racine */
  premier_texte = 5;
  dernier_texte = nbr_objets;

  /* On essaye de trouver la largeur optimale du sous-menu */
  if ( arbre_pere >= 0 ) { /* L'objet parent existe */
    Fenetre *tmp;

#ifdef _DEVELOPPE_
    (void) printf("SSMenu_AvecAscenseur : objet parent = arbre %d, objet %u\n", arbre_pere, objet_pere);
#endif
    largeur = (unsigned int) OB_WIDTH( arbres[arbre_pere].adresse, objet_pere );
    largeur /= BI_larg_car; /* Car arbre déjà converti en coordonnées écran ! */

    tmp = ARB_QuelleFenetre( arbres + arbre_pere, FAUX );
    if ( tmp != NULL ) {
      fenetre_mere = tmp->handle;
    }
  }

  larg_max = strlen( textes[0] );
  for (i = 1 ; i < nbr_txt ; i++)
    if ( strlen( textes[i] ) > larg_max )
      larg_max = strlen( textes[i] );
  if (larg_max > 50)
    larg_max = 50; /* Largeur maximale du sous-menu */

  if ( largeur < larg_max )
    largeur = larg_max + 2;

  /* On crée l'arbre contenant le menu */
  retour = BI_CreerArbre( BI_ArbreSousMenu, nbr_objets, largeur, nbr_visibles);
  if (retour != 0) {
    (void) fprintf(stderr, "SSMenu_AvecAscenseur : erreur dans BI_creer_arbre()\n");
    return -4;
  }
  /* Arbre interne, textes statiques ! */
  arbres[ BI_ArbreSousMenu ].drapeaux.interne = VRAI;

  /* On crée l'ascenseur & on l'initialise : on commence en haut */
  ARB_CreerAscenseur( BI_ArbreSousMenu, 1, (int) (largeur - 2), 0, ASC_VERTICAL, nbr_visibles );

  /* On crée les textes */
  for ( i = 0 ; i < nbr_visibles ; i++ ) {
    ARB_CreerTexte( BI_ArbreSousMenu, 5 + i, 0, (int) i, largeur - 2, textes[ i ] );
  }

  /* On repère où ouvrir le menu */
  SSMenu_Placer( arbre_pere, objet_pere, nbr_txt, &x_menu, &y_menu );

  /* On adapte l'arbre à l'écran */
  ARB_Initialiser(BI_ArbreSousMenu, NULL, ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
  ARB_EnjoliverDefileur( BI_ArbreSousMenu, ROOT, 5, 5 + nbr_visibles - 1,
			 1, 2, 3, 4, ARB_ASCENSEUR_DROITE );

  ARB_ascenseur(BI_ArbreSousMenu, 1, 2, 3, 4, ASC_VERTICAL, nbr_txt, nbr_visibles, 0, ROOT, 0, 0, METTRE);

  /* On crée la fenêtre pour ce menu */
  fenetre = FEN_OuvrirArbre( NULL, NAME|MOVER, BI_ArbreSousMenu, titre, x_menu, y_menu );
  if ( NULL == fenetre ) {
    (void) fprintf( stderr, "SSMenu_AvecAscenseur : erreur dans creer_gem_fenetre\n" );
    BI_LibererArbre( BI_ArbreSousMenu );
    return -1;
  }
  XSetTransientForHint( BI_Display, fenetre->handle, fenetre_mere );

  /* On gère les événements pour cette fenêtre */
  while (sortie == FAUX) {
    XNextEvent(BI_Display, &evenement);

    BI_Fen_redessin = evenement.xany.window;
    if (BI_Fen_redessin != vx_fenetre) {
      vx_fenetre = BI_Fen_redessin;
      fen_evenement = BIX_QuelleFenetre( BI_Fen_redessin );
    }

    switch ( evenement.type )
    {
    case KeyRelease: /* On ignore */
      break;

    case Expose:
      EVNT_redessin(fen_evenement, &(evenement.xexpose));
      break;

    case ButtonPress:
      if ( fen_evenement == fenetre ) {
	unsigned int objet_clic = ROOT;

	if (evenement.xbutton.button == Button4)
	  objet_clic = 1;
	else if (evenement.xbutton.button == Button5)
	  objet_clic = 4;
	else {
	  objet_clic = ARB_QuelObjet( BI_ArbreSousMenu, evenement.xbutton.window);
	}

	if ( evenement.xbutton.window == fenetre->handle ) { /* Clic sur le fond */
#ifdef _DEVELOPPE_
	  (void) printf("SSMenu_AvecAscenseur : clic sur le fond, sans effet\n");
#endif
	}

	if ( (objet_clic >= 1) && (objet_clic <= 4) ) {	/* Clic sur l'ascenseur */
	  prem_affiche = ARB_ascenseur( BI_ArbreSousMenu, 1, 2, 3, 4, ASC_VERTICAL, nbr_txt, nbr_visibles, prem_affiche,
					objet_clic, evenement.xbutton.x, evenement.xbutton.y, ECHANGE);
	  for (i = 0 ; i < nbr_visibles  ; i++) {
	    met_chaine( BI_ArbreSousMenu, 5 + i, 0, textes[ prem_affiche + i ], TXT_REMPLACE);
/* 	    OBJ_dessine(arbres[BI_ArbreSousMenu].objets[i], sous_menu + i); */
	  }
	} else if ( objet_clic != ROOT ) {
	  sortie = VRAI;
	  obj_choisi = prem_affiche + (objet_clic - 5);
	}
      } else
	FEN_premierplan( fenetre ); /* Repasse au premier plan */
      break;

    case ConfigureNotify:
      if ( (fen_evenement) && (fen_evenement != fenetre) ) {
	int delta_x, delta_y;

	delta_x = evenement.xconfigure.width - fen_evenement->larg;
	delta_y = evenement.xconfigure.height - fen_evenement->haut;
	if ( (delta_x == 0) && (delta_y == 0) )	{
	  break;
	}

	/* Si fenêtre demande un message spécial en cas de changement de taille, on le fait */
	if ( fen_evenement->msg_masque.taille == VRAI ) {
	  Message message;

	  message.type = WM_SIZED;
	  message.fenetre.application = 0;
	  message.fenetre.en_plus = 0;
	  message.fenetre.handle = 0; /*quel_handle(fen_evenement);*/
	  message.fenetre.x = fen_evenement->x;
	  message.fenetre.y = fen_evenement->y;
	  message.fenetre.larg = evenement.xconfigure.width;
	  message.fenetre.haut = evenement.xconfigure.height;
	  (fen_evenement->message)(fen_evenement, &message);
	}

	/* On effectue le changement de taille */	  
	change_taille_fenetre(fen_evenement, evenement.xconfigure.width, evenement.xconfigure.height);
      }
      break;

    case MotionNotify: /* Déplacement du curseur */
      EVNT_pointeur(fen_evenement, &(evenement.xmotion) );
      break;

    case EnterNotify:
      EVNT_entree(fen_evenement, &(evenement.xcrossing) );
      break;

    case LeaveNotify:
      EVNT_sortie(fen_evenement, &(evenement.xcrossing) );
      break;
    }
  }

  FEN_Detruire( fenetre );
  BI_LibererArbre( BI_ArbreSousMenu );

  return obj_choisi;
}

static int SSMenu_SansAscenseur(const char *textes[], unsigned int nbr_txt, const char *titre,
				int arbre_pere, unsigned int objet_pere)
{
  Menu *sous_menu;
  unsigned int largeur = 0, larg_max, prem_option = 1, option_choisie = 0;
  unsigned int i;
  int x_menu = 0, y_menu = 0;
  int retour;

  /* Contrôle préliminaire : pas d'appel récursif */
  if ( arbres[ BI_ArbreSousMenu ].adresse != NULL ) {
    (void) fprintf(stderr, "SSMenu_SansAscenseur : il y a déjà un sous-menu ouvert !\n");
    return -2;
  }

  /* On essaye de trouver la largeur optimale du sous-menu */
  if ( objet_pere != ROOT ) { /* L'objet parent existe */
#ifdef _DEVELOPPE_
    (void) printf("SSMenu_SansAscenseur : objet parent = arbre %d, objet %u\n", arbre_pere, objet_pere);
#endif
    largeur = (unsigned int) OB_WIDTH( arbres[ arbre_pere ].adresse, objet_pere );
    largeur /= BI_larg_car; /* Car arbre déjà converti en coordonnées écran ! */
  }

  larg_max = strlen( textes[0] );
  for (i = 1 ; i < nbr_txt ; i++) {
    if ( NULL == textes[ i ] ) {
#ifdef _DEVELOPPE_
      (void) printf("%s : texte %u NULL\n", __FUNCTION__, i );
#endif
    } else {
      if ( strlen( textes[i] ) > larg_max )
	larg_max = strlen( textes[i] );
    }
  }
  if (larg_max > 50)
    larg_max = 50; /* Largeur maximale du sous-menu */

  if ( largeur < larg_max )
    largeur = larg_max;

#ifdef _DEVELOPPE_
  (void) printf("SSMenu_SansAscenseur : largeur du menu = %u (max = %u)\n", largeur, larg_max);
#endif

  if (0 == largeur) {
    (void) fprintf(stderr, "SSMenu_SansAscenseur : pas d'objet père, pas de texte, largeur nulle !\n");
    return -1;
  }

  /* On crée l'arbre contenant le menu */
  retour = BI_CreerArbre( BI_ArbreSousMenu, nbr_txt + 1, largeur, nbr_txt);
  if (retour != 0) {
    (void) fprintf(stderr, "SSMenu_SansAscenseur : erreur dans BI_creer_arbre()\n");
    return -4;
  }
  /* Arbre interne, textes statiques ! */
  arbres[ BI_ArbreSousMenu ].drapeaux.interne = VRAI;

  /* On crée les textes */
  for (i = 1 ; i <= nbr_txt ; i++) {
    if ( NULL == textes[ i - 1 ] ) {
      ARB_CreerTexte(BI_ArbreSousMenu, i, 0, (int) (i - 1), largeur, NULL );
      OBJ_active( BI_ArbreSousMenu, i, FORCE_NON );
    } else if ( '-' == *textes[ i - 1 ] ) {
      ARB_CreerTexte(BI_ArbreSousMenu, i, 0, (int) (i - 1), largeur, textes[ i - 1] + 1);
      OBJ_active( BI_ArbreSousMenu, i, FORCE_NON );
    } else {
      ARB_CreerTexte(BI_ArbreSousMenu, i, 0, (int) (i - 1), largeur, textes[ i - 1]);
    }
  }

  /* On crée la structure de menu correcte */
  sous_menu = MENU_ReserverMenu( BI_ArbreSousMenu, FAUX );
  if ( NULL == sous_menu ) {
    (void) fprintf(stderr, "SSMenu_SansAscenseur : erreur dans MENU_ReserverMenu()\n");
    BI_LibererArbre( BI_ArbreSousMenu );
    return -5;
  }
  sous_menu->nbr_titres = 0;
  sous_menu->prem_option = &prem_option;
  sous_menu->nbr_options = &nbr_txt;
  sous_menu->titre = titre;

  /* On repère où ouvrir le menu */
  SSMenu_Placer(arbre_pere, objet_pere, nbr_txt, &x_menu, &y_menu);

  /* On adapte les tailles */
  ARB_Initialiser(BI_ArbreSousMenu, NULL, ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
  MENU_PreparerMenu( arbres[ BI_ArbreSousMenu ].adresse , ROOT, VRAI );

  /* On appelle ce menu */
  retour = MENU_PreparerOptions( sous_menu, 0, x_menu, y_menu, titre );
  if (retour != 0) {
    (void) fprintf(stderr, "SSMenu_SansAscenseur : erreur dans MENU_PreparerOptions [code %u]\n", retour);
    free(sous_menu);
    BI_LibererArbre( BI_ArbreSousMenu );
    return -6;
  }

  retour = MENU_GererOptions(sous_menu, 0, &option_choisie);
  /* On renvoie l'option choisie */
  if ( MENU_CHOIX_OPTION == retour ) {
    if ( '-'== *(textes[ option_choisie - 1]) ) 
      retour = -10; /* Option inactive, ou séparateur */
    else
      retour = (int) (option_choisie - 1);
  } else {
    retour = -20; /* Annulation ou problème */
  }

  /* On a fini */
  (void) MENU_LibererOptions(sous_menu, 0);
  free( sous_menu ); sous_menu = NULL;
  BI_LibererArbre( BI_ArbreSousMenu );
  return retour;
}

static int SSMenu_Placer(int arbre_pere, unsigned int objet_pere, unsigned int nbr_textes, int *x, int *y)
{
  Window appelante;
  int x_menu = 0, y_menu = 0;

  if ( (arbre_pere < 0) || (objet_pere == ROOT) ||
       ( FAUX == arbres[arbre_pere].drapeaux.ouvert ) ) {
    int rien;
    unsigned int masque;

    (void) XQueryPointer(BI_Display, BI_Fen_racine, &appelante, &appelante, x, y, &rien, &rien, &masque);
    return 0;
  }

  /* Recherche des coordonnées adéquates */
#ifdef _DEVELOPPE_
  if ( objet_pere >= arbres[arbre_pere].nbr_objets ) {
    (void) fprintf(stderr, "SSMenu_Placer : objet père [%d] pas dans l'arbre père [%d - %d objets]\n",
		   objet_pere, arbre_pere, arbres[arbre_pere].nbr_objets);
  }
#endif

  appelante = arbres[arbre_pere].objets[objet_pere]; /* Fenêtre de l'appelant */
  while (appelante != BI_Fen_racine) {
    int dx, dy;
    unsigned int larg_menu, haut_menu, _rien_;
    Window w, *fils;

#ifdef _DEVELOPPE_
    if (appelante == None) {
      (void) fprintf(stderr, "SSMenu_SansAscenseur : fenêtre None en recherchant l'appelant [racine : %lu]\n", 
		     (unsigned long) BI_Fen_racine);
      XSynchronize(BI_Display, True); /* On force la synchro pour vérifier */
    }
#endif

    XGetGeometry(BI_Display, appelante, &w, &dx, &dy, &larg_menu, &haut_menu, &_rien_, &_rien_);

    x_menu += dx;
    y_menu += dy;
    
    XQueryTree(BI_Display, appelante, &w, &appelante, &fils, &_rien_);
    XFree(fils);
  }

  /* Là, s'ouvre au bon endroit, mais peut déborder vers le bas */
  if ( ( y_menu + (BI_haut_car + 1) * nbr_textes ) > BI_haut_ecran ) {
    y_menu = BI_haut_ecran - (BI_haut_car + 1) * nbr_textes;
  }
  *x = x_menu;
  *y = y_menu;
  return 0;
}

/* --------------- Gestion d'un sous-menu ---------------------------------\
 |  Renvoie l'objet du sous-menu sur lequel on a cliqué (et quitte donc)   |
 |          -1 si on quitte sans changer (Esc ou bouton de fermeture)      |
 |  cas simple où il n'y a pas d'ascenseur à gérer                         |
 |                                                                         |
 | arbre      : arbre servant de sous-menu                                 |
 | arbre_pere : arbre contenant l'objet père - <0 si aucun                 |
 | objet_pere : objet père - donne la largeur et la position initiale      |
 | delta_y    : décalage vertical éventuel (en unité "objets")             |
 \------------------------------------------------------------------------ */

int GER_ssmenu(unsigned int arbre, unsigned int arbre_pere, unsigned int objet_pere, int delta_y)
{
  Menu *sous_menu;
  int x_menu = 0, y_menu = 0;
  int retour;
  unsigned int premiere_option = 1, nombre_options, option_choisie;

  /* Contrôle préliminaire */
  if ( arbre > BI_nbr_arbres ) {
    (void) fprintf(stderr, "GER_ssmenu : numéro d'arbre incorrect [%u / %u max.]\n", arbre, BI_nbr_arbres);
    return -1;
  }

  /* On initialise l'arbre qui sert de sous-menu */
  retour = ARB_Initialiser( arbre, NULL,  ( boolean (*)(unsigned int, int, int) ) NULL, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Initialisation de l'arbre a échoué [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    return -42;
  }

  nombre_options = arbres[ arbre ].nbr_objets - 1;

  /* On crée la structure de menu correcte */
  sous_menu = MENU_ReserverMenu( arbre, FAUX );
  if ( NULL == sous_menu ) {
    (void) fprintf(stderr, "GER_ssmenu : erreur dans MENU_ReserverMenu()\n");
    return -5;
  }
  sous_menu->nbr_titres = 0;
  sous_menu->prem_option = &premiere_option;
  sous_menu->nbr_options = &nombre_options;
  sous_menu->titre = "Votre choix...";

  /* On repère où ouvrir le menu */
  SSMenu_Placer((int) arbre_pere, objet_pere, nombre_options, &x_menu, &y_menu);
  y_menu -= (delta_y * BI_haut_car);
  if (y_menu < 0)
    y_menu = 0;

  /* On adapte les tailles */
  if ( FAUX == arbres[ arbre ].drapeaux.prepare ) {
    MENU_PreparerMenu( arbres[ arbre ].adresse , ROOT, arbres[ arbre ].drapeaux.interne );
    arbres[ arbre ].drapeaux.prepare = VRAI;
  }

  if (objet_pere > ROOT) { /* Il y a un objet père */
    unsigned int objet, largeur;

    OBJ_taille(arbre_pere, objet_pere, -1, -1, &largeur, NULL);
    for (objet = ROOT; objet < arbres[arbre].nbr_objets ; objet++)
      OBJ_taille(arbre, objet, largeur, -1, NULL, NULL);
    /* Suppose implicitement que le menu est ultra-classique... */
  }

  /* On appelle ce menu */
  retour = MENU_PreparerOptions(sous_menu, 0, x_menu, y_menu, "Votre choix..." );
  if (retour != 0) {
    (void) fprintf(stderr, "GER_ssmenu : erreur dans MENU_PreparerOptions [code %u]\n", retour);
    free(sous_menu);
    return -6;
  }

  retour = MENU_GererOptions(sous_menu, 0, &option_choisie);

  /* On a fini */
  (void) MENU_LibererOptions(sous_menu, 0);
  free( sous_menu ); sous_menu = NULL;

  /* On renvoie l'option choisie */
  if ( MENU_CHOIX_OPTION == retour ) {
    return (int) option_choisie;
  }

  /* On a annulé... */
  return -20;
}
