/* Bibliothque X-Window pour les interfaces
            Routines de gestion des objets GEM [~rcriture de l'AES...]
	    Fichier  inclure
   (C) 1998-2003, Emmanuel CURIS
*/

#ifndef X_OBJETS_H_

#define X_OBJETS_H_ /* Protection contre l'inclusion multiple */ 

/* -------- Infos supplmentaires : objets de type texte */

typedef struct texte_infini
{
  char *texte;         /* Le texte que l'on doit afficher               */
  size_t max_cars;     /* Nombre maximal de caractres autoris (0 final non compt) */
  size_t prem_car;     /* Premier caractre affich du texte en mmoire */
  size_t max_visibles; /* Nombre maximal de caractres affichables      */
} Texte_Infini;

struct infos_editable {
  struct texte_infini texte;	/* Le texte affich & modifi */
  unsigned int proprietes;	/* Quels critres pour l'entre des valeurs (dfinies dans Bib_Commun/demande.h) */
  double min, max;		/* Valeurs extrmes, si besoin est */
  char *unite;			/* Texte pour l'unit de mesure */
};

struct infos_ssmenu {
  unsigned long nombre_options;
  unsigned long option_choisie;
  const char *texte;		/* Si non-NULL, on l'affiche ; sinon, on affiche celui de l'option choisie */
  const char *titre;		/* Le titre  donner au sous-menu qui apparat [NULL : on utilise texte] */
  const char **options;
  void (*gestion)(unsigned int numero_arbre, unsigned int numero_objet, unsigned long option_choisie);
};

#define IBM              3 /* font types */
#define SMALL            5

#define TE_LEFT          0 /* editable text justification */
#define TE_RIGHT         1
#define TE_CNTR          2

typedef struct text_edinfo
{
  union {
    char *editable;       /* Texte modifiable, pour un objet ditable (FTEXT) */
    const char *constant; /* Texte affich pour un objet non ditable (TEXT) */
    unsigned long numero_chaine; /* Pour le chargement depuis le RSC */
  } te_ptext;
  char *te_ptmplt;  /* Masque, non modifiable        */
  char *te_pvalid;  /* Caractres autoriss : constant */
  int  te_font;           /* font                          */
  int  te_junk1;          /* junk word                     */
  int  te_just;           /* justification- left, right... */
  unsigned int te_color;  /* color information word        */
  int  te_junk2;          /* junk word                     */
  int  te_thickness;      /* border thickness              */
  unsigned int te_txtlen; /* Taille de te_ptext et te_pvalid [0 final INCLUS] */
  unsigned int te_tmplen; /* Taille de te_ptmplt [0 final INCLUS] */
} TEDINFO;

/* ------ Les trois structures d'icne sont l pour compatibilit.
   En pratique, on utilise les XPM pour le stockage rel des icnes
   donc un char *[] qui dcrit tout.
   Ces structures ne devraient donc servir que pour l'initialisation des arbres
   lors du chargement depuis un ressource [voir pour les arbres statiques  icnes...]
*/

typedef struct icon_block {
  char *ib_pmask;       /* ptr to mask of icon                */
  char *ib_pdata;       /* ptr to data of icon                */
  char *ib_ptext;       /* ptr to text of icon                */
  unsigned int ib_char; /* character in icon                  */
  int   ib_xchar;       /* x-coordinate of ib_char            */
  int   ib_ychar;       /* y-coordinate of ib_char            */
  int   ib_xicon;       /* x-coordinate of icon               */
  int   ib_yicon;       /* y-coordinate of icon               */
  int   ib_wicon;       /* width of icon in pixels            */
  int   ib_hicon;       /* height of icon in pixels           */
  int   ib_xtext;       /* x-coordinate of the icon's text    */
  int   ib_ytext;       /* y-coordinate of the icon's text    */
  int   ib_wtext;       /* width of rectangle for icon's text */
  int   ib_htext;       /* height of icon's text in pixels    */
} ICONBLK;

typedef struct cicon_data {
  int num_planes; /* number of planes in the following data          */
  int *col_data;
  int *col_mask;  /* pointer to single plane mask of col_data        */
  int *sel_data;  /* pointer to color bitmap of selected icon        */
  int *sel_mask;  /* pointer to single plane mask of selected icon   */
  struct cicon_data *next_res;	/* pointer to next icon for a different resolution */
} CICON;

typedef struct cicon_blk {
  ICONBLK monoblk; /* default monochrome icon                         */
  CICON *mainlist; /* list of color icons for different resolutions */
} CICONBLK;

/* ------ Cette structure est l pour compatibilit uniquement ------ */

typedef struct bit_block {
  int *bi_pdata;   /* ptr to bit forms data  */
  int  bi_wb;      /* width of form in bytes */
  int  bi_hl;      /* height in scan lines   */
  int  bi_x;       /* source x in bit form   */
  int  bi_y;       /* source y in bit form   */
  int  bi_color;   /* fg color of blt        */
} BITBLK;

struct parm_blk; /* La dfinition suit */

typedef struct user_blk {
  int (*ub_code) (struct parm_blk *pb); /* pointer to drawing function */
  long ub_parm;              /* parm for drawing function   */
} USERBLK;

typedef struct gbox_blk {
  unsigned couleur_fond  : 4;
  unsigned motif         : 3;
  unsigned opaque        : 1; /* VRAI si texte opaque */
  unsigned couleur_texte : 4;
  unsigned couleur_cadre : 4;
  signed cadre           : 8; /* paisseur du cadre */
  unsigned caractere     : 8; /* Pour BoxChar uniquement */
  /* -- Total : 1 long mot (32 bits), comme sur atari */
} INFOS_GBOX;

typedef struct
{
  int ob_next;  /* Numro de l'objet suivant [-1 pour ROOT ; n du pre si dernier fils] */
  int ob_head;  /* Premier objet fils [-1 : aucun fils] */
  int ob_tail;  /* Dernier objet fils [-1 : aucun fils] */
  int ob_type;  /* Nature de l'objet         */
  int ob_flags; /* Indicateurs divers        */
  int ob_state; /* tat de l'objet           */
  union {
    const char   *texte;      /* Pour Button, String, Title
				 En const car pas de raison de modifier la chane, seulement la remplacer
				 a rellement un sens... D'ailleurs, la modifier est une porte ouverte
				  avoir des seg. fault. avec GCC/linux... Sauf si la chane a t rserve
				 avec malloc(), ce qui n'est pas forcment le cas !
			      */
    TEDINFO      *tedinfo;    /* Pour Text, Boxtext, Ftext et FBoxtext */
    USERBLK      *util;       /* Pour Userdef                          */
    ICONBLK      *icone;      /* Pour Icon                             */
    CICONBLK     *icone_coul; /* Pour les icnes couleur, Cicon        */
    BITBLK       *image;      /* Pour les images noir et blanc         */
    Texte_Infini *txt_infini; /* Pour les textes de longueur infinie   */
    struct infos_editable *nombre; /* Pour G_NOMBRE : ditable contenant un nombre */
    struct infos_ssmenu *menu;	/* Pour G_SSMENU : bouton appelant un sous-menu */
    INFOS_GBOX    gbox;       /* Pour les botes de toute sorte */
    char         *txt_malloc; /* Comme texte, mais pour rservations avec malloc/free sans problme */
    char        **icone_xpm;  /* Stockage rel d'un icne, au format XPM */
    unsigned int style_ligne;	/* Pour G_STYLELIGNE */
    unsigned int style_marque;	/* Pour G_STYLEMARQUE */
    long          autre;
  } ob_spec; /* Caractristiques diverses */
  int ob_x;     /* Position de l'objet RELATIVE AU PRE */
  int ob_y;
  unsigned int ob_width; /* Dimensions de l'objet */
  unsigned int ob_height;
} OBJECT;

#define OB_NEXT(arbre, id)     arbre [id].ob_next
#define OB_HEAD(arbre, id)     arbre [id].ob_head
#define OB_TAIL(arbre, id)     arbre [id].ob_tail
#define OB_TYPE(arbre, id)     arbre [id].ob_type
#define OB_EXTYPE(arbre, id)  (arbre [id].ob_type >> 8)
#define OB_FLAGS(arbre, id)    arbre [id].ob_flags
#define OB_STATE(arbre, id)    arbre [id].ob_state
#define OB_SPEC(arbre, id)     arbre [id].ob_spec.autre /* Par compatibilit... */
#define OB_X(arbre, id)        arbre [id].ob_x
#define OB_Y(arbre, id)        arbre [id].ob_y
#define OB_WIDTH(arbre, id)    arbre [id].ob_width
#define OB_HEIGHT(arbre, id)   arbre [id].ob_height

typedef struct parm_blk
{
  OBJECT *pb_tree;                    /* ptr to obj tree for user defined obj */
  int     pb_obj;                     /* index of user defined object         */
  int     pb_prevstate;               /* old state to be changed              */
  int     pb_currstate;               /* changed (new) state of object        */
  int     pb_x, pb_y, pb_w, pb_h;     /* location of object on screen         */
  int     pb_xc, pb_yc, pb_wc, pb_hc; /* current clipping rectangle on screen */
  long    pb_parm;                    /* same as ub_parm in USERBLK struct    */
} PARMBLK;

/* -------- Les diffrents types d'objets possibles -------- */

#define ROOT 0
#define NIL             -1 /* nil object index */
#define MAX_LEN         81 /* max string length */
#define MAX_DEPTH        8 /* max depth of search or draw for objects */

#define G_CLOSER        -1 /* Objets internes : gadgets de fentre */
#define G_INFOS         -2
#define G_FULLER        -3
#define G_FLECHE        -4
#define G_ASCENSEUR     -5

#define G_SEPARATEUR   -20 /* Objets internes : extensions aux menus */
#define G_STYLELIGNE   -21 /* Ligne avec le style indiqu */
#define G_STYLEMARQUE  -22 /* Ligne avec les marques indiques */

#define G_TXT_INFINI   -30 /* Texte ditable de longueur infinie */
#define G_NOMBRE       -31 /* Champ ditable pour un nombre */

#define G_SSMENU       -40 /* Bouton appelant un sous-menu */
#define G_CHOIXFONTE   -41 /* Bouton permettant d'appeler un slecteur de fonte */

#define G_BOX           20 /* graphic types of obs */
#define G_TEXT          21
#define G_BOXTEXT       22
#define G_IMAGE         23
#define G_USERDEF       24
#define G_PROGDEF       24 /* for compatibility */
#define G_IBOX          25
#define G_BUTTON        26
#define G_BOXCHAR       27
#define G_STRING        28
#define G_FTEXT         29
#define G_FBOXTEXT      30
#define G_ICON          31
#define G_TITLE         32
#define G_CICON         33

/* -------- Attributs possibles des objets -------- */

#define NONE        0x0000 /* Object flags */
#define SELECTABLE  0x0001
#define DEFAULT     0x0002
#define EXIT        0x0004
#define EDITABLE    0x0008
#define RBUTTON     0x0010
#define LASTOB      0x0020
#define TOUCHEXIT   0x0040
#define HIDETREE    0x0080
#define INDIRECT    0x0100

/* -------- tats possibles des objets ---------- */

#define NORMAL      0x0000 /* Object states */
#define SELECTED    0x0001
#define CROSSED     0x0002
#define CHECKED     0x0004
#define DISABLED    0x0008
#define OUTLINED    0x0010
#define SHADOWED    0x0020
#define WHITEBAK    0x0040
#define DRAW3D      0x0080

/* -------- Sous-types pour les objets internes complexes -------- */

/* Types pour G_FLECHE */
#define BI_OBJ_FLECHE_HAUTE  0
#define BI_OBJ_FLECHE_BASSE  1
#define BI_OBJ_FLECHE_DROITE 2
#define BI_OBJ_FLECHE_GAUCHE 3
#define BI_OBJ_FLECHE_HB     4
#define BI_OBJ_FLECHE_GD     5
#define BI_OBJ_FLECHE_HG     6
#define BI_OBJ_FLECHE_HD     7
#define BI_OBJ_FLECHE_BG     8
#define BI_OBJ_FLECHE_BD     9
#define BI_OBJ_FLECHE_HG_BD 10
#define BI_OBJ_FLECHE_HD_BG 11

/* -------- Infos supplmentaires sur les icnes -------- */

/* --- Paramtres de objc_edit() --- */

#define ED_INIT        1
#define ED_CHAR        2
#define ED_END         3

/* ~~~~~~~~ Modifier les caractristiques d'un objet ~~~~~~~~ */

/* -------- Contrles de validit -------- */

#ifdef _LIBRAIRIE_X_
int OBJ_ControlerValeurs(unsigned int arbre, unsigned int objet);
#endif

/* -------- Position, taille -------- */

void OBJ_agrandit(unsigned int arbre, unsigned int objet, int larg, int haut);
void OBJ_deplace (unsigned int arbre, unsigned int objet, int x, int y, boolean mode);
void OBJ_position(unsigned int arbre, unsigned int objet, int x, int y, int *nv_x, int *nv_y);
void OBJ_taille  (unsigned int arbre, unsigned int objet, int larg, int haut, unsigned int *nv_larg, unsigned int *nv_haut);

/* -------- tat -------- */

#define DEPLACE_PIXELS  1 /* Unit des dplacements */
#define DEPLACE_RELATIF 2

#define COULEUR_BORD  1   /* De quelle partie faut-il changer la couleur ? */
#define COULEUR_FOND  2
#define COULEUR_TEXTE 3

int     OBJ_couleur    (unsigned int arbre, unsigned int objet, int couleur, boolean partie, boolean mode);
void    OBJ_icone      (unsigned int arbre, unsigned int objet, int icone);
long    OBJ_utilisateur(unsigned int arbre, unsigned int objet, int (*fonction)(PARMBLK *), long parametre, boolean mode);

/* -------- Redessin d'un objet GEM -------- */

void BI_DessinerObjet(unsigned int arbre, unsigned int objet);
#ifdef _LIBRAIRIE_X_
int OBJ_dessine(Window fenetre, OBJECT *objet);
#endif

/****************** criture d'une valeur numrique ***********
 *                                                            *
 * arbre  : arbre o crire la valeur                         *
 * objet  : objet de cet arbre o crire la valeur            *
 * valeur : valeur  crire                                   *
 **************************************************************/

#define TXT_COPIER   0
#define TXT_REMPLACE 1

/* Les types d'icne prdfinis */

#define ERR_ICONE_AUCUN    0
#define ERR_ICONE_DISQUE   1
#define ERR_ICONE_STOP     2
#define ERR_ICONE_QUESTION 3
#define ERR_ICONE_TROMBONE 4
#define ERR_ICONE_INFO     5
#define ERR_ICONE_EXCLAME  6

void met_chaine  (unsigned int arbre, unsigned int objet, unsigned long l_max, const char *texte, int mode);
void met_icone   (unsigned int arbre, unsigned int objet, int icone);
void met_svaleur (unsigned int arbre, unsigned int objet, double valeur);

void OBJ_TraduireTexte(unsigned int arbre, unsigned int objet, const char *chaine);

/******************* Lecture d'une valeur numrique ***********
 *  Renvoie la valeur lue                                     *
 *                                                            *
 * arbre : arbre o lire la valeur                            *
 * objet : objet de cet arbre o lire la valeur               *
 **************************************************************/

char         *lit_chaine    (unsigned int arbre, unsigned int objet, unsigned long l_max, char *texte, int mode);

#ifdef _LIBRAIRIE_X_ /* Fonctions internes  la librairie X */

int USR_separateur(struct parm_blk *parametres); /* Dessin d'un sparateur (ligne hor. ou ver.) */

/* -------- Fonctions utilises dans le dessin des objets -------- */

char *faire_chaine     (TEDINFO *tedinfo);
unsigned int OBJ_der_caractere(TEDINFO *tedinfo);

#endif /* _LIBRAIRIE_X_ */

#endif /* _X_OBJETS_H_ */
