/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Export des rsultats  divers formats

   Export d'une image (carto X par exemple)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "images.h"
#include "fichiers.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_export_image = NULL; /* Dossiers pour la sauvegarde */

/* Chemins par dfaut : dfinies dans variables.c */
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Export d'une image sous forme de fichier 3D GNUplot -------- */

int IMG_export_gnuplot(Image *image)
{
  char *nom_fichier;
  FILE *fichier;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    0, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  if ( NULL == image ) return FAUX;
  if ( image->genre != IMG_GENRE_REELLE ) {
    (void) fprintf(stderr, "%s,%s l.%u : Cette image ne contient pas des valeurs relles [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->nom);
    return -1;
  }

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) "Exporter vers GNUplot";
  infos_selecteur.chemin = chemin_export_image;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, fen_images );
  /* On mmorise les chemins */
  chemin_export_image = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_export_image );

  if (nom_fichier == NULL) {
    return -2;
  }

  fichier = fopen(nom_fichier, "wt");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, NULL, NULL);
    return -3;
  }

  /* En-tte : quelques infos sur l'image */
  fprintf(fichier,
	  "# LASE (c) Emmanuel CURIS, juillet 2003\n"
	  "# Export d'une image relle pour gnuplot\n#\n"
	  "# Image : %s\n", image->nom );

  /* Sauvegarde des donnes */
  if ( image->drapeaux.grille_libre == FAUX ) {
    unsigned int x, y;
    double x_mm, y_mm;
    double *p_case;

    p_case = image->valeurs.R;
    for ( x = 0 ; x < image->larg ; x++ ) {
      x_mm  = image->l_reel * x + image->x_min; /* On travaille par lignes */

      for ( y = 0 ; y < image->haut ; y++, p_case++ ) {
	y_mm  = image->h_reel * y + image->y_min;

	(void) fprintf(fichier, "%f %f %f\n", x_mm, y_mm, *p_case);
      }
      (void) fprintf(fichier, "\n"); /* Ligne vide : passage  la ligne suivante [gnuplot] */
    }
  } else {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Image relle sur grille libre\n",
		   __FILE__, __FUNCTION__, __LINE__);
  }

  (void) fclose(fichier);
  return 0;
}

