/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2003 Emmanuel CURIS
   Cration : 27 novembre 2003

   Export des rsultats  divers formats

   Export d'une srie complte sous forme de fichier texte ASCII (1 colonne par graphe, 1re colonne : les X)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"
#include "txt_utils.h"

#include "fichiers.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_export_serie = NULL;

/* Chemins par dfaut : dfinies dans variables.c */
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Routine vue de l'extrieur ~-~-~-~-~-~-~-~-~ */

int EA_SerieComplete(struct serie *serie)
{
  char *nom_fichier;
  const char *masque = "*.txt";
  unsigned long nbr_graphes = 0, idx_graphe;
  unsigned long nombre_points = 0;
  double **tables_y = NULL;
  struct EA_infos_export options_export = { 0, 0, 10 };
  FILE *fichier;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    2, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  if ( NULL == serie ) return -1;
  if ( NULL == serie->premier_graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun graphe dans la srie !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

/*   if ( FAUX == serie->drapeaux.table_X ) */
  /* Combien de graphes dans cette srie ? */
  if ( NULL == serie->table_X ) { /* Pas de table des X commune ! */
    Graphe *graphe;
    unsigned long nombre_points = 0;

    for( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( NULL == graphe->x ) {
	if ( 0 == nombre_points ) {
	  /* 1er graphe comme cela dtect */
	  nbr_graphes++;
	  nombre_points = graphe->nombre_point;
	} else {
	  if ( graphe->nombre_point == nombre_points ) {
	    /* Le graphe convient */
	    nbr_graphes++;
	  } else {
	    (void) fprintf(stderr, "%s,%s l.%u : Les graphes n'ont pas tous le mme nombre de points !\n",
			   __FILE__, __FUNCTION__, __LINE__);
	    return -1975;
	  }
	}
      }
    }
  } else {
    Graphe *graphe;

    for( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( graphe->x == serie->table_X ) {
	nbr_graphes++;
      }
    }
  }
  if ( 0 == nbr_graphes ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun graphe convenable dans cette srie\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +10;
  }


  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) "Exporter srie...";
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_export_serie;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, principale );
  /* On mmorise les chemins */
  chemin_export_serie = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_export_serie );

  if ( NULL == nom_fichier ) { /* Annulation */
    return +1;
  }

  /* On ouvre le fichier */
  fichier = fopen( nom_fichier, "wb" );
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir le fichier\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -15;
  }
  (void) fprintf( fichier, "# Srie complte de graphes de mme X [LASE, (c) Emmanuel Curis, novembre 2003]\n" );
  {
    char *txt_date;

    txt_date = TXT_CreerDate( NULL, NULL );
    (void) fprintf( fichier, "# Date : %s\n", txt_date );
    free( txt_date ); txt_date = NULL;
  }
  (void) fprintf( fichier, "# Srie : %s\n", serie->nom );

  /* On rserve la table des tables de Y [plus celle des X] */
  tables_y = (double **) calloc( nbr_graphes + 1, sizeof(double *) );
  if ( NULL == tables_y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    (void) fclose( fichier );
    return -20;
  }

  /* On remplit cette table et prpare l'en-tte du fichier */
  if ( NULL == serie->table_X ) { /* Pas de table des X commune ! */
    Graphe *graphe;

    nombre_points = 0;
    for( graphe = serie->premier_graphe, idx_graphe = 0 ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( graphe->x == NULL ) {
	tables_y[ idx_graphe ] = graphe->y;
	idx_graphe++;
	(void) fprintf( fichier, "# Colonne %lu : %s [%s]\n", idx_graphe, graphe->nom, GRF_QuelleLegende( graphe, AXE_Y ) );

	if ( ( 0 == nombre_points ) || ( nombre_points > graphe->nombre_point ) ) {
	  /* On prend le plus petit de tous, pour tre sr de ne pas dborder */
	  nombre_points = graphe->nombre_point;
	}
      }
    }
  } else {
    Graphe *graphe;

    tables_y[ 0 ] = serie->table_X;
    (void) fprintf( fichier, "# Colonne 1 : X\n" );
    for( graphe = serie->premier_graphe, idx_graphe = 1 ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( graphe->x == serie->table_X ) {
	tables_y[ idx_graphe ] = graphe->y;
	idx_graphe++;
	(void) fprintf( fichier, "# Colonne %lu : %s [%s]\n", idx_graphe, graphe->nom, GRF_QuelleLegende( graphe, AXE_Y ) );
      }
    }

    nombre_points = serie->nbr_points;
  }

  /* On remplit le fichier */
  EA_MultiColonnes( fichier, nombre_points, nbr_graphes + 1, tables_y, NULL, &options_export );

  /* On a fini */
  free( tables_y ); tables_y = NULL;
  (void) fclose( fichier );

  return 0;
}
