/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996  1999, Emmanuel CURIS
   Cration : 27 mai 1998

   Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   Les messages d'erreur qui peuvent survenir
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "systeme.h"

#define NBR_MESSAGES 9

#ifdef _ENGLISH_

static const char *textes[] =
{ 
  "[3][Absorption signal computing| "
     "| One find I0 <= 0! This value"
     "|is absurd! May be the beam"
     "|disappeared while experiment][ Cancel |Continue]",

  /*   v----------------------------v */
  "[1][---- Multi-detector file ----| "
     "| You didn't precise the number"
     "| of elements! The operation"
     "| is not possible.][ Cancel ]",

  "[3][--- Chargement d'un signal ---| "
     "| Le fichier indiqu"e_AIGU" ne contient"
     "|pas d'en-t"e_CIRC"te ! Le format est"
     "|peut-"e_CIRC"tre incorrect.][Continuer|Annuler]",

  "[3][-- Import d'un fichier texte --| "
     "|  Caract"e_GRAVE"re inconnu : %c"
     "|  Le r"e_AIGU"sultat est impr"e_AIGU"visible.][Arr"e_CIRC"ter|Continuer]",

  /*   v----------------------------v */
  "[1][-- Traitement des doublons --"
     "| Aucun doublon n'a "e_AIGU"t"e_AIGU" mis en"
     "|"e_AIGU"vidence dans ce spectre.][ Suite ]",

  /*   v----------------------------v */
  "[1][-- Double-points treatment --"
     "| %ld doublon%c ha%s been"
     "|detected in that spectrum.][ Suite ]",

  "[3][- Experimental data loading -"
     "|         You did not mark any"
     "|information to keep!][ End ]",
  /*   v----------------------------v */
  "[3][- Experimental data loading -"
     "|  One finds I1 negative! This"
     "|often means that a detector is"
     "|saturated (here, detector %d)"
     "][ Continue ]"

};

static const BI_Boite_Alerte err_Aucune_info = {
  ERR_ICONE_STOP,
  "No information", /* Title */
  "The selected parameters lead to use no data from the file.", /* Message */
  1, 1, -1,
  {"Finished", NULL, NULL, NULL, NULL} /* Buttons */
};

#else

static const char *textes[] =
{ "[3][----- Signal exp"e_AIGU"rimental -----|"
    " |    On trouve I0 = 0 ! Perte du|"
      "faisceau ?][Sauter|Continuer|Arr"e_CIRC"ter]",

  "[3][---- Signal en absorbance ----|"
    " |    On trouve I1 = 0 ! Perte du|"
      "faisceau ?|"
      "    Signal arbitrairement nul.][Sauter|Continuer|Arr"e_CIRC"ter]",

  "[1][---- Fichier multi-"e_AIGU"l"e_AIGU"ment ----| "
     "|   Le nombre d'"e_AIGU"l"e_AIGU"ments n'a pas"
     "|"e_AIGU"t"e_AIGU" indiqu"e_AIGU" !"
     "|   Op"e_AIGU"ration impossible][Abandon]",

  "[3][--- Chargement d'un signal ---| "
     "| Le fichier indiqu"e_AIGU" ne contient"
     "|pas d'en-t"e_CIRC"te ! Le format est"
     "|peut-"e_CIRC"tre incorrect.][Continuer|Annuler]",

  "[3][-- Import d'un fichier texte --| "
     "|  Caract"e_GRAVE"re inconnu : %c"
     "|  Le r"e_AIGU"sultat est impr"e_AIGU"visible.][Arr"e_CIRC"ter|Continuer]",

  "[1][-- Traitement des doublons --"
     "| Aucun doublon n'a "e_AIGU"t"e_AIGU" mis en"
     "|"e_AIGU"vidence dans ce spectre.][ Suite ]",

  "[1][-- Traitement des doublons --"
     "| %ld doublon%c %s "e_AIGU"t"e_AIGU
     "|trait"e_AIGU"%c dans ce spectre.][ Suite ]",

  /*   v----------------------------v */
  "[3][- Chargement de donn"e_AIGU"es exp. -"
     "|  Vous n'avez retenu aucune"
     "|information "a_GRAVE" conserver !][ Fin ]",
  /*   v----------------------------v */
  "[3][- Chargement de donn"e_AIGU"es exp. -"
     "|  On trouve I1 n"e_AIGU"gatif ! Cela"
     "|indique souvent une saturation"
     "|du d"e_AIGU"tecteur (ici, le %d)][ Suite ]"
};

#endif /* _ENGLISH_ */

extern char vrai_message[]; /* Dans erreurs.c */

int EXT_ERREUR(unsigned int numero_erreur, ...)
{
  va_list params;
  unsigned char c_1;
  unsigned long ul_1;
  int retour;

#ifdef _ENGLISH_
  if (numero_erreur >= NBR_MESSAGES)
    return SYS_ERREUR(ERREUR_INCONNUE, numero_erreur , "Experimental signal extraction");
#else
  if (numero_erreur >= NBR_MESSAGES)
    return SYS_ERREUR(ERREUR_INCONNUE, numero_erreur , "Extraction des signaux exp"e_AIGU"rimentaux");
#endif

  va_start(params, numero_erreur);

  switch (numero_erreur)
  {
  case CAR_INCONNU:
    c_1 = (unsigned char) (va_arg(params, int) );
    if ( c_1 == '|' ) /* Sinon, confusion avec le sparateur de lignes... */
      c_1 = ' ';
    sprintf(vrai_message, textes[CAR_INCONNU], c_1);
    retour = form_alert(1, vrai_message);
    break;

  case NOMBRE_DOUBLONS:
    ul_1 = va_arg(params, unsigned long);
#ifdef _ENGLISH_
    if (ul_1 > 1)
      sprintf(vrai_message, textes[NOMBRE_DOUBLONS], ul_1, 's', "ve");
    else
      sprintf(vrai_message, textes[NOMBRE_DOUBLONS], ul_1, ' ', "s");
#else
    if (ul_1 > 1)
      sprintf(vrai_message, textes[NOMBRE_DOUBLONS], ul_1, 's', "ont", 's');
    else
      sprintf(vrai_message, textes[NOMBRE_DOUBLONS], ul_1, ' ', "a", ' ');
#endif
    retour = form_alert(1, vrai_message);
    break;

  case EXT_SATURATION:
    ul_1 = va_arg(params, unsigned long);
    sprintf(vrai_message, textes[numero_erreur], ul_1);
    retour = form_alert(1, vrai_message);
    break;

  default:
    retour = form_alert(1, textes[numero_erreur]);
  }

  va_end(params);
  return retour;
}
