/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996  2001 - Emmanuel CURIS
   Cration de ce fichier : 20 novembre 2001 [ partir de FICHIERS.c qui disparat]

   Extraction de donnes exprimentales

   Fichiers contenant plusieurs spectres  la suite : fusion des spectres
   
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "fichiers.h"

#include "extrait_fusion.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ------------ Exafs II : fusionner les spectres choisis en un seul --------------------------------------- */

boolean LII_fondre(char *nom, FILE *fichier, unsigned long nbr_lignes, Infos_Extraction *infos)
{
  Graphe *spectre = NULL, *gr_i0 = NULL, *gr_i1 = NULL;
  char *titre;
  double energie, i_0, i_1;
  register double *zone_x = NULL, *zone_y = NULL;
  unsigned int nbr_elements, tot_element = 0;
  unsigned long nbr_pts, index, nbr_intervalles; /* Nombre de points par lment */
  unsigned int i, j;
  Intervalle_I *detecteurs = NULL; /* Numros des dtecteurs  conserver */
  TRI_infos *infos_tri = NULL, *inf_i0 = NULL, *inf_i1 = NULL;   /* Infos de tri... */
  boolean reserve;

  if (tot_element == 0) {
    (void) BI_ERR_alerte(NULL, &err_AucunSpectre, VRAI);
    (void) fclose(fichier);

    HIST_Titre(-1, "Chargement annul"e_AIGU" : nombre de spectres inconnu\n", "ERREUR");
    return FAUX;
  }

  if (nbr_lignes % tot_element) {
    BI_ERR_alerte(NULL, &err_nbr_lignes_incorrect, VRAI, nbr_lignes, tot_element, nom);
    (void) fclose(fichier);

    HIST_Titre(-1, "Chargement annul"e_AIGU" : nombre de spectres incompatible avec la taille du fichier\n", "ERREUR");
    return FAUX;
  }
  nbr_pts = (int) ( nbr_lignes / tot_element );

  if (detecteurs == NULL) {
    EXT_ERREUR(AUCUN_ELEMENT);
    (void) fclose(fichier);

    HIST_Titre(-1, "Chargement annul"e_AIGU" : aucun spectre pris en compte\n", "ERREUR");
    return FAUX; /* On a fini ! */
  }

  /* On rserve tout ce qu'il faut pour charger */
  spectre = GRF_creer(nbr_elements * nbr_pts, GR_BRUT, GR_NORMAL, NULL, NULL, "Absorption", UNIT_EV, SS_UNITE);
  if (spectre == NULL) {
    (void) fclose(fichier);

    HIST_Titre(-1, "Chargement annul"e_AIGU" : m"e_AIGU"moire insuffisante\n", "ERREUR");
    free(detecteurs);
    return FAUX;
  }

  infos_tri = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_elements );
  if (infos_tri == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "FICHIERS.c", "LII_fondre", "infos_tri");
    (void) fclose(fichier);

    HIST_Titre(-1, "Chargement annul"e_AIGU" : m"e_AIGU"moire insuffisante\n", "ERREUR");
    GRF_efface(spectre, VRAI, VRAI);
    free(detecteurs);
    return FAUX;
  }

  if ( type->garde_i0 ) {
    gr_i0 = GRF_creer(nbr_elements * nbr_pts, GR_I0, GR_NORMAL, NULL, NULL, "Coups dans la chambre I0", UNIT_EV, SS_UNITE);
    if ( gr_i0 == NULL ) {
      type->garde_i0 = FAUX;
    }
  }
  if ( type->garde_i0 ) {
    inf_i0 = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_elements );
    if (inf_i0 == NULL) {
      SYS_ERREUR(MANQUE_MEMOIRE, "FICHIERS.c", "LII_fondre", "inf_i0");
      GRF_efface(gr_i0, VRAI, VRAI);
      type->garde_i0 = FAUX;
    }
  }

  if ( type->garde_i1 ) {
    gr_i1 = GRF_creer(nbr_elements * nbr_pts, GR_I1, GR_NORMAL, NULL, NULL, "Coups dans la chambre I1", UNIT_EV, SS_UNITE);
    if ( gr_i1 == NULL ) {
      type->garde_i1 = FAUX;
    }
  }
  if ( type->garde_i1 ) {
    inf_i1 = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_elements );
    if (inf_i1 == NULL) {
      SYS_ERREUR(MANQUE_MEMOIRE, "FICHIERS.c", "LII_fondre", "inf_i1");
      GRF_efface(gr_i1, VRAI, VRAI);
      type->garde_i1 = FAUX;
    }
  }

  /* Lecture des graphes dans le fichier */
  j = 0; /* Numro rel d'lment */
  for (i = 0 ; i < tot_element ; i++) {
    reserve = TXT_est_dans_intervalle(i + 1, detecteurs, nbr_intervalles); /* Doit-on charger le spectre ? */

    if (reserve == VRAI) {
      infos_tri[j].xpoint_depart = (double *) malloc( (size_t) (sizeof(double) * nbr_pts ) );
      infos_tri[j].ypoint_depart = (double *) malloc( (size_t) (sizeof(double) * nbr_pts ) );

      if ( (infos_tri[j].xpoint_depart == NULL) || (infos_tri[j].ypoint_depart == NULL) ) {
	SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/fusion.c", "LII_fondre", "infos_tri[j], x ou y");
	for (i = 0 ; i <= j ; i++) {
	  free(infos_tri[i].xpoint_depart);
	  free(infos_tri[i].ypoint_depart);
	}
	free(infos_tri);
	if (type->garde_i0) free(inf_i0);
	if (type->garde_i1) free(inf_i1);
	GRF_efface(spectre, VRAI, VRAI);
	(void) fclose(fichier);

	HIST_Titre(-1, "Chargement annul"e_AIGU" : m"e_AIGU"moire insuffisante\n", "ERREUR");
	free(detecteurs);
	return FAUX;
      }
      zone_x = infos_tri[j].xpoint_depart;
      zone_y = infos_tri[j].ypoint_depart;

      infos_tri[j].nbr_points = nbr_pts;
      infos_tri[j].position = 0; /* On commence en position 0, c'est plus sr */
      
      if ( type->garde_i0 ) {
	inf_i0[j].ypoint_depart = (double *) malloc( sizeof(double) * nbr_pts );
	if ( inf_i0[j].ypoint_depart == NULL ) {
	  unsigned int k;

	  (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/fusion.c", "LII_fondre", "inf_i0[j].ypoint_depart");
	  for (k = 0 ; k <= j ; k++) {
	    free(inf_i0[k].ypoint_depart);
	  }
	  free(inf_i0);
	  GRF_efface(gr_i0, VRAI, VRAI);
	  type->garde_i0 = FAUX;
	} else {
	  inf_i0[j].xpoint_depart = zone_x;
	  inf_i0[j].nbr_points = nbr_pts;
	  inf_i0[j].position = 0;
	}
      }
      
      if ( type->garde_i1 ) {
	inf_i1[j].ypoint_depart = (double *) malloc( sizeof(double) * nbr_pts );
	if ( inf_i1[j].ypoint_depart == NULL ) {
	  unsigned int k;

	  (void) SYS_ERREUR(MANQUE_MEMOIRE, "Extrait/fusion.c", "LII_fondre", "inf_i1[j].ypoint_depart");
	  for (k = 0 ; k <= j ; k++) {
	    free(inf_i1[k].ypoint_depart);
	  }
	  free(inf_i1);
	  GRF_efface(gr_i1, VRAI, VRAI);
	  type->garde_i1 = FAUX;
	} else {
	  inf_i1[j].xpoint_depart = zone_x;
	  inf_i1[j].nbr_points = nbr_pts;
	  inf_i1[j].position = 0;
	}
      }
    }

    for (index = 0 ; index < nbr_pts ; index++) {
      fscanf(fichier,"%lf %lf %lf",&energie,&i_0,&i_1); /* On lit la ligne */
      if (i_0 <= 0) {
	unsigned int k;
	
	EXT_ERREUR(EXT_I0_NUL); /* Impossible de charger -> on annule le chargement */	      
	if (type->garde_i1) {
	  for (k = 0 ; k <= j ; k++) {
	    free(inf_i1[k].ypoint_depart);
	  }
	  free(inf_i1);
	  GRF_efface(gr_i1, VRAI, VRAI);
	}	

	if (type->garde_i1) {
	  for (k = 0 ; k <= j ; k++) {
	    free(inf_i0[k].ypoint_depart);
	  }
	  free(inf_i0);
	  GRF_efface(gr_i0, VRAI, VRAI);
	}

	for (k = 0 ; k <= j ; k++) {
	  free(infos_tri[k].xpoint_depart);
	  free(infos_tri[k].ypoint_depart);
	}
	free(infos_tri);
	GRF_efface(spectre, VRAI, VRAI);
	      
	(void) fclose(fichier);

	HIST_Titre(-1, "Chargement annul"e_AIGU" : I0 n"e_AIGU"gatif, coupure de faisceau ?\n", "ERREUR");
	return FAUX;
      }

      if (reserve == VRAI) { /* On veut la mmoriser */
	*zone_x = energie;
	if (type->mode_detection == DETECT_FLUORESCENCE)
	  *zone_y = log( i_0 / i_1 );
	else
	  *zone_y = i_1/i_0;
	zone_x++; zone_y++;
	
	if ( type->garde_i0 )
	  (inf_i0[j].ypoint_depart)[index] = i_0;
	
	if ( type->garde_i1 )
	  (inf_i1[j].ypoint_depart)[index] = i_1;
      }
    }

    if (reserve == VRAI)
      j++; /* On passe au graphe suivant */
    else {
      HIST_Texte(4, 0, "Spectre num"e_AIGU"ro %d inutilis"e_AIGU"\n", i + 1);
    }
  }

  (void) fclose(fichier); /* Fini de lire les donnes */
  free(detecteurs);

  /* Ici, on a donc les graphes voulus chargs dans les graphes[]->x/y */
  /* On ralise donc leur tri */

  tri_blocs(spectre->x, spectre->y, infos_tri, nbr_elements, spectre->nombre_point);
  tri_doublons(spectre, VRAI ); /* On limine les doublons */
  GRF_bornes(spectre, DESORDONNE, DESORDONNE); /* On installe les bornes */

  if ( type->garde_i0 ) {
    tri_blocs(gr_i0->x, gr_i0->y, inf_i0, nbr_elements, gr_i0->nombre_point);
    tri_doublons(gr_i0, VRAI );
    GRF_bornes(gr_i0, DESORDONNE, DESORDONNE);

    titre = TXT_genere_chaine(txt_titre_i0, nom);
    GRF_titre(gr_i0, titre);
    free(titre);
  }

  if ( type->garde_i1 ) {
    tri_blocs(gr_i1->x, gr_i1->y, inf_i1, nbr_elements, gr_i1->nombre_point);
    tri_doublons(gr_i1, VRAI );
    GRF_bornes(gr_i1, DESORDONNE, DESORDONNE);

    titre = TXT_genere_chaine(txt_titre_i1, nom);
    GRF_titre(gr_i0, titre);
    free(titre);
  }

  titre = TXT_genere_chaine(txt_titre_signal, nom);
  GRF_titre(spectre, titre);
  free(titre);

  spectre->drapeaux.sauve = FAUX; /* Pas sauv */
  free(infos_tri);
  if (type->garde_i0) free(inf_i0);
  if (type->garde_i1) free(inf_i1);

  GRF_aller(spectre); /* On affiche ce graphe */
  return VRAI;
}
