/* LASE - Traitement de l'E.X.A.F.S. sous TOS, MagiC, ... & X-Window
   Interface graphique pour le paramtrage de FEFF

   Gestion des objets ncessaires aux descriptions structurales
   (c) Emmanuel CURIS 1996 - 1998

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "atomes.h"
#include "exafs.h"

#include "feff.h"

/* ---------------------- Routines de gestion des recouvrements FEFF ------------------- *
 * Cration et effacement d'un recouvrement FEFF                                         *
 *****************************************************************************************/

/* ---------------------- Cration d'un recouvrement FEFF -------------------------------\
 | Renvoie l'adresse du recouvrement cr (NULL si erreur)                               |
 |                                                                                       |
 | potentiel    : le potentiel pour lequel il faut crer le recouvrement                 |
 | pot_recouvre : potentiel de l'atome avec lequel il se recouvre                        |
 | nbr_atomes   : nombre d'atomes de ce type avec lesquels il se recouvre                |
 | distance     : distance de ces atomes                                                 |
 \-------------------------------------------------------------------------------------- */

FEFF_OVERLAP *FEFF_CreerOverlap(Modele_FEFF *modele, int potentiel, int pot_recouvre, int nbr_atomes, double distance)
{
  FEFF_OVERLAP *Overlap;

  if (NULL == modele) return NULL;

  if ( (potentiel < 0) || (potentiel > 8) ) {
    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
    return NULL;
  }
  if ( (pot_recouvre < 0) || (pot_recouvre > 8) ) {
    FEFF_ERREUR(POTENTIEL_FAUX, pot_recouvre);
    return NULL;
  }
  if ( nbr_atomes < 1 ) {
    fprintf(stderr, "FEFF_CreerOverlap : Il faut au moins un atome d'un autre type pour dfinir un recouvrement!\n");
    return NULL;
  }
  if ( distance <= 0.0 ) {
    FEFF_ERREUR(FF_DIST_NEGATIVE, (int) distance);
    return NULL;
  }

  Overlap = (FEFF_OVERLAP *) malloc( sizeof(FEFF_OVERLAP) );
  if (Overlap == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "FEFF/F_STRUCT.c", "OVL_creer", "Overlap");
    return NULL;
  }

  /* La couche de recouvrement a pu tre dfinie */

  Overlap->potentiel  = pot_recouvre; /* Paramtres du recouvrement */
  Overlap->nbr_atomes = nbr_atomes;
  Overlap->distance   = distance;

  /* On la place en dbut de liste */

  Overlap->suivante = (modele->parametres.prem_couche)[potentiel];
  (modele->parametres.prem_couche)[potentiel] = Overlap;

  return Overlap;
}
