/* Analyse d'une chane de texte pour en dduire une formule mathmatique
   = relation entre graphes, transformations de coordonnes,...

   (C) 2001 Emmanuel Curis

   Module gnral de gestion des expressions "formelles"
   (Tout sauf la construction, en fait)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h> /* Pour tolower() */
#include <math.h>
#ifdef AVEC_GSL_
# include <gsl/gsl_sf.h>	/* Fonctions spciales fournies dans la GSL */
#endif

#include "systeme.h"
#include "utils_tables.h"

#include "c_formel.h"
#include "science.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean CF_CalculerPuissance(double x, double exposant, double *resultat);

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Crer une table de valeurs  partir d'une autre et d'une formule --------
   Renvoie la table ainsi cre (NULL en cas d'erreur)

   table_reference : la table des valeurs pour la variable X [NULL : la variable X sera gale  I, ou inutilise]
   nombre_valeurs  : le nombre de valeurs  calculer
   formule         : la formule  utiliser
   n_parametres    : le nombre de variables utilisateur
   parametres      : la table de ces paramtres [leur valeur ne peut PAS changer d'un calcul  l'autre
 */

double *TABLE_Creer_Formule(double *table_reference, unsigned long nombre_valeurs, const char *formule, unsigned long n_parametres, struct parametre *parametres)
{
  Expression *arbre;
  double *table;
  unsigned long i;

  if ( NULL == formule ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de formule indique !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( 0 == nombre_valeurs ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune valeur  calculer !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On construit l'arbre  partir de la formule */
  arbre = CF_ConstruireArbre( formule, n_parametres, parametres );
  if ( NULL == arbre ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de convertir la formule !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On construit la table des valeurs [rsultat] */
  table = TABLE_CreerTable( nombre_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer la table\n",
		   __FILE__, __FUNCTION__, __LINE__);
    effacer_arbre( arbre ); arbre = NULL;
    return NULL;
  }

  /* On effectue le calcul */
  for ( i = 0 ; i < nombre_valeurs ; i++ ) {
    double valeur;

    Var_I.valeur_initiale = (double) i;
    Var_I.contenu.valeur  = (double) i;

    if ( table_reference != NULL ) {
      Var_X.valeur_initiale = table_reference[ i ];
      Var_X.contenu.valeur  = table_reference[ i ];
    } else {
      Var_X = Var_I;
    }

    /* On calcule l'arbre */
    if ( FAUX == calcule_arbre( arbre, &valeur ) ) {
      (void) fprintf(stderr, "%s, %s l.%u : Erreur de calcul, point %lu (0  %lu) [X=%g]\n",
		     __FILE__, __FUNCTION__, __LINE__, i, nombre_valeurs - 1, Var_X.valeur_initiale);
      table[ i ] = 0.0;
    } else {
      table[ i ] = valeur;
    }
  }

  /* Calcul termin : plus besoin de l'arbre */
  effacer_arbre( arbre ); arbre = NULL;

  /* On renvoie la table cre */
  return table;
}

/* -------- Ralisation du calcul.
   Le rsultat est stock dans resultat
   La fonction renvoie VRAI si tout c'est bien pass, FAUX en cas de problme (div par 0, incohrence,...)

   On utilise une mthode rcursive, pour simplifier les choses
*/

boolean calcule_arbre(Expression *arbre, double *resultat)
{
  boolean correct = VRAI, binaire;
  double val_1, val_2 = 0.0;

  if ( (arbre == NULL) || (resultat == NULL) ) return FAUX;

  binaire = OperationADeuxParametres(arbre->code); /* VRAI : on a besoin des deux paramtres */

  switch (arbre->type)
  {
  case EXP_CTE_CTE:
    val_1 = arbre->arg_1.valeur;
    val_2 = arbre->arg_2.valeur;
    break;

  case  EXP_CTE_ARG:
    val_1 = arbre->arg_1.valeur;
    if (binaire == VRAI) {
      correct &= calcule_arbre(arbre->arg_2.expression, &val_2);
    }
    break;

  case EXP_ARG_ARG:
    correct = calcule_arbre(arbre->arg_1.expression, &val_1);
    if (binaire == VRAI) {
      correct &= calcule_arbre(arbre->arg_2.expression, &val_2);
    }
    break;

  case EXP_CTE_VAR:
    val_1 = arbre->arg_1.valeur;
    if (binaire == VRAI) {
      val_2 = VAR_LireValeur( arbre->arg_2.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    }
    break;

  case EXP_VAR_VAR:
    val_1 = VAR_LireValeur( arbre->arg_1.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    if (binaire == VRAI) {
      val_2 = VAR_LireValeur( arbre->arg_2.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    }
    break;

  case EXP_ARG_VAR:
    correct = calcule_arbre(arbre->arg_1.expression, &val_1);
    if (binaire == VRAI) {
      val_2 = VAR_LireValeur(arbre->arg_2.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    }
    break;

  case EXP_VAR_ARG:
    val_1 = VAR_LireValeur(arbre->arg_1.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    if (binaire == VRAI) {
      correct = calcule_arbre(arbre->arg_2.expression, &val_2);
    }
    break;

  case EXP_ARG_CTE:
    correct = calcule_arbre(arbre->arg_1.expression, &val_1);
    val_2 = arbre->arg_2.valeur;
/*    printf("Valeurs trouves ARG/CTE : %f et %f\n", val_1, val_2); */
    break;

  case EXP_VAR_CTE:
    val_1 = VAR_LireValeur(arbre->arg_1.variable, VARIABLE_LIRE_VALEUR_AJUSTEE );
    val_2 = arbre->arg_2.valeur;
/*    printf("Valeurs trouves VAR/CTE : %f et %f\n", val_1, val_2); */
    break;

  default:
    fprintf(stderr, "Type d'expression inconnu : %d - CALCUL ANNUL\n", arbre->type);
    return FAUX;
  }

  if (correct == FAUX) /* Une erreur s'est produite */
    return FAUX;

  /* Maintenant que l'on a les deux valeurs, on fait le calcul */
  switch (arbre->code)
  {
  case EXP_NOMBRE_ISOLE: *resultat = val_1; break; /* Devrait tre rare, normalement ! */

  case EXP_ADDITION:     *resultat = val_1 + val_2; break;
  case EXP_SOUSTRACTION: *resultat = val_1 - val_2; break;
  case EXP_PRODUIT:      *resultat = val_1 * val_2; break;
  case EXP_DIVISION:
    if (val_2 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/quotient", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = val_1 / val_2;
    break;

  case EXP_PUISSANCE:
    return CF_CalculerPuissance( val_1, val_2, resultat );
    break;

  case EXP_SINUS:    *resultat = sin(val_1); break;
  case EXP_COSINUS:  *resultat = cos(val_1); break;
  case EXP_TANGENTE: *resultat = tan(val_1); break; /* Il faudrait empcher une division par zro */
  case EXP_COTANGENTE:
    val_2 = tan(val_1);
    if (val_2 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/cotg", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / val_2;
    break;
  case EXP_SECANTE:
    val_2 = cos(val_1);
    if (val_2 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/sec", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / val_2;
    break;
  case EXP_COSECANTE:
    val_2 = sin(val_1);
    if (val_2 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/cosec", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / val_2;
    break;

  case EXP_COSH: *resultat = cosh(val_1); break;
  case EXP_SINH: *resultat = sinh(val_1); break;
  case EXP_TANH: *resultat = tanh(val_1); break;
  case EXP_COTANH:
    if (val_1 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/coth", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / tanh(val_1);
    break;
  case EXP_SECH: *resultat = 1.0 / cosh(val_1); break;
  case EXP_COSECH:
    if (val_1 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/cosech", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / sinh(val_1);
    break;

  case EXP_EXP:   *resultat = exp(val_1); break;
  case EXP_EXP10: *resultat = pow(10.0, val_1); break;
  case EXP_LOG:
    if (val_1 <= 0.0) {
      fprintf(stderr, "Logarithme d'un nombre ngatif ou nul\n");
      return FAUX;
    } else
      *resultat = log(val_1);
    break;
  case EXP_LOG10:
    if (val_1 <= 0.0) {
      fprintf(stderr, "Logarithme dcimal d'un nombre ngatif ou nul\n");
      return FAUX;
    } else
      *resultat = log10(val_1);
    break;

  case EXP_RACINE:
    if (val_1 < 0.0) {
      fprintf(stderr, "Racine carre d'un nombre ngatif\n");
      return FAUX;
    } else
      *resultat = sqrt(val_1);
    break;
  case EXP_CARRE: *resultat = val_1 * val_1; break;
  case EXP_CUBE:  *resultat = val_1 * val_1 * val_1; break;
  case EXP_INVERSE:
    if (val_1 == 0.0) {
      EXF_ERREUR(DIV_ZERO, "Expression/inverse", txt_err_calc_annule);
      return FAUX;
    } else
      *resultat = 1.0 / val_1;
    break;

  case EXP_ASIN:
    if ( (val_1 < -1.0) || (val_1 > 1.0) ) {
      fprintf(stderr, "Arc sinus d'un nombre plus grand que 1, en valeur absolue\n");
      return FAUX;
    } else
      *resultat = asin(val_1);
    break;
  case EXP_ACOS:
    if ( (val_1 < -1.0) || (val_1 > 1.0) ) {
      fprintf(stderr, "Arc cosinus d'un nombre plus grand que 1, en valeur absolue\n");
      return FAUX;
    } else
      *resultat = acos(val_1);
    break;
  case EXP_ATAN: *resultat = atan(val_1); break;

  case EXP_ASINH: *resultat = asinh(val_1); break;
  case EXP_ACOSH:
    if ( val_1 < 1.0 ) {
      fprintf(stderr, "Arg ch d'un nombre plus petit que 1\n");
      return FAUX;
    } else
      *resultat = acosh(val_1);
    break;
  case EXP_ATANH:
    if ( (val_1 < -1.0) || (val_1 > 1.0) ) {
      fprintf(stderr, "Arg th d'un nombre plus grand que 1, en valeur absolue\n");
      return FAUX;
    } else
      *resultat = atanh(val_1);
    break;

    /* Fonctions spciales : dpend de l'installation de la GNU Scientific Library... */
  case EXP_LAMBERTW0:
#ifdef AVEC_GSL_
    return gsl_sf_lambert_W0( val_1 );
#endif
    break;

  default:
    fprintf(stderr, "Fonction inconnue : %d - CALCUL ANNUL\n", arbre->code);
    return FAUX;
  }

  return VRAI; /* Tout s'est bien pass */
}

/* -------- Calcul d'une puissance quelconque de faon suppose optimale -------- */

static boolean CF_CalculerPuissance(double x, double exposant, double *resultat)
{
  boolean exp_negatif = FAUX;

  /* Quelques cas particuliers */
  /* a. exposant = 0 : x^0 = 1, 0^0 = indfini */
  if ( 0.0 == exposant ) {
      *resultat = 1.0;
      if ( 0.0 == x ) {
	(void) fprintf(stderr, "%s, %s l.%u : 0^0, indtermin !\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return FAUX;
      }
      return VRAI;
  }

  /* b. exposant = 1 : x^1 = x */
  if ( 1.0 == exposant ) {
    *resultat = x;
    return VRAI;
  }

  /* Signe de l'exposant --- pour toujours avoir un exposant positif ensuite */
  if ( exposant < 0.0 ) {
    if ( 0.0 == x ) {
      (void) fprintf(stderr, "%s, %s l.%u : Division par 0 [0^%g]\n",
		     __FILE__, __FUNCTION__, __LINE__, exposant);
      return FAUX;
    }

    exp_negatif = VRAI;
    exposant = -exposant;
  }

  /* c. exposant entier ? */
  if ( fabs( rint( exposant ) - exposant ) < 1e-10 ) {
    unsigned int i, n;

    n = rint( exposant );
    *resultat = 1.0;

    for ( i = 1 ; i <= n ; i++ ) {
      *resultat *= x;
    }
  } else {			/* Non : a priori, dfini uniquement pour x > 0.0 */
    /* Remarque : si exposant fractionnaire et dnominateur impair, dfini pour x < 0.0 aussi ! */
    if ( x < 0.0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Log. d'un nombre ngatif [%g^%g]\n",
		     __FILE__, __FUNCTION__, __LINE__, x, exposant);
      return FAUX;
    }

    *resultat = pow( x, exposant );
  }

  /* Fin : on prend l'inverse en cas d'exposant ngatif */
  if ( VRAI == exp_negatif ) {
    *resultat = 1.0 / *resultat;
  }

  /* Calcul russi */
  return VRAI;
}

/* -------- Dtruire un arbre... -------- */

void effacer_arbre(Expression *arbre)
{
  if (arbre == NULL) return;

  if ( (arbre->type == EXP_ARG_ARG) || (arbre->type == EXP_ARG_VAR) || (arbre->type == EXP_ARG_CTE) )
    effacer_arbre(arbre->arg_1.expression);

  if ( ( OperationADeuxParametres(arbre->code) ) &&
       ( (arbre->type == EXP_ARG_ARG) || (arbre->type == EXP_CTE_ARG) || (arbre->type == EXP_VAR_ARG) ) )
    effacer_arbre(arbre->arg_2.expression);

  free(arbre);
}
