/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Aller  un nouveau graphe = le placer comme graphe courant
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "interface.h"		/* Pour I_MettreAJourPrincipale() */
#include "serie.h"
#include "tracer.h"		/* Pour TRC_MettreAJourSuperposition */

#include "grf_aller.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

extern const char *txt_info_interne, *txt_info_sanstype; /* Dans variables.fr */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void couper_titres_menu  (void);
static void adapter_options_menu(Graphe *graphe);

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

void GRF_aller(Graphe *graphe)
{
  Serie_Graphes *serie;
  Graphe *i;
  unsigned long num_graphe;

  /* On enregistre les modifs. ventuelles du graphe courant : titre, lgendes,... */
  GRF_EnregistrerParametres();

  if (graphe == NULL) {
    if ( (serie_courante == NULL) || (serie_courante->premier_graphe == NULL) ) {
      graphe_courant = NULL;
      couper_titres_menu();
      FEN_ascenseur(principale, ASC_VERTICAL, 0, 1000);
      FEN_redessin(principale, BI_RDS_TOUT);
      if (serie_courante != NULL) {
	FEN_titre(principale, serie_courante->nom);
	FEN_infos(principale, infos_principale_serie_vide);
      } else {
	FEN_titre(principale, titre_principale_aucune_serie);
	FEN_infos(principale, infos_principale_aucune_serie);
      }
      GRF_PreparerListe( VRAI );
    }

    return;
  }

  serie = GRF_QuelleSerie(graphe);
  if (serie == NULL) {
    (void) fprintf(stderr, "%s, %s l.%u : Graphe %s hors de toute srie !\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->nom );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }
  if ( serie != serie_courante ) {
    I_SERIE_GRF_Aller( serie );
  }
  /* On est maintenant sr que le graphe indiqu appartient  la srie courante */

  for ( i = serie->premier_graphe, num_graphe = 0 ; i != graphe ; i = i->suivant, num_graphe++ ) {
  }
  
  /* On positionne l'ascenseur */
  if (serie->nbr_graphes < 2) {
    FEN_ascenseur(principale, ASC_VERTICAL, 0, 1000);
  } else {
    float pos_asc, taille_asc;

    pos_asc  = (float) (num_graphe * 1000.0);
    pos_asc /= (float) (serie->nbr_graphes - 1);
    taille_asc = 1000.0 / (float) (serie->nbr_graphes);
    FEN_ascenseur(principale, ASC_VERTICAL, (int) pos_asc, (int) taille_asc);
  }

  graphe_courant = graphe;

  /* Titre de la fentre */
  FEN_titre(principale, graphe_courant->nom);

  /* Ligne d'informations de la fentre */
  FEN_infos( principale, GRF_TexteTypeGraphe( graphe_courant->type ) );

  /* Paramtres de trac */
  I_MettreAJourPrincipale( VRAI ); /* On change de graphe -> on remet  l'chelle ! */
  GRF_PreparerListe(VRAI);

  /* On met  jour la superposition, si elle est ouverte */
  TRC_MettreAJourSuperposition();

  /* On met  jour les menus */
  adapter_options_menu( graphe );
}

/* -------- On s'occupe des options dans les menus -------- */

/* ------ Aucun graphe : on coupe les menus inutiles ------ */

static void couper_titres_menu(void)
{
  /* Titres du menu  extraction , inutiles si aucun graphe dans la srie */
  OBJ_active(MENU, TIT_FILTRAGE  , FORCE_NON); 
  OBJ_active(MENU, TIT_TRACE     , FORCE_NON);

  /* Options du menu  extraction  inutiles si aucun graphe dans la srie */
  OBJ_active(MENU, SAUVER          , FORCE_NON);
  OBJ_active(MENU, MEX_EXPORTER_GRF, FORCE_NON);
  OBJ_active(MENU, MEN_EXPORTIMAGE , FORCE_NON);
  OBJ_active(MENU, IMP_GRAPHE      , FORCE_NON);

  OBJ_active(MENU, MEN_TT_EFFACER, FORCE_NON);
  OBJ_active(MENU, MEX_POINTS, FORCE_NON);
  OBJ_active(MENU, MEN_FONDRE, FORCE_NON);
  OBJ_active(MENU, OP_DIVERSES, FORCE_NON);
  OBJ_active(MENU, MEN_MOYENNER, FORCE_NON);
  OBJ_active(MENU, COMP_GRAPHES, FORCE_NON);
  OBJ_active(MENU, MEX_CREER_GROUPE, FORCE_NON);
  OBJ_active(MENU, MEX_MOD_GROUPE, FORCE_NON);
  OBJ_active(MENU, MEX_EFF_GROUPE, FORCE_NON);

  OBJ_active(MENU, MEN_CLC_SIGNAL, FORCE_NON);
  OBJ_active(MENU, MEN_CORR_NRJ, FORCE_NON);
  OBJ_active(MENU, MEX_CORR_I, FORCE_NON);
  OBJ_active(MENU, XANES_TOUT, FORCE_NON);
  OBJ_active(MENU, MATRICE, FORCE_NON);
  OBJ_active(MENU, XAN_NORMALISE, FORCE_NON);
  OBJ_active(MENU, DERIVEE_UN, FORCE_NON);
  OBJ_active(MENU, DERIVEE_DEUX, FORCE_NON);
  OBJ_active(MENU, INT_PRESEUIL, FORCE_NON);

  /* Options du menu  ajustements  inutiles si aucun graphe dans la srie */
  OBJ_active(MEN_AJUSTE, AJ_DEMARRE, FORCE_NON); /* Impossible de lancer un ajustement */
}

/* ------ N'activer que les options qui ont un sens avec le graphe courant ------ */

static void adapter_options_menu(Graphe *graphe)
{
  boolean absorbe = FAUX, exafs = FAUX, autre = FAUX, derive = FAUX, module = FAUX, phase = FAUX;

  if (graphe == NULL) return;

  /* Options disponibles quel que soit le type de graphe */
  OBJ_active(MENU, TIT_FILTRAGE   , FORCE_OUI); 
  OBJ_active(MENU, TIT_ERREURS    , FORCE_OUI);
  OBJ_active(MENU, TIT_TRACE      , FORCE_OUI);  

  OBJ_active(MENU, SAUVER          , FORCE_OUI);
  OBJ_active(MENU, MEX_EXPORTER_GRF, FORCE_OUI);
  OBJ_active(MENU, MEN_EXPORTIMAGE , FORCE_OUI);
#ifdef _TOS_
  OBJ_active(MENU, IMP_GRAPHE     , gdos);
#endif
  OBJ_active(MENU, MEN_CLC_SIGNAL , FORCE_OUI);

  OBJ_active(MENU, MEN_TT_EFFACER, FORCE_OUI);
#ifdef _DEVELOPPE_
  OBJ_active(MENU, MEX_POINTS, FORCE_OUI);
  OBJ_active(MENU, COMP_GRAPHES, FORCE_OUI);
  OBJ_active(MENU, MEX_CREER_GROUPE, FORCE_OUI);
  OBJ_active(MENU, MEX_MOD_GROUPE, FORCE_OUI);
  OBJ_active(MENU, MEX_EFF_GROUPE, FORCE_OUI);
#endif
  OBJ_active(MENU, MEN_FONDRE, FORCE_OUI);
  OBJ_active(MENU, OP_DIVERSES, FORCE_OUI);
  OBJ_active(MENU, MEN_MOYENNER, FORCE_OUI);

  /* Options qui ne s'appliquent qu'aux graphes rels */
  autre = (graphe->genre_donnees == GR_NORMAL); /* VRAI si le graphe est rel */
  OBJ_active(MENU, RECOMP_TF      , autre); /* Recomposition d'un spectre complexe */
  OBJ_active(MENU, MEN_CORR_NRJ   , autre); /* Corrections des nergies   */
  OBJ_active(MENU, INT_PRESEUIL   , autre); /* Intgration du prseuil    */

#ifdef _DEVELOPPE_ /* Options en cours de dveloppement -> sont _toujours_ inactives... */
  OBJ_active(MENU, MEX_CORR_I, autre);
#endif

  /* Options spcifiques d'un ou deux types de graphes */

  /* Options spcifiques  l'extraction d'un signal E.X.A.F.S. */
  autre = (graphe->type == GR_SSTYPE) || (graphe->type == GR_AUTRETYPE); /* Graphe sans type : tout est possible... */

  absorbe = ( (graphe->type == GR_BRUT)     || (graphe->type == GR_MOINSMAT) );
  derive  = ( (graphe->type == GR_DERIVEE)  || (graphe->type == GR_DERIVEE2) );
  exafs   = ( (graphe->type == GR_EXAFS1)   || (graphe->type == GR_EXAFS2)
	   || (graphe->type == GR_EXAFS)    || (graphe->drapeaux.exafs == VRAI) );
  module  = ( (graphe->type == GR_EXAFSAMP) || (graphe->type == GR_COMPMOD ) );
  phase   = ( (graphe->type == GR_EXAFSPHA) || (graphe->type == GR_COMPARG ) );

  OBJ_active(MENU, MATRICE, autre || (graphe->type == GR_BRUT)); /* Soustration du terme de la matrice */
  OBJ_active(MENU, XANES_TOUT,    autre|absorbe); /* Traitement complet du XANES          */
  OBJ_active(MENU, CHERCHE_E0,    autre|absorbe|derive); /* Recherche d'un E0             */
  OBJ_active(MENU, DERIVEE_UN,    autre|absorbe|derive); /* Calcul de la drive premire */
  OBJ_active(MENU, DERIVEE_DEUX,  autre|absorbe); /* Calcul direct de la drive seconde  */
  OBJ_active(MENU, XAN_NORMALISE, autre|absorbe); /* Normalisation du X.A.N.E.S.          */
  OBJ_active(MENU, EXAFS_TOUT,    autre|absorbe); /* Traitement complet de l'E.X.A.F.S.   */
  OBJ_active(MENU, OTE_MU0_1,     autre|absorbe); /* Soustraction de mu0, premire tape  */
  OBJ_active(MENU, OTE_MU0_2,     autre|exafs);   /* Soustraction de mu0, deuxime tape  */
  OBJ_active(MENU, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  OBJ_active(MENU, MEN_MU0_TOTAL, autre|absorbe); /* Soustraction de mu0, dj calcul    */
  OBJ_active(MENU, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  OBJ_active(MENU, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  OBJ_active(MENU, CORR_AMPLITUDE, autre|module); /* Correction d'une amplitude */
  OBJ_active(MENU, CORR_PHASE,     autre|phase);  /* Correction d'une phase     */

  /* Conversion nergie (eV) <-> nombre d'onde (1/) */
  /*  OBJ_active(MENU, CONVERSION,
      (graphe->unit_x == UNIT_EV) || (graphe->unit_x == UNIT_1_ANG) ); */
  
  OBJ_active(MENU, MEX_KSMIRNOV, (autre) || (graphe->type == GR_EPREUVE));
  OBJ_active(MENU, MEX_CVMISES , (autre) || (graphe->type == GR_EPREUVE));

  OBJ_active( MENU, MEX_HOM_VARIANCE, ( GR_DEVIATION == graphe->type ) );

  /* ~~~~~~~~ Adaptation du menu d'ajustements, si ncessaire ~~~~~~~~ */

  OBJ_active(MEN_AJUSTE, AJ_DEMARRE, exafs | (graphe->type == GR_TF) | autre ); /* Graphes ajustables */
}
