/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Copier un graphe (avec changement de srie ventuel)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"
#include "filtrage.h" /* Pour les infos complmentaires sur la TF */

#include "grf_copier.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Copie : interface gnrale ------------\
 |  Renvoie le graphe copie, NULL en cas d'erreur  |
 |                                                 |
 | source : le graphe  copier                     |
 | serie  : la srie dans laquelle placer la copie |
 |            (NULL -> dans la mme srie)         |
 | isole  : VRAI -> la copie est un graphe isol   |
 \------------------------------------------------ */

Graphe *GRF_Copier(const Graphe *source, struct serie *serie, boolean isole)
{
  Serie_Graphes *serie_source;
  Graphe *copie;

  if ( NULL == source ) {
    /* Situation normale lors de la copie des barres d'erreur => ni message, ni arrt brutal */
    return NULL;
  }

  if ( FAUX == source->drapeaux.isole ) {
    serie_source = GRF_QuelleSerie( source );
  } else { /* Copie d'un graphe isol */
    serie_source = NULL;
    if ( NULL == serie ) {
      isole = VRAI; /* donne donc forcment une copie isole, sauf si la srie est explicite ! */
    }
  }

  if ( ( NULL == serie ) && ( FAUX == isole ) ) {
    serie = serie_source;
    if ( NULL == serie ) { /* Copie d'un graphe isol en graphe isol */
      AVERTISSEMENT( "Original (%s) ET copie sont des graphes isols", source->nom );
      isole = VRAI;
    }

    /* Si la srie de copie est protge : on copie dans une nouvelle srie */
    if ( VRAI == serie->drapeaux.protegee ) {
      serie = SERIE_GRF_CreerSerie( serie, "Copies..." );
      if ( NULL == serie ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer une srie d'accueil pour la copie\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return NULL;
      }
    }
  }

  if ( VRAI == isole ) {
    copie = GRF_nv_isole( source->nombre_point, source->type, source->genre_donnees );
  } else {
    char *titre;
    
    titre = TXT_genere_chaine( txt_titre_copie, source->nom ); /* Le nom de la copie */
    copie = Serie_nv_graphe( serie, NULL, source->nombre_point, source->type, source->genre_donnees, titre );
    free( titre ); titre = NULL;
  }
  if ( NULL == copie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le graphe copie\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Copie du contenu */
  /* 1. les X */
  if ( copie->genre_donnees == GRF_SERIE ) {
    copie->x = source->x; /* On partage les mmes  faux X  */
    if ( ( serie != serie_source ) && ( source->x != NULL ) ) {
      /* On doit passer en graphe normal, car les X sont dans une autre srie (ou quelque part ailleurs)
	 => lors de leur effacement, le lien ne serait pas mis  jour ici : plantage assur tt ou tard... */
      GRF_ChangerGenre( copie, GRF_FCT_REELLE );
    }
  } else if ( ( GRF_ERREUR_C == copie->genre_donnees ) || ( GRF_ERREUR_R == copie->genre_donnees ) ) {
    /* Rien  copier car pas de X mmoriss */
  } else {
    if ( copie->x != NULL ) {
      memcpy( copie->x, source->x, sizeof(double) * source->nombre_point );
    } else {
      (void) fprintf(stderr, "%s,%s l.%u : PROBLME GRAVE, on n'a pas de X cible mais ce n'est pas une srie !\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }
  }
  copie->x_min = source->x_min;
  copie->x_max = source->x_max;

  /* 2. les Y */
  if ( GRF_FCT_COMPLEXE == source->genre_donnees ) {
    memcpy( copie->y, source->y, sizeof(Complexe) * source->nombre_point );
  } else if ( GRF_FCT_REELLE == source->genre_donnees ) {
    memcpy( copie->y, source->y, sizeof(double) * source->nombre_point );
  } else if ( GRF_SERIE == source->genre_donnees ) {
    memcpy( copie->y, source->y, sizeof(double) * source->nombre_point );
  } else if ( GRF_ERREUR_R == source->genre_donnees ) {
    memcpy( copie->y, source->y, sizeof(double) * source->nombre_point );
    memcpy( copie->z.ddl, source->z.ddl, sizeof(int) * source->nombre_point );
  } else if ( GRF_ERREUR_C == source->genre_donnees ) {
    memcpy( copie->y, source->y, sizeof(Complexe) * source->nombre_point );
    memcpy( copie->z.ddl, source->z.ddl, sizeof(int) * source->nombre_point );
  } else {
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE, copier le contenu des graphes de type %d\n",
		   __FILE__, __FUNCTION__, __LINE__, source->genre_donnees );
  }
  copie->y_min = source->y_min;
  copie->y_max = source->y_max;

  if ( GR_DEVIATION == source->type ) { /* Les degrs de libert */
    memcpy( copie->z.ddl, source->z.ddl, sizeof(int) * source->nombre_point );
  }

  /* On s'occupe de copier les barres d'erreur */
  copie->erreur_x = GRF_Copier( source->erreur_x, NULL, VRAI );
  copie->erreur_y = GRF_Copier( source->erreur_y, NULL, VRAI );

  /* Seulement pour les graphes non-isols */
  if ( ( FAUX == isole ) && ( FAUX == source->drapeaux.isole ) ) {
    int nv_couleur = NOIR, nv_style = 1, nv_marque = 0;

    /* Historique */
    HIST_Titre( 4, "Copie de graphe", "GRF" );
    HIST_Texte( 4, 0, "Original : %s\n", source->nom );
    HIST_Texte( 4, 0, "Copie    : %s\n", copie->nom );

    /* Couleur, style : ce qui suit ceux de l'original */
    if ( source->affichage != NULL ) {
      nv_couleur = source->affichage->couleur + 1;
      nv_style = source->affichage->style;
      nv_marque = source->affichage->marque;
    }
    if ( nv_couleur > 15 ) {
      nv_couleur = NOIR;
      nv_style++;
    }
    if ( nv_style > BI_NOMBRE_STYLES_LIGNES ) {
      nv_style = 0;
      nv_marque++;
    }
    GRF_couleur( copie, nv_couleur, nv_style, nv_marque );

    /* Lgendes, units : comme l'original */
    GRF_CopierAffichage( copie, source,
			 GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_LEGENDE_Y | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_UNITE_Y );
  }

  /* On a fini, tout s'est bien pass */
  return copie;
}

/* -------- Copie ventuelle des paramres d'affichage -------- */

int GRF_CopierAffichage(Graphe *copie, const Graphe *source, int elements)
{

  if ( 0 == elements ) return 0; /* Rien  copier, donc pas de souci ! */
  if ( ( NULL == copie ) || ( NULL == source ) ) return 0; /* Idem */

  if ( NULL == source->affichage ) {
    (void) fprintf(stderr, "GRF_CopierAffichage : pas d'infos  copier dans l'original [%s]\n", source->nom);
    return +2;
  }

  if ( NULL == copie->affichage ) {
    int retour;

    retour = GRF_CreerAffichage( copie );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "GRF_CopierAffichage : ERREUR dans GRF_CreerAffichage()\n");
      return -1;
    }
  }

  /* On peut maintenant raliser la copie en toute confiance */
  if ( elements & GRF_CP_AFF_LEGENDE_X ) {
    if ( source->affichage->axe_x != NULL ) { /* Sinon, lgende par dfaut : sans intrt */
      GRF_ChangerLegende( copie, AXE_X, source->affichage->axe_x, VRAI );
    } else {
      /* Normal lors de la copie de graphe : NULL -> NULL */
    }
  }

  if ( elements & GRF_CP_AFF_LEGENDE_Y ) {
    if ( source->affichage->axe_y != NULL ) { /* Sinon, lgende par dfaut : sans intrt */
      GRF_ChangerLegende( copie, AXE_Y, source->affichage->axe_y, VRAI );
    } else {
      /* Normal lors de la copie de graphe : NULL -> NULL */
    }
  }

  if ( elements & GRF_CP_AFF_UNITE_X ) {
    GRF_ChangerUnite( copie, AXE_X, GRF_QuelleUnite( source, AXE_X ) );
  }

  if ( elements & GRF_CP_AFF_UNITE_Y ) {
    GRF_ChangerUnite( copie, AXE_Y, GRF_QuelleUnite( source, AXE_Y ) );
 }

  if ( elements & ( GRF_CP_AFF_COULEUR|GRF_CP_AFF_STYLE|GRF_CP_AFF_MARQUE ) ) {
    GRF_couleur( copie, source->affichage->couleur, source->affichage->style, source->affichage->marque );
  } else if ( elements & (GRF_CP_AFF_COULEUR|GRF_CP_AFF_STYLE) ) {
    GRF_couleur( copie, source->affichage->couleur, source->affichage->style, -1 );
  } else if ( elements & (GRF_CP_AFF_COULEUR|GRF_CP_AFF_MARQUE) ) {
    GRF_couleur( copie, source->affichage->couleur, -1, source->affichage->marque );
  } else if ( elements & (GRF_CP_AFF_STYLE|GRF_CP_AFF_MARQUE) ) {
    GRF_couleur( copie, -1, source->affichage->style, source->affichage->marque );
  } else if ( elements & GRF_CP_AFF_STYLE ) {
    GRF_couleur( copie, -1, source->affichage->style, -1 );
  } else if ( elements & GRF_CP_AFF_COULEUR ) {
    GRF_couleur( copie, source->affichage->couleur, -1, -1 );
  } else if ( elements & GRF_CP_AFF_MARQUE ) {
    GRF_couleur( copie, -1, -1, source->affichage->marque );
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -~ Copie des informations complmentaires ventuelles -~ */

void *GRF_CopierComplements(Graphe *original)
{
  void *copie = NULL;

  if ( NULL == original->pointeur.autre ) /* Pas d'informations complmentaires */
    return NULL;

  switch ( original->type )
  {
  case GR_TF:
    {
      Infos_FonctionApodisation *infos;

      infos =  malloc( sizeof(Infos_FonctionApodisation) );
      if (infos == NULL) {
	(void) SYS_ERREUR(MANQUE_MEMOIRE, "Graphes/copier.c", "GRF_CopierComplements", "infos / TF");
      } else {
	*infos = *(original->pointeur.infos_tf);
      }
      copie = infos;
    }
    break;

  default:
    (void) fprintf(stderr, "GRF_CopierComplements : type de complment inconnu [graphe de type %d]\n", original->type);
    break;
  }

  return copie;
}
