/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Oprations diverses sur les graphes [pas leur contenu]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "grf_divers.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static inline int GRF_AdapterTaille_Y_R(Graphe *graphe, unsigned long nv_nbr_pts);

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Adapter la taille de la table des Y, suppose relle -------- */

static inline int GRF_AdapterTaille_Y_R(Graphe *graphe, unsigned long nv_nbr_pts)
{
  double *nv_y;

  nv_y = (double *) realloc( graphe->y, sizeof(double) * nv_nbr_pts );
  if ( nv_y == NULL ) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, __FILE__, __FUNCTION__, "nv_y, r"e_AIGU"el");
    (void) fprintf(stderr, "%s, %s l.%u : Mmoire insuffisante pour rallouer la table des Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
  graphe->y = nv_y;

  /* On met  0 la nouvelle partie, si ncessaire */
  if ( nv_nbr_pts > graphe->nombre_point ) {
    unsigned long i;

    for ( i = graphe->nombre_point ; i < nv_nbr_pts ; i++ ) {
      nv_y[ i ] = 0.0;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Adapter la taille d'un graphe ----------------------------------\
 |    Renvoie VRAI si la modification a pu tre faite, FAUX sinon           |
 |    Dans le cas ou renvoie FAUX, le graphe n'a pas t modifi en gnral |
 |                                                                          |
 | graphe : le graphe que l'on veut modifier                                |
 | nv_nbr_pts : combien de points doit-il avoir maintenant ?                |
 \------------------------------------------------------------------------- */

boolean GRF_AdapteTaille(Graphe *graphe, unsigned long nv_nbr_pts)
{
  double *nv_x;
  int retour;

  if (graphe == NULL) return FAUX;
  if (nv_nbr_pts == graphe->nombre_point) return VRAI; /* Rien  faire */
  if (nv_nbr_pts < 2) {
    (void) BI_ERR_alerte(NULL, &ERR_TropPeuPoints, VRAI, nv_nbr_pts);
    return FAUX;
  }

  if ( GRF_SERIE == graphe->genre_donnees ) {
    if ( graphe->x != NULL ) { /* La table est externe -> ne peut pas tre modifie
				  Mais on la perd, pour tre sr de ne pas dborder aprs */
      graphe->x = NULL;
      graphe->x_min = 0.0;
      graphe->x_max = (double) ( graphe->nombre_point );
    }

    /* On adapte la table des Y */
    retour = GRF_AdapterTaille_Y_R( graphe, nv_nbr_pts );

    /* Termin */
    return ( retour != 0 ) ? ( FAUX ) : ( VRAI );
  }

  /* Doit-on adapter la table des X ? */
  if ( graphe->x != NULL ) {
    nv_x = (double *) realloc(graphe->x, sizeof(double) * nv_nbr_pts);
    if (nv_x == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "graphes.c", "GRF_adapte", "nv_x");
      return FAUX; /* Et le graphe n'a pas t modifi */
    }
    graphe->x = nv_x;
  }

  if ( GRF_FCT_REELLE == graphe->genre_donnees ) {
    retour = GRF_AdapterTaille_Y_R( graphe, nv_nbr_pts );
    if ( retour != 0 ) {
      if ( nv_nbr_pts < graphe->nombre_point )
	graphe->nombre_point = nv_nbr_pts; /* vite de dborder en X, car les X ont dj t rduits */
      return FAUX; /* Le graphe a t  moiti modifi, mais je ne vois pas trop
		      comment contourner le problme...
		      La demi-modif est gnante uniquement si nv_nbr_pts < nombre_point
		      mais c'est un cas qui devrait tre rare : realloc ne devrait pas chouer
		      si l'on cherche  _diminuer_ la place occupe !
		      Dans le cas ou nv_nbr_pts > nombre_point, il y aura juste de la place perdue pour les X
		      mais comme on travaille toujours avec nombre_point, ce n'est pas gnant. */
    }
  } else if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) { /* Graphe de type Complexe */
    Complexe *nv_y;

    nv_y = (Complexe *) realloc(graphe->y, sizeof(Complexe) * nv_nbr_pts);
    if (nv_y == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "graphes.c", "GRF_AdapteTaille", "nv_y, complexe");
      if ( nv_nbr_pts < graphe->nombre_point )
	graphe->nombre_point = nv_nbr_pts; /* vite de dborder en X, car les X ont dj t rduits */
      return FAUX; /* Le graphe a t  moiti modifi - mme remarque que ci-dessus. */
    }
    graphe->y = (double *) nv_y;
  } else {
    (void) fprintf(stderr, "GRF_AdapteTaille :  FAIRE --- graphes de type %d\n", graphe->genre_donnees);
  }

  /* Si l'on est l, tout s'est bien pass */
  if (nv_nbr_pts > graphe->nombre_point) { /* On met  zro la nouvelle partie */
    unsigned long i;
    register double *x;

    if ( GRF_FCT_REELLE == graphe->genre_donnees ) {
      for ( i = graphe->nombre_point, x = graphe->x + graphe->nombre_point; i < nv_nbr_pts ; i++, x++ ) {
	*x = 0.0;
      }
    } else if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) {
      register Complexe *y = ( (Complexe *) graphe->y ) + graphe->nombre_point;
      for (i = graphe->nombre_point, x = graphe->x + graphe->nombre_point; i < nv_nbr_pts ; i++, x++, y++) {
	*x = 0.0; y->re = 0.0; y->im = 0.0;
      }
    } else {
    }
  }

  graphe->nombre_point = nv_nbr_pts;

  /* On adapte les barres d'erreur, si ncessaire */
  GRF_AdapteTaille( graphe->erreur_x, nv_nbr_pts );
  GRF_AdapteTaille( graphe->erreur_y, nv_nbr_pts );

  /* Idalement, il faudrait voir aussi pour la matrice de covariance;
     Dans l'tat actuel, il semble improbable que le cas arrive, ceci dit... */

  return VRAI; /* A pu se faire sans problme */
}

/* -~-~-~-~-~-~-~-~ Oprations impliquant les types des graphes -~-~-~-~-~-~-~-~ */

/* -------- Envoie le nom associ au type de graphe considr -------- */

const char *GRF_TexteTypeGraphe(int type)
{
  if ( type >= 0 ) {
    if ( type < NOMBRE_TYPES_GRAPHES ) {
      return nom_types_graphes[ type ];
    } else {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : Type de graphe inconnu [%d],  ajouter dans la table\n",
		     __FILE__, __FUNCTION__, __LINE__, type);
#endif
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
      return "Type  rajouter";
    }
  }

  if ( GR_INTERNE   == type ) return nom_type_interne;
  if ( GR_SSTYPE    == type ) return nom_type_sstype;
  if ( GR_AUTRETYPE == type ) return nom_type_autre;

#ifdef _DEVELOPPE_
  (void) fprintf(stderr, "%s,%s l.%u : Type de graphe inconnu [%d]\n",
		 __FILE__, __FUNCTION__, __LINE__, type);
#endif
  return "Type de graphe inconnu";
}

/* -------- Vrifie s'il existe (au moins) un graphe d'un type donn ------\
 |  Renvoie VRAI s'il existe au moins un tel graphe, FAUX sinon            |
 |                                                                         |
 | type = le type choisi                                                   |
 | strict = VRAI si on est strict, FAUX si on accepte les GR_SSTYPE        |
 | serie  = la srie dans laquelle chercher (NULL = toutes les sries)     |
 \------------------------------------------------------------------------ */

boolean GRF_type_existe(int type, boolean strict, Serie_Graphes *serie)
{
  Graphe *graphe;

  if (serie == NULL) { /* On fait toutes les sries */
    Serie_Graphes *s;

    for (s = premiere_serie ; s != NULL ; s = s->suivante) {
      if ( GRF_type_existe(type, strict, s) == VRAI )
	return VRAI;
    }

    return FAUX; /* Trouv dans aucune des sries... */
  }

  /* On cherche dans la seule srie indique */
  if ( type >= 0 ) { /* Types classiques */
    for (graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
      if (graphe->type == type) {
	return VRAI;
      }

      if ( (strict == FAUX) && (GR_SSTYPE == graphe->type) )
	return VRAI;
    }
  }

  return FAUX;
}

/* -------- Compte le nombre de graphes d'un type donn en mmoire --------\
 |  Renvoie le nombre de graphes compt                                    |
 |                                                                         |
 | type = le type choisi                                                   |
 | strict = VRAI si on est strict, FAUX si on accepte les GR_SSTYPE        |
 | serie  = la srie dans laquelle chercher (NULL = toutes les sries)     |
 \------------------------------------------------------------------------ */

unsigned long GRF_compter_taille(unsigned long nombre_points, struct serie *serie)
{
  unsigned long nombre = 0;
  Graphe *graphe;

  if ( NULL == serie ) {  /* Dcompte dans toutes les sries */
    struct serie *idx_serie;

    for ( idx_serie = premiere_serie ; idx_serie != NULL ; idx_serie = idx_serie->suivante ) {
      nombre += GRF_compter_taille( nombre_points, idx_serie );
    }
    return nombre;
  }

  /* Ici, dcompte dans la srie indique */
  for ( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    if ( nombre_points == graphe->nombre_point ) {
      nombre++;
    }
  }

  /* Termin */
  return nombre;
}

/* -------- Compte le nombre de graphes d'un type donn en mmoire --------\
 |  Renvoie le nombre de graphes compt                                    |
 |                                                                         |
 | type = le type choisi                                                   |
 | strict = VRAI si on est strict, FAUX si on accepte les GR_SSTYPE        |
 | serie  = la srie dans laquelle chercher (NULL = toutes les sries)     |
 \------------------------------------------------------------------------ */

unsigned long GRF_compter_type(int type, boolean strict, Serie_Graphes *serie)
{
  Graphe *graphe;
  unsigned long compte = 0;

  if (serie == NULL) { /* On fait toutes les sries */
    Serie_Graphes *s;

    for (s = premiere_serie ; s != NULL ; s = s->suivante) {
      compte += GRF_compter_type(type, strict, s);
    }

    return compte;
  }

  /* On compte dans la seule srie indique */
  if ( (type >= 0) || (type == GR_SSTYPE) || (type == GR_AUTRETYPE) ) { /* Types classiques */
    for (graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
      if (graphe->type == type) {
	compte++;
      } else if ( (strict == FAUX) && (GR_SSTYPE == graphe->type) ) {
	compte++;
      }
    }
  } else if (type == GR_TOUS) { /* Tous les graphes, sauf les internes */
    compte = serie->nbr_graphes;
    for ( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
      if (GR_INTERNE == graphe->type) compte--;
    }
  } else if (type == GR_TT_EXAFS) {
    for (graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
      if (graphe->drapeaux.exafs == VRAI) {
	compte++;
	continue;
      }

      if ( (strict == FAUX) && (GR_SSTYPE == graphe->type) )
	compte++;
    }    
  }

  return compte;
}

/* ~-~-~-~-~-~-~-~- Divers contrles sur les graphes ~-~-~-~-~-~-~-~- */

/* -------- Contrle des genres de donnes -------- */

boolean GRF_ControleGenre(Graphe **graphes, unsigned long nombre, int *genre)
{
  unsigned long i;
  int genre_ref;

  if ( NULL == graphes ) return FAUX;

  genre_ref = graphes[ 0 ]->genre_donnees;
  for ( i = 1 ; i < nombre ; i++ ) {
    if ( graphes[ i ]->genre_donnees != genre_ref )
      return FAUX;
  }

  /* Si on est l : tous les graphes ont le meme genre */
  if ( genre != NULL ) *genre = genre_ref;
  return VRAI;
}

/* -------- Contrle des nombres de points ---------------------------------------\
 |  Renvoie VRAI si tous les graphes ont le mme nombre de points                 |
 |            ou si la diffrence est acceptable                                  |
 |                                                                                |
 | graphes    : les graphes  contrler - il y en a nombre                        |
 | correction : si les nombres de points sont diffrents, que doit-on faire ?     |
 | resultat   : aprs troncature ou somme, le nombre de points finalement retenu  |
 | message    : si VRAI, affiche un message pour avertir qu'il y a un problme    |
 \------------------------------------------------------------------------------- */

boolean GRF_ControleTaille(Graphe **graphes, unsigned long nombre, int correction, unsigned long *resultat, boolean message)
{
  unsigned long nombre_points, i;

  if ( NULL == graphes ) return FAUX;

  nombre_points = graphes[0]->nombre_point;

  if ( GRF_CONTROLE_N_SOMME == correction ) { /* Somme des N : marche toujours... */
    if ( NULL == resultat ) return FAUX; /* Mais on a forcment besoin de a... */

    for (i = 1 ; i < nombre ; i++)
      nombre_points += graphes[i]->nombre_point;

    *resultat = nombre_points;
  }

  if ( GRF_CONTROLE_N_STRICT == correction ) { /* Il faut exactement le mme nombre de points */
    for (i = 1 ; i < nombre ; i++) {
      if ( graphes[i]->nombre_point != nombre_points ) {
	if ( VRAI == message )
	  (void) BI_ERR_alerte(NULL, &ERR_TailleDifferenteStrict, VRAI);
	return FAUX;
      }
    }

    if (resultat != NULL) *resultat = nombre_points;
  }

  if ( GRF_CONTROLE_N_MINIMAL == correction ) { /* On peut s'arranger en prenant le minimum */
    int reponse = -1;

    for (i = 1 ; i < nombre ; i++) {
      if ( graphes[i]->nombre_point != nombre_points ) {
	if ( reponse < 0 ) {
	  reponse = BI_ERR_alerte(NULL, &ERR_TailleDifferente, VRAI);

	  if (2 == reponse) return FAUX; /* On n'accepte pas l'arrangement */
	}

	if ( graphes[i]->nombre_point < nombre_points) nombre_points = graphes[i]->nombre_point;
      }
    }

    if (resultat != NULL) *resultat = nombre_points;
  }

  if ( GRF_CONTROLE_N_MAXIMAL == correction ) { /* On peut s'arranger en prenant le maximum */
    int reponse = -1;

    for (i = 1 ; i < nombre ; i++) {
      if ( graphes[i]->nombre_point != nombre_points ) {
	if ( reponse < 0 ) {
	  (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Adapter le message\n",
			 __FILE__, __FUNCTION__, __LINE__);
	  reponse = BI_ERR_alerte(NULL, &ERR_TailleDifferente, VRAI);

	  if (2 == reponse) return FAUX; /* On n'accepte pas l'arrangement */
	}

	if ( graphes[i]->nombre_point > nombre_points) nombre_points = graphes[i]->nombre_point;
      }
    }

    if (resultat != NULL) *resultat = nombre_points;
  }

  return VRAI;
}

/* -------- Contrle des types de donnes -------- */

boolean GRF_ControleType(Graphe **graphes, unsigned long nombre, boolean strict, int *type)
{
  unsigned long i;
  int type_ref;

  if ( NULL == graphes ) return FAUX;

  type_ref = graphes[ 0 ]->type;
  for ( i = 1 ; i < nombre ; i++ ) {
    if ( graphes[ i ]->type != type_ref ) {
      if ( VRAI == strict ) return FAUX; /* Il faut vraiment exactement le meme type */

      if ( GR_SSTYPE == type_ref )
	type_ref = graphes[ i ]->type;
      else if ( graphes[ i ]->type != GR_SSTYPE ) /* Ils sont vraiment diffrents */
	return FAUX;
    }
  }

  /* Si on est l : tous les graphes ont le meme genre */
  if ( type != NULL ) *type = type_ref;
  return VRAI;
}

/* -------- Contrle des intervalles en X ----------------------------------------\
 |  Renvoie VRAI si l'intervalle final obtenu est non-vide                        |
 |                                                                                |
 | graphes      : les graphes  contrler - il y en a nombre                      |
 | etendue      : doit-on regarder la runion ou l'intersection des intervalles ? |
 | x_min, x_max : l'intervalle finalement obtenu                                  |
 \------------------------------------------------------------------------------- */

boolean GRF_ControleX(Graphe **graphes, unsigned long nombre, int etendue, double *x_min, double *x_max)
{
  unsigned long i;
  double min, max;

  if ( NULL == graphes ) return FAUX;

  min = graphes[0]->x_min;
  max = graphes[0]->x_max;

  if ( GRF_CONTROLE_X_INTER == etendue ) { /* L'intersection */
    for ( i = 1 ; i < nombre ; i++ ) {
      if ( graphes[i]->x_min > min ) min = graphes[i]->x_min;
      if ( graphes[i]->x_max < max ) max = graphes[i]->x_max;
    }

    /* Contrle : l'intervalle ainsi obtenu est-il valide (max > min) ? */
    if ( min >= max ) {
      (void) BI_ERR_alerte(NULL, &ERR_GraphesDisjoints, VRAI);
      return FAUX;
    }
  }

  if ( GRF_CONTROLE_X_UNION == etendue ) { /* La runion : tout simple et marche toujours... */
    for ( i = 1 ; i < nombre ; i++ ) {
      if ( graphes[i]->x_min < min ) min = graphes[i]->x_min;
      if ( graphes[i]->x_max > max ) max = graphes[i]->x_max;
    }
  }

  if ( x_min != NULL ) *x_min = min;
  if ( x_max != NULL ) *x_max = max;

  return VRAI;
}

/* -------- Comparer les intervalles en X de deux graphes ---------------\
 |  Renvoie VRAI si les deux graphes ont exactement les mmes X          |
 |                                                                       |
 | graphe_1, graphe_2 : les deux graphes  comparer                      |
 \---------------------------------------------------------------------- */

boolean GRF_ComparerX(const Graphe *graphe_1, const Graphe *graphe_2)
{
  unsigned long i;
  double *x_1, *x_2;

  if ( graphe_1->nombre_point != graphe_2->nombre_point ) {
    return FAUX; /* Les X ne peuvent tre compatibles : il n'y en a pas le mme nombre ! */
  }

  if ( ( graphe_1->x_min != graphe_2->x_min ) || ( graphe_1->x_max != graphe_2->x_max ) ) {
    return FAUX; /* Les X ne peuvent tre compatibles : l'un des graphes, au moins, dborde ! */
  }

  for ( i = 0, x_1 = graphe_1->x, x_2 = graphe_2->x ; i < graphe_1->nombre_point ; i++, x_1++, x_2++ ) {
    if ( *x_1 != *x_2 )
      return FAUX;
  }

  /* On a pass toutes les vrifications : on est cohrent en X */
  return VRAI;
}

/* -------- Associer un graphe  une fentre --------
   Renvoie 0    si tout s'est bien pass
           != 0 en cas de problme

   fenetre = la fentre  laquelle associer un graphe
   graphe  = le graphe  associer [NULL : faire sauter la liaison !]
 */

int FEN_GRF_LierGraphe(Fenetre *fenetre, const Graphe *graphe)
{
  void *info_existante;

  if ( NULL == fenetre ) return -1;

  info_existante = FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  if ( info_existante != NULL ) {
    if ( FAUX == fenetre->drapeaux.util_3 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Il y a dj autre chose associ  la fentre, qui n'est pas un graphe !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return -5;
    }

    /* On cherche  remplacer un graphe par un autre ! */
    if ( graphe != NULL ) {
      AVERTISSEMENT( "On va remplacer le graphe %s par le graphe %s", ( (Graphe *) info_existante )->nom, graphe->nom );
    }
  }

  if ( NULL == graphe ) {
    /* On supprime, en fait, le lien entre le graphe et la fentre */
    FEN_StockerComplement( fenetre, FEN_INFOS_GRAPHE, NULL );
    fenetre->drapeaux.util_3 = FAUX;
  } else {
    /* On associe le graphe  la fentre */
    FEN_StockerComplement( fenetre, FEN_INFOS_GRAPHE, graphe );
    fenetre->drapeaux.util_3 = VRAI;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Associer un chemin ou un nom de fichier au graphe --------
   On copie la chaine passee [car : 1) appelant peut l'effacer et
                                    2) on doit la manipuler...]
 */

int GRF_AssocierFichier(Graphe *graphe, const char *chemin, boolean avec_dossier)
{
  char *fin_racine, *nom_fichier;

  if ( NULL == graphe ) return +1;
  if ( NULL == chemin ) {
    /* On efface le chemin associe au graphe */
    free( graphe->nom_fichier ); graphe->nom_fichier = NULL;
    if ( VRAI == avec_dossier ) {
      free( graphe->nom_dossier ); graphe->nom_dossier = NULL;
    }

    /* Termine, sans souci */
    return 0;
  }

  /* On fait l'association */
  fin_racine = strrchr( chemin, '/' );
  if ( NULL == fin_racine ) {
    /* Nom de fichier uniquement */
    free( graphe->nom_fichier );
    graphe->nom_fichier = strdup( chemin );

    /* Termine, sans souci */
    return 0;
  }

  /* On recupere le nom du fichier */
  nom_fichier = strdup( fin_racine + 1 );
  (void) fprintf( stderr, "%s,%s l.%u : nom du fichier = %s\n",
		  __FILE__, __FUNCTION__, __LINE__, nom_fichier );
  if ( NULL == nom_fichier ) {
    (void) fprintf( stderr, "%s,%s l.%u : Impossible d'allouer la memoire pour le nom du fichier...\n",
		    __FILE__, __FUNCTION__, __LINE__);
  } else {
    free( graphe->nom_fichier );
    graphe->nom_fichier = strdup( chemin );
  }

  if ( VRAI == avec_dossier ) {
    /* On veut aussi memoriser le nom de dossier */
    char *nom_dossier;

    nom_dossier = strdup( chemin );
    nom_dossier[ fin_racine - chemin + 1 ] = 0;
    (void) fprintf( stderr, "%s,%s l.%u : nom du dossier = %s\n",
		    __FILE__, __FUNCTION__, __LINE__, nom_dossier );
    if ( NULL == nom_dossier ) {
      
    } else {
      free( graphe->nom_dossier );
      graphe->nom_dossier = nom_dossier;
    }
  }

  /* Termine, sans souci */
  return 0;
}
