/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Regroupement des graphes en sries
   Les sries forment un niveau suprieur aux graphes
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "interface.h"		/*  SUPPRIMER  TERME ! Pour I_SERIE_GRF_Aller() */

#include "atomes.h"
#include "images.h"
#include "fichiers.h" /* Pour sauver() */
#include "serie.h"

#include "series.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

Serie_Graphes *SERIE_GRF_CreerSerie(Serie_Graphes *avant, const char *nom)
{
  Serie_Graphes *serie;

  /* calloc() -> init  0 */
  serie = (Serie_Graphes *) calloc( 1, sizeof(Serie_Graphes) );
  if ( NULL == serie ) {
    ERR_MemoireInsuffisante( __FILE__, __FUNCTION__, __LINE__, sizeof( Serie_Graphes ), NULL );
    return NULL;
  }

  if ( nom != NULL ) {
    serie->nom = strdup(nom);
  } else {
    serie->nom = strdup(nom_serie_par_defaut);
  }
  if ( NULL == serie->nom ) {
    free( serie ); serie = NULL;
    ERR_MemoireInsuffisante( __FILE__, __FUNCTION__, __LINE__, (nom != NULL)?(strlen(nom)):(strlen(nom_serie_par_defaut)), NULL );
    return NULL;
  }

  /* Inits complmentaires : autre que 0 */
  serie->suivante = NULL;

  serie->premier_graphe = NULL;
  serie->dernier_graphe = NULL;

  serie->drapeaux.homogene = FAUX;
  serie->drapeaux.commun   = FAUX;
  serie->drapeaux.protegee = FAUX;

  serie->table_X = NULL;
  serie->erreurs_X = NULL;
  serie->legende_X = NULL;
  serie->unite_X = NULL;

  serie->modele = NULL;
  serie->carto  = NULL;

  if ( avant != NULL ) { /* On insre aprs celle indique */
    serie->suivante = avant->suivante;
    avant->suivante = serie;
  } else {
    if (premiere_serie == NULL) { /* On cre la premire srie */
      serie->suivante = NULL;
      premiere_serie = serie;
      I_SERIE_GRF_Aller( serie ); /* On y va pour tout initialiser */
    } else { /* On se place  la fin */
      Serie_Graphes *s;

      for (s = premiere_serie ; s->suivante != NULL ; s = s->suivante);
      serie->suivante = NULL;
      s->suivante = serie;
    }
  }

  return serie;
}

/* -------- Effacer une srie -------- */

boolean SERIE_GRF_EffacerSerie(Serie_Graphes *serie, boolean forcer)
{
  Serie_Graphes *s;

  if ( NULL == serie ) return VRAI;
  if ( ( VRAI == serie->drapeaux.protegee ) && ( FAUX == forcer ) ) {
    /* Srie protge : effacement impossible... */
    return FAUX;
  }

  /* On efface le modle associ au besoin... */
  MOL_effacer( serie->modele, VRAI, VRAI );
  serie->modele = NULL; /* Si on annule aprs... */

  /* On efface l'image associe au besoin... */
  IMG_EffacerImage( serie->carto, FAUX /* pour ne pas avoir de boucle sans fin ! */ );
  serie->carto = NULL;

  /* Effacer les graphes --- AVANT d'isoler la srie, sinon GRF_efface() croit qu'il s'agit d'un graphe isol
     On contrle quand mme que les graphes ont t sauvs avant d'effacer ! */
  SERIE_EffacerGraphes( serie );

  /* Dfaire le chanage */
  if ( serie == premiere_serie ) {
    premiere_serie = serie->suivante;
    if ( NULL == serie->suivante ) { /* On efface la dernire srie */
      serie_courante = NULL;
    } else if ( serie == serie_courante ) {
      serie_courante = serie->suivante;
    }
  } else {
    for ( s = premiere_serie ; s->suivante != serie ; s = s->suivante ) {
      if ( NULL == s ) { /* Problme dans le chanage : double effacement ? */
	fprintf( stderr, "%s : Incohrence dans le chanage des sries\n", __FUNCTION__ );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return FAUX;
      }
    }
    s->suivante = serie->suivante;
    if ( serie == serie_courante ) {
      if ( serie->suivante ) {
	serie_courante = serie->suivante;
      } else {
	serie_courante = s;
      }
    }
  }
  I_SERIE_GRF_Aller( serie_courante );

  /* On efface la table des X, si elle existe */
  if ( serie->table_X != NULL ) {
#ifdef _DEVELOPPE_
    if ( FAUX == serie->drapeaux.table_X ) {
      (void) fprintf(stderr, "%s,%s l.%u : Table des X, mais marque sans table des X\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }
#endif
    free( serie->table_X ); /* Elle ne devrait tre utilise nulle part, puisque tous les graphes ont t effacs ! */
    serie->table_X = NULL;
  }
  GRF_eff_isole( serie->erreurs_X );
  serie->erreurs_X = NULL;

  /* On efface le nom, s'il existe */
  free( serie->nom ); serie->nom = NULL;

  free( serie ); /* On n'utilise plus cette srie */
  return VRAI;
}

/* -------- Sauver les graphes d'une srie --------
 | Sauve tous les graphes d'une srie, chacun dans son propre fichier
 |  Renvoie 0 si tout s'est bien pass
 |
 | serie   : la srie dont on veut sauver les graphes
 | radical : le radical pour gnrer les noms de fichier
 | tous    :  VRAI = on sauve tous les graphes
 |            FAUX = on ne sauve que les graphes non-sauvs
*/

int SERIE_SauverGraphes(Serie_Graphes *serie, const char *radical, boolean tous)
{
  char *nom_fichier;
  Graphe *graphe;
  unsigned long compteur = 0;

  if ( NULL == serie ) return -1;
  if ( NULL == radical ) radical = "svauto_grf_";

  for ( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    if ( ( FAUX == tous ) && ( VRAI == graphe->drapeaux.sauve ) ) {
      /* Inutile de sauve ce graphe-ci */
      continue;
    }

    /* On sauve le graphe dans radical_xxx.lase */
    compteur++;
    nom_fichier = TXT_genere_chaine( "%s%5lu.lase", radical, compteur );
    GRF_SauverIsole( nom_fichier, graphe );
    free( nom_fichier ); nom_fichier = NULL;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Effacer les graphes d'une srie -------- */

int SERIE_EffacerGraphes( Serie_Graphes *serie )
{
  if ( NULL == serie ) return -1;
  if ( NULL == serie->premier_graphe ) return 0; /* Graphes dj effacs */

#ifdef _ancien_
  if ( VRAI == controle ) {
    Graphe *graphe;
    boolean tout_sauver = FAUX;
    int reponse;
    char *radical = NULL, *nom_fichier;
    unsigned long compteur;

    for ( graphe = serie->premier_graphe, compteur = 0 ; graphe != NULL ; graphe = graphe->suivant ) {
      if ( VRAI == graphe->drapeaux.sauve ) continue;

      /* Ici, le graphe n'a pas t sauv */
      if ( FAUX == tout_sauver ) {
	reponse = BI_ERR_alerte( NULL, &Question_SauverGraphes, VRAI, graphe->nom );
	if ( 5 == reponse ) return +1; /* Annulation : erreur, mais mineure */
	if ( 4 == reponse ) break; /* On efface ce graphe et les suivants */
	if ( 3 == reponse ) {
	  tout_sauver = VRAI;
	}
	if ( 2 == reponse ) continue; /* On passe au suivant */
      }

      /* Ici, on veut donc sauver le graphe */
      if ( NULL == radical ) {
	radical = SV_selecteur("*", "Radical des fichiers");
      }
      compteur++;
      nom_fichier = TXT_genere_chaine("%s%l.sav", radical, compteur);
      GRF_SauverIsole( nom_fichier, graphe );
      free( nom_fichier ); nom_fichier = NULL;
    }
  }
#endif
  /* Si l'on est l, on a sauv ce que l'on voulait sauver -> on peut effacer tranquillement */
  while ( serie->premier_graphe != NULL ) {
    GRF_efface( serie->premier_graphe, VRAI, FAUX );
  }

  /* Tout s'est bien pass */
  return 0;
}

/* --------  quelle srie appartient un graphe ? -------- */

Serie_Graphes *GRF_QuelleSerie(const Graphe *graphe)
{
  Serie_Graphes *serie;

  if (graphe == NULL) {
    return NULL;
  }

  /* Est-il isol ? */
  if ( VRAI == graphe->drapeaux.isole ) {
    /* Pas de message : mthode autorise pour dtecter un graphe isol */
    return NULL;
  }

  /* On retrouve le premier graphe de la chane */
  while ( graphe->precedent != NULL ) {
    graphe = graphe->precedent;
  }

  /* Quelle srie contient ce graphe comme premier graphe ? */
  for ( serie = premiere_serie ; serie != NULL ; serie = serie->suivante ) {
    if ( serie->premier_graphe == graphe ) {
      return serie;
    }
  }

  (void) fprintf(stderr, "%s,%s l.%u : Graphe non isol, mais reliable  aucune srie [%s]\n",
		 __FILE__, __FUNCTION__, __LINE__, graphe->nom);
  FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  return NULL; /* Srie non trouve ! */
}

/* -------- Changer le nom d'une srie -------- */

void Serie_titre(Serie_Graphes *serie, const char *titre)
{
  if ( NULL == serie ) return;
  if ( NULL == titre ) return;

  free( serie->nom ); /* On libre l'ancien nom */
  serie->nom = strdup(titre);
}

/* ~-~-~-~-~-~-~-~-~ Grer une table de X commune aux graphes de la srie */

/* -------- Choisir une table de X et la mettre -------- */

int Serie_AssignerX(Serie_Graphes *serie)
{
  Graphe *idx_graphe, **graphes, **grf_choisi, *erreurs_X;
  boolean que_serie = VRAI;
  int retour;
  double *table = NULL, x_min, x_max;
  unsigned long nombre_points = 0, nombre_graphes = 0, nombre_choisis = 0;

  if ( NULL == serie ) return -1;

  /* 1. On contrle que cela a un sens... */
  if ( NULL == serie->premier_graphe ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de graphe dans la srie -> assigner des X n'a pas de sens\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +1;
  }

  /* 1-a : tous les graphes de la srie doivent avoir le mme nombre de point */
  nombre_points = serie->premier_graphe->nombre_point;
  for ( idx_graphe = serie->premier_graphe ; idx_graphe != NULL ; idx_graphe = idx_graphe->suivant ) {
    if ( nombre_points != idx_graphe->nombre_point ) {
      (void) BI_ERR_alerte( principale, &ERR_NombrePointsDifferent, VRAI );
      return -2;
    }
    if ( idx_graphe->genre_donnees != GRF_SERIE ) {
      que_serie = FAUX;
    }
  }

  /* 1-b : on ne perd pas d'information */
  if ( FAUX == que_serie ) {
    OPTION_A_FAIRE( "Demander si l'on accepte de perdre les X existants" );
  }

  /* 2. On choisit le graphe qui va donner ses X  la srie */
  /* 2-a : on prpare la liste des graphes acceptables */
  graphes = GRF_ChoisirTaille( NULL, nombre_points, &nombre_graphes );
  if ( NULL == graphes ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun graphe n'a le bon nombre de point, ou erreur\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }

  /* 2-b : on demande  l'utilisateur quel graphe choisir parmi ceux-ci */
  grf_choisi = GRF_ChoisirDansTable( graphes, nombre_graphes, 0, &nombre_choisis );
  free( graphes ); graphes = NULL;
  if ( NULL == grf_choisi ) {
    (void) fprintf(stderr, "%s, %s l.%u : Annul ou problme\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +5; /* Annul = erreur mineure */
  }

  /* 3. Qu'utiliser dans le graphe : X ou Y ? */
  if ( GRF_SERIE == (*grf_choisi)->genre_donnees ) {
    table = (*grf_choisi)->y;
    x_min = (*grf_choisi)->y_min; x_max = (*grf_choisi)->y_max;
    erreurs_X = (*grf_choisi)->erreur_y;
  } else if ( GRF_FCT_REELLE == (*grf_choisi)->genre_donnees ) {
    retour = BI_ERR_alerte( NULL, &Question_XouY, VRAI );
    if ( 1 == retour ) {	/* Utiliser les X */
      table = (*grf_choisi)->x;
      x_min = (*grf_choisi)->x_min; x_max = (*grf_choisi)->x_max;
      erreurs_X = (*grf_choisi)->erreur_x;
    } else {			/* Utiliser les Y */
      table = (*grf_choisi)->y;
      x_min = (*grf_choisi)->y_min; x_max = (*grf_choisi)->y_max;
      erreurs_X = (*grf_choisi)->erreur_y;
    }
  } else {
    table = (*grf_choisi)->x;
    x_min = (*grf_choisi)->x_min; x_max = (*grf_choisi)->x_max;
    erreurs_X = (*grf_choisi)->erreur_x;
  }

  /* Plus besoin de lui ! */
  free( grf_choisi ); grf_choisi = NULL;

  /* On enlve une ventuelle ancienne table */
  Serie_EnleverTable( serie );

  /* On assigne  la srie la table choisie [en faisant une _copie_] */
  retour = Serie_AssocierTable( serie, nombre_points, table, VRAI );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible d'associer la table !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1054;
  }
  if ( erreurs_X != NULL ) {
    serie->erreurs_X = GRF_Copier( erreurs_X, NULL, VRAI );
  }

  /* On associe ces X  chaque graphe de la srie */
  for ( idx_graphe = serie->premier_graphe ; idx_graphe != NULL ; idx_graphe = idx_graphe->suivant ) {
    /* On convertit le graphe en graphe de type srie (X implicites ou, ici, externes) */
    retour = GRF_ChangerGenre( idx_graphe, GRF_SERIE );
    if ( 0 == retour ) {
      /* On assigne les X de la srie  ce graphe, si la transformation a russi */
      idx_graphe->x = serie->table_X;
      idx_graphe->x_min = x_min;
      idx_graphe->x_max = x_max;

      idx_graphe->drapeaux.X_serie = VRAI;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Associer une table des X -------- */

int Serie_AssocierTable(Serie_Graphes *serie, unsigned long nbr_points, double *table_X, boolean copier)
{
  if ( ( NULL == serie) || ( NULL == table_X ) || ( 0 == nbr_points ) ) return -1;

  if ( VRAI == serie->drapeaux.table_X ) {
    (void) fprintf(stderr, "%s,%s l.%u : La srie %s a dj une table des X\n",
		   __FILE__, __FUNCTION__, __LINE__, serie->nom );
#ifdef _DEVELOPPE_
    if ( NULL == serie->table_X ) {
      (void) fprintf(stderr, "%s,%s l.%u : Srie marque avec table, mais sans table !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    }
#endif
    return -2;
  }

  if ( VRAI == copier ) {
    serie->table_X = (double *) malloc( sizeof( double ) * nbr_points );
    if ( NULL == serie->table_X ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante pour copier la table\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -3;
    }

    memcpy( serie->table_X, table_X, sizeof( double ) * nbr_points );
  } else {
    serie->table_X = table_X;
  }
  serie->nbr_points = nbr_points;

  /* La srie contient donc une table des X */
  serie->drapeaux.table_X = VRAI;

  /* Tout s'est pass sans problme */
  return 0;
}

/* -------- Enlever une table des X -------- */

int Serie_EnleverTable(Serie_Graphes *serie)
{
  Graphe *graphe;

  if ( NULL == serie ) return -1;

  if ( FAUX == serie->drapeaux.table_X ) {
    if ( serie->table_X != NULL ) {
      (void) fprintf(stderr, "%s,%s l.%u : Srie %s avec table des X, non-marque comme telle\n",
		     __FILE__, __FUNCTION__, __LINE__, serie->nom);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return -50;
    }
    return 0;
  }

  /* Si des graphes l'utilisent, on les adapte ! */
  for ( graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant ) {
    if ( graphe->genre_donnees != GRF_SERIE ) continue; /* Seul un graphe de type srie peut avoir une table des X externe ! */

    if ( graphe->x == serie->table_X ) {
      graphe->x = NULL;
      graphe->x_min = 0; /* X implicites : numro de mesure */
      graphe->x_max = (double) (graphe->nombre_point - 1);
      graphe->erreur_x = NULL;

      /* Les X ne sont plus partags par la srie */
      graphe->drapeaux.X_serie = FAUX;

      GRF_MiseAJour( graphe, VRAI );
    }
  }

  /* On peut maintenant la librer */
  GRF_eff_isole( serie->erreurs_X ); serie->erreurs_X = NULL;
  free( serie->table_X ); serie->table_X = NULL;
  serie->nbr_points = 0;

  /* La srie ne contient plus de table */
  serie->drapeaux.table_X = FAUX;

  /* Tout s'est pass sans problme */
  return 0;
}

/* ~-~-~-~-~-~-~-~ Liens avec les images ~-~-~-~-~-~-~-~ */

int Serie_AssocierImage(Serie_Graphes *serie, struct image *image)
{
  (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Associer l'image %s  la srie %s\n",
		 __FILE__, __FUNCTION__, __LINE__, image->nom, serie->nom);

  return -25;
}

int Serie_DissocierImage(Serie_Graphes *serie)
{
  Image *image;

  if ( NULL == serie ) return 0;
  if ( NULL == serie->carto ) return +1; /* Erreur mineure */

  image = serie->carto;
  if ( ( IMG_GENRE_GRAPHE != image->genre ) && ( IMG_GENRE_DIAGRAMME != image->genre ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : L'image ne contient pas des graphes !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On dprotge la srie
     ATTENTION, pas de contrle  l'effacement des graphes
     ==> si les graphes de image->valeurs.G ne sont pas mis  NULL, plantage assur ! */
  serie->drapeaux.protegee = FAUX;

  /* On enlve le lien  l'image */
  serie->carto = NULL;

  /* Tout s'est bien pass */
  return 0;
}
