/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Prise en compte des dimensions non-ponctuelles du faisceau incident
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "images.h"
#include "science.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Image *conv_ss_grille(Image *original, float lg_faisceau, float ht_faisceau);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Image *convoluer_image(Image *original)
{
  float larg_faisceau, haut_faisceau, angle_faisceau;
  Image *image = NULL;

  if (original == NULL) return NULL;

  printf("Largeur du faisceau (m) :\n");
  scanf("%f", &larg_faisceau);
  printf("hauteur du faisceau (m) :\n");
  scanf("%f", &haut_faisceau);
  printf("Angle faisceau/chantillon () :\n");
  scanf("%f", &angle_faisceau);

  /* On passe entre 0 et 360 */
  while (angle_faisceau >= 360.0) angle_faisceau -= 360.0;
  while (angle_faisceau < 0.0) angle_faisceau += 360.0;
  if ( (angle_faisceau == 0.0) || (angle_faisceau == 180.0) ) {
    EXF_ERREUR(DIV_ZERO, "Angle nul", txt_err_calc_annule);
    return NULL;
  }
  larg_faisceau /= sin( angle_faisceau * M_PI / 180.0);

  if ( (larg_faisceau <= original->l_reel) && (haut_faisceau <= original->h_reel) ) {
    printf("Prcision de l'image originale insuffisante - calcul annul\n");
    return NULL;
  }

  conv_ss_grille(original, larg_faisceau, haut_faisceau);

  return image;
}

/* -------- Convolution sans grille impose --------
     Renvoie l'image rsultante

     L'image finale a le mme nombre de points et la mme grille que l'originale
     En chaque point, on fait la moyenne des points autour
     C'est assez brutal, mais en attendant mieux...
   -------- */

static Image *conv_ss_grille(Image *original, float lg_faisceau, float ht_faisceau)
{
  int nc_x, nc_y; /* tendu du faisceau en cases */
  int x, y, i, j, nbr_cases;
  unsigned long n_case;
  Image *image;

  if (original == NULL) return NULL;
  if ( original->genre != IMG_GENRE_REELLE ) {
    (void) fprintf(stderr, "%s,%s l.%u : L'image %s ne contient pas des valeurs relles\n",
		   __FILE__, __FUNCTION__, __LINE__, original->nom );
    return NULL;
  }

  nc_x = (int) rint(lg_faisceau / original->l_reel);
  nc_y = (int) rint(ht_faisceau / original->h_reel);
  if ( (nc_x < 3) && (nc_y < 3) ) {
    fprintf(stderr, "conv_ss_grille : le faisceau couvre moins de trois cases dans chaque direction, calcul impossible\n");
    return NULL;
  }
  nc_x >>= 1; nc_y >>= 1;
  nbr_cases = (2 * nc_x + 1) * (2 * nc_y + 1);

  image = IMG_CreerImage( NULL, original, original->genre, original->type, original->larg, original->haut);
  if (image == NULL) return NULL;

  image->l_reel = original->l_reel;
  image->h_reel = original->h_reel;
/*  image->x_centre = original->x_centre;
  image->y_centre = original->y_centre; */

  for (x = 0 ; x < (int) image->larg ; x++) {
    for (y = 0 ; y < (int) image->haut ; y++) {

      IMG_IJ_v_N( image, x, y, &n_case );

      image->valeurs.R[ n_case ] = 0.0;

      for (i = -nc_x ; i <= nc_x ; i++) {
	if ( ( (x + i) < 0 ) || ( (x + i) >= (int) image->larg ) )
	  continue;

	for (j = -nc_y ; j <= nc_y ; j++) {
	  unsigned long n_case_C;

	  if ( ( (y + j) < 0 ) || ( (y + j) >= (int) image->haut ) )
	    continue;

	  IMG_IJ_v_N( image, x + i, y + j, &n_case_C );
	  image->valeurs.R[ n_case ] += original->valeurs.R[ n_case_C ];
	}
      }

      image->valeurs.R[ n_case ] /= nbr_cases;
    }
  }

  IMG_bornes(image);
  return image;
}
