/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Création : 11 avril 2001

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Création, effacement des images (gestion "fondamentale")
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"
#include "images.h"
#include "matrices.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Conversion d'une matrice de covariance ------- */

Image *IMG_CovarianceVersImage(Graphe *graphe, boolean correlation)
{
  boolean erreur __attribute__((__unused__)) = FAUX;
  double pas;
  Image *image;

  if (NULL == graphe) return NULL;
  if (NULL == graphe->covariance) {
    (void) fprintf(stderr, "Le graphe n'a pas de matrice de covariance...\n");
    return NULL;
  }

  /* Conversion de la matrice */
  image = IMG_MatriceVersImage( graphe->covariance );
  if (NULL == image) return NULL;

  /* Le centre : dépend du graphe... */
  image->x_min = graphe->x_min;
  image->y_min = graphe->x_min;

  /* On essaye de trouver le pas.
     Comme la grille de l'image _doit_ être régulière, correct seulement pour un pas constant dans le graphe... */
  pas = ( graphe->x_max - graphe->x_min) / ( (double) (graphe->nombre_point - 1) );
  if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) {
    /* À cause des parties réelles & imaginaires, il y a 2 fois plus de points...
       Le truc ci-dessous permet de bien aller de min à max, mais ce n'est pas tip-top... */
    pas /= 2.0;
  }
  if (pas > 0.0) {
    image->l_reel = pas;
    image->h_reel = pas;
  }

  if ( VRAI == correlation ) { /* On veut la matrice de corrélation... */
    unsigned int i, j;
    unsigned long n_case, n_case_D;

    for (i = 0 ; i < image->larg ; i++) {
      double sigma;

      IMG_IJ_v_N( image, i, i, &n_case_D );

      sigma = (double) image->valeurs.R[ n_case_D ];
      if ( sigma <= 0.0 ) {
	erreur = VRAI;
	continue;
      }
      sigma = sqrt(sigma); /* Écart-type */

      /* Moitié inférieure seulement */
      /* On s'occupe de la ligne */
      for (j = 0 ; j < i ; j++) {
	IMG_IJ_v_N( image, i, j, &n_case );

	image->valeurs.R[ n_case ] /= sigma;
      }
      /* On s'occupe de la colonne */
      for (j = i + 1 ; j < image->haut ; j++) {
	IMG_IJ_v_N( image, j, i, &n_case );

	image->valeurs.R[ n_case ] /= sigma;
      }
    }

    /* Par symétrie, la moitié supérieure */
    for (i = 0 ; i < image->larg ; i++) {
      for (j = 0 ; j < i ; j++) {
	IMG_IJ_v_N( image, i, j, &n_case );
	IMG_IJ_v_N( image, j, i, &n_case_D );

	image->valeurs.R[ n_case_D ] = image->valeurs.R[ n_case ];
      }

      /* Diagonale : forcément 1 */
      IMG_IJ_v_N( image, i, i, &n_case_D );
      image->valeurs.R[ n_case_D ] = 1.0;
    }

    image->val_min = -1.0;
    image->val_max = 1.0;
  }

  IMG_AllerImage( image );
  return image;
}

/* -------- Conversion d'une matrice, cas général -------- */

Image *IMG_MatriceVersImage(Matrice *matrice)
{
  Image *image;
  unsigned int x, y;
  unsigned long n_case;

  if (NULL == matrice) return NULL;

  image = IMG_CreerImage( NULL, NULL, IMG_GENRE_REELLE, IMG_TYPE_MATRICE, matrice->n_lignes, matrice->n_colonnes);
  if ( NULL == image )
    return NULL;

  image->l_reel = 1.0;
  image->h_reel = 1.0;

  for (x = 0 ; x < matrice->n_lignes ; x++ ) {
    for (y = 0 ; y < matrice->n_colonnes ; y++) {
      IMG_IJ_v_N( image, x, y, &n_case );
      image->valeurs.R[ n_case ] = lit_element(matrice, x, y);
    }
  }
  IMG_bornes(image);

  return image;
}
