/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS
   Cration : 16 juillet 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Cration des images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "echantillon.h"	/* Pour les cartes de composition ! */
#include "graphes.h" /* Pour GRF_Copier() */
#include "images.h"
#include "serie.h" /* Pour les images de type graphe ou apparent */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Image *IMG_ReserverImage(int genre, int type, unsigned int larg, unsigned int haut);
static int    IMG_ReserverContenu(Image *image);

static int IMG_R_v_C    (Image *image);
static int IMG_R_v_Compo(Image *image, struct composant *matrice, struct composant *compose);

static int IMG_C_v_R(Image *image);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Cration d'une image dans une srie -------- */

Image *IMG_CreerImage(Serie_Images *serie, Image *avant, int genre, int type, unsigned int larg, unsigned int haut)
{
  Image *image;
  int retour;

  ENTREE_FONCTION( __FUNCTION__, "serie=%p avant=%p genre=%d type=%d larg=%u haut=%u", serie, avant, genre, type, larg, haut);

  /* On essaye de reprer la srie */
  if ( NULL == serie ) {
    if ( avant != NULL ) {
      serie = IMG_QuelleSerie( avant );
      if ( NULL == serie ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver la srie de l'image  la suite de laquelle on doit insrer...\n",
		       __FILE__, __FUNCTION__, __LINE__);
	SORTIE_FONCTION( __FUNCTION__, "%p [Srie introuvable / avant]", NULL);
	return NULL;
      }
    } else {
      serie = IMG_serie_courante;
      if ( NULL == serie ) { /* On cre donc une nouvelle srie */
	serie = IMG_SERIE_CreerSerie( NULL, "Images en vrac" );
      }
    }
  }
  if ( NULL == serie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver une srie convenable\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%p [Srie introuvable / avant]", NULL);
    return NULL;
  }

#ifdef _DEVELOPPE_
  if ( ( serie != NULL ) && ( avant != NULL ) ) {
    Image *img;

    /* Contrle de cohrence */
    for ( img = serie->premiere_image ; img != avant ; img = img->suivante ) {
      if ( NULL == img ) {
	(void) fprintf(stderr, "%s,%s l.%u : avant [%s] n'appartient pas  la srie indique [%s]\n",
		       __FILE__, __FUNCTION__, __LINE__, avant->nom, serie->nom );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
    }
  }
#endif

  /* On cre l'image demande, comme isole dans un premier temps */
  image = IMG_CreerImageIsolee( genre, type, larg, haut );
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_CreerImageIsolee()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%p [ERREUR dans IMG_CreerImageIsolee()]", NULL);
    return NULL;
  }

  /* On essaye de l'insrer dans la srie indique, aprs l'image indique */
  retour = IMG_InsererDansSerie( image, serie, avant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_ChainerImage()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    IMG_EffacerImage( image, FAUX );
    SORTIE_FONCTION( __FUNCTION__, "%p [ERREUR dans IMG_ChainerImage()]", NULL);
    return NULL;
  }

  /* La cration a russi, tout va bien ! */
  SORTIE_FONCTION( __FUNCTION__, "%p", image);
  return image;
}

/* -------- Cration d'une image isole -------- */

Image *IMG_CreerImageIsolee(int genre, int type, unsigned int larg, unsigned int haut)
{
  Image *image;

  ENTREE_FONCTION( __FUNCTION__, "genre=%d type=%d larg=%u haut=%u", genre, type, larg, haut);

  /* On rserve l'image */
  image = IMG_ReserverImage( genre, type, larg, haut );
  if ( NULL == image ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_ReserverImage()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%p [ERREUR dans IMG_ReserverImage()]", NULL);
    return NULL;
  }

  /* L'image est pour l'instant hors de tout chanage */
  image->drapeaux.isolee = VRAI;

  /* On a fini ! */
  SORTIE_FONCTION( __FUNCTION__, "%p", image);
  return image;
}

/* -------- Copie d'une image -------- */

Image *IMG_CopierImage(Image *originale, Serie_Images *serie_arrivee, boolean isolee)
{
  Image *copie;
  unsigned long nbr_valeurs;

  if ( NULL == originale ) return NULL;

  if ( VRAI == isolee ) {
    copie = IMG_CreerImageIsolee( originale->genre, originale->type, originale->larg, originale->haut );
  } else {
    char *titre;

    if ( NULL == serie_arrivee )
      copie = IMG_CreerImage( NULL, originale, originale->genre, originale->type, originale->larg, originale->haut );
    else
      copie = IMG_CreerImage( serie_arrivee, NULL, originale->genre, originale->type, originale->larg, originale->haut );

    titre = TXT_genere_chaine( "Copie de %s", originale->nom );
    IMG_Titre( copie, titre );
    free( titre );
  }
  if ( NULL == copie ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer l'image-copie\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Mmes infos associes */
  copie->l_reel = originale->l_reel;
  copie->h_reel = originale->h_reel;
  copie->x_min = originale->x_min;
  copie->y_min = originale->y_min;

  /* Mmes valeurs */
  nbr_valeurs = copie->larg * (unsigned long) copie->haut;
  switch ( copie->genre )
  {
  case IMG_GENRE_REELLE:
    memcpy( copie->valeurs.R, originale->valeurs.R, sizeof( double ) * nbr_valeurs );
    break;

  case IMG_GENRE_COMPLEXE:
    memcpy( copie->valeurs.C, originale->valeurs.C, sizeof( Complexe ) * nbr_valeurs );
    break;

  case IMG_GENRE_GRAPHE:
  case IMG_GENRE_DIAGRAMME:
    memcpy( copie->valeurs.G, originale->valeurs.G, sizeof( struct graphe * ) * nbr_valeurs );
    break;

  case IMG_GENRE_PHOTO:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- photo\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Genre inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, copie->genre);
  }
  copie->val_min = originale->val_min;
  copie->val_max = originale->val_max;

  /* Infos complmentaires ventuelles */
  switch ( copie->type )
  {
  case IMG_TYPE_ERREURS:
    memcpy( copie->ddl, originale->ddl, sizeof( int ) * nbr_valeurs );
    break;
  }

  /* Les erreurs sont copies aussi, si possible */
  copie->erreurs = IMG_CopierImage( originale->erreurs, NULL, VRAI );

  /* ventuellement, infos d'affichage */
  if ( ( copie->infos_affichage != NULL ) && ( originale->infos_affichage != NULL ) ) {
    struct aff_image *infos_copie, *infos_originale;

    infos_copie = copie->infos_affichage;
    infos_originale = originale->infos_affichage;

    /* La coupe */
    infos_copie->xp_1 = infos_originale->xp_1;
    infos_copie->yp_1 = infos_originale->yp_1;
    infos_copie->xp_2 = infos_originale->xp_2;
    infos_copie->yp_2 = infos_originale->yp_2;

    infos_copie->coupe = GRF_Copier( infos_originale->coupe, NULL, VRAI );

    /* La couleur */
    infos_copie->couleur = infos_originale->couleur;

    /* Ce que l'on doit afficher */
    infos_copie->proprietes = infos_originale->proprietes;

    /* Les lgendes */
    IMG_ChangerLegende( copie, AXE_X, infos_originale->legende_X );
    IMG_ChangerLegende( copie, AXE_Y, infos_originale->legende_Y );
    IMG_ChangerLegende( copie, AXE_Z, infos_originale->legende_Z );

    /* La palette */
    if ( infos_copie->proprietes & IMG_AVEC_PALETTE ) {
      (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- copie de la palette\n",
		     __FILE__, __FUNCTION__, __LINE__);
    }
  }

  /* La copie a russi */
  return copie;
}

/* -------- Rserver et initialiser l'image -------- */

static Image *IMG_ReserverImage(int genre, int type, unsigned int larg, unsigned int haut)
{
  Image *nv_image = NULL;
  int retour;
  unsigned long nbr_valeurs;

  /* On rserve avec calloc -> moins d'initialisations... */
  nv_image = (Image *) calloc( 1, sizeof( Image ) );
  if (nv_image == NULL) {
    (void) fprintf(stderr, "IMG_ReserverImage : Impossible de rserver une image...\n");
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Images/gestion.c", "creer_image", "nv_image");
    return NULL;
  }

  nv_image->genre = genre;
  nv_image->type = type;

  /* Mmorisation des dimensions */
  nv_image->larg = larg;
  nv_image->haut = haut;

  /* Rservation de la mmoire pour le contenu */
  retour = IMG_ReserverContenu( nv_image );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_ReserverContenu(), code %d\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    free( nv_image );
    return NULL;
  }

  /* Combien de valeurs dans l'image ? */
  nbr_valeurs = larg * (unsigned long) haut;

  /* En fonction du type : infos complmentaires ? */
  switch ( type )
  {
  case IMG_TYPE_ERREURS:
    nv_image->ddl = (int *) calloc( sizeof(int), nbr_valeurs );
    if ( NULL == nv_image->ddl ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver les ddl\n",
		     __FILE__, __FUNCTION__, __LINE__);
      free( nv_image->valeurs.R );
      free( nv_image );
      return NULL;
    }
    break;
  }

  /* Initialisation des float : probablement inutile, mais rien ne prouve que 0.0 soit cod par des 0 ! */
  nv_image->l_reel = 0.0;
  nv_image->h_reel = 0.0;
  nv_image->val_max = 0.0; /* Image blanche au dbut */
  nv_image->val_min = 0.0;
  nv_image->x_min = 0.0; /* Par dfaut, x vont de 0  larg par pas de l_reel */
  nv_image->y_min = 0.0;

  /* Initialisation russie */
  return nv_image;
}

static int IMG_ReserverContenu(Image *image)
{
  unsigned long nbr_valeurs;

  /* Combien de valeurs faut-il rserver ? */
  nbr_valeurs = image->larg * (unsigned long) image->haut;
  if ( 0 == nbr_valeurs ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune valeur pour l'image !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On rserve les valeurs, en fonction du genre d'images ; calloc => init  0 [pas sre pour les doubles, mais...] */
  switch ( image->genre )
  {
  case IMG_GENRE_REELLE:
  case IMG_GENRE_ERREUR:
    image->valeurs.R = (double *) calloc( sizeof(double), nbr_valeurs );
    break;

  case IMG_GENRE_COMPLEXE:
    image->valeurs.C = (Complexe *) calloc( sizeof(Complexe), nbr_valeurs );
    break;

  case IMG_GENRE_GRAPHE:
    image->valeurs.G = (struct graphe **) calloc( sizeof( struct graphe * ), nbr_valeurs );
    printf( "Image de graphes, 1er graphe = %p\n", image->valeurs.G[ 0 ] );
    break;

  case IMG_GENRE_DIAGRAMME:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Init pour les diagrammes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case IMG_GENRE_PHOTO:
    (void) fprintf(stderr, "%s,%s l.%u :  FAIRE --- Init pour les photos\n",
		   __FILE__, __FUNCTION__, __LINE__);
    break;

  case IMG_GENRE_COMPO:
    image->valeurs.M = (struct matiere **) calloc( sizeof( struct matiere * ), nbr_valeurs );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Genre d'image inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, image->genre);
  }
  if ( NULL == image->valeurs.R ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver les donnes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -10;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Changer le genre des donnes d'une image ~-~-~-~-~-~-~-~-~ */

/* -------- Routine gnrique -------- */

int IMG_ChangerGenre(Image *image, int nv_genre, struct composant *matrice, struct composant *compose)
{
  int retour = 12345;

  if ( NULL == image ) return +1;
  if ( nv_genre == image->genre ) return 0; /* Rien  faire */

  /* Les cas spciaux */
  switch ( image->genre )
  {
  case IMG_GENRE_REELLE:
    switch ( nv_genre )
    {
    case IMG_GENRE_COMPLEXE:
      retour = IMG_R_v_C( image );
      break;

    case IMG_GENRE_COMPO:
      retour = IMG_R_v_Compo( image, matrice, compose );
      break;
    }
    break;

  case IMG_GENRE_COMPLEXE:
    switch ( nv_genre )
    {
    case IMG_GENRE_REELLE:
      retour = IMG_C_v_R( image );
      break;
    }
    break;

  case IMG_GENRE_GRAPHE:
    break;

  case IMG_GENRE_DIAGRAMME:
    break;

  case IMG_GENRE_PHOTO:
    break;

  case IMG_GENRE_COMPO:
    break;

  case IMG_GENRE_ERREUR:
    break;

  case IMG_GENRE_CONTINGENCE:
    break;
  }

  if ( 12345 == retour ) {
    /* On n'a pas fait de conversion intelligente : on fait donc dans le brutal */

    /* On efface les donnes de l'ancien genre */
    if ( IMG_GENRE_GRAPHE == image->genre ) {
      struct serie *serie_graphes;

      /* De quelle srie est faite l'image ? */
      serie_graphes = IMG_TrouverSerieGraphes( image );
      if ( serie_graphes != NULL ) {
	int retour;

	retour = Serie_DissocierImage( serie_graphes );
	if ( retour != 0 ) {
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Serie_DissocierImage(), code %d\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
	  return -1;
	}

	SERIE_GRF_EffacerSerie( serie_graphes, VRAI );
      }
    }
    free( image->valeurs.R );
    image->valeurs.R = NULL;

    /* On mmorise le nouveau genre */
    image->genre = nv_genre;

    /* On cre les donnes pour le nouveau genre */
    retour = IMG_ReserverContenu( image );
  }

  /* On contrle que tout s'est bien pass */
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_ReserverContenu() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return retour;
  }

  /* S'il y a des erreurs : on fait pareil */
  if ( image->erreurs != NULL ) {
    retour = IMG_ChangerGenre( image->erreurs, nv_genre, matrice, compose );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans IMG_ChangerGenre() / Erreurs\n",
		     __FILE__, __FUNCTION__, __LINE__);
      image->erreurs = NULL; /* Il n'y a plus d'erreur associe ! */
      return +1;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Image  valeurs relles -> valeurs complexes -------- */

static int IMG_R_v_C(Image *image)
{
  Complexe *table_C;
  double *table_R;
  unsigned long nbr_valeurs, i;

  /* Combien de valeurs ? */
  nbr_valeurs = image->larg * (unsigned long) image->haut;

  /* Table des complexes */
  table_C = (Complexe *) malloc( sizeof( Complexe ) * nbr_valeurs );
  if ( NULL == table_C ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante [table C]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On initialise cette table : Re(Z) = R, Im(Z) = 0 */
  for ( i = 0, table_R = image->valeurs.R ; i < nbr_valeurs ; i++ ) {
    table_C[ i ].re = table_R[ i ];
    table_C[ i ].im = 0.0;
  }

  /* On permute... */
  image->valeurs.C = table_C;
  image->genre = IMG_GENRE_COMPLEXE;

  /* On efface l'ancienne table */
  free( table_R );

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Image  valeurs relles -> carte de composition --------
   L'image est renormalise ; donne la fraction de compose dans la matrice
 */

static int IMG_R_v_Compo(Image *image, struct composant *matrice, struct composant *compose)
{
  struct matiere **table_compositions, **echelle;
  char *titre;
  double facteur, *table_R;
  unsigned long nbr_valeurs, i, pc_matrice;

  if ( ( matrice == NULL ) && ( compose == NULL ) ) {
    return 12345;		/* Revient  faire une conversion sans tenir compte des valeurs ! */
  }
  if ( image->val_max - image->val_min <= 0.0 ) {
    return 12345;		/* Idem */
  }

  /* Combien de valeurs ? */
  nbr_valeurs = image->larg * (unsigned long) image->haut;

  /* Table des compositions */
  table_compositions = (struct matiere **) malloc( sizeof( struct matiere * ) * nbr_valeurs );
  if ( NULL == table_compositions ) {
    ERR_Memoire( "Table des compositions", sizeof( struct matiere * ) * nbr_valeurs );
    return -1;
  }

  /* Table des combinaisons linaires des deux composants */
  echelle = (struct matiere **) malloc( sizeof( struct matiere * ) * 101 ); /* De 0  100% par pas de 1% */
  if ( NULL == echelle ) {
    ERR_Memoire( "Combinaisons linaires", sizeof( struct matiere * ) * 101 );
    free( table_compositions ); table_compositions = NULL;
    return -2;
  }

  OPTION_A_FAIRE( "Il faudrait faire une recherche dans les compositions dj existantes pour viter les doublons..." );

  /* On commence  0% de compos d'intrt */
  echelle[ 0 ] = Matiere_CreerComposition( NULL, 1 );
  if ( NULL == echelle[ 0 ] ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en faisant la 1re composition\n",
		   __FILE__, __FUNCTION__, __LINE__);

    free( table_compositions ); table_compositions = NULL;
    free( echelle ); echelle = NULL;
    return -3;
  }
  echelle[ 0 ]->composants[ 0 ] = matrice;
  echelle[ 0 ]->fraction_massique[ 0 ] = 1.0;
  echelle[ 0 ]->couleur = BLANC;

  titre = TXT_genere_chaine( "%s : 100%%", (matrice != NULL) ? (matrice->nom) : ( "Trou" ) );
  Matiere_Nommer( echelle[ 0 ], titre );
  free( titre ); titre = NULL;

  /* On monte dans l'chelle */
  for ( i = 1 ; i < 100 ; i++ ) {
    echelle[ i ] = Matiere_CreerComposition( echelle[ i - 1 ], 2 );
    if ( NULL == echelle[ i ] ) {
      unsigned int j;

      (void) fprintf(stderr, "%s,%s l.%u : Erreur en faisant la composition %lu\n",
		     __FILE__, __FUNCTION__, __LINE__, i);
      for ( j = 0 ; j < i ; i++ ) {
	Matiere_EffacerComposition( echelle[ j ], VRAI );
	echelle[ j ] = NULL;
      }
      free( table_compositions ); table_compositions = NULL;
      free( echelle ); echelle = NULL;
      return -3;
    }

    echelle[ i ]->composants[ 0 ] = matrice;
    echelle[ i ]->fraction_massique[ 0 ] = 1.0 - ( i / 100.0 );
    
    echelle[ i ]->composants[ 1 ] = compose;
    echelle[ i ]->fraction_massique[ 1 ] = i / 100.0;

    titre = TXT_genere_chaine( "%s/%s %lu:%lu ; %s", (matrice != NULL) ? (matrice->nom) : ( "Trou" ), (compose != NULL) ? (compose->nom) : ( "Trou" ), i, 100 - i );
    Matiere_Nommer( echelle[ i ], titre );
    free( titre ); titre = NULL;

    echelle[ i ]->couleur = (unsigned int) rint( 255.0 * i / 100.0 );
  }

  /* On finit  100% de compos d'intrt */
  echelle[ 100 ] = Matiere_CreerComposition( NULL, 1 );
  if ( NULL == echelle[ 100 ] ) {
    unsigned int j;

    (void) fprintf(stderr, "%s,%s l.%u : Erreur en faisant la 1re composition\n",
		   __FILE__, __FUNCTION__, __LINE__);

    for ( j = 0 ; j < 100 ; i++ ) {
      Matiere_EffacerComposition( echelle[ j ], VRAI );
      echelle[ j ] = NULL;
    }
    free( table_compositions ); table_compositions = NULL;
    free( echelle ); echelle = NULL;
    return -3;
  }
  echelle[ 100 ]->composants[ 0 ] = compose;
  echelle[ 100 ]->fraction_massique[ 0 ] = 1.0;
  echelle[ 100 ]->couleur = 255;

  titre = TXT_genere_chaine( "%s : 100%%", (compose != NULL) ? (compose->nom) : ( "Trou" ) );
  Matiere_Nommer( echelle[ 100 ], titre );
  free( titre ); titre = NULL;

  /* Le facteur de renormalisation */
  facteur = image->val_max - image->val_min;

  /* Maintenant que l'chelle est prte, on construit l'image */
  for ( i = 0, table_R = image->valeurs.R ; i < nbr_valeurs ; i++, table_R++ ) {
    pc_matrice = (unsigned int) rint( 100.0 * ( image->val_min + *table_R / facteur ) );
    printf( "i=%lu : valeur = %g, min = %g, max = %g ==> %g soit %g = %lu %%\n",
	    i, *table_R, image->val_min, image->val_max, image->val_min + *table_R / facteur, 100.0 * ( image->val_min + *table_R / facteur ), pc_matrice);

    table_compositions[ i ] = echelle[ pc_matrice ];
  }

  /* On fait la permutation */
  table_R = image->valeurs.R;
  image->valeurs.M = table_compositions;
  image->genre = IMG_GENRE_COMPO;

  /* On efface ce qui ne sert plus */
  free( table_R ); table_R = NULL;
  free( echelle ); echelle = NULL;

  /* Les erreurs sont forcment perdues */
  IMG_EffacerImage( image->erreurs, FAUX ); image->erreurs = NULL;

  /* On met les bonnes bornes */
  IMG_bornes( image );

  /* Termin */
  return 0;
}

/* -------- Image  valeurs complexes -> valeurs relles (partie relle) -------- */

static int IMG_C_v_R(Image *image)
{
  Complexe *table_C;
  double *table_R;
  unsigned long nbr_valeurs, i;

  /* Combien de valeurs ? */
  nbr_valeurs = image->larg * (unsigned long) image->haut;

  /* Table des rels */
  table_R = (double *) malloc( sizeof( double ) * nbr_valeurs );
  if ( NULL == table_R ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante [table R]\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On initialise cette table : R = Re(Z) */
  for ( i = 0, table_C = image->valeurs.C ; i < nbr_valeurs ; i++ ) {
    table_R[ i ] = table_C[ i ].re;
  }

  /* On permute... */
  image->valeurs.R = table_R;
  image->genre = IMG_GENRE_COMPLEXE;

  /* On efface l'ancienne table */
  free( table_C );

  /* Tout s'est bien pass */
  return 0;
}

