/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Création : 28 juin 2002

   Imports/export, des fichiers expérimentaux entre autres
   Import d'un fichier réalisé sur ID21, à l'ESRF
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h> /* Pour stat */
#include <unistd.h>

#include "bin_utils.h"
#include "txt_utils.h"
#include "systeme.h"

#include "exafs.h"

#include "esrf.h"
#include "fichiers.h"
#include "images.h"
#include "serie.h"

#include "id21.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Lecture d'une image provenant de ID21 = zapimage -------- */

Image *id21_lit_zapimg(FILE *fichier, int numero_scan, const char *nom_fichier_spec)
{
  char *mvmt_Y;
  char *repertoire = NULL, *radical = NULL;
  char *nom_fichier = NULL;
  char *nom_image = NULL;
  double x_deb, x_fin, y_deb, y_fin, tps_comptage;
  int c;
  unsigned int nbr_x, nbr_y, i;
  long num_scan = -1, num_image = -1;
  struct stat infos_rep;
  Image *image = NULL;
  Serie_Graphes *serie_mca = NULL;

  if (fichier == NULL) return NULL;

  /* On lit de quoi il s'agit exactement */
  mvmt_Y = lit_commande(fichier);
  if (mvmt_Y == NULL) {
    return NULL;
  }
  fscanf(fichier, "%lf %lf %u %*s %lf %lf %u %lf", &y_deb, &y_fin, &nbr_y, &x_deb, &x_fin, &nbr_x, &tps_comptage);
#ifdef _DEVELOPPE_
  printf("Image : %u * %u cases\n", nbr_x, nbr_y);
#endif

  /* On saute l'en-tête */
  do {
    c = ignore_ligne(fichier);

    if (c == '#') {
      (void) fgetc(fichier); /* #, car ignoré par ignore_ligne */
      c = fgetc(fichier); /* Caractère suivant */
    }
  } while (c != 'C');

  /* On lit les commandes */
  do {
    (void) fgetc(fichier); /* espace */
    c = fgetc(fichier);
    if (c == 'D') { /* #C D = repertoire des images */
      if (repertoire != NULL) {
	fprintf(stderr, "zap_image : ligne DIRECTORY multiple ?\n");
	free(repertoire);
      }
      fscanf(fichier, "%*s :");
      repertoire = lit_commande(fichier);
      if (repertoire == NULL) {
	free(radical);
	return NULL;
      }
    } else if (c == 'R') { /* #C R = radical du fichier */
      if (radical != NULL) {
	fprintf(stderr, "zap_image : ligne RADIX multiple ?\n");
	free(radical);
      }
      fscanf(fichier, "%*s :");
      radical = lit_commande(fichier);
      if (radical == NULL) {
	free(repertoire);
	return NULL;
      }
    } else if (c == 'Z') { /* #C Z */
      c = ignore_car(fichier, 4);
      if (c == 'S') {
	fscanf(fichier, "%*s %*s : %ld", &num_scan);
      } else if (c == 'I') {
	fscanf(fichier, "%*s %*s : %ld", &num_image);
	break; /* Normalement, derniere ligne */
      }
    }
    c = ignore_ligne(fichier);
    if (c != '#') {
      fprintf(stderr, "zapimage : commentaires inachevés [c = %c (%d)]\n", c, c);
      free(repertoire);
      free(radical);
      return NULL;
    }
    (void) fgetc(fichier);
    c = fgetc(fichier);
    if (c != 'C') {
      fprintf(stderr, "zapimage : commentaires inachevés / autre scan commence ? [c = %c (%d)]\n", c, c);
      free(repertoire);
      free(radical);
      return NULL;
    }
  } while ( (repertoire == NULL) || (radical == NULL) || (num_scan < 0) || (num_image < 0) );

#ifdef _DEVELOPPE_
  printf("Répertoire de travail : %s\n", repertoire);
#endif
  if ( stat(repertoire, &infos_rep) != 0 ) {
    int retour = BI_ERR_alerte( principale, &ERR_RepertoireImage, VRAI, repertoire );
    free(repertoire);
    if (retour == 2) { /* On annule le chargement */
      free(radical);
      return NULL;
    }

    /* On demande le nouveau répertoire */
    repertoire = CHEM_selecteur( "Dossier des images...", principale );
    if ( NULL == repertoire ) { /* Annulation ou erreur */
      free( radical ); radical = NULL;
      return NULL;
    }
  }
#ifdef _DEVELOPPE_
  printf("Radical               : %s\n", radical);
  printf("Image numero %ld/%ld [scan %d]\n", num_scan, num_image, numero_scan);
#endif

  nom_image = TXT_genere_chaine("%F - Scan %u, image %s/%u", nom_fichier_spec, num_scan, radical, num_image);
  image = IMG_CreerImage( NULL, NULL, IMG_GENRE_GRAPHE, IMG_TYPE_CARTOGRAPHIE, nbr_x, nbr_y);
  IMG_Titre(image, nom_image);
  free( nom_image ); nom_image = NULL;
  if ( image == NULL ) {
    free(repertoire);
    free(radical);
    return NULL;
  }
  image->l_reel = (x_fin - x_deb) / (nbr_x);
  image->h_reel = (y_fin - y_deb) / (nbr_y);

  nom_image = TXT_genere_chaine("Spectres MCA pour %s", image->nom);
  serie_mca = SERIE_GRF_CreerSerie(NULL, nom_image);
  free( nom_image ); nom_image = NULL;
  if (NULL == serie_mca) {
    IMG_EffacerImage( image, VRAI );
    free(repertoire);
    free(radical);
    return NULL;
  }

  for (i = 0 ; i < nbr_y ; i++) {
    int retour;

    nom_fichier = (char *) malloc( strlen(repertoire) + strlen(radical) + 25 ); /* 0 final, _mca_, radical, 3*4 chiffres + 2 _, .edf */
    if (nom_fichier == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "zap_image", "nom_fichier");
      free(radical);
      free(repertoire);
      return NULL;
    }
    strcpy(nom_fichier, repertoire);
    strcat(nom_fichier, "/");
    strcat(nom_fichier, radical);
    strcat(nom_fichier, "_mca_");
    sprintf(nom_fichier + strlen(nom_fichier), "%.4ld_0000_%.4u.edf", num_scan, i);
#ifdef _DEVELOPPE_
    printf("%s\n", nom_fichier);
#endif

    /*    printf("Image / ligne %d - %d colonnes\n", i, nbr_x); */
    retour = id21_ligne_image(nom_fichier, image, i, nbr_x, serie_mca);
    free(nom_fichier);

    if (retour == FAUX) {
      free(radical);
      free(repertoire);
      IMG_bornes(image);
      return image;
    }
  }

  /* On libere la memoire */
  free(radical); radical = NULL;
  free(repertoire); repertoire = NULL;

  /* On prepare l'affichage de cette carte */
  IMG_Moyennes( image );
  IMG_bornes( image );

  /* On affiche cette image */
  IMG_AllerImage( image );

  /* On s'occupe de la serie de graphes */
  serie_mca->carto = image; /* Cette série est liée à l'image créée */
  serie_mca->drapeaux.protegee = VRAI; /* Impossible d'enlever/ajouter des graphes */
  return image;
}

/* -------- Lecture ligne à ligne de l'image ID 21 -------- */

int id21_ligne_image(const char *nom_fichier, Image *image, unsigned int num_ligne, unsigned int nbr_colonnes,
		     Serie_Graphes *serie_mca)
{
  boolean flottant __attribute__((__unused__)) = FAUX;
  char *nom_spectre = NULL;
  unsigned long nbr_canaux = 0, prem_canal = 0, canal;
  unsigned int j, nbr_x = 0;
  int c;
  int mode_lecture = BIN_MODE_SOURCE_LSB;
  FILE *ligne;

  if ( (nom_fichier == NULL) || (image == NULL) || (serie_mca == NULL) ) return FAUX;

  ligne = fopen(nom_fichier, "rb");
  if (ligne == NULL) {
    SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, NULL);
    return FAUX;
  }

  /* On lit l'en-tete */
  c = fgetc(ligne);
  if (c != '{') {
    fprintf(stderr, "Commence pas par {\n [%s]", nom_fichier);
    (void) fclose(ligne);
    return FAUX;
  }
  (void) ignore_ligne(ligne);
  do {
    char *commande;

    commande = lit_mot( ligne );
    if (commande == NULL) {
      (void) fprintf( stderr, "%s, %s l.%u : erreur dans lit_mot\n", __FILE__, __FUNCTION__, __LINE__ );
      (void) fclose( ligne );
      return FAUX;
    }

    /*    printf("Commande : %s\n", commande); */
    if ( strcmp(commande, "Image") == 0 ) {
      int num_image;
      fscanf(ligne, " = %d", &num_image);
      /*      printf("Image numero %d\n", num_image); */
    } else if ( strcmp(commande, "ByteOrder") == 0 ) {
      char sens;

      fscanf(ligne, " = %c", &sens);
      if (sens == 'H') { /* Codage BIGENDIAN */
	mode_lecture = BIN_MODE_SOURCE_MSB;
      } else if (sens == 'L') {	/* Codage Little Endian */
	mode_lecture = BIN_MODE_SOURCE_LSB;
      }
    } else if ( strcmp(commande, "DataType") == 0 ) {
      char type;

      fscanf(ligne, " = %c", &type);
      if (type == 'F') flottant = VRAI;
    } else if ( strcmp(commande, "Dim_1") == 0 ) {
      fscanf(ligne, " = %lu", &nbr_canaux);
    } else if ( strcmp(commande, "Dim_2") == 0 ) {
      fscanf(ligne, " = %u", &nbr_x);
      if (nbr_x != nbr_colonnes) {
	fprintf(stderr, "Incohérence des tailles : %u/%u\n", nbr_colonnes, nbr_x);
	free(commande);
	(void) fclose(ligne);
	return FAUX;
      }
    } else if ( strcmp(commande, "MCA end ch") == 0 ) {
      fscanf(ligne, " = %lu", &prem_canal);
    } else if ( '}' == *commande ) {
      /* Si la  derniere ligne commence par des espaces... */
      free( commande ); commande = NULL;
      ungetc( '{', ligne );
      break;
    } else {
      printf( "Commande ignoree : %s\n", commande );
    }

    free(commande);
    c = ignore_ligne(ligne); /* On ignore la fin de la ligne */
  } while ( c != '}' );
  (void) fgetc(ligne); /* Accolade fermante */
  (void) fgetc(ligne); /* Retour à la ligne/unix */

  /* Ici, les données brutes commencent... */
  printf("Graphe a %ld canaux...\n", nbr_canaux);

  /* Preparation de la table des X, commune a tous les graphes */
  if ( NULL == serie_mca->table_X ) {
    int retour;
    double *x;

    /* Reservation */
    x = (double *) malloc( sizeof(double) * nbr_canaux );
    if ( NULL == x ) {
      (void) fprintf( stderr, "%s, %s l.%u : Memoire insuffisante, table des X communs\n", __FILE__, __FUNCTION__, __LINE__ );
      (void) fclose(ligne);
      return FAUX;
    }

    /* Remplissage */
    for ( canal = 0 ; canal < nbr_canaux ; canal++ ) {
      x[ canal ] = (double) ( prem_canal + canal );
    }

    /* Association a la serie */
    retour = Serie_AssocierTable( serie_mca, nbr_canaux, x, FAUX );
    if ( retour != 0 ) {
      (void) fprintf( stderr, "%s, %s l.%u : Impossible d'associer les X a la serie\n", __FILE__, __FUNCTION__, __LINE__ );
      (void) fclose(ligne);
      return FAUX;
    }
  } else {
    /* Quelques controles */
    if ( nbr_canaux != serie_mca->nbr_points ) {
      (void) fprintf( stderr, "%s, %s l.%u : Incoherence de X [%lu dans le fichier, %lu dans la serie\n", __FILE__, __FUNCTION__, __LINE__,
		      nbr_canaux, serie_mca->nbr_points );
      (void) fclose(ligne);
      return FAUX;
    }
  }

  /* Lecture de chaque spectre (un par point de la ligne) */
  for ( j = 0 ; j < nbr_x ; j++ ) {
    Graphe *spectre;
    double *y;

    /* Construction du nom du graphe */
    nom_spectre = TXT_genere_chaine( "MCA de (X=%f ; Y=%f)", image->x_min + image->l_reel * j, image->y_min + image->h_reel * num_ligne );
    spectre = Serie_nv_graphe( serie_mca, NULL, nbr_canaux, GR_FLUOX, GRF_SERIE, nom_spectre );
    free( nom_spectre ); nom_spectre = NULL;
    if ( NULL == spectre ) {
      (void) fclose(ligne);
      return FAUX;
    }
    image->valeurs.G[ num_ligne * nbr_x + j ] = spectre;

    /* Les X sont ceux de la table de la serie ! */
    spectre->x = serie_mca->table_X;

    /* Lecture du spectre de fluo X */
    for (canal = 0, y = spectre->y ; canal < nbr_canaux ; canal++, y++) {
      float coups;

      coups = BIN_LireFlottant( ligne, mode_lecture );
      *y = (double) coups;
    }

    GRF_bornes(spectre, ORDONNE, DESORDONNE);
  }

  (void) fclose(ligne);
  return VRAI;
}
