/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Imports/export, des fichiers exprimentaux entre autres
   Import d'un fichier contenant une carto ralise sur ID22  l'ESRF
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h> /* Pour isspace */

#include "bin_utils.h"
#include "systeme.h"

#include "fichiers.h"
#include "images.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Image *IMG_esrf_edf(const char *nom_fichier)
{
  boolean transposer = FAUX; /* VRAI => x et y inverses dans la carto */
  char *nom_image = NULL;
  double lg_case = 1.0, ht_case = 1.0, unite_x = 1.0, unite_y = 1.0;
  /*  float nrj_a, nrj_b, nrj_c; */
  int format_source = BIN_MODE_SOURCE_LSB;
  unsigned int num_image;
  unsigned long i, larg, haut;
  Image *image = NULL;
  int c;
  FILE *fichier;

  fichier = fopen(nom_fichier, "rb");
  if (fichier == NULL) return NULL;

  c = fgetc(fichier);
  if (c != '{') {
    (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, "Format de fichier incorrect");
    (void) fclose(fichier);
    return NULL;
  }

  /* Lecture de l'en-tete : lignes "mot-clef = valeur" jusqu'a l'accolade fermante */
  do {
    char *commande;

    commande = lit_mot(fichier); /* Peut contenir des espaces... */
    if (commande == NULL) {
      (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, "Format de fichier incorrect");
      (void) fclose(fichier);
      return NULL;
    }
    (void) fscanf(fichier, " ="); /* On saute le egal */

    /* Analyse des mots-clefs */
    if ( strcmp(commande, "HeaderID") == 0 ) {
    } else if ( strcmp(commande, "Image") == 0 ) { /* Numero de l'image */
      fscanf(fichier, " %u", &num_image);
    } else if ( strcmp(commande, "ByteOrder") == 0 ) { /* Ordre des octets */
      do {
	c = fgetc(fichier);
      } while ( isspace(c) );
      printf("Caractere lu / ByteOrder : %c\n", c);
      if (c == 'H') { /* Codage Big Endian */
	format_source = BIN_MODE_SOURCE_MSB;
      } else { /* Codage Little Endian */
	format_source = BIN_MODE_SOURCE_LSB;
      }
    } else if ( strcmp(commande, "DataType") == 0 ) { /* Type de valeurs */
    } else if ( strcmp(commande, "Dim_1") == 0 ) { /* Dimensions 1 utilisee en X */
      fscanf(fichier, "%lu", &larg);
    } else if ( strcmp(commande, "Dim_2") == 0 ) { /* Dimensions 1 utilisee en Y */
      fscanf(fichier, "%lu", &haut);
    } else if ( strcmp(commande, "Pixel_X") == 0 ) { /* Dimension horizontale d'une case (en um ?) */
      fscanf(fichier, "%lf", &lg_case);
    } else if ( strcmp(commande, "Pixel_Y") == 0 ) { /* Dimension verticale d'une case (en um ?) */
      fscanf(fichier, "%lf", &ht_case);
    } else if ( strcmp(commande, "Title") == 0 ) { /* Titre => sur ID21, commande utilisee... */
      char *mvmt;

      mvmt = lit_commande(fichier);
      if (mvmt == NULL) {
	free(commande);
	(void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, "\"Title\" non fini");
	(void) fclose(fichier);
	return NULL;
      }
      if ( strcmp(mvmt, "zapimage") == 0) {
	free(mvmt);

	/* REMARQUE
	   Version actuelle [12/02/2002] : zapimage mvmt_Y_image (lent) mvmt_X_image (rapide)
                   dans Title et dans le .spec / #S zapimage...
	   mais la doc & la commande tapee est zapimage mvmt_X_image mvmt_Y_image

	   ==> le champ Title est susceptible de changer pour coller a la commande
	*/

	/* Premier mouvement - correspond au Y dans l'image stockee (mouvement lent) */
	mvmt = lit_commande(fichier);
	if (mvmt == NULL) {
	  free(commande);
	  (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, "\"Title/zapimage\" non fini");
	  (void) fclose(fichier);
	  return NULL;
	}
	if ( strcmp(mvmt, "samz") == 0 ) { /* en y sur l'echantillon */
	  unite_y = 1000.0; /* Deplacement moteur code en mm - LASE interne en microns */
	  transposer = FAUX;
	} else if ( strcmp(mvmt, "sampz") == 0 ) { /* en y sur l'echantillon, deja code en microns */
	  transposer = FAUX;
	} else if ( strcmp(mvmt, "samy") == 0 ) { /* en _x_ sur l'echantillon */
	  unite_x = 1000.0; /* Deplacement moteur code en mm - LASE interne en microns */
	  transposer = VRAI;
	} else if ( strcmp(mvmt, "sampy") == 0 ) { /* en _x_ sur l'echantillon, deja code en microns */
	  transposer = VRAI;
	} else {
	  printf("Carto ID21 *.edf, MOUVEMENT INCONNU : %s\n", mvmt);
	}
      }
      free(mvmt);
    }
    free(commande);
    c = ignore_ligne(fichier);
    if (c == EOF) {
      (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier, NULL, "Format incorrect, } final manque");
      (void) fclose(fichier);
      return NULL;
    }
  } while (c != '}');
  (void) fgetc(fichier); /* } */
  (void) fgetc(fichier); /* Fin de ligne qui suit */

  image = IMG_CreerImage( NULL, NULL, IMG_GENRE_REELLE, IMG_TYPE_CARTOGRAPHIE, larg, haut);
  if (image == NULL) {
    (void) fclose(fichier);
    return NULL;
  }

  nom_image = TXT_genere_chaine("%F", nom_fichier);
  IMG_Titre(image, nom_image);
  free(nom_image);

  /* Codage en flottant */
  for ( i = 0 ; i < larg * (unsigned long) haut ; i++ ) {
    image->valeurs.R[ i ] = (double) BIN_LireFlottant( fichier, format_source );
  }

  /* Chargement termin */
  (void) fclose( fichier );

  IMG_bornes(image);
  image->l_reel = fabs(lg_case) * unite_x;
  image->h_reel = fabs(ht_case) * unite_y;

  if (lg_case < 0) { /* Coordonnee en X en sens inverse => miroir vertical */
    IMG_PseudoMiroir( image, IMG_MIROIR_VERTICAL );
  }

  if (ht_case < 0) { /* Coordonnee en Y en sens inverse => miroir horizontal */
    IMG_PseudoMiroir( image, IMG_MIROIR_HORIZONTAL );
  }

  if ( transposer == VRAI ) { /* (X, Y) de l'image correspondent a (y, x) sur l'echantillon */
    unsigned long x, y;
    unsigned int ech;
    double *valeurs;

    printf("ON TRANSPOSE L'IMAGE\n");

    valeurs = (double *) malloc( sizeof(double) * (image->larg * image->haut) );
    if (valeurs == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/esrf_image.c", "IMG_esrf_edf", "valeurs/transposition");
      return image;
    }

    for (x = 0 ; x < image->larg ; x++)
      for (y = 0 ; y < image->haut ; y++) {
	valeurs[x * image->haut + y] = image->valeurs.R[y * image->larg + x];
      }

    free( image->valeurs.R );
    image->valeurs.R = valeurs;
    ech = image->larg; image->larg = image->haut; image->haut = ech;
    image->l_reel = fabs(ht_case);
    image->h_reel = fabs(lg_case);
  }

  return image;
}

Image *import_edf(void)
{
  Image *image = NULL;
  char **liste_noms;
  unsigned int nbr_fichier = 0, i;

  liste_noms = CH_selecteur( "*.edf", "Import d'une image ESRF", 0, NULL, &nbr_fichier, principale );
  if (liste_noms == NULL)
    return FAUX;

  for (i = 0 ; i < nbr_fichier ; i++) {
    Image *retour;

    retour = IMG_esrf_edf( liste_noms[i] );
    if (retour != NULL) image = retour;
    free( liste_noms[i] );
  }

  IMG_AllerImage( image );
  free( liste_noms );
  return image;
}

