/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 2003

   Modles structurels d'une station exprimentale
   Fichiers de dclarations
*/

#ifndef EXPERIENCE_H_
# define EXPERIENCE_H_ /* Protection contre l'inclusion multiple */

# include "detecteur.h"
# include "echantillon.h"
# include "faisceau.h"

struct graphe; /* Dans graphes.h */
struct image;  /* Dans images.h */
struct serie;  /* Dans serie.h */

/* -------- Type d'exprience ralise -------- */

#define EXPERIENCE_XAS_TRANSMISSION 0 /* Spectroscopie d'absorption X, par transmission */
#define EXPERIENCE_XAS_FLUORESCENCE 1 /* Spectroscopie d'absorption X, par fluorescence */
#define EXPERIENCE_XRF              2 /* Analyse de traces par fluorescence X           */
#define EXPERIENCE_XRF_TOMOGRAPHIE  3 /* Tomographie par fluorescence X                 */
#define EXPERIENCE_PIXE             4 /* Carte lmentaire par PIXE                     */

#define NOMBRE_TYPE_EXPERIENCE 5

/* -------- Informations complmentaires sur l'exprience -------- */

#define EXPERIENCE_ABSORBANTE   1 /* Bit mis => l'exprience elle-mme absorbe une partie du faisceau */
#define EXPERIENCE_CARTOGRAPHIE 2 /* Bit mis => l'exprience est rpte en bougeant pour faire une carte */

/* -------- Description d'une station exprimentale -------- */

typedef struct experience {
  double E_debut, E_fin, pas_E; /* Sur quel intervalle d'nergie fait-on l'excitation ? */
  int methode;                  /* Quelle technique exprimentale ? */
  int proprietes;               /* Caractristiques de l'exprience */
  unsigned int ordre;           /* Jusqu' quel ordre faut-il construire les termes correctifs ? */

  unsigned int Z, seuil;           /* Seuil mesur */
  double Ef_debut, Ef_fin, pas_Ef; /* Quel intervalle d'nergie analyse-t-on pour les spectres fluorescs ? */
  double ROI_debut, ROI_fin;       /* La sous-partie intressante [mme pas que ci-dessus] */

  Faisceau *faisceau;       /* Description du faisceau utilis (avant tout dtecteur) */
  Detecteur *i0;            /* Chambre I0 : contrle avant l'chantillon de l'intensit du faisceau */
  Echantillon *echantillon; /* chantillon tudi */
  Detecteur *i1;            /* Dtecteur de mesure aprs l'chantillon (I1 ou dtecteur de fluo.) */

  struct matiere *composition; /* Si l'exprience est absorbante... */

  double *table_X_mu;     /* Valeurs des X pour lesquelles calculer mu */
  unsigned long nbr_X_mu; /* Nombre de points dans ces X */
  double *table_X_fluo;   /* Idem, mais pour la partie fluorescence seule */
  unsigned long nbr_X_fluo;

  double x_min, x_max, pas_x;	/* Rgion cartographie [en m] */
  double y_min, y_max, pas_y;
} Experience;

#ifndef VARIABLES_GENERALES_
struct experience *experience_courante; /* Pour l'instant, une seule exprience  la fois ! */
#endif

/* -------- Manipulation interne d'une exprience -------- */

Experience *EXP_CreerExperience(int methode);

struct graphe *EXP_SpectreXAS_Fluorescence( Experience *experience );
struct graphe *EXP_SpectreXAS_Transmission( Experience *experience );
struct graphe *EXP_SpectreXRF(Experience *experience, struct serie *serie_resultats);

boolean EXP_EstSpectroscopie   (int methode);
boolean EXP_UtiliseFluorescence(Experience *experience);

/* -------- Prdiction des rsultats d'une exprience ---- */

struct graphe *EXP_ConstruireSpectre( Experience *experience );
struct graphe *FLUO_CalculerSpectre ( Experience *experience, struct faisceau *f_detecteur, double energie );

struct image *EXP_ConstruireCartographie(Experience *experience);

/* -------- Interface graphique -------- */

int EXP_Parametrer   ( Experience *experience );
int EXP_OuvrirFenetre( void );

Experience *EXP_ChargerExperience( FILE *fichier, int version, const char *nom_fichier );
int         EXP_SauverIsolee( const char *nom_fichier, Experience *experience );

#endif /* EXPERIENCE_H_ */
