/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 2006, Emmanuel CURIS

   Création de figures (présentation des résultats)
   Fichier de déclarations
*/

#ifndef LASE_FIGURES_H_
# define LASE_FIGURES_H_ /* Protection contre l'inclusion multiple */

# include "tracer.h"		/* On en a _toujours_ besoin quand on fait une figure ! */

/* ~-~-~-~-~-~-~-~-~ Le principe ~-~-~-~-~-~-~-~-~ */

/* Une figure est prévue pour être imprimée. Ses dimensions sont donc en mm.
   L'origine du repère est le coin inférieur, droit de la page ==> axe des X vers la droite, des Y vers le haut "mathématique"

   La figure est constituée d'un ensemble de blocs, rectangulaires,
   Ils sont définis par leur coin inférieur, gauche, leur largeur et leur hauteur, exprimés en mm aussi.
 */

/* Données pour une figure contenant des graphes */
struct axes_graphe {
  struct graphe *graphes;
  unsigned int axes_secondaires; /* Si AXE_? mis, alors on utilise l'axe secondaire pour l'affichage */
  int epaisseur; /* 1/10e mm */
  int ht_marque; /* 1/10e mm */
};

struct infos_graphe {
  /* Les infos d'échelle sont stockées dans la région directement */
  /* Les infos de couleur & style sont celles du graphe */
  unsigned long nbr_graphes;
  struct axes_graphe *infos_graphe;
};

/* Définition d'un bloc sur la page.
   Les blocs forment une liste simplement chaînée. NULL en marque la fin

   Chaque bloc peut contenir un objet, dont la nature dépend du type de bloc
   Chaque bloc peut être déplacé, il est défini par sa position
 */

#define BLOCFIGURE_TYPE_GRAPHES 0 /* Contient une superposition de graphes */
#define BLOCFIGURE_TYPE_TEXTE   1 /* Contient un texte */
#define BLOCFIGURE_TYPE_LEGENDE 2 /* Contient la légende d'une superposition de graphes */
#define BLOCFIGURE_TYPE_IMAGE   3 /* Contient une image 2D */
#define BLOCFIGURE_TYPE_SURFACE 4 /* Contient une surface géométrique simple */

#define NOMBRE_TYPES_BLOC_FIGURE 5

typedef struct bloc_figure {
  struct bloc_figure *suivant;
  unsigned int type;		/* Cf. les valeurs possibles ci-dessus */

  struct infos_dessin2D dessin;	/* Pour le dessin du cadre, du fond,... */

  union {
    struct infos_graphe graphes; /* Graphes à superposer */
    struct bloc_figure *a_legender; /* Légende : de quel bloc est-ce la légende ? */
    char *texte;		/* Texte */
    struct image *image;	/* Image */
  } donnees;			/* Ce que l'on doit afficher dans le cadre */
} Bloc_Figure;

/* Définition de la figure proprement dite */
#define FIGURE_A_REDESSINER   1 /* Bit mis : il faut redessiner la figure */
#define FIGURE_A_REECHELONNER 2	/* Bit mis : il faut refaire la conversion % -> écran */

typedef struct figure {
  /* Dimension de la page [=figure totale] [mm] */
  unsigned int largeur, hauteur;

  /* Position de la figure réelle sur la page [mm] */
  unsigned int marge_gauche, marge_haute;
  unsigned int marge_droite, marge_basse;

  /* Propriétés & masques de la figure */
  unsigned int proprietes; /* Masque de bits */

  /* Nom (titre) --- pour affichage uniquement */
  char *titre;

  /* Liste chaînée des blocs sur la figure */
  struct bloc_figure *premier_bloc;

  /* Bloc actif « type PAO » */
  struct bloc_figure *bloc_actif;
} Figure;

Figure  *FIG_CreerFigure   (void);
int      FIG_EffacerFigure (Figure *figure);
Fenetre *FIG_OuvrirFenetre (Figure *figure);
int      FIG_RenommerFigure(Figure *figure, const char *nouveau_titre);

void         FIG_ChangerBlocActif(Figure *figure, Bloc_Figure *nv_actif);
Bloc_Figure *FIG_CreerBloc   (Figure *figure, Bloc_Figure *avant, unsigned int type, double xBG_mm, double yBG_mm, double larg_mm, double haut_mm);
void         FIG_DessinerBloc(Fenetre *fenetre, Bloc_Figure *bloc, int x_zone, int y_zone, unsigned int larg_zone, unsigned int haut_zone);
int          FIG_EffacerBloc (Figure *figure, Bloc_Figure *bloc);

int          BLOC_AjouterGraphes(Bloc_Figure *bloc, struct graphe **graphes, unsigned long nbr_graphes);
unsigned int BLOC_PositionPoint (Bloc_Figure *bloc, int x, int y, unsigned int marge);
void         BLOC_TracerReticule(Fenetre *fenetre, Bloc_Figure *bloc, unsigned int axes, int x, int y);

/* -------- Optimisation des bornes -------- */
#define BORNES_XY     0
#define BORNES_X      1
#define BORNES_Y      2
#define BORNES_XpourY 3
#define BORNES_YpourX 4

int BLOC_ChoisirBornesOptimales(unsigned int methode, double bornes[ 6 ], void *infos);
int BLOC_OptimiserBornes       (Bloc_Figure *bloc, int methode);

#endif	/* LASE_FIGURES_H_ */
