/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2001, Emmanuel CURIS
   Cration              : 10 avril 2001
   Dernire modification : 

   Modules gnraux : dfinition des fonctions
   Calcul matriciel, avec gestion de la mmoire optimise
*/

#ifndef MATRICES_H_

#define MATRICES_H_ /* Protection contre l'inclusion multiple */

/* -------- Rservation d'une matrice -------- */

typedef struct matrice {
  int type;        /* Voir les dfinitions ci-dessous */
  unsigned int n_lignes;
  unsigned int n_colonnes;
  double *valeurs; /* Pour les matrices usuelles */
  /* Voir pour d'ventuelles matrices spciales demandant plus d'infos */
} Matrice;

#define MATRICE_QUELCONQUE       0
#define MATRICE_SYMETRIQUE       1
#define MATRICE_ANTISYMETRIQUE   2
#define MATRICE_TRIANGULAIRE_SUP 3
#define MATRICE_TRIANGULAIRE_INF 4
#define MATRICE_DIAGONALE        5

Matrice *creer_matrice (unsigned int n_lignes, unsigned int n_colonnes, int special);
Matrice *copier_matrice(Matrice *matrice);
void   effacer_matrice(Matrice *matrice);

int MAT_ChangerType(Matrice *matrice, int nv_type);

#ifdef _DEVELOPPE_ /* Contrles ! */
void MAT_AfficherMatrice(Matrice *matrice);
#endif

/* -------- Accs  la matrice : gnral et quelques macros pour aller plus vite -------- */

double lit_element(Matrice *matrice, unsigned int i, unsigned int j);
void   met_element(Matrice *matrice, unsigned int i, unsigned int j, double valeur);

unsigned long MAT_IndexElement(Matrice *matrice, unsigned int i, unsigned int j);

/* Correspondance a(i, j) <-> index du tableau associ pour une matrice triangulaire infrieure */
#define MAT_TINF_element(i, j) ( ( ( (i) * ((i) + 1) ) >> 1 ) + (j) )
#define MAT_SYM_element(i, j)  MAT_TINF_element(i, j)

/* -------- Oprations fondamentales sur les matrices -------- */

Matrice *MAT_Transposer(Matrice *matrice, boolean en_place);
Matrice *inverse_matrice(Matrice *matrice, boolean def_pos);

Matrice *MAT_ExtraitBloc(Matrice *matrice, unsigned int i_deb, unsigned int j_deb, unsigned int larg, unsigned int haut);

int MAT_CL_Colonnes(Matrice *matrice, unsigned int col_cible, unsigned int col_2, double coef_2);
int MAT_CL_Lignes  (Matrice *matrice, unsigned int col_cible, unsigned int col_2, double coef_2);
int MAT_EchangerColonnes(Matrice *matrice, unsigned int col_1, unsigned int col_2);
int MAT_EchangerLignes  (Matrice *matrice, unsigned int lgn_1, unsigned int lgn_2);

/* -------- Factorisations diverses -------- */

Matrice *MAT_DecompositionCholesky(Matrice *matrice);

/* -------- Rsolution de systmes linaires A*X = B -------- */

#define MATRICE_LINEAIRE_GAUSS 0

#define MATRICE_LINEAIRE_PAS_DE_SOLUTION 1

int MAT_ResoudSysteme(Matrice *A, Matrice *X, Matrice *B, unsigned int methode);
int MAT_ResoudGauss  (Matrice *A, Matrice *X, Matrice *B);

/* -------- Moindres carrs linaires ------ */


#endif
