/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Regroupement des graphes en sries
   Les sries forment un niveau suprieur aux graphes (voir plus bas)
   Cependant, en gnral cet tage n'est pas utile -> fonctions distinctes
*/

#ifndef LASE_SERIES_H_

#define LASE_SERIES_H_ /* Protection contre l'inclusion multiple */

#include "txt_utils.h" /* On en aura probablement besoin pour les noms... */

#include "graphes.h"

/* -------- Structures dfinies ailleurs, d'usage conditionnel -------- */

struct molecule; /* dans atomes.h */
struct image;    /* dans images.h */

/* Les sries forment une liste simplement chane.
   Chaque srie contient un nombre quelconque de graphes en liste
   doublement chane [ventuellement 0]

   Un graphe appartient forcment  une et une seule srie,
     forcment la mme que celle des graphes vers lesquels il pointe
     (barres d'erreur par exemple)

   Elle contient aussi des informations complmentaires :
   - Molcule contenant le modle
   - Image de carto associe (srie de spectres MCA)
*/
typedef struct serie {
  struct serie *suivante;     /* Pointeur sur la srie suivante
				 NULL = c'est la dernire */
  char *nom; /* Nom de la srie */

  unsigned long nbr_graphes; /* Nombre dans la srie */
  Graphe *premier_graphe;    /* Premier graphe de la liste */ 
  Graphe *dernier_graphe;    /* Dernier graphe de la liste */

  struct {
    unsigned homogene :1; /* VRAI : tous les graphes sont de mme type         */
    unsigned commun   :1; /* VRAI : tous les graphes sont traits simultanment
			     N'a de sens que si le prcdent est VRAI aussi    */
    unsigned protegee :1; /* VRAI : aucun graphe effaable/ajoutable par l'utilisateur */
    unsigned table_X  :1; /* VRAI : contient une table des X utilisable par les spectres */
  } drapeaux;

  unsigned long nbr_points; /* dans la table des X */
  double *table_X; /* Cette table ne peut tre utilise QUE par des graphes de cette srie */
  struct graphe *erreurs_X;	/* Les erreurs associes  ces X communs, s'il y en a */
  char *legende_X, *unite_X;	/* Pour contrles et affichage */

  struct molecule *modele; /* Modle structural correspondant (NULL = aucun) */
  struct image    *carto;  /* Cartographie correspondante */
} Serie_Graphes;

/* -------- Accs aux membres de la srie -------- */

#define SERIE_EstProtegee(serie) ( ( VRAI == (serie)->drapeaux.protegee ) ? ( VRAI ) : ( FAUX ) )

/* Gestion de base des sries */

Serie_Graphes *SERIE_GRF_CreerSerie  (Serie_Graphes *avant, const char *nom);
boolean        SERIE_GRF_EffacerSerie(Serie_Graphes *serie, boolean forcer);

Serie_Graphes *choisir_serie(int arbre, unsigned int objet, unsigned long lgmax_texte);
Serie_Graphes *GRF_QuelleSerie(const Graphe *graphe);

boolean        aller_serie(Serie_Graphes *serie);

Graphe        *Serie_nv_graphe(Serie_Graphes *serie, Graphe *avant,
			       unsigned long nbr_pts, int type, int mode, const char *nom);

void           Serie_titre(Serie_Graphes *serie, const char *titre);

# ifndef VARIABLES_GENERALES_
extern Serie_Graphes *premiere_serie, *serie_courante;
# endif

/* Quelques fonctions pour une utilisation plus subtile */

int GRF_ChangeSerie     (Graphe *graphe, Serie_Graphes *nv_serie, Graphe *nv_avant);
int GRF_IsolerGraphe    (Serie_Graphes *serie, Graphe *graphe);
int GRF_InsererDansSerie(Graphe *graphe, Serie_Graphes *serie, Graphe *avant);

int GRF_DefaireChainage(Serie_Graphes *serie, Graphe *graphe);
int GRF_ChainerGraphe  (Graphe *graphe, Graphe *avant, Serie_Graphes *serie);

int SERIE_EffacerGraphes(Serie_Graphes *serie);
int SERIE_SauverGraphes (Serie_Graphes *serie, const char *radical, boolean tous);

#define SERIE_OPT_SAUVER_X_SEULS   1 /* Bit mis : on ne sauve que ceux dont les X sont ceux de la srie */
#define SERIE_OPT_SAUVER_NON_SAUVE 2 /* Bit mis : on ne sauve pas ceux qui sont dj sauvs */

Serie_Graphes *Serie_Charger     (FILE *fichier, int version, const char *nom_fichier);
int            Serie_Sauver      (Serie_Graphes *serie, FILE *fichier, unsigned int options);
int            Serie_SauverIsolee(Serie_Graphes *serie, const char *nom_fichier, unsigned int options);

int Serie_AssignerX    (Serie_Graphes *serie);
int Serie_AssocierTable(Serie_Graphes *serie, unsigned long nbr_points, double *table_X, boolean copier);
int Serie_EnleverTable (Serie_Graphes *serie);

int Serie_AssocierImage (Serie_Graphes *serie, struct image *image);
int Serie_DissocierImage(Serie_Graphes *serie);

#endif /* LASE_SERIES_H_ */
