/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 2006, Emmanuel CURIS
   Cration : 2 juin 2006

   Interface graphique

   Soustraction d'un blanc  un graphe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"
#include "serie.h"

/* ~~~~~~~~ Structures locales ~~~~~~~ */

/* -------- Variables locales -------- */

#define LGMAX_TXT_TYPE_GRAPHE (49)
#define LGMAX_TXT_NOM_SERIE   (59) /* Nom de la srie dans le slecteur */

/* -------- Les textes [isols pour traduction] -------- */

#include "int_grf_choisir.txt"

/* -------- Fonctions locales -------- */

static int  I_GRF_CHX_GererArbre_Clavier(unsigned int touche, int etat, int *renvoi);
static int  I_GRF_CHX_GererArbre_Clic   (unsigned int numero_objet, int x, int y);
static void I_GRF_CHX_InitialiserArbre  (const Infos_ChoixGraphe *const infos);
static void I_GRF_CHX_PreparerArbre     (void);

static void I_GRF_CHX_AfficherListe   (Infos_ChoixGraphe *const infos);
static int  I_GRF_CHX_PreparerListe   (Infos_ChoixGraphe *infos);

/* ~-~-~-~-~-~-~-~-~ Appel par l'utilisateur ~-~-~-~-~-~-~-~-~ */

/* -------- Slection d'un seul graphe parmi ceux indiqus -------- *
   Renvoie NULL en cas d'annulation ou de problme
           le graphe choisi sinon

   modele : sert pour savoir parmi quels graphes ils faut chercher
   masque : masque de bits indiquant suivant quels critres un graphe est valide ou non [cf. interface.h]
   strict : VRAI ==> les critres ci-dessus doivent tres satisfaits de faon stricte
            FAUX ==> accorde une certaine souplesse
   multi_serie : VRAI ==> on peut choisir dans n'importe quelle srie
   serie_a_chercher : dans quelle srie chercher le graphe ? [NULL => srie courante]
   titre_choix      : le titre qui sera affich pour la bote de dialogue
*/

Graphe *I_GRF_ChoisirUnique(Graphe *modele, unsigned int masque, boolean strict, boolean multi_serie,
			    struct serie *serie_a_chercher, const char *titre_choix, Fenetre *fenetre_mere)
{
  Infos_ChoixGraphe infos = { NULL, 0U, 0U, NULL, 0UL, 0UL, 0UL, 0UL, 0U, 1U, 1U, 0, NULL, NULL };
  Graphe *retour = NULL;
  int reponse;

  /* 1. Contrle des paramtres passs */
  if ( NULL == serie_a_chercher ) {
    serie_a_chercher = serie_courante;
  }

  if ( ( NULL == modele ) && ( masque != 0 ) ) {
    AVERTISSEMENT( "Aucun modle indiqu, alors qu'un masque est demand [masque : %u]", masque );
    return NULL;
  }

  if ( VRAI == multi_serie ) {
    infos.options |= GRF_LISTE_TYPES_MULTISERIE;
  }

  if ( VRAI == strict ) {
    infos.options |= GRF_LISTE_TYPES_STRICT;
  }

  if ( NULL == titre_choix ) {
    titre_choix = titre_choix_par_defaut;
  }

  /* 2. Initialisation de l'arbre, en fonction de la liste & des paramtres choisis */
  I_GRF_CHX_InitialiserArbre( &infos );

  /* 3. Construction de la liste des graphes valables */
  infos.modele = modele;
  infos.masque = masque;
  infos.serie = serie_a_chercher;

  reponse = I_GRF_CHX_PreparerListe( &infos );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans la prparation de la liste [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, reponse );
  }

  /* 4. Affichage de la liste des graphes dans le slecteur */
  I_GRF_CHX_AfficherListe( &infos );

  /* 5. On appelle l'arbre & on attend le choix de l'utilisateur */
  ARB_MettreComplement( CHX_GRAPHE, &infos );
  reponse = ARB_Bloquant( CHX_GRAPHE, titre_choix, fenetre_mere, CHX_GRAPHE );
  ARB_MettreComplement( CHX_GRAPHE, NULL );

  /* Si l'on annule : on libre tout et l'on renvoie... */
  if ( CHXG_ANNULE == reponse ) {
    free( infos.choisi ); infos.choisi = NULL; /* Normalement, ne devrait pas tre utile puisque slection simple, mais bon... */
    free( infos.graphes ); infos.graphes = NULL;
    infos.serie = NULL;
    infos.modele = NULL;
    return NULL;
  }

  /* Slection effectue */
  return retour;
}

/* -------- Gestion de l'arbre [clavier, puis souris] -------- */

/* .. Clavier .. */

static int I_GRF_CHX_GererArbre_Clavier(unsigned int touche, int etat, int *renvoi)
{
  unsigned int nombre_noms;
  Infos_ChoixGraphe *const infos = (Infos_ChoixGraphe *) ARB_LireComplement( CHX_GRAPHE );

  /* Contrles... */
  if ( renvoi != NULL ) *renvoi = 0; /* Aucun bouton en particulier... */
  if ( NULL == infos->graphes ) return BI_EVENEMENT_NON_TRAITE;

  /* Initialisation */
  nombre_noms = (CHXG_DERNIER_NOM - CHXG_PREMIER_NOM) + 1;

  switch ( touche )
  {
  case Fleche_Bas: /* On choisit le spectre suivant */
    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) {
      /* On dplace d'une page, sauf 1 ligne */
      infos->numero_choisi += nombre_noms - 1;
    } else {
      /* On dplace d'une ligne */
      infos->numero_choisi++;
    }

    /* On s'arrte au dernier graphe quoi qu'il arrive ! */
    if ( infos->numero_choisi >= infos->nombre_graphes ) {
      OPTION_A_FAIRE( "Faire un Bip !" );
      infos->numero_choisi = infos->nombre_graphes - 1;
    }

    /* On s'arrange pour faire le dcalage du dfileur au besoin */
    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) {
      infos->premier_graphe += nombre_noms - 1;
    }
    infos->premier_graphe = ARB_ChoisirDebutDefileur( nombre_noms, infos->nombre_graphes, infos->numero_choisi, infos->premier_graphe );

    /* On raffiche la liste */
    I_GRF_CHX_AfficherListe( infos );

    return BI_EVENEMENT_TRAITE;

  case Fleche_Haut: /* On choisit le spectre prcdent */
    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) {
      /* On dplace d'une page, sauf 1 ligne */
      if ( infos->numero_choisi > ( nombre_noms - 1 ) ) {
	infos->numero_choisi -= nombre_noms - 1;
      } else {
	infos->numero_choisi = 0;
      }
    } else {
      /* On dplace d'une ligne */
      if ( infos->numero_choisi > 0 ) {
	infos->numero_choisi--;
      } else {
	OPTION_A_FAIRE( "Faire un Bip !" );
      }
    }

    /* On s'arrange pour faire le dcalage du dfileur au besoin */
    if ( etat & ( K_RSHIFT | K_LSHIFT ) ) {
      if ( infos->premier_graphe > ( nombre_noms - 1 ) ) {
	infos->premier_graphe += nombre_noms - 1;
      } else {
	infos->premier_graphe = 0;
      }
    }
    infos->premier_graphe = ARB_ChoisirDebutDefileur( nombre_noms, infos->nombre_graphes, infos->numero_choisi, infos->premier_graphe );

    /* On raffiche la liste */
    I_GRF_CHX_AfficherListe( infos );

    return BI_EVENEMENT_TRAITE;

  case Fleche_Gauche: /* Dfiler vers la droite */
    return ARB_SimulerClic( CHX_GRAPHE, CG_TXT_AVANT, 1 );
  case Fleche_Droite:
    return ARB_SimulerClic( CHX_GRAPHE, CG_TXT_APRES, 1 );

  case 'a': /* Aucun */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_AUCUN, 1 );

  case 's': /* Choix de la srie */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_SERIE, 1 );

  case 't': /* Tous */
    return ARB_SimulerClic( CHX_GRAPHE, CHXG_TOUS, 1 );
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* .. Souris .. */

static int I_GRF_CHX_GererArbre_Clic(unsigned int numero_objet, int x, int y)
{
  Infos_ChoixGraphe *infos;

  infos = (Infos_ChoixGraphe *) ARB_LireComplement( CHX_GRAPHE );

  if ( CHXG_SERIE == numero_objet ) {
    /* Choix de la srie de graphes. On peut avoir liste = NULL */
    Serie_Graphes *nv_serie;

    nv_serie = I_SERIE_GRF_Choisir( CHX_GRAPHE, CHXG_SERIE, LGMAX_TXT_NOM_SERIE );
    if ( ( nv_serie != NULL ) && ( nv_serie != infos->serie ) ) {
      int reponse;

      /* On a bien chang de srie : il faut refaire la liste */
      /* a. On note le changement de srie */
      infos->serie = nv_serie;

      /* b. On prpare la nouvelle liste */
      reponse = I_GRF_CHX_PreparerListe( infos );
#ifdef _DEVELOPPE_
      printf("Table renvoye : %p\n", (void *) infos->graphes);
#endif
      if ( reponse != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Erreur en rservant la liste [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, reponse);
      }

      /* On affiche cette nouvelle srie */
      I_GRF_CHX_AfficherListe( infos );
    }
    return BI_EVENEMENT_TRAITE;
  }

  if ( NULL == infos->graphes ) return BI_EVENEMENT_TRAITE; /* Rien  faire quand la liste est vide ! */

  if ( infos->choisi != NULL ) {
    unsigned long i;

    if ( CHXG_TOUS == numero_objet ) {
      for ( i = 0 ; i < infos->nombre_graphes ; i++ )
	infos->choisi[ i ] = VRAI;

      infos->nombre_choisis = infos->nombre_graphes;
      I_GRF_CHX_AfficherListe( infos );

      return BI_EVENEMENT_TRAITE;
    }

    if ( CHXG_AUCUN == numero_objet ) {
      for ( i = 0 ; i < infos->nombre_graphes ; i++ )
	infos->choisi[ i ] = FAUX;

      infos->nombre_choisis = 0;
      I_GRF_CHX_AfficherListe( infos );

      return BI_EVENEMENT_TRAITE;
    }
  }

  /* Pour le dcalage des noms : ascenseur horizontal */
  if ( ( CG_TXT_AVANT == numero_objet ) || ( CG_FND_ASC_TXT == numero_objet ) ||
       ( CG_ASC_TXT   == numero_objet ) || ( CG_TXT_APRES   == numero_objet ) ) {
    infos->delta_x = ARB_ascenseur( CHX_GRAPHE,
				    CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
				    infos->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos->delta_x,
				    numero_objet, x, y, ECHANGE );
    I_GRF_CHX_AfficherListe( infos );

    return BI_EVENEMENT_TRAITE;
  }

  /* Gestion du dfileur : ascenseur vertical + slection */
  if ( ( CHXG_AVANT == numero_objet ) || ( CHXG_FOND_ASC == numero_objet ) ||
       ( CHXG_APRES == numero_objet ) || ( CHXG_POUSSOIR == numero_objet ) ||
       ( ( numero_objet >= CHXG_PREMIER_NOM ) && ( numero_objet <= CHXG_DERNIER_NOM ) ) ) {
    GRF_liste liste;

    /* Hack temporaire : conversion vers GRF_liste pour utiliser GRF_selecteur() */
    OPTION_A_FAIRE( "Rcriture de GRF_selecteur pour grer directement Infos_ChoixGraphe" );
    liste.nbr_graphe  = infos->nombre_graphes;
    liste.prem_graphe = infos->premier_graphe;
    liste.num_select  = infos->numero_choisi;
    liste.delta_x     = infos->delta_x;
    liste.lgmax_txt   = infos->lgmax_txt;
    liste.choisi      = infos->choisi;
    liste.graphes     = infos->graphes;

    (void) GRF_selecteur( &liste, CHX_GRAPHE,
			  CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
			  CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR,
			  numero_objet, x, y);

    infos->premier_graphe = liste.prem_graphe;
    infos->numero_choisi  = liste.num_select;

    if ( ( numero_objet >= CHXG_PREMIER_NOM ) && ( numero_objet <= CHXG_DERNIER_NOM ) ) {
      /* A pu changer le(s) graphe(s) choisis */
      if ( infos->choisi != NULL ) {
	unsigned long i;

	infos->nombre_choisis = 0;
	for ( i = 0 ; i < infos->nombre_graphes ; i++ )
	  if ( VRAI == infos->choisi[ i ] ) (infos->nombre_choisis)++;
      }

      I_GRF_CHX_AfficherListe( infos );
    }

    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE; /* Autrement, traitement par defaut */
}

/* .. Affichage de la liste .. */

static void I_GRF_CHX_AfficherListe(Infos_ChoixGraphe *const infos)
{
  GRF_liste liste;

  /* Contrles prliminaires */
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Infos de slection absentes !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* Hack temporaire : conversion vers GRF_liste pour utiliser GRF_affiche() */
  OPTION_A_FAIRE( "Rcriture de GRF_liste pour grer directement Infos_ChoixGraphe" );
  liste.nbr_graphe  = infos->nombre_graphes;
  liste.prem_graphe = infos->premier_graphe;
  liste.num_select  = infos->numero_choisi;
  liste.delta_x     = infos->delta_x;
  liste.lgmax_txt   = infos->lgmax_txt;
  liste.choisi      = infos->choisi;
  liste.graphes     = infos->graphes;

  /* Affichage du dfileur/slecteur : graphes + ascenseur vertical */
  GRF_affiche( &liste, CHX_GRAPHE,
	       CHXG_PREMIER_NOM, CHXG_DERNIER_NOM, LGMAX_TXT_NOM_OBJET_SELECTEUR,
	       CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR );

  /* Affichage de l'ascenseur horizontal */
  if ( 0 == infos->nombre_graphes ) {
    ARB_ascenseur( CHX_GRAPHE,
		   CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		   1, LGMAX_TXT_NOM_OBJET_SELECTEUR, 0,
		   ROOT, -1, -1, METTRE );
  } else {
    ARB_ascenseur( CHX_GRAPHE,
		   CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL,
		   infos->lgmax_txt, LGMAX_TXT_NOM_OBJET_SELECTEUR, infos->delta_x,
		   ROOT, -1, -1, METTRE );
  }

  /* Mise en place de l'aperu */
  if ( infos->options & GRF_LISTE_TYPES_MULTISELECTION ) {
  } else {
    /* Un seul graphe slectionnable */
    if ( infos->numero_choisi < infos->nombre_graphes ) {
      OBJ_utilisateur( CHX_GRAPHE, CG_AFFICHAGE, USR_graphe,
		       (long) (infos->graphes[ infos->numero_choisi ]), METTRE );
    } else {
      OBJ_utilisateur( CHX_GRAPHE, CG_AFFICHAGE, USR_graphe,
		       (long) (NULL), METTRE );
    }
  }

  /* Contrle du nombre de slectionns */
  if ( infos->options & GRF_LISTE_TYPES_MULTISELECTION ) {
    if ( ( infos->nombre_choisis >= infos->min ) && ( infos->nombre_choisis <= infos->max ) ) {
      OBJ_active( CHX_GRAPHE, CHXG_OK, FORCE_OUI );
    } else {
      OBJ_active( CHX_GRAPHE, CHXG_OK, FORCE_NON );
    }
  } else {
    /* Un seul graphe slectionnable */
    if ( infos->numero_choisi < infos->nombre_graphes ) {
      OBJ_active( CHX_GRAPHE, CHXG_OK, FORCE_OUI );
    } else {
      OBJ_active( CHX_GRAPHE, CHXG_OK, FORCE_NON );
    }
  }  
}

/* -------- Initialisation de l'arbre -------- */

static void I_GRF_CHX_InitialiserArbre(const Infos_ChoixGraphe *const infos)
{
  /* Optimisations graphiques [ne sera rellement fait qu'au premier appel] */
  I_GRF_CHX_PreparerArbre( );

  /* Autorise-t-on la slection multiple ? */
  if ( infos->options & GRF_LISTE_TYPES_MULTISELECTION ) {
    /* Oui ==> Les boutons "Tous" et "Aucun" ont un sens */
    OBJ_active( CHX_GRAPHE, CHXG_TOUS , FORCE_OUI );
    OBJ_active( CHX_GRAPHE, CHXG_AUCUN, FORCE_OUI );
  } else {
    /* Non ==> Ils n'ont pas de sens */
    OBJ_active( CHX_GRAPHE, CHXG_TOUS , FORCE_NON );
    OBJ_active( CHX_GRAPHE, CHXG_AUCUN, FORCE_NON );
  }

  /* Autorise-t-on le choix de la srie ? */
  if ( infos->options & GRF_LISTE_TYPES_MULTISERIE ) {
    if ( ( premiere_serie != NULL ) && ( premiere_serie->suivante != NULL ) ) {
      OBJ_active( CHX_GRAPHE, CHXG_SERIE, FORCE_OUI );
    } else {
      /* 0 ou 1 srie : inutile d'en changer ! */
      OBJ_active( CHX_GRAPHE, CHXG_SERIE, FORCE_NON );
    }
  } else {
    OBJ_active( CHX_GRAPHE, CHXG_SERIE, FORCE_NON );
  }
}

/* -------- Prparation de l'arbre -------- */

static void I_GRF_CHX_PreparerArbre(void)
{
  /* Partie commune au slecteur utilis pour n'importe-quoi... */
  I_GRFIMG_CHX_PreparerArbreCommun();

  /* On installe le bon type pour l'objet utilisateur : prvisualisation */
  OBJ_utilisateur( CHX_GRAPHE, CG_AFFICHAGE, USR_graphe, 0, METTRE );

  /* On installe les bonnes routines de gestion */
  arbres[ CHX_GRAPHE ].clavier = I_GRF_CHX_GererArbre_Clavier;
  arbres[ CHX_GRAPHE ].fonction.arbre = I_GRF_CHX_GererArbre_Clic;
}

void I_GRFIMG_CHX_PreparerArbreCommun(void)
{
  int retour;

  if ( VRAI == arbres[ CHX_GRAPHE ].drapeaux.prepare ) return; /* Dj initialis ! */

  /* ATTENTION, l'objet utilisateur doit dj avoir t initialis pour ne pas le perdre ! */
  /* Initialisation de l'arbre */
  retour = ARB_Initialiser( CHX_GRAPHE, "communs.html", NULL, NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'initialiser l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  OBJ_deplace ( CHX_GRAPHE, CHXG_TXT_SERIE, 0, +500, DEPLACE_RELATIF );
  OBJ_deplace ( CHX_GRAPHE, CHXG_SERIE    , 0, +500, DEPLACE_RELATIF );
  OBJ_deplace ( CHX_GRAPHE, CHXG_TITRE   , 0, -500, DEPLACE_RELATIF );
  OBJ_deplace ( CHX_GRAPHE, CHXG_TXT_TYPE, 0, -500, DEPLACE_RELATIF );

  ARB_EnjoliverDefileur( CHX_GRAPHE, CHXG_FOND, CHXG_PREMIER_NOM, CHXG_DERNIER_NOM,
			 CHXG_AVANT, CHXG_FOND_ASC, CHXG_POUSSOIR, CHXG_APRES, ARB_ASCENSEUR_DROITE );
  ARB_EnjoliverAscenseur( CHX_GRAPHE, CG_TXT_AVANT, CG_FND_ASC_TXT, CG_ASC_TXT, CG_TXT_APRES, ASC_HORIZONTAL, 0 );

#ifdef TRADUCTION_COMPLETE_
  OBJ_TraduireTexte( CHX_GRAPHE, CHXG_OK   , txt_chx_grf_ok    );
  OBJ_TraduireTexte( CHX_GRAPHE, CHXG_TITRE, titre_choix_graphe);
  OBJ_TraduireTexte( CHX_GRAPHE, CHXG_TOUS , txt_chx_grf_tous2 );
  OBJ_TraduireTexte( CHX_GRAPHE, CHXG_AUCUN, txt_chx_grf_aucun );
#endif

  arbres[ CHX_GRAPHE ].drapeaux.prepare = VRAI;
}

/* ~- Routines de slection de graphes pour former les listes */

int I_GRF_CHX_PreparerListe(Infos_ChoixGraphe *infos)
{
  unsigned long i;

  /* 1. On libre les anciennes tables */
  free( infos->choisi  ); infos->choisi  = NULL;
  free( infos->graphes ); infos->graphes = NULL;

  /* 2. On rserve pour les graphes [peut tre NULL si aucun graphe ne convient !] */
  infos->graphes = GRF_ChoisirMasque( infos->serie, infos->modele, infos->masque, infos->options, &(infos->nombre_graphes) );

  /* Y a-t-il une slection multiple ? */
  infos->nombre_choisis = 0;
  if ( infos->options & GRF_LISTE_TYPES_MULTISELECTION ) {
    infos->numero_choisi  = 0;
    infos->choisi = (boolean *) calloc( infos->nombre_graphes, sizeof( boolean ) );
    if ( NULL == infos->choisi ) {
      ERR_Memoire( "Table de slection", infos->nombre_graphes * sizeof( boolean ) );
    }
  } else {
    infos->choisi = NULL;
    infos->numero_choisi = (infos->nombre_graphes); /* Aucun slectionn */
  }

  /* Taille du plus long nom de la liste */
  infos->lgmax_txt = 0;
  for ( i = 0 ; i < infos->nombre_graphes ; i++ ) {
    if ( ( infos->graphes[ i ] != NULL ) && ( infos->graphes[ i ]->nom != NULL ) ) {
      unsigned long lg_nom;

      lg_nom = strlen( infos->graphes[ i ]->nom );
      if ( lg_nom > infos->lgmax_txt )
	infos->lgmax_txt = lg_nom;
    }
  }

  /* On initialise le reste
     min, max sont  dfinir par la fonction appelante */
  infos->premier_graphe = 0;	/* Affichage du dbut */
  infos->delta_x = 0;		/* Aucun dcalage */

  /* Termin */
  return 0;
}
