/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Gnralits du programme LASE (interface graphique)

   Gestion et prparation du menu des calculs & modles thoriques
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "atomes.h"
#include "couches.h"
#include "graphes.h"
#include "interface.h"
#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int CLV_men_ajuste(unsigned int touche, int etat, int *renvoi);
int MEN_PreparerAjuste( void );

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int MEN_PreparerAjuste( void )
{
  arbres[MEN_AJUSTE].clavier = CLV_men_ajuste;

# ifdef _TOS_ /* Pour que les accessoires soient accessibles */
  OBJ_active(MEN_AJUSTE,MAJ_V_AJUSTE, FORCE_OUI);
# endif

  OBJ_active(MEN_AJUSTE, MAJ_VALIDITE, FORCE_NON); /* Actif ssi fentre d'ajustements ouverte */

# ifndef _DEVELOPPE_ /* Version de dveloppement uniquement */
  OBJ_active(MEN_AJUSTE, MAJ_TIT_GENERAL, FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_TIT_STRUCT , FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_TIT_SEUIL  , FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_TIT_RDF    , FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_V_BANQUE   , FORCE_NON);
  OBJ_active(MEN_AJUSTE, MST_CHOIX_CHEMIN, FORCE_NON);
  OBJ_active(MEN_AJUSTE, MENAJ_IMP_EDAFIT, FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_KHI_DEUX    , FORCE_NON);
  OBJ_active(MEN_AJUSTE, MAJ_TEST_F      , FORCE_NON);
# endif

  /* Initialisation russie */
  return 0;
}

int MEN_ajuste(unsigned int option)
{
  switch (option)
  {
  case MEN_AJ_INFOS:   /* Appeler les informations     */
    ARB_AppelerInformations();
    return BI_EVENEMENT_TRAITE;

  case MAJ_V_EXTRAIRE:
    GER_infos(TRAITE_SPECTRES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MAJ_V_THEORIE:
    GER_infos(CALC_THEORIQUES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MAJ_V_STRUCTURE:
    GER_infos(STRUCTURE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case AJ_GENERAUX:
    FEN_premierplan( fenetre_onglets );
    BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_MODELES );
    return BI_EVENEMENT_TRAITE;

  case AJ_INITIAL:
    COUCHE_OuvrirEditeur(NULL);
    return BI_EVENEMENT_TRAITE;

  case AJ_DEMARRE: /* On lance l'ajustement, sur le graphe courant */
    NC_AjusterSpectre( graphe_courant );
    break;

  case AJ_AJ_ABSORBEUR: /* On ajuste, sur le spectre de rfrence de l'absorbeur courant */
    if ( (mol_courante != NULL) && (mol_courante->absorbeurs != NULL) )
      ABS_Ajuster( mol_courante->absorbeurs + mol_courante->absorbeur_courant );
    else {
      form_alert(1,"[3][--- Ajustement impossible ---"
		      "|    La mol"e_AIGU"cule courante ne"
		      "|contient aucun absorbeur.][Annuler]");
    }
    break;

  case MAJ_AJ_MOLECULE: /* On ajuste sur tous les absorbeurs de la molcule courante */
    MOL_ajuste(mol_courante);
    break;

  case MAJ_MAT_CORREL: /* Calcul de la matrice de corrlation avec les hypothses classiques */
    fprintf(stderr, "MEN_ajuste, MAJ_MAT_CORREL : modifier l'appel avec les nouveaux modles\n");
#ifdef ancien
    {
      Fenetre *fenetre = ARB_fenetre( arbres + RESULT_N_COUCHES );
      if (fenetre != NULL) {
	Infos_ncouches *infos = (Infos_ncouches *) (fenetre->pointeur);
	if (infos->en_cours == VRAI) {
	  /*                v----------------------------v */
	  form_alert(1,"[2][---- Ajustement en cours ----"
		          "|    Attendez que l'ajustement"
		          "|soit termin"e_AIGU" pour calculer"
		          "|la matrice de covariance !][ Bien ]");
	}
	else
	  NC_covariance(infos);
      }
    }
#endif
    break;

  case MAJ_VALIDITE: /* On vrifie que le modle cr est plausible */
    fprintf(stderr, "MEN_ajuste, MAJ_VALIDITE : modifier l'appel avec les nouveaux modles\n");
#ifdef ancien
    if (arbres[RESULT_N_COUCHES].drapeaux.ouvert == VRAI) /* Il y a bien des rsultats disponibles */
    {
      Fenetre *fenetre = ARB_fenetre( arbres + RESULT_N_COUCHES );
      if (fenetre != NULL)
      {
	Infos_ncouches *infos = (Infos_ncouches *) (fenetre->pointeur);
	if (infos->en_cours == VRAI)
	{
	  /*                v----------------------------v */
	  form_alert(1,"[2][---- Ajustement en cours ----"
		          "|    Attendez que l'ajustement"
		          "|soit termin"e_AIGU" pour en v"e_AIGU"rifier"
		          "|les r"e_AIGU"sultats !][ Bien ]");
	}
	else
	  NC_verifie(infos);
      }
    }
#endif
    break;

  case MAJ_COMBINER: /* Combiner des chemins de gomtrie semblable */
    I_CHEM_CombinerChemins( NULL );
    break;

  case MST_CHOIX_CHEMIN: /* Slection de chemins sur critres */
    (void) fprintf(stderr, "MEN_ajuste : MST_CHOIX_CHEMIN, appel de la fonction choisit_couche()  VRIFIER...\n");
    choisit_couche(/* A VOIR premiere_couche*/ NULL);
    break;

  case MAJ_CONTRIB_CHEM: /* Contribution individuelle de chaque couche */
    {
      struct modele_ajuste *modele;

      if (NULL == mol_courante) {
	(void) fprintf(stderr, "MEN_ajuste, MAJ_CONTRIB_CHEM : aucune molcule en mmoire, pas de modle de type EXAFS\n");
	return BI_EVENEMENT_TRAITE;
      }
      if (NULL == mol_courante->absorbeurs) {
	(void) fprintf(stderr, "MEN_ajuste, MAJ_CONTRIB_CHEM : aucun absorbeur dans la molcule, pas de modle de type EXAFS\n");
	return BI_EVENEMENT_TRAITE;
      }
      modele = mol_courante->absorbeurs[mol_courante->absorbeur_courant].modele_actif;
      if (NULL == modele) {
	(void) fprintf(stderr, "MEN_ajuste, MAJ_CONTRIB_CHEM : aucun modle dans l'absorbeur actif !\n");
	return BI_EVENEMENT_TRAITE;
      }
      if ( modele->type != MODELE_EXAFS_CHEMINS ) {
	fprintf(stderr, "MEN_ajuste, MAJ_CONTRIB_CHEM :  FAIRE, message d'erreur / modle de type incorrect\n");
	return BI_EVENEMENT_TRAITE;
      }
      if (NULL == modele->infos_modele.feff) {
	(void) fprintf(stderr, "MEN_ajuste, MAJ_CONTRIB_CHEM : modle de chemins sans infos complmentaires !\n");
	return BI_EVENEMENT_TRAITE;
      }

      NC_CalculeContributions(modele->infos_modele.feff, modele->reference);
    }
    break;

  case MAJ_MONTECARLO: /* On fait un essai Monte-Carlo */
    MODELE_InterfaceMonteCarlo(NULL);
    break;
  }

  return BI_EVENEMENT_TRAITE;
}

/* -------- Raccourcis clavier -------- */

static int CLV_men_ajuste(unsigned int touche, int etat, int *renvoi)
{
  *renvoi = 0; /* On ne quitte pas un menu de toute faon... */

  if (etat & K_CTRL) /* On appuyait sur Control */
  {
    if ( touche <= 255 )
      touche = (unsigned int) toupper( (int) touche); /* Comportement indfini pour autres codes ! */
    switch ( touche )
    {
    case 'C':   /* ^C = Charger */
      return MEN_extrait(CHARGER);

    case 'E':   /* ^E = appeler le menu d'Extraction d'un graphe */
      GER_infos( TRAITE_SPECTRES, 0, 0 );
      return BI_EVENEMENT_TRAITE;

    case 'G':   /* ^G = appeler le menu de cration de structures */
      GER_infos( STRUCTURE, 0, 0 );
      return BI_EVENEMENT_TRAITE;

    case 'L':   /* ^L = appeler la liste des graphes */
      FEN_premierplan( fenetre_onglets );
      BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_MODELES );
      return BI_EVENEMENT_TRAITE;

    case 'Q':		/* ^Q = Quitter */
      return MEN_extrait( QUITTER );

    case 'T':   /* ^T = appeler le menu de calculs thoriques    */
      GER_infos( CALC_THEORIQUES, 0, 0 );
      return BI_EVENEMENT_TRAITE;

#ifdef _DEVELOPPE_
    default:
      printf( "[%s] On appuyait sur la lettre no : %d (%c)\n",
	      __FUNCTION__, touche, (char) touche );
      break;
#endif
    }
  }

  return BI_EVENEMENT_NON_TRAITE; /* Par dfaut, message non-trait */
}
