/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion du slecteur universel
   Gestion de la partie commune du slecteur de graphes/d'images
*/

#include <stdlib.h>
#include <stdio.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_NOM_OBJET 30

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  GER_selecteur(unsigned int objet, int x, int y);
static void prepare_selecteur(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int I_SelecteurUniversel(const char *titre, InfosSelecteur *infos, Fenetre *fenetre_mere)
{
  int retour;
  InfosSelecteur *infos_avant = NULL;

  if ( NULL == titre ) titre = "Slecteur universel";
  if ( NULL == infos ) return -1;

  if ( VRAI == arbres[ SELECT_UNIVERSEL ].drapeaux.ouvert ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : slecteur dj ouvert\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    infos_avant = (InfosSelecteur *) ARB_LireComplement( SELECT_UNIVERSEL );
  }

  if ( FAUX == arbres[ SELECT_UNIVERSEL ].drapeaux.prepare ) {
    prepare_selecteur();
  }

  /* Combien d'objets dans la liste ? */
  if ( ( unsigned long (*)(void) ) NULL == infos->compter ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de fonction pour compter les objets\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  infos->nbr_objets = infos->compter();
  infos->obj_choisi = 0;
  infos->premier = 0;

  /* Aucun objet : on essaye d'en crer un */
  if ( 0 == infos->nbr_objets ) {
    if ( ( int (*)(unsigned long) ) NULL == infos->nouveau ) {
      (void) fprintf(stderr, "%s,%s l.%u : Aucun objet, pas de routine de cration. Slection annule\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -3;
    }
    retour = infos->nouveau( 0 );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans infos->nouveau(), premier objet\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -4;
    }
    infos->nbr_objets = 1;
  }
  OBJ_active( SELECT_UNIVERSEL, SU_CHOISIR, FORCE_OUI );

  /* On prpare la liste des objets */
  retour = ARB_PreparerDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
				   infos->nbr_objets, NULL, infos->obj_choisi, infos->premier, infos->objet_v_texte );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans ARB_PreparerDefileur_O()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -5;
  }

  /* Options complmentaires disponibles */
  OBJ_active( SELECT_UNIVERSEL, SU_NOUVEAU  , infos->nouveau != ( int (*)(unsigned long) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_EFFACER  , infos->effacer != ( int (*)(unsigned long) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_TTEFFACER, infos->effacer != ( int (*)(unsigned long) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_MODIFIER , infos->modifier != ( int (*)(unsigned long) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_COPIER   , infos->copier != ( int (*)(unsigned long) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_CHARGER  , infos->charger != ( int (*)(void) ) NULL );
  OBJ_active( SELECT_UNIVERSEL, SU_SAUVER   , infos->sauver != ( int (*)(unsigned long) ) NULL );
  /*   OBJ_active( SELECT_UNIVERSEL, SU_APERCU   , infos->apercu != ( int (*)(unsigned long) ) NULL ); */

  /* On appelle le slecteur */
  ARB_MettreComplement( SELECT_UNIVERSEL, infos );
  retour = ARB_Bloquant( SELECT_UNIVERSEL, titre, fenetre_mere, SELECT_UNIVERSEL );
  ARB_MettreComplement( SELECT_UNIVERSEL, infos_avant );

  /* On rinstalle pour l'ancien slecteur, si ncessaire */
  if ( infos_avant != NULL ) {
    ARB_PreparerDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			    infos_avant->nbr_objets, NULL, infos_avant->obj_choisi, infos_avant->premier, infos_avant->objet_v_texte );

    /* On rouvre la fentre qui vient d'tre ferme... */
    FEN_Reouvrir( ARB_QuelleFenetre( arbres + SELECT_UNIVERSEL, VRAI ) );
  }

  if ( SU_ANNULER == retour ) return +1; /* On annule :  erreur  mineure */

  /* Tout s'est bien pass ; l'objet choisi est dans infos->obj_choisi */
  return 0;
}

static int GER_selecteur(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  InfosSelecteur *infos;
  int retour;

  infos = ( InfosSelecteur * ) ARB_LireComplement( SELECT_UNIVERSEL );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : infos = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_TRAITE;
  }

  if ( ( objet >= SU_OBJET_1 ) && ( objet <= SU_OBJET_N ) ) {
    ARB_GererDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			 infos->nbr_objets, NULL, &(infos->obj_choisi), &(infos->premier), infos->objet_v_texte, objet, y );
    return BI_EVENEMENT_TRAITE;
  }

  switch ( objet )
  {
  case SU_OBJ_AVANT:
  case SU_FOND_ASC:
  case SU_BOUTON_ASC:
  case SU_OBJ_APRES:
    ARB_GererDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			 infos->nbr_objets, NULL, &(infos->obj_choisi), &(infos->premier), infos->objet_v_texte, objet, y );
    return BI_EVENEMENT_TRAITE;

  case SU_NOUVEAU:
    OBJ_choisit( SELECT_UNIVERSEL, SU_NOUVEAU, FORCE_OUI );
    retour = infos->nouveau( infos->obj_choisi );
    if ( 0 == retour ) {
      infos->nbr_objets++; /* Un objet de plus */
      OBJ_active( SELECT_UNIVERSEL, SU_CHOISIR, FORCE_OUI );
      ARB_PreparerDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			      infos->nbr_objets, NULL, infos->obj_choisi, infos->premier, infos->objet_v_texte );
    }
    OBJ_choisit( SELECT_UNIVERSEL, SU_NOUVEAU, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case SU_MODIFIER:
    retour = infos->modifier( infos->obj_choisi );
    if ( 0 == retour ) {
      /* On raffiche car le nom a pu changer */
      ARB_PreparerDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			      infos->nbr_objets, NULL, infos->obj_choisi, infos->premier, infos->objet_v_texte );
    }
    return BI_EVENEMENT_TRAITE;

  case SU_EFFACER:
    retour = infos->effacer( infos->obj_choisi );
    if ( 0 == retour ) {
      infos->nbr_objets--; /* Un objet de moins */
      ARB_PreparerDefileur_O( SELECT_UNIVERSEL, SU_OBJET_1, SU_OBJET_N, LGMAX_NOM_OBJET, SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES,
			      infos->nbr_objets, NULL, infos->obj_choisi, infos->premier, infos->objet_v_texte );
    }
    return BI_EVENEMENT_TRAITE;

  }

  return BI_EVENEMENT_NON_TRAITE;
}

static void prepare_selecteur(void)
{
  ARB_EnjoliverDefileur( SELECT_UNIVERSEL, SU_FOND_LISTE, SU_OBJET_1, SU_OBJET_N,
			 SU_OBJ_AVANT, SU_FOND_ASC, SU_BOUTON_ASC, SU_OBJ_APRES, ARB_ASCENSEUR_DROITE );

  arbres[ SELECT_UNIVERSEL ].drapeaux.prepare = VRAI;
  arbres[ SELECT_UNIVERSEL ].fonction.arbre = GER_selecteur;
}
