/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Généralités du programme LASE (interface graphique)

   Gestion et préparation du menu des calculs & modèles théoriques
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "interface.h"
#include "modeles.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#include "int_validite_modele.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Serie_Graphes *I_MOD_VALIDITE_EtudeDesValeursInitiales(struct modele_ajuste *modele);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Appel par l'utilisateur ~-~-~-~-~-~-~-~-~ */

/* -------- Choix de la routine en fonction de la verification demandee
 *  (normalement, appelee automatiquement depuis le sous-menu de la liste des graphes)
 *
 * modele  : le modele a controler
 * methode : la methode demandee
 */

int I_MOD_VALIDITE_ControlerValidite(struct modele_ajuste *modele, int methode)
{
  int retour = 0; /* Le modèle est valide, par défaut */

  if (NULL == modele) return -1;
  if ( VRAI == modele->drapeaux.en_cours ) {
    (void) BI_ERR_alerte(NULL, & ERR_AjustementEnCours, VRAI);
    return -2;
  }

  switch (methode)
  {
  case MODELE_VAL_BORNES:
    retour = MODELE_ControlerBornes(modele);
    if( 0 == retour ) {/* Pas de problème particulier trouvé */
      (void) BI_ERR_alerte(NULL, &MSG_BornesCorrectes, VRAI);
    } else {
      OPTION_A_FAIRE( "modèle modifié -- proposer de relancer l'ajustement" );
    }
    break;

  case MODELE_VAL_PHYSIQUE:
    retour = MODELE_ControlerPhysique( modele );
    if( 0 == retour ) {/* Pas de problème particulier trouvé */
      (void) BI_ERR_alerte( NULL, &MSG_PhysiqueCorrecte, VRAI );
    } else if (retour < 0) {
      OPTION_A_FAIRE( "modèle modifié -- proposer de relancer l'ajustement" );
    }
    break;

  case MODELE_VAL_STATS:
    MODELE_TestKhiDeux( modele );
    MODELE_TestFisherSnedecor( modele );
    return 0;

  case MODELE_VAL_MONTECARLO: /* Appeler la méthode de Monte-Carlo */
    MODELE_InterfaceMonteCarlo( modele );
    return 0;

  case MODELE_VAL_MC_RESIDUS: /* Id., mais pour une étude sur les résidus */
    {
      Graphe *vx_experience;
      int retour;

      if ( NULL == modele->modele ) {
	(void) BI_ERR_alerte( NULL, &ERR_AucunModele, VRAI );
	return -3;
      }

      /* On assigne au modèle les erreurs expérimentales */
      OPTION_A_FAIRE( "Mettre les erreurs experimentales au modele !" )

      /* On remplace le spectre expérimental par le modèle obtenu */
      vx_experience = modele->experience;
      retour = MODELE_MettreReference( modele, modele->modele, FAUX );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s : ERREUR dans MODELE_MettreReference [code %d]\n",
		       __FUNCTION__, retour);
	return -4;
      }

      /* On fait Monte-Carlo là-dessus */
      MODELE_InterfaceMonteCarlo(modele);

      /* On remet l'ancien graphe expérimental */
      (void) MODELE_MettreReference( modele, vx_experience, FAUX );
    }
    return 0;

  case MODELE_VAL_INITIALES:
    I_MOD_VALIDITE_EtudeDesValeursInitiales( modele );
    return 0;

  default:
    AVERTISSEMENT( "méthode inconnue [%d]", methode );
  }

  /* On a fini */
  return retour;
}

/* -------- Etude de l'influence des valeurs initiales sur le resultat trouve
 *  Renvoie : la serie qui contient les resultats
 *
 * modele : le modele dont on veut etudier la validite
 */

static Serie_Graphes *I_MOD_VALIDITE_EtudeDesValeursInitiales(struct modele_ajuste *modele)
{
  Serie_Graphes *serie = NULL;
  int reponse;

  if ( NULL == modele ) return NULL;

  /* On demande si l'on veut une grille reguliere ou du Monte-Carlo */
  reponse = BI_ERR_alerte( NULL, &Q_ChoixMethodeVI, VRAI );
  if ( 1 == reponse ) {
    OPTION_A_FAIRE( "Exploration par une grille reguliere de l'espace des valeurs initiales" );
  } else {
    unsigned long nombre_tirages = 1000;

    OPTION_A_FAIRE( "Exploration par une grille aleatoire de l'espace des valeurs initiales : demander le nombre de tirages !" );
    serie = MODELE_VALIDITE_ValeursInitiales_MC( modele, nombre_tirages );
  }

  OPTION_A_FAIRE( "Preparer script R d'analyse des resultats" );

  /* On renvoie la serie des resultats obtenus */
  return serie;
}
