/* Traitement de l'E.X.A.F.S. sur Atari
   Tout ce qui concerne le lissage
   (c) 1997 - Emmanuel CURIS

 12 janvier 1998
   * Inclus les messages d'erreur GEM

  2 dcembre 1997
   * Corrig le calcul des splines : c[0] et c[1] taient faux ! Merci  Bruno.

*/

#define _LISSAGE_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "lissage.h"

/*********** Lissage par un polynme, le plus adapt possible **********************************
 *  Utilisent diffrentes versions des moindres carrs                                         *
 *  Renvoient l'erreur entre la courbe calcule et la courbe de rfrence                      *
 ***********************************************************************************************/

/* ----- Moindres carrs, mthode de Juris (plus lente, mais meilleure ?) ---------------------\
 |  Renvoie l'erreur si le polynme a pu tre calcul correctement, -1 sinon
 |
 | poly      : le tableau des coefficients du polynme en retour ([0] + [1]*x + [2]*x^2 + ...) |
 | degre     : le degr du polynme d'interpolation (ventuellement ngatif...)
 | graphe    : le graphe sur lequel on va travailler
 | nbr_zones : le nombre d'intervalles de points valables
 | zone      : tableau des bornes des intervalles (0 : min1, 1 : max1, 2 : min2, 3 : max2,...) |
 \-------------------------------------------------------------------------------------------- */

double MC_Polynome_SurPartieGraphe_Juris(const Graphe *graphe, int degre, int nbr_zones, double *zones, double *poly)
{
  register double *zone_x,*zone_y;
  double delta_x,a[13][13],inter,x,d,y[13];
  double x_min,x_max,erreur = 0, somme_y2 = 0;
  int k,i,l;
  unsigned long index;
  boolean convient = VRAI;

  if ( (nbr_zones == 0) || (zones == NULL) ) /* Aucune zone entre */
    return -1.0;

  /* Recherche des bornes "totales" */
  x_min = zones[0]; x_max = zones[1];
  for (i = 2 ; i < 2*nbr_zones ; i++) {
    if (zones[i] < x_min) x_min = zones[1];
    if (zones[i] > x_max) x_max = zones[2];
  }

  /* Prparation des tableaux pour la recherche des coefficients */
  if (x_max == x_min) /* Zones dfinies sont vides ! */
    return -2;

  delta_x = x_max - x_min; delta_x *= delta_x; /* Donc vaut (max - min)^2 */
  for (k = 0 ; k <= degre ; k++) {
    for (i = 0 ; i <= degre ; i++) {
      inter = 0;
      zone_x = graphe->x + 1;
      for (index = 1 ; index < graphe->nombre_point ; index++) {
	x = *zone_x;
				
	/* Le point convient-il ? */
	convient = FAUX;
	for (l = 0 ; l < 2*nbr_zones ; l += 2) {
	  if ( ( x >= zones[l] ) && (x <= zones[l + 1]) ) {
	    convient = VRAI;
	    break;
	  }
	}
	
	if ( convient )	{
	  double rien = 1;
	  d = x - *(zone_x - 1);
	  d *= d / delta_x;

	  x /= x_max;
	  for (l = 0 ; l < k + i ; l++) { /* Calcul de (x/x_max)^(k+i) */
	    rien *= x;
	  }
	  inter += d * rien;
	}
	zone_x++;
      }
      a[k+1][i+1] = inter;
    }

    inter = 0;
    zone_x = graphe->x + 1;
    zone_y = graphe->y + 1;
    for (index = 1 ; index < graphe->nombre_point ; index++) {
      x = *zone_x;

      /* Le point convient-il ? */
      convient = FAUX;
      for (l = 0 ; l < 2*nbr_zones ; l += 2) {
	if ( ( x >= zones[l] ) && (x <= zones[l + 1]) )	{
	  convient = VRAI;
	  break;
	}
      }

      if ( convient ) {
	double rien = 1;

	d = x - *(zone_x - 1);
	d *= d / delta_x;
	x /= x_max;
	for (l = 0 ; l < k ; l++) { /* Calcul de (x/x_max)^k */
	  rien *= x;
	}
	inter += d * rien * *zone_y;
      }
      zone_x++; zone_y++;
    }
    a[k+1][degre+2] = inter;
  }

  if ( a[1][1] == 0 ) {
    EXF_ERREUR(DIV_ZERO, "Coefficient du lissage", txt_err_calc_annule);
    return -1.0;
  }
  for (k = 2 ; k <= degre + 2 ; k++)
    a[1][k] /= a[1][1];

  y[0] = a[1][degre + 2];
  for (k = 2 ; k <= degre + 1 ; k++) {
    for (i = k ; i <= degre + 1 ; i++) {
      x = 0;
      for (l = 1 ; l < k ; l++) /* Produit de matrices... */
	x += a[i][l] * a[l][k];
      a[i][k] -= x;
    }

    if (k <= degre + 1) {
      for (i = k + 1 ; i <= degre + 1 ; i++) {
	x = 0;
	for (l = 1 ; l < k ; l++) /* Produit de matrices... */
	  x += a[k][l] * a[l][i];
	a[k][i] -= x;
	
	if ( a[k][k] == 0) {
	  EXF_ERREUR(DIV_ZERO, "Coefficient du lissage", txt_err_calc_annule);
	  return -1;
	}
	a[k][i] /= a[k][k];
      }
    }
		
    x = 0;
    for (l = 1 ; l < k ; l++)
      x += a[k][l] * y[l-1];
    y[k-1] = ( a[k][degre+2] - x ) / a[k][k];
  }

  for (k = 0 ; k < degre ; poly[k++] = 0); /* On initialise les coefs. */
  poly[degre] = y[degre];

  for (k = degre ; k > 0 ; k--) {
    x = 0;
    for (l = k ; l <= degre ; l++)
      x += a[k][l+1] * poly[l];
    poly[k-1] = y[k-1] - x;
  }
	
  x = 1;
  for (k = 0 ; k <= degre ; k++) {
    poly[k] /= x;
    x *= x_max;
  }

  /* Calcul de l'erreur */
  zone_x = graphe->x; zone_y = graphe->y;
  for (index = 0 ; index < graphe->nombre_point ; index++) {
    x = *(zone_x);

    /* Le point convient-il ? */
    convient = FAUX;
    for (i = 0 ; i < 2*nbr_zones ; i+=2) {
      if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	convient = VRAI; /* Le point appartient  un des intervalles proposs */
    }

    if ( convient == VRAI ) { /* On est dans les bornes */
      double rien = poly[0];

      somme_y2 += *zone_y * *zone_y;
      for (i = 1 ; i <= degre ; i++) {
	rien += poly[i] * x;
	x *= *(zone_x);
      }
      rien -= *zone_y;
      erreur += rien * rien;
    }
    zone_x++; zone_y++;		
  }

  return erreur / somme_y2;
}
