/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Lissage de valeurs

   Mthodes d'interpolation linaire gnriques 1D, 2D,...
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"
#include "lissage.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Interpolation linaire 1D de base

                     y2 - y1
   y = y1 + (x - x1) -------
                     x2 - x1
*/

double LIS_lineaire(double x1, double x2, double y1, double y2, double x)
{
  double pente;

  if (x2 == x1) {
    EXF_ERREUR(DIV_ZERO, "Lissage/lineaire.c", "LIS_lineaire", txt_err_val_nulle);
    return 0.0;
  }

#ifdef _DEVELOPPE_
  if ( ( (x2 > x1) && ( (x > x2) || (x < x1) ) ) ||
       ( (x1 > x2) && ( (x > x1) || (x < x2) ) ) ) {
    fprintf(stdout/*stderr*/, "LIS_lineaire : ATTENTION, il s'agit d'une EXTRAPOLATION\n");
  }
#endif

  pente = (y2 - y1) / (x2 - x1);
  return y1 + pente * (x - x1);
}

/* -------- Interpolation linaire pour un graphe [suppos tri] -------- */

double GRF_InterpoleLineaire(const Graphe *graphe, double x, unsigned long *depart)
{
  unsigned long i = 0, i_avant;

  if (graphe == NULL) return 0.0;

  if (x <= graphe->x_min) return graphe->y[0];
  if (x >= graphe->x_max) return graphe->y[graphe->nombre_point - 1];

  if (depart != NULL) {
    if (*depart >= graphe->nombre_point) *depart = 0; /* Pour ne pas dborder */

    i = *depart;
  }

  i_avant = LIS_RepererPointAvant(graphe->x, graphe->nombre_point, x, i);
  if ( x == graphe->x[i_avant] ) return graphe->y[i_avant];

  if (depart != NULL) *depart = i_avant + 1; /* Car sera probablement le i_avant du suivant ! */

/*  printf("on doit interpoler ! x demand : %.20g encadr par %.20g et %.20g\n", x, graphe->x[i_avant], graphe->x[i_avant + 1]); */
  return LIS_lineaire(graphe->x[i_avant], graphe->x[i_avant + 1], graphe->y[i_avant], graphe->y[i_avant + 1], x);
}

/* -------- Estimation de l'erreur sur une interpolation linaire -------- */

double GRF_InterpoleLineaire_Erreur(const  Graphe *graphe, double x, unsigned long *depart)
{
  unsigned long i_avant, i = 0;
  double erreur = 0.0;

  if (NULL == graphe) return -1.0;
  if ( (NULL == graphe->erreur_x) && (NULL == graphe->erreur_y) ) {
    (void) fprintf(stderr, "GRF_InterpoleLineaire_Erreur : pas d'erreurs associes au graphe %s !\n", graphe->nom);
    return 0.0;
  }

  if (depart != NULL) {
    if (*depart >= graphe->nombre_point) *depart = 0; /* Pour ne pas dborder */

    i = *depart;
  }

  i_avant = LIS_RepererPointAvant(graphe->x, graphe->nombre_point, x, i);
  if ( x == graphe->x[i_avant] ) { /* valeur exacte : erreur inchange */
    if (NULL == graphe->erreur_y)
      return 0.0; /* valeur exacte */
    else
      return graphe->erreur_y->y[i_avant];
  }

  if ( graphe->erreur_x == NULL ) {
    double facteur;
    double sigma_y;

    facteur = graphe->x[i_avant + 1] - graphe->x[i_avant];
    if (0.0 == facteur) {
      (void) fprintf(stderr, "GRF_InterpoleLineaire_Erreur : points confondus, pas d'interpolation\n");
      return -1.0;
    }
    facteur = (x - graphe->x[i_avant]) / facteur;

    sigma_y = graphe->erreur_y->y[i_avant + 1];
    erreur  = facteur * facteur * sigma_y * sigma_y;

    sigma_y = graphe->erreur_y->y[i_avant];
    facteur = 1.0 - facteur;
    erreur += facteur * facteur * sigma_y * sigma_y;

    erreur = sqrt(erreur);
  } else {
    printf("GRF_InterpoleLineaire_Erreur :  FAIRE, cas d'erreurs sur les X prsentes\n");
  }

  return erreur;  
}

/* -------- Interpolation "bilinaire" 2D selon un plan --------

   Plan : | x - x1  x2 - x1  x3 - x1 |           | y2 - y1  y3 - y1 |           | x2 - x1  x3 - x1 |           | x2 - x1  x3 - x1 |
          | y - y1  y2 - y1  y3 - y1 | = (x - x1)|                  | - (y - y1)|                  | + (z - z1)|                  | = 0
          | z - z1  z2 - z1  z3 - z1 |           | z2 - z1  z3 - z1 |           | z2 - z1  z3 - z1 |           | y2 - y1  y3 - y1 |

   soit D_x * x - D_y * y + D_z * z - x1 * D_x + y1 * D_y - z1 * D_z = 0

 */

double LIS_planaire(double x1, double y1, double z1, double x2, double y2, double z2, double x3, double y3, double z3,
		    double x, double y)
{
  double D_x, D_y, D_z, D;
  double x21, y21, z21, x31, y31, z31;
  double valeur;

  x21 = x2 - x1; y21 = y2 - y1; z21 = z2 - z1;
  x31 = x3 - x1; y31 = y3 - y1; z31 = z3 - z1;

  D_z = x21 * y31 - y21 * x31;
  if (D_z == 0.0) {
    EXF_ERREUR(DIV_ZERO, "Lissage/lineaire.c", "LIS_planaire", txt_err_val_nulle);
    return 0.0;
  }
  D_x = y21 * z31 - z21 * y31;
  D_y = x21 * z31 - z21 * x31;
  D = -D_x * x1 + D_y * y1 - D_z * z1;

  valeur = - D_x * x + D_y * y - D;
  valeur /= D_z;
  return valeur;
}

/* ~~~~~~~~ Reprage d'un x dans la table des valeurs indique ~~~~~~~~ */

unsigned long LIS_RepererPointAvant(double *valeurs, unsigned long nbr_valeurs, double x, unsigned long i_depart)
{
  if (NULL == valeurs) return 0;

  /* Cas simples : table suppose trie par X croissants */
  if ( x <= valeurs[ 0 ] ) return 0;
  if ( x >= valeurs[ nbr_valeurs - 1 ] ) return nbr_valeurs - 1;

  /* On s'assure que l'on dmarre bien dans la table */
  if (i_depart >= nbr_valeurs)
    i_depart = 0;

  if ( valeurs[ i_depart ] > x ) {
    while ( valeurs[ i_depart ] > x ) {
      if (0 == i_depart) {  /* Comme forcment x[0] < x, pas de risque - sauf si non-tri ! */
	(void) fprintf(stderr, "LIS_RepererPointAvant : table non trie par X croissants !\n");
	return 0;
      }
      i_depart--;
    }
  } else if ( valeurs[ i_depart ] < x ) {
    while ( valeurs[ i_depart ] < x ) {
      i_depart++;
      if (i_depart == nbr_valeurs) {
	(void) fprintf(stderr, "LIS_RepererPointAvant : table non trie par X croissants !\n");
	return 0;
      }
    }
    if (x < valeurs[ i_depart ]) /* On peut avoir galit, auquel cas i_depart est bon */
      i_depart--;
  }

  return i_depart; /* Numro de la dernire case telle que v(i) <= x ==> x(i) <= x < x(i + 1) */
}
