/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-2001, Emmanuel CURIS
   Cration              : 11 avril 2001
   Dernire modification : 

   Modules de calcul matriciel
   Messages d'erreur
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>

#include "systeme.h"
#include "erreurs.h"

#define NOMBRE_MESSAGES 6

#include "mat_erreurs.txt"

int MAT_ERREUR(unsigned int code, ...)
{
  const char *p1;
  double d1;
  int retour;
  unsigned int i1, i2;
  va_list params;

  if (code >= NOMBRE_MESSAGES)
    return SYS_ERREUR(ERREUR_INCONNUE, code, "Calcul matriciel");

  va_start(params, code);
  switch (code)
  {
  case MAT_VAL_ABSURDE: /* Quatre params : unsigned int, unsigned int, char, double */
    i1 = va_arg(params, unsigned int);
    i2 = va_arg(params, unsigned int);
    p1 = va_arg(params, const char *);
    d1 = va_arg(params, double);
    retour = BI_ERR_alerte( NULL, ERR_Matrices + code, VRAI, p1, i1, i2, d1);
    break;

  case MAT_TYPE_IMPOSSIBLE:/* Un paramtre : char * */
    p1 = va_arg(params, const char *);
    retour = BI_ERR_alerte( NULL, ERR_Matrices + code, VRAI, p1);
    break;

  case MAT_ELEMENT_INEXISTANT: /*Deux paramtres : unsigned int, unsigned int */
    i1 = va_arg(params, unsigned int);
    i2 = va_arg(params, unsigned int);
    retour = BI_ERR_alerte( NULL, ERR_Matrices + code, VRAI, i1, i2);
    break;

  default: /* Aucun paramtre */
    retour = BI_ERR_alerte( NULL, ERR_Matrices + code, VRAI);
  }

  va_end(params);
  return retour;
}
