/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996-2002, Emmanuel CURIS
   Cration de ce fichier : 6 septembre 2002

   Routines gnriques pour les ajustements

   Ralisation complte d'un ajustement, initialisations comprises
   Pas forcment optimal, car trs gnrique, mais simple d'usage...

   ATTENTION : comme trs gnrique, il n'y a pas de redessin ni de contrle...
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "systeme.h"

#include "modeles.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

int MODELE_AjustementComplet(ModeleAjustement *modele, int (*calcule_modele)(struct modele_ajuste *),
			     boolean dessins, boolean afficher_resultats, int action_ancien_modele)
{
  Fenetre *fenetre = NULL;
  int retour;

  /* Vrifications prliminaires */
  if ( NULL == modele ) return +1;

  /* 1. Prparer les graphes... [si modle en cours d'dition, les nouveaux paramtres seront lus ici]...
        Vrifie aussi que le modle n'est pas dj en cours d'ajustement (donc inutile de le refaire ici)
   */
  retour = MODELE_PreparerAjustement( modele, action_ancien_modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "MODELE_AjustementComplet : ERREUR dans MODELE_PreparerAjustement(), code %d\n", retour );
    (void) MODELE_TerminerAjustement( modele, FAUX );
    return retour;
  }

  /* 2. Initialisation */
  retour = MODELE_Initialiser( modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "MODELE_AjustementComplet : ERREUR dans MODELE_Initialiser(), code %d\n", retour );
    (void) MODELE_TerminerAjustement( modele, FAUX );
    return retour;
  }

  /* 2 bis : contrler la validit du modle ! */
  retour = MODELE_ControlerModele( modele, FAUX );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s : ERREUR dans MODELE_ControlerModele(), code %d\n", __FUNCTION__, retour );
    (void) MODELE_TerminerAjustement( modele, FAUX );
    return retour;
  }

  /* 3. Prparation des variables */
  retour = MODELE_InitVariables( modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "MODELE_AjustementComplet : ERREUR dans MODELE_InitVariables(), code %d\n", retour );
    (void) MODELE_TerminerAjustement( modele, FAUX );
    return retour;
  }

  /* 4. Si on dessine les diffrentes tapes : cration de la fentre... */
  if ( VRAI == dessins ) {
    OPTION_A_FAIRE( "Ouverture de la fentre de redessin" );
  }

  /* 5. Minimisation : le vrai truc intressant... */
  retour = MODELE_AjusterModele( modele, calcule_modele );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s,%s l.%u : ERREUR dans MODELE_AjusterModele(), code %d\n",
		    __FILE__, __FUNCTION__, __LINE__, retour );
  }

  /* 6. On affiche les rsultats, tout de mme... */
  retour = MODELE_TerminerAjustement( modele, afficher_resultats );

  /* 7. On a fini : on ferme la fentre, si elle existe */
  FEN_Detruire( fenetre );

  /* Tout s'est bien pass */
  return PAS_DE_PROBLEME;
}
