/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Création : 4 décembre 1998

   Opérations diverses sur les graphes, pas spécifiques de l'exafs

   Élimination d'un point aberrant par interpolation polynomiale ou par une spline
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "operations.h"

#include "interface.h" /* RCT_pointeur */
#include "lissage.h" /* Utilisation des splines */
#include "tracer.h"

#include "op_aberrance.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

ABER_param param_aber, *prem_aber = NULL;

static double x_vue, y_vue, larg_vue, haut_vue;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void RDS_aberrance(Fenetre *fen, int x, int y, unsigned int larg, unsigned int haut);
static int  MSG_aberrance(Fenetre *fen, Message *message);
static int  CLC_aberrance(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

static int  GER_aberrance(unsigned int num_objet, int x, int y);
static int  prepare_arbre_aberrance(unsigned int);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* -------- Appel de la fenêtre ------------------------------------------- */

void aberrance(Graphe *graphe)
{
  Fenetre *fenetre;

  if (graphe == NULL) return;    /* Pas de graphe ! */
  if (graphe->nombre_point < 11) {
    EXF_ERREUR(TROP_PEU_POINTS);
    return;
  }

  if ( graphe->genre_donnees != GRF_FCT_REELLE ) {
    EXF_ERREUR(GRAPHE_COMPLEXE);
    return;
  }

  fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, titre_fenetre_aberrance, NULL,
				ELIM_ABERRANCE, BI_FEN_ARBRE_HAUT, prepare_arbre_aberrance,
				RDS_aberrance, ( int (*)(Fenetre *, unsigned int, int) ) NULL,
				CLC_aberrance, MSG_aberrance, RCT_pointeur,
				( int (*)(Fenetre *) ) NULL );
  if (fenetre == NULL) return;

  OPTION_A_FAIRE( "Réserver au vol les informations sur les aberrances !" );

  fenetre->drapeaux.redessin = VRAI;
  fenetre->msg_masque.fermeture    = VRAI; /* Message de fermeture...  */

  param_aber.degre = OBJ_LireEntier_U( ELIM_ABERRANCE, ABER_VALEUR );
  param_aber.graphe = graphe;
  param_aber.x_borne[0] = *(graphe->x);
  param_aber.x_borne[1] = *(graphe->x + 10) ;
  param_aber.x_borne[2] = *(graphe->x + graphe->nombre_point - 11);
  param_aber.x_borne[3] = *(graphe->x + graphe->nombre_point - 1);

  x_vue = graphe->x_min;            /* Au début, tout le spectre est visible */
  y_vue = graphe->y_min;
  larg_vue = graphe->x_max - x_vue;
  haut_vue = graphe->y_max - y_vue;

  OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN1, param_aber.x_borne[0], 0.0 );
  OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX1, param_aber.x_borne[1], 0.0 );
  OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN2, param_aber.x_borne[2], 0.0 );
  OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX2, param_aber.x_borne[3], 0.0 );

  /* On associe les infos à la fenêtr */
  FEN_GRF_LierGraphe( fenetre, graphe );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, &param_aber );
}

/* ~~~~~~~~~~~~~~~ Redessin de cette fenêtre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static void RDS_aberrance(Fenetre *fenetre, int x_rds, int y_rds, unsigned int larg, unsigned int haut)
{
  ABER_param *param_aber;
  char nombre[ 2 ] = { 0, 0 };
  double x_echelle,y_echelle;
  int i;
  double *b = NULL, *c = NULL, *d = NULL, *tableau = NULL;
  unsigned long nbr_spline;
  double erreur;

  /* Préparation du dessin */
  TRC_InitDessins( fenetre, x_rds, y_rds, larg, haut );

  param_aber = (ABER_param *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == param_aber ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'information associée à la fenêtre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  /* Calcul des facteurs d'échelle */
  x_echelle = ( (double) fenetre->w_larg) / (larg_vue);
  y_echelle = ( (double) fenetre->w_haut) / (haut_vue);

  /* On trace la courbe expérimentale - en trait continu, noir */
  TRC_Graphe( fenetre, param_aber->graphe, NOIR, 1, 0, 0, fenetre->w_x, fenetre->w_y,
	      x_vue, x_vue + larg_vue, y_vue, y_vue + haut_vue,
	      x_echelle, y_echelle );

  /* On calcule le polynôme d'interpolation entre les deux points */

  if ( OBJ_choisit(ELIM_ABERRANCE, ABER_POLY1, QUESTION) )
    erreur = MC_Polynome_SurPartieGraphe( param_aber->graphe, param_aber->degre, 2, param_aber->x_borne, param_aber->coefs );
  else if ( OBJ_choisit(ELIM_ABERRANCE, ABER_POLY2, QUESTION) )
    erreur = MC_Polynome_SurPartieGraphe_Juris( param_aber->graphe, param_aber->degre, 2, param_aber->x_borne, param_aber->coefs );
  else
    erreur = SPL_prepare(param_aber->graphe, 2, param_aber->x_borne, &b, &c, &d, &tableau, &nbr_spline);

  if (erreur >= 0) {
#ifdef ancien
    unsigned long i;
    register double *x;

    met_svaleur(ELIM_ABERRANCE, ABER_ERREUR, erreur); /* Erreur calculée */
    vsl_color(_PRG_station->handle, BLEU);

    compteur = 0;
    for (i = 0, x = param_aber->graphe->x ;
	 i < param_aber->graphe->nombre_point ; i++, x++) {
      if ( (*x >= param_aber->x_borne[0]) && (*x <= param_aber->x_borne[3]) ) {
	if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) ) /* Calcul par Spline */
	{
	  y = SPL_calcule(*x, nbr_spline, tableau, b, c, d);
	} else {
	  short int j;
	  double x_p_n = *x;

	  y = param_aber->coefs[0];
	  for (j = 1 ; j <= param_aber->degre ; j++) {
	    y += x_p_n * param_aber->coefs[j];
	    x_p_n *= *x;
	  }
	}
	points[compteur]   = fenetre->w_x + x_echelle * ( *x - x_vue);
	points[compteur+1] = fenetre->w_y + y_echelle * (y_vue + haut_vue - y);
	compteur += 2;	
      }
      
      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle,128,points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }
    v_pline(_PRG_station->handle, (compteur + 1) >> 1,points);
#endif
    OPTION_A_FAIRE( "Tracer le modèle : graphe isolé, polynome, spline ?" );
  }

  /* On libère la mémoire éventuellement utilisée pour une SPLINE */
  if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) ) {
    free(b); free(c); free(d);
    free(tableau);
  }

  /* On place les quatre curseurs indiquant les limites */
  TRC_Mode( MD_INVERSE ); /* Mode inverser */
  TRC_Couleur( ROUGE );
  for ( i = 0 ; i < 4 ; i++ ) {
    TRC_LigneV( fenetre, x_echelle * ( param_aber->x_borne[ i ] - x_vue ), 0, fenetre->w_haut );
  }

  /* On met les numéros associés */
  TRC_Couleur( BLEU );
  for ( i = 0 ; i < 4 ; i++ ) {
    nombre[ 0 ] = '1' + i;
    TRC_Texte( fenetre, x_echelle * ( param_aber->x_borne[ i ] - x_vue ), 5, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, nombre );
  }
}

/* ----------- Gestion du clic dans cette fenêtre ---------------------------------------- */

static int CLC_aberrance(Fenetre *fenetre, int bouton, int n_clic __attribute__((__unused__)), int etat __attribute__((__unused__)), int x_souris, int y_souris)
{
  ABER_param *param_aber;
  double x_clic;
  double x_echelle;
  int x_borne[4], x_mem __attribute__((__unused__)), retour, vieux_x;
  short int i;
  unsigned int rien;
  boolean lieu_clic = 0;
  char numero[2] = { '1', 0 };

#ifdef _GEM_
  int handle;

  wind_get(0, WF_TOP, &handle);
  if (handle != fenetre->handle) {
    wind_set(fenetre->handle, WF_TOP);
    return BI_EVENEMENT_TRAITE;
  }
#endif

  param_aber = (ABER_param *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == param_aber ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'information associée à la fenêtre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_TRAITE;
  }

  x_echelle = (double) (fenetre->w_larg) / (larg_vue);

  TRC_Mode( BI_MODE_INVERSE );

  if (bouton == 2) /* Clic du bouton droit = sélectionner une région et la grossir */
  {
    int x = x_souris, y = y_souris, bouton;

    bouton = prepare_zoom(fenetre, x_souris, y_souris, &x, &y);

    if ( bouton == 1) /* On valide ce qu'on vient de faire */
    {
      double min, max;
      double echelle;
      boolean modifie = FAUX;

      /* On s'occupe des X */
      min = x_vue + (x_souris - fenetre->w_x) / x_echelle;
      max = x_vue + (x        - fenetre->w_x) / x_echelle;
      if (max < min) {
	double x = max; max = min; min = x;
      }
      if (max != min) {
	x_vue = min;
	larg_vue = max - min;
	modifie = VRAI;
      }

      /* On s'occupe des Y */
      echelle = ( (double) fenetre->w_haut) / haut_vue;
      min = y_vue + haut_vue - (y_souris - fenetre->w_y) / echelle;
      max = y_vue + haut_vue - (y        - fenetre->w_y) / echelle;
      if (max < min) {
	double y = max; max = min; min = y;
      }
      if (max != min) {
	y_vue = min;
	haut_vue = max - min;
	modifie = VRAI;
      }

      if (modifie == VRAI) /* On a changé l'échelle */
	FEN_redessin(fenetre, BI_RDS_UTIL); /* donc on redessine */
    } else {
/*       v_pline(_PRG_station->handle, 5, points); */
    }

    return BI_EVENEMENT_TRAITE;
  }

  /* Si on est là, clic gauche = on essaye de positionner une barre de paramétrage */

  x_borne[0] = (param_aber->x_borne[0] - x_vue) * x_echelle + fenetre->w_x;
  x_borne[1] = (param_aber->x_borne[1] - x_vue) * x_echelle + fenetre->w_x;
  x_borne[2] = (param_aber->x_borne[2] - x_vue) * x_echelle + fenetre->w_x;
  x_borne[3] = (param_aber->x_borne[3] - x_vue) * x_echelle + fenetre->w_x;

  for ( i = 0 ; i < 4 ; i++ ) {
    if ( (x_souris > x_borne[i] - 5) && (x_souris < x_borne[i] + 5) ) {
      lieu_clic = i + 1;
    }
  }

  if (lieu_clic == 0) { /* On n'a cliqué nulle part */
    return BI_EVENEMENT_TRAITE;
  }

  if (lieu_clic > 0) {
    TRC_Couleur( NOIR );
    TRC_LigneV( fenetre, x_souris, 0, fenetre->w_haut );
    x_mem = x_souris;

    x_souris = x_borne[lieu_clic - 1];
    numero[0] += (lieu_clic - 1);

    vsl_color(_PRG_station->handle, ROUGE);  /* On travaille en rouge, c'est plus joli... */
    vst_color(_PRG_station->handle, BLEU );  /* Avec le numéro en bleu                    */

    retour = BI_AttendClic( fenetre, 0, RCT_pointeur, &x_souris, &y_souris, NULL );
    if (retour == 1) /* On valide le changement de position du truc */
    {
      x_borne[lieu_clic - 1] = x_souris;
      /* On efface les anciens numéros */
      for (i = 0 ; i < 4 ; i++) {
	numero[0] = '1' + i;

	TRC_Texte( fenetre, x_echelle * ( param_aber->x_borne[ i ] - x_vue ), 5, BI_TEXTE_CENTRER_H, BI_TEXTE_FER_EN_HAUT, 0, numero );
      }

      /* On enregistre _la_ nouvelle valeur */
      param_aber->x_borne[lieu_clic - 1] = x_vue 
	                                 + (x_borne[lieu_clic - 1] - fenetre->w_x) / x_echelle;

      /* On trie les valeurs pour avoir x1_min < x1_max < x2_min < x2_max */
      for (i = 0 ; i < 4 ; i++)
	for (rien = i + 1 ; rien < 4 ; rien++)
	  if (param_aber->x_borne[rien] < param_aber->x_borne[i]) /* On intervertit les deux */
	  {
	    x_clic = param_aber->x_borne[rien];
	    vieux_x = x_borne[rien];
	    param_aber->x_borne[rien] = param_aber->x_borne[i];
	    x_borne[rien] = x_borne[i];
	    param_aber->x_borne[i] = x_clic;
	    x_borne[i] = vieux_x;
	  }
      
      /* On réaffiche les boutons */
      OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN1, param_aber->x_borne[0], 0.0 );
      OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX1, param_aber->x_borne[1], 0.0 );
      OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN2, param_aber->x_borne[2], 0.0 );
      OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX2, param_aber->x_borne[3], 0.0 );

      /* On fait redessiner */
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
  }

  return BI_EVENEMENT_TRAITE;
}

/* ----------- Gestion des messages pour cette fenêtre ----------------------------------- */

static int MSG_aberrance(Fenetre *fenetre, Message *message)
{
  ABER_param *param_aber;
  Graphe *resultat,*source;
  double x,y,*b,*c,*d,*tableau;
  register double *zone_x,*zone_y;
  int reponse;
  unsigned long i, nbr_pts;
  char *titre;

  param_aber = (ABER_param *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  switch (message->type) /* Message à considérer */
  {
  case WM_CLOSED: /* Fermeture de la fenêtre */
    if ( message->fenetre.haut == -1 ) /* Fermeture forcée de la fenêtre */
      reponse = 2;
    else {
      reponse = EXF_ERREUR(FIN_MODULE);
      if (reponse == 3) return BI_EVENEMENT_TRAITE; /* On ne fait rien */
    }

    if (reponse == 2) { /* On ne veut pas éliminer l'aberrance */
      FEN_Detruire( fenetre );
      return BI_EVENEMENT_FEN_FERMEE;
    }

    source = param_aber->graphe;

    /* On élimine l'aberrance - suppose le graphe trié */

    titre = TXT_genere_chaine(titre_graphe_corrige, source->nom);
    resultat = GRF_nouveau(source, source->nombre_point, source->type, source->genre_donnees, titre);
    free( titre ); titre = NULL;
    if (resultat == NULL) /* On quitte sans fermer, pour que l'utilisateur ne perde rien */
      return BI_EVENEMENT_TRAITE;
    
    FEN_Detruire( fenetre );

    zone_x = source->x; zone_y = resultat->y;

    if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) ) /* On veut une spline */
    {
      SPL_prepare(source,2,param_aber->x_borne,&b,&c,&d,&tableau,&nbr_pts); /* On la prépare */
      for (i = 0 ; i < source->nombre_point ; i++) {
	x = *(zone_x);
	*(resultat->x + i) = x;
	if ( ( x < param_aber->x_borne[1] ) || ( x > param_aber->x_borne[2] ) )
	  y = *(source->y + i);
	else {
	  y = SPL_calcule(x,nbr_pts,tableau,b,c,d);
	}
	*zone_y = y;
	zone_x++; zone_y++;
      }
      free(b); free(c); free(d); free(tableau); /* On libère tout */
    } else /* On veut un polynôme, les coefficients en sont déjà calculés */
    {
      for (i = 0 ; i < source->nombre_point ; i++) {
	x = *(zone_x);
	*(resultat->x + i) = x;
	if ( ( x < param_aber->x_borne[1] ) || ( x > param_aber->x_borne[2] ) )
	  y = *(source->y + i);
	else {
	  short int j;

	  y = param_aber->coefs[0];
	  for (j = 1 ; j <= param_aber->degre ; j++) {
	    y += param_aber->coefs[j] * x;
	    x *= *(zone_x);
	  }
	}
	*zone_y = y;
	zone_x++; zone_y++;
      }
    }

    resultat->x_min = source->x_min; /* Les bornes en X n'ont pas changé */
    resultat->x_max = source->x_max;
    GRF_bornes(resultat, NE_PAS_FAIRE, DESORDONNE);

    /* Barres d'erreur : aucune modification en X
       En Y : on suppose que seule la valeur moyenne est mesurée fausse,
       mais que le fait d'avoir un point aberrant ne modifie pas la statistique
       à l'origine des erreurs -> pas d'effet sur l'écart-type */
    resultat->erreur_x = GRF_Copier( source->erreur_x, NULL, VRAI );
    resultat->erreur_y = GRF_Copier( source->erreur_y, NULL, VRAI );

    HIST_Titre(1, E_AIGU"liminer un point aberrant", "Aberrance");
    HIST_Texte(1, 0, "Graphe original : %s\n", source->nom);
    HIST_Texte(1, 0, "Graphe trait"e_AIGU" : %s\n", resultat->nom);
    HIST_Texte(4, 0, "M"e_AIGU"thode d'interpolation : ");
    if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) == VRAI ) {
      HIST_Texte(4, 0, "spline cubique\n");
    } else {
      HIST_Texte(4, 0, "polyn"o_CIRC"me");
      HIST_Texte(5, 0, " (degr"e_AIGU" : %u)", param_aber->degre);
      HIST_Texte(4, 0, "\n");
    }
    HIST_Texte(5, 0, "Intervalle d'estimation avant : [%g, %g]\n", param_aber->x_borne[0], param_aber->x_borne[1]);
    HIST_Texte(5, 0, "Intervalle d'interpolation : [%g, %g]\n", param_aber->x_borne[1], param_aber->x_borne[2]);
    HIST_Texte(5, 0, "Intervalle d'estimation apr"e_GRAVE"s : [%g, %g]\n", param_aber->x_borne[2], param_aber->x_borne[3]);

    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non traité [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Élimination d'une aberrance : interface -------- */

static int GER_aberrance(unsigned int num_objet, int x, int y)
{
  Fenetre *fenetre; /* Fenêtre courante */
  ABER_param *param_aber;
  int reponse;
  struct infos_demande nv_valeur = { BI_CHAMP_DEMANDE_REEL, NULL, NULL, 0, 0.0, 0.0, 0.0, 0, NULL, { 0 }, { 0 }, { 0 } };

  fenetre = ARB_QuelleFenetre(arbres + ELIM_ABERRANCE, FAUX);
  param_aber = (ABER_param *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == param_aber ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas d'information associée à la fenêtre !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return BI_EVENEMENT_TRAITE;
  }
  
  switch (num_objet)
  {
  case ABER_XMIN1: /* Bornes de l'intervale d'avant */
  case ABER_XMAX1:
  case ABER_XMIN2: /* Bornes de l'intervale d'après */
  case ABER_XMAX2:
    nv_valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    nv_valeur.texte = NULL;
    nv_valeur.min = param_aber->graphe->x_min;
    nv_valeur.max = param_aber->graphe->x_max;

    if ( num_objet == ABER_XMIN1 ) {
      nv_valeur.valeur_initiale.reel = param_aber->x_borne[ 0 ];
      reponse = BI_DemandeValeur( fenetre, &Q_DemandeBorne, &nv_valeur, 1 );
      if ( Q_DemandeBorne.bouton_annuler == (unsigned int) reponse ) return VRAI; /* On annule */
      param_aber->x_borne[ 0 ] = nv_valeur.valeur_donnee.reel;
    } else if ( num_objet == ABER_XMAX1 ) {
      nv_valeur.valeur_initiale.reel = param_aber->x_borne[ 1 ];
      reponse = BI_DemandeValeur( fenetre, &Q_DemandeBorne, &nv_valeur, 2 );
      if ( Q_DemandeBorne.bouton_annuler == (unsigned int) reponse ) return VRAI; /* On annule */
      param_aber->x_borne[ 1 ] = nv_valeur.valeur_donnee.reel;
    } else if ( num_objet == ABER_XMIN2 ) {
      nv_valeur.valeur_initiale.reel = param_aber->x_borne[ 2 ];
      reponse = BI_DemandeValeur( fenetre, &Q_DemandeBorne, &nv_valeur, 3 );
      if ( Q_DemandeBorne.bouton_annuler == (unsigned int) reponse ) return VRAI; /* On annule */
      param_aber->x_borne[ 2 ] = nv_valeur.valeur_donnee.reel;
    } else {
      nv_valeur.valeur_initiale.reel = param_aber->x_borne[ 3 ];
      reponse = BI_DemandeValeur( fenetre, &Q_DemandeBorne, &nv_valeur, 4 );
      if ( Q_DemandeBorne.bouton_annuler == (unsigned int) reponse ) return VRAI; /* On annule */
      param_aber->x_borne[ 3 ] = nv_valeur.valeur_donnee.reel;
    }

    /* On vérifie l'ordre des bornes, on l'améliore si nécessaire */
    for (x = 0 ; x < 4 ; x++)
      for (y = x + 1 ; y < 4 ; y++)
	if (param_aber->x_borne[y] < param_aber->x_borne[x]) { /* On intervertit les deux */
	  int vx = param_aber->x_borne[y];
	  param_aber->x_borne[y] = param_aber->x_borne[x];
	  param_aber->x_borne[x] = vx;
	}

    /* On met les nouvelles valeurs */
    OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN1, param_aber->x_borne[0], 0.0 );
    OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX1, param_aber->x_borne[1], 0.0 );
    OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMIN2, param_aber->x_borne[2], 0.0 );
    OBJ_MettreNombre( ELIM_ABERRANCE, ABER_XMAX2, param_aber->x_borne[3], 0.0 );

    /* On recommence tout avec ces valeurs */
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;

  case ABER_MOINS:  /* Ascenseur de choix du degré */
  case ABER_FOND:
  case ABER_VALEUR:
  case ABER_PLUS:
    param_aber->degre = gere_ascenseur(ELIM_ABERRANCE, ABER_MOINS, ABER_FOND, ABER_VALEUR, ABER_PLUS, ASC_HORIZONTAL,
				       1, 9, 1, param_aber->degre, num_objet, x, y);
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;

  case ABER_POLY1: /* Type de courbe d'interpolation */
  case ABER_POLY2:
    OBJ_choisit(ELIM_ABERRANCE, num_objet, FORCE_OUI);
    OBJ_active (ELIM_ABERRANCE, ABER_MOINS, FORCE_OUI);
    OBJ_active (ELIM_ABERRANCE, ABER_FOND, FORCE_OUI);
    OBJ_active (ELIM_ABERRANCE, ABER_VALEUR, FORCE_OUI);
    OBJ_active (ELIM_ABERRANCE, ABER_PLUS, FORCE_OUI);
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;

  case ABER_SPLINE:
    OBJ_choisit(ELIM_ABERRANCE, num_objet, FORCE_OUI);
    OBJ_active (ELIM_ABERRANCE, ABER_MOINS, FORCE_NON);
    OBJ_active (ELIM_ABERRANCE, ABER_FOND, FORCE_NON);
    OBJ_active (ELIM_ABERRANCE, ABER_VALEUR, FORCE_NON);
    OBJ_active (ELIM_ABERRANCE, ABER_PLUS, FORCE_NON);
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;

  case EPA_VUE_TOTALE: /* On demande de voir tout le spectre : on annule le zoom */
    x_vue = param_aber->graphe->x_min;
    larg_vue = param_aber->graphe->x_max - x_vue;
    y_vue = param_aber->graphe->y_min;
    haut_vue = param_aber->graphe->y_max - y_vue;
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE; /* On a traité tous les cas intéressants */
}

/* -------- Préparation de l'arbre d'élimination des aberrances ~~~~~~~~ */

static int prepare_arbre_aberrance(unsigned int numero_arbre __attribute__((__unused__)))
{
  OBJ_agrandit(ELIM_ABERRANCE, ROOT, 0, 2);
  OBJ_deplace(ELIM_ABERRANCE, ABER_XMIN2, 0, 1, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_XMAX2, 0, 1, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, EPA_VUE_TOTALE, 0, 1, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_FOND, 1, 0, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_PLUS, 2, 0, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_SPLINE,  0, 1, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_POLY1, 1, 1, DEPLACE_PIXELS);
  OBJ_deplace(ELIM_ABERRANCE, ABER_POLY2, 2, 1, DEPLACE_PIXELS);

#ifdef AVEC_TRADUCTION_
#endif

  arbres[ELIM_ABERRANCE].fonction.arbre = GER_aberrance;
  arbres[ELIM_ABERRANCE].drapeaux.prepare = VRAI;
  return 0;
}
