/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998, Emmanuel CURIS
   Cration : 14 mai 1999

   Oprations diverses sur les graphes, pas spcifiques de l'exafs

   Dcomposition d'un graphe sur une base linaire
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"
#include "serie.h"

#include "interface.h"
#include "lissage.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ----------------- Dcomposition sur une base de deux spectres : calcul effectif ------\
 |    Renvoie 0 si tout va bien, -NUM_ERREUR en cas d'erreur                             |
 |    Suppose que les graphes sur lesquels on dcompose soient tris (SPLINES...)        |
 |                                                                                       |
 | graphe : le graphe que l'on veut dcomposer                                           |
 | infos  : les infos pour la dcomposition                                              |
 \-------------------------------------------------------------------------------------- */
#ifdef ancien

int GRF_DecomposerSurBase(Graphe *spectre, DCMP_infos *infos)
{
  int retour;
  unsigned long i, nbr_pts = 0;
  register double *x, *y;

  /* Contrles prliminaires */
  if ( ( NULL == spectre ) || ( NULL == infos ) ) {
    return -1;
  }
  if ( ( NULL == infos->base ) || ( 0 == infos->nombre_bases ) ) {
    /* Pas de base */
    return -2;
  }
  if ( infos->x_min == infos->x_max ) {
    AVERTISSEMENT( "Bornes identiques ! [min = %g, max = %g", infos->x_min, infos->x_max );
    return -BORNES_FAUSSES;
  }

  /* Initialisation */
  for ( i = 0 ; i < infos->nombre_bases ; i++ ) {
    infos->base[ i ].coefficient.valeur_initiale = 1.0 / ( (double) infos->nombre_bases );
    infos->base[ i ].erreur = 0.0;

    if ( NULL == infos->base[ i ].graphe ) {
      AVERTISSEMENT( "Appel avec le graphe %lu de la base NULL !", i );
      return -10;
    }
    if ( ( infos->x_min >= infos->base[ i ].graphe->x_max ) ||
	 ( infos->x_max <= infos->base[ i ].graphe->x_min ) ) {
      /* Un spectre de la base ne recouvre pas : il ne contribue donc pas  la dcomposition, par principe ! */
      infos->base[ i ].coefficient.type = VAR_CONSTANTE;
      infos->base[ i ].coefficient.valeur_initiale = 0.0;
      infos->base[ i ].coefficient.contenu.valeur = 0.0;
    }
  }

  switch ( infos->methode ) /* Calcul effectif des coefficients */
  {
  case DCMP_METH_MOYENNE:
    retour = DCMP_moyenne(spectre, infos);
    break;

  case DCMP_METH_VARIANCE:
    retour = DCMP_variance(spectre, infos);
    break;

  case DCMP_METH_MDR_CARR:
    retour = DCMP_moindres(spectre, infos);
    break;

  case DCMP_METH_SUIVI:
    retour = DCMP_suivi(spectre, infos);
    break;

  case DCMP_METH_LINEAIRE:
    retour = DCMP_lineaire(spectre, infos);
    break;

  case DCMP_METH_MDR_CARR_SOMME1:
    retour = DCMP_moindres_somme1(spectre, infos);
    break;

  default:
    AVERTISSEMENT( "Mthode inconnue : %d", infos->methode );
    retour = -50;
  }

  OBJ_MettreNombre( DECOMP_GRAPHE, DCMP_COEF_A, infos->coef_1, 0.0 ); /* On affiche les valeurs trouves */
  OBJ_MettreNombre( DECOMP_GRAPHE, DCMP_COEF_B, infos->coef_2, 0.0 );

  /* --- Maintenant, on rempli les graphes des rsultats --- */

  if ( (infos->methode == DCMP_METH_MOYENNE) || (infos->methode == DCMP_METH_VARIANCE) ||
       (infos->methode == DCMP_METH_MDR_CARR) ||  (infos->methode == DCMP_METH_MDR_CARR_SOMME1) ) /* Mthodes "simples" */
  {
    double erreur = 0.0, somme_y2 = 0.0;
    register double *x_clin = infos->c_lineaire->x, *y_clin = infos->c_lineaire->y,
      *x_residu = infos->residu->x, *y_residu = infos->residu->y;

    for (i = 0, x = spectre->x, y = spectre->y ;
	 i < spectre->nombre_point;
	 i++, x++, y++) {
      double x_lu = *x;

      if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) ) /* Le point compte */
      {
	double y_1 = SPL_tt_calc(infos->ref_1, x_lu, infos->a_1, infos->b_1, infos->c_1);
	double y_2 = SPL_tt_calc(infos->ref_2, x_lu, infos->a_2, infos->b_2, infos->c_2);

	*x_clin = x_lu; x_clin++;
	*y_clin = infos->coef_1 * y_1 + infos->coef_2 * y_2;
	
	*x_residu = x_lu; x_residu++;
	*y_residu = *y - *y_clin;

	somme_y2 += *y * *y;
	erreur   += *y_residu * *y_residu;

	y_residu++; y_clin++;
	nbr_pts++; /* Un point en plus */
      }
    }
    infos->c_lineaire->nombre_point = nbr_pts; /* Nombre de points effectif */
    infos->residu->nombre_point = nbr_pts;
    GRF_bornes(infos->c_lineaire, NE_PAS_FAIRE, DESORDONNE);
    GRF_bornes(infos->residu, NE_PAS_FAIRE, DESORDONNE);

    erreur /= somme_y2;
    met_svaleur(DECOMP_GRAPHE, DCMP_ERREUR, erreur);
  } else if ( infos->methode == DCMP_METH_LINEAIRE ) {
    double erreur = 0.0, somme_y2 = 0.0;
    register double *x_clin = infos->c_lineaire->x, *y_clin = infos->c_lineaire->y,
      *x_residu = infos->residu->x, *y_residu = infos->residu->y;

    for (i = 0, x = spectre->x, y = spectre->y ;
	 i < spectre->nombre_point;
	 i++, x++, y++) {
      double x_lu = *x;

      if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) ) /* Le point compte */
      {
	double alpha;
	double y_1 = SPL_tt_calc(infos->ref_1, x_lu, infos->a_1, infos->b_1, infos->c_1);
	double y_2 = SPL_tt_calc(infos->ref_2, x_lu, infos->a_2, infos->b_2, infos->c_2);

	alpha = infos->coef_1 * x_lu + infos->coef_2; /* volution linaire */

	*x_clin = x_lu; x_clin++;
	*y_clin = alpha * y_1 + (1 - alpha) * y_2;
	
	*x_residu = x_lu; x_residu++;
	*y_residu = *y - *y_clin;

	somme_y2 += *y * *y;
	erreur   += *y_residu * *y_residu;

	y_residu++; y_clin++;
	nbr_pts++; /* Un point en plus */
      }
    }
    infos->c_lineaire->nombre_point = nbr_pts; /* Nombre de points effectif */
    infos->residu->nombre_point = nbr_pts;
    GRF_bornes(infos->c_lineaire, NE_PAS_FAIRE, DESORDONNE);
    GRF_bornes(infos->residu, NE_PAS_FAIRE, DESORDONNE);

    erreur /= somme_y2;
    met_svaleur(DECOMP_GRAPHE, DCMP_ERREUR, erreur);
  }
       
  return 0; /* Pas d'erreur */
}

/* ~~~~~~~~ Ralisation de l'interface ~~~~~~~~ */

/* ----------------- Ouverture de la fentre d'interface ------------------------------- */

void GRF_decompose(Graphe *graphe)
{
  DCMP_infos *infos = NULL;
  Fenetre *fenetre;

  if (graphe == NULL) return;

  if ( arbres[DECOMP_GRAPHE].drapeaux.ouvert == VRAI ) /* Arbre dj ouvert */
  {
    Graphe *vieux;   /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_QuelleFenetre( &(arbres[DECOMP_GRAPHE]), FAUX );
    vieux = (Graphe *)     FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
    infos = (DCMP_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

    if ( ( reponse == 1 ) || ( vieux == graphe ) ) { /* On active seulement */
      FEN_premierplan(fenetre);
      return; /* Rien d'autre  faire */
    } else if ( reponse == 2 ) { /* On rappelle, pour le nouveau graphe */
      if ( graphe->drapeaux.protege ) {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      /* On dissocie les infos de la fentre, pour viter des soucis... */
      FEN_GRF_LierGraphe( fenetre, NULL );
      FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

      /* On efface les graphes inutiles */
      GRF_efface( infos->residu, VRAI, VRAI );
      GRF_efface( infos->c_lineaire, VRAI, VRAI);

      /* On a fini de travailler avec */
      vieux->drapeaux.protege = FAUX;

      FEN_premierplan( fenetre );
      FEN_redessin( fenetre, BI_RDS_UTIL );
    } else { /* On annule l'appel */
      return;
    }
  } else { /* Arbre non ouvert : on l'ouvre */
    if ( graphe->drapeaux.protege ) {
      EXF_ERREUR(GRF_PROTEGE);
      return;
    }

    fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, , NULL,
				  DECOMP_GRAPHE, BI_FEN_ARBRE_HAUT, prepare_arbre_decompose,
				  RDS_decompose, CLV_decompose,
				  CLC_decompose, MSG_decompose, RCT_pointeur,
				  ( int (*)(Fenetre *) ) NULL);
    if (fenetre == NULL) return; /* Rien  faire... */

    fenetre->drapeaux.redessin = VRAI;
    fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */

    OBJ_MettreNombre( DECOMP_GRAPHE, DCMP_COEF_A, 0.0, 0.0 ); /* On affiche les valeurs trouves */
    OBJ_MettreNombre( DECOMP_GRAPHE, DCMP_COEF_B, 0.0, 0.0 );
    met_svaleur(DECOMP_GRAPHE, DCMP_ERREUR, 0.0);
    met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_A, LG_MAX_NOM_SPECTRE, "Pas de spectre choisi", TXT_COPIER);
    met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_B, LG_MAX_NOM_SPECTRE, "Pas de spectre choisi", TXT_COPIER);
  }

  if (infos->residu == NULL) {
    unsigned long i, nbr_pts = 0;
    register double *x_gr, *x_res;

    infos->residu = GRF_nouveau(graphe, graphe->nombre_point, GR_ECART, GR_NORMAL,
				"R"e_AIGU"sidu de la d"e_AIGU"composition");
    if (infos->residu == NULL) {
      GRF_efface(infos->c_lineaire, VRAI, VRAI); /* Au cas o */
      free(infos->a_1); free(infos->b_1); free(infos->c_1);
      free(infos->a_2); free(infos->b_2); free(infos->c_2);
      free( infos ); infos = NULL;
      FEN_Detruire( fenetre );
      return;
    }

    infos->residu->x_min = graphe->x_min;
    infos->residu->x_max = graphe->x_max;
    infos->residu->drapeaux.protege = VRAI;
    x_gr = graphe->x; x_res = infos->residu->x;
    for (i = 0 ; i < graphe->nombre_point ; i++, x_res++, x_gr++) {
      double x = *x_gr;
      if ( (x >= infos->x_min) && (x<= infos->x_max) ) {
	*x_res = x;
	nbr_pts++;
      }
    }	
    infos->residu->nombre_point = nbr_pts; /* Nombre de points effectif */
  }

  if (infos->c_lineaire == NULL) {
    unsigned long i, nbr_pts = 0;
    register double *x_gr, *x_clin;

    infos->c_lineaire = GRF_nouveau(graphe, graphe->nombre_point, graphe->type, GR_NORMAL,
				    "Graphe obtenu par la combinaison lin"e_AIGU"aire trouv"e_AIGU"e");

    infos->c_lineaire->x_min = infos->x_min;
    infos->c_lineaire->x_max = infos->x_max;
    infos->c_lineaire->drapeaux.protege = VRAI;
    x_gr = graphe->x; x_clin = infos->c_lineaire->x;
    for (i = 0 ; i < graphe->nombre_point ; i++, x_clin++, x_gr++) {
      double x = *x_gr;
      if ( (x >= infos->x_min) && (x<= infos->x_max) ) {
	*x_clin = x;
	nbr_pts++;
      }
    }	
    infos->c_lineaire->nombre_point = nbr_pts; /* Nombre de points effectif */
  }

  /* On associe les infos  la fentre */
  FEN_GRF_LierGraphe( fenetre, graphe );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos );

  graphe->drapeaux.protege = VRAI;

  /* On fait le calcul */
/*   DCMP_calc(graphe, infos); */
}
#endif
