/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2001, Emmanuel CURIS
   Cration du fichier : 11 fvrier 2001

   Oprations diverses sur les graphes, plus gnrales que l'EXAFS lui-mme
   (aussi bien mathmatiques que statistiques simples : tri en particulier)
   (les oprations statistiques avances sont dans ../Stats)

   Estimation d'une drive  partir d'une famille de points
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "operations.h"

#include "lissage.h"

#include "deriver.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int DeriveeSimple_Reel    (const Graphe *graphe, Graphe *derivee);
static int DeriveeSimple_Complexe(const Graphe *graphe, Graphe *derivee);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Cas d'un graphe rel, mthode simple -------- */

static int DeriveeSimple_Reel(const Graphe *graphe, Graphe *derivee)
{
  unsigned long i;
  double *x, *x_der, *y, *y_der;

  for (i = 0, x = graphe->x, x_der = derivee->x, y = graphe->y, y_der = derivee->y ;
       i < derivee->nombre_point ; i++, x++, y++, x_der++, y_der++) {
    double dx = *(x + 1) - *x;
    if (dx == 0.0) {
      (void) EXF_ERREUR(ERR_DOUBLONS); /* Calcul impossible -> on quitte */
      return -1;
    }

    *x_der = ( *(x + 1) + *x ) / 2.0;
    *y_der = ( *(y + 1) - *y ) / dx;
  }

  GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* Les nouvelles bornes */
  if ( (graphe->erreur_y == NULL) && ( graphe->erreur_x == NULL ) ) /* Pas d'erreur : tout est fini */
    return 0;

  if ( graphe->erreur_x != NULL ) { /* Erreur sur les X */
    Graphe *erreurs;
    double *x_err, *xd_err;

    if ( graphe->erreur_y != NULL ) {
      fprintf(stderr, "DeriveeSimple_Reel : erreurs en X et en Y, propagation des erreurs  FAIRE\n");
      return 1;
    }

    /* Propagation des erreurs sur les X */
    erreurs = GRF_creer_erreurs(derivee, AXE_X);
    if (NULL == erreurs) {
      (void) fprintf(stderr, "DeriveeSimple_Reel : impossible de propager les erreurs en X.\n");
      return 2;
    }
    for (i = 0, x_err = graphe->erreur_x->y, xd_err = erreurs->y ; i < derivee->nombre_point ; i++, x_err++, xd_err++) {
      double sigma;

      sigma = (*x_err) * (*x_err) + *(x_err + 1) * *(x_err + 1);
      sigma = sqrt(sigma) / 2.0;
      *xd_err = sigma;
    }

    /* Influence sur les Y */
    fprintf(stderr, "DeriveeSimple_Reel : erreurs en X, propagation des erreurs en Y  FAIRE\n");

    GRF_bornes(derivee->erreur_x, NE_PAS_FAIRE, DESORDONNE); /* Les nouvelles bornes */
    return 0;
  }

  /* Si on est l, c'est qu'il n'y a des erreurs qu'en Y */
  if ( graphe->erreur_y != NULL ) {
    Graphe *erreurs;
    double *y_err, *yd_err;

    if ( graphe->covariance != NULL ) {
      fprintf(stderr, "DeriveeSimple_Reel : erreurs en Y avec matrice de covariance, propagation des erreurs  FAIRE\n");
      return 1;
    }

    erreurs = GRF_creer_erreurs(derivee, AXE_Y);
    if (NULL == erreurs) {
      (void) fprintf(stderr, "DeriveeSimple_Reel : impossible de propager les erreurs en Y.\n");
      return 2;
    }
    for (i = 0, x = graphe->x, y_err = graphe->erreur_y->y, yd_err = erreurs->y ; i < derivee->nombre_point ; i++, x++, y_err++, yd_err++) {
      double sigma, dx;

      dx = *(x + 1) - *x; /* Forcment non nul, sinon on aurait quitt dans le calcul de la drive */
      sigma = (*y_err) * (*y_err) + *(y_err + 1) * *(y_err + 1);
      sigma = sqrt(sigma) / dx;
      *yd_err = sigma;
    }

    GRF_bornes(derivee->erreur_y, NE_PAS_FAIRE, DESORDONNE); /* Les nouvelles bornes */
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Cas d'un graphe complexe, mthode simple -------- */

static int DeriveeSimple_Complexe(const Graphe *graphe, Graphe *derivee)
{
  unsigned long i;
  double *x, *x_der;
  Complexe *y = (Complexe *) graphe->y, *y_der = (Complexe *) derivee->y;

  for (i = 0, x = graphe->x, x_der = derivee->x ;
       i < derivee->nombre_point ; i++, x++, y++, x_der++, y_der++) {
    double dx = *(x + 1) - *x;

    if (dx == 0.0) {
      (void) EXF_ERREUR(ERR_DOUBLONS); /* Calcul impossible -> on quitte */
      return -1;
    }

    *x_der    = ( *(x + 1)     + *x     ) / 2.0;
    y_der->re = (  (y + 1)->re -  y->re ) / dx;
    y_der->im = (  (y + 1)->im -  y->im ) / dx;
  }

  GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* Les nouvelles bornes */
  if ( (graphe->erreur_y == NULL) && ( graphe->erreur_x == NULL ) ) /* Pas d'erreur : tout est fini */
    return 0;

  if ( graphe->erreur_x != NULL ) { /* Erreur sur les X */
    Graphe *erreurs;
    double *x_err, *xd_err;

    if ( graphe->erreur_y != NULL ) {
      fprintf(stderr, "DeriveeSimple_Reel : erreurs en X et en Y, propagation des erreurs  FAIRE\n");
      return 1;
    }

    /* Propagation des erreurs sur les X */
    erreurs = GRF_creer_erreurs(derivee, AXE_X);
    if (NULL == erreurs) {
      (void) fprintf(stderr, "DeriveeSimple_Complexe : impossible de propager les erreurs en X.\n");
      return 2;
    }
    for (i = 0, x_err = graphe->erreur_x->y, xd_err = erreurs->y ; i < derivee->nombre_point ; i++, x_err++, xd_err++) {
      double sigma;

      sigma = (*x_err) * (*x_err) + *(x_err + 1) * *(x_err + 1);
      sigma = sqrt(sigma) / 2.0;
      *xd_err = sigma;
    }

    /* Influence sur les Y */
    fprintf(stderr, "DeriveeSimple_Complexe : erreurs en X, propagation des erreurs en Y  FAIRE\n");

    GRF_bornes(derivee->erreur_x, NE_PAS_FAIRE, DESORDONNE); /* Les nouvelles bornes */
    return 0;
  }

  /* Si on est l, c'est qu'il n'y a des erreurs qu'en Y */
  if ( graphe->erreur_y != NULL ) {
    fprintf(stderr, "DeriveeSimple_Complexe : erreurs en Y, propagation des erreurs  FAIRE\n");
  }

  /* Tout s'est bien pass */
  return 0;		
}

/* ----------------------- Procdure gnrale : choisit la mthode et tout et tout --------------\
 |   Renvoie le graphe contenant la drive (NULL : problme)
 |
 | graphe  : le graphe dont on veut la drive
 |           Suppose que ce graphe soit tri par X croissants
 | methode : mthode de calcul de la drive (DER_RAPIDE ou DER_SPLINE)
 \---------------------------------------------------------------------------------------------- */

Graphe *GRF_Deriver(const Graphe *graphe, int methode)
{
  Graphe *derive = NULL;

  if (graphe == NULL) return NULL;

  if (methode == DER_RAPIDE) /* Drive rapide */
    return GRF_DeriveeSimple( graphe );

  if (methode == DER_SPLINE) { /* On veut une drive par lissage */
    double *b, *c, *d; /* Tableaux pour le lissage */
    int retour;

    retour = SPL_tt_prep(graphe, &b, &c, &d);
    if (retour != 0) {
      (void) LIS_ERREUR(retour, graphe->nom);
      return NULL; /* Problme survenu */
    }

    derive = GRF_DeriveeSpline( graphe, graphe->nombre_point, b, c, d );
    free(b); free(c); free(d);
  }

  /* Ici, prvoir d'autres mthodes : noyau par exemple */

  return derive;
}

/* -------- Traitement d'un graphe aprs calcul de la drive, pour optimiser un peu les affichages -------- */

int GRF_Derivee_PostCalcul(const Graphe *graphe, Graphe *derivee, int methode __attribute__((__unused__)) )
{
  char *titre = NULL;
  char *unite_Y, *legende_Y = NULL;

  if ( NULL == derivee ) return -1;
  if ( NULL == graphe ) return 0;
  if ( VRAI == derivee->drapeaux.isole ) {
    (void) fprintf(stderr, "%s,%s l.%u : Appel pour un graphe isol : sans intrt !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -2;
  }

  /* On optimise le type du graphe */
  if ( GR_DERIVEE == graphe->type ) {
    /* On calcule donc en ralit une drive seconde */
    GRF_ChangerType( derivee, GR_DERIVEE2 );
    OPTION_A_FAIRE( "Drive seconde --- adapter le titre en consquence" );
#ifdef _DEVELOPPE_
    printf( "Comparaison : %d\n", strncmp( graphe->nom, "D"e_AIGU"riv"e_AIGU"e", 7 ) );
/*     if ( strncmp(graphe->nom, txt_titre_derivee, (unsigned) (strchr(txt_titre_derivee, '%') - txt_titre_derivee)) ) */
/*       sprintf(titre, txt_titre_derivee2, graphe->nom + 10); */
#endif
  }

  /* On amliore le titre du graphe [si pas de titre spcial dfini ci-dessus] */
  if ( NULL == titre ) {
    titre = TXT_genere_chaine( txt_titre_derivee, graphe->nom );
  }
  GRF_titre( derivee, titre );

  /* Plus besoin du titre */
  free( titre ); titre = NULL;

  /* Affichage : mmes paramtres de dessin ; mme axe X [unit, lgende] */
  GRF_CopierAffichage( derivee, graphe, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_DESSIN );

  /* Axe Y : on adapte la lgende et l'unit */
  /* a. L'unit : unit drive = [unit Y] / [unit X] */
  unite_Y = TXT_ProduitUnites( GRF_QuelleUnite( graphe, AXE_Y ), GRF_QuelleUnite( graphe, AXE_X ), -1 );
  GRF_ChangerUnite( derivee, AXE_Y, unite_Y );
  free( unite_Y ); unite_Y = NULL;

  /* b. La lgende [si pas dfinie par ailleurs !] */
  if ( NULL == legende_Y ) {
    legende_Y = TXT_genere_chaine( "d{%s}/d{%s}", GRF_QuelleLegende( graphe, AXE_Y ), GRF_QuelleLegende( graphe, AXE_X ) );
  }
  GRF_ChangerLegende( derivee, AXE_Y, legende_Y, FAUX );
  /* La lgende sera libre avec le graphe [ou si elle est change par la suite] */

  /* Tout s'est bien pass ! */
  return 0;
}

/* -------- Calcul de la drive par la mthode la plus simple --------------------\
 |   Renvoie le graphe contenant la drive                                        |
 |   Ce graphe est isol.                                                          |
 |                                                                                 |
 | graphe : le graphe dont on veut la drive                                      |
 |          Suppose qu'il est tri selon les X - Algorithme : y' = Dy/Dx           |
 \-------------------------------------------------------------------------------- */

Graphe *GRF_DeriveeSimple(const Graphe *graphe)
{
  Graphe *derivee;
  int retour;
  
  if (graphe == NULL) return NULL; /* Rien de spcial  faire... */
  if (graphe->nombre_point < 3) {
    EXF_ERREUR(TROP_PEU_POINTS); /* Au moins trois point pour que a ait un vague intrt... */
    return NULL;
  }
  
  /* Cration du graphe qui contiendra la drive */
  derivee = GRF_nv_isole( graphe->nombre_point - 1, GR_DERIVEE, graphe->genre_donnees );
  if ( NULL == derivee ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le graphe contenant la drive !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
#ifdef _DEVELOPPE_
  /* Titre, pour identifier le graphe en cas de problme ! */
  GRF_titre( derivee, "Graphe drive simple, isol" );
#endif

  /* Calcul effectif de la drive */
  if ( GRF_FCT_REELLE == graphe->genre_donnees ) { /* Graphe rel */
    retour = DeriveeSimple_Reel(graphe, derivee);
  } else if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) { /* Le graphe est un graphe complexe -> drive de Re() et Im() */
    retour = DeriveeSimple_Complexe(graphe, derivee);
  } else {
    (void) fprintf(stderr, "DER_simple : type de graphe non-trait [%d]\n", graphe->genre_donnees);
    retour = -1;
  }
  if (retour < 0) { /* Erreur grave survenue, calcul annul */
    GRF_efface(derivee, VRAI, VRAI);
    return NULL;
  }

  /* Log. dans l'historique */
  HIST_Titre(1, "D"e_AIGU"river un graphe", "MATH");
  HIST_Texte(1, 0, "Graphe original : %s\n", graphe->nom);
  HIST_Texte(1, 0, "Graphe d"e_AIGU"riv"e_AIGU" : %s\n", derivee->nom);
  HIST_Texte(4, 0, "M"e_AIGU"thode de calcul : estimation simple\n");

  return derivee;
}

/* -------- Calcul de la drive avec lissage par splines -----------------------------\
 |   Renvoie le graphe contenant la drive                                            |
 |   Suppose que le graphe est tri selon les X et qu'il est REEL                      |
 |   Mthode : on a la SPLINE -> on drive l'expression, donne ce qu'il faut...        |
 |                                                                                     |
 | graphe  : le graphe dont on veut la drive                                         |
 | nbr_pts : le nombre de points dans la drive [bornes : celles du graphe de dpart] |
 | b, c, d : tableaux des coefficients du lissage par spline, dj calculs...         |
 \------------------------------------------------------------------------------------ */

Graphe *GRF_DeriveeSpline(const Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d)
{
  Graphe *derivee;
  register double *x_der, *y_der;
  double x, pas;
  unsigned long i;

  if (graphe == NULL) return NULL;
  if ( (b == NULL) || (c == NULL) || (d == NULL) ) /* Coefficients non dfinis */
    return NULL;

  if (graphe->genre_donnees != GRF_FCT_REELLE) {
    EXF_ERREUR(GRAPHE_COMPLEXE);
    return NULL;
  }

  /* Cration du graphe qui contiendra la drive */
  derivee = GRF_nv_isole( nbr_pts, GR_DERIVEE, graphe->genre_donnees );
  if ( NULL == derivee ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le graphe contenant la drive !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
#ifdef _DEVELOPPE_
  /* Titre, pour identifier le graphe en cas de problme ! */
  GRF_titre( derivee, "Graphe drive par spline, isol" );
#endif

  /* Calcul effectif de la drive */
  x   = graphe->x_min;
  pas = (graphe->x_max - graphe->x_min) / (nbr_pts - 1);

  x_der = derivee->x; y_der = derivee->y;
  for (i = 0 ; i < nbr_pts ; i++, x_der++, y_der++) {
    unsigned long j;
    double delta_x;

    *x_der = x;

    /* On recherche le point en x le plus proche */
    if ( x <= graphe->x_min) {
      delta_x = x - graphe->x_min;
      j = 0;
    } else if (x >= graphe->x_max) {
      delta_x = x - graphe->x_max;
      j = graphe->nombre_point - 1;
    } else { /* Recherche du point le plus proche - PAS ENCORE par dichotomie :  FAIRE ! */
      register double *zone_x = graphe->x;
      
      j = 0;
      while (x > *zone_x) {
	zone_x++; j++;
      }
      /* Ici, zone_x pointe sur l'lment juste au-dessus de X et on veut celui d'avant... */
      
      delta_x = x - *(zone_x - 1);
      j--;
    }

    if (fabs(delta_x) > 0) {
      *y_der  = delta_x * d[j] * 3;
      *y_der += 2 * c[j];
      *y_der *= delta_x;
    } else {
      *y_der = 0;
    }
    *y_der += b[j]; /* drive = b + 2*c*DX + 3*d*DX^2 */

    x += pas; /* On passe au point suivant */
  }

  /* Log. dans l'historique */
  HIST_Titre(1, "D"e_AIGU"river un graphe", "MATH");
  HIST_Texte(1, 0, "Graphe original : %s\n", graphe->nom);
  HIST_Texte(1, 0, "Graphe d"e_AIGU"riv"e_AIGU" : %s\n", derivee->nom);
  HIST_Texte(4, 0, "M"e_AIGU"thode de calcul : d"e_AIGU"rivation de la spline cubique d'interpolation\n");

  /* -- Ce qui suit : mthode de Juris = utilise Dy/Dx avec des points pris sur la SPLINE (!) --
     Cela me semble moins prcis, mais bon... */
#ifdef methode_Juris
  zone_x = derivee->x;
  zone_y = derivee->y;
  pas    = (source->x_max - source->x_min) / (derivee->nombre_point - 1);

  x_derive = source->x_min;
  *zone_x = x_derive;
  for (j = 1 ; j < derivee->nombre_point-1 ; j++) {
    zone_x++; zone_y++;
    x_derive += pas;
    *zone_x = x_derive;

    /* On cherche le x prcdent dans le graphe source */
    index2 = 0;
    while ( *(source->x + index2) < x_derive )
      index2++;

    index2--;
    dx  = x_derive - (source->x)[index2];
    dx1 = dx - pas; dx2 = dx + pas;
    
    yd1 = dx2 * (b[index2] + dx2 * (c[index2] + d[index2] * dx2));
    yd2 = dx1 * (b[index2] + dx1 * (c[index2] + d[index2] * dx1));
    y_derive = (yd1 - yd2) / (2 * pas);
    *zone_y = y_derive;			
  }
  (derivee->x)[derivee->nombre_point - 1] = source->x_max;
  (derivee->y)[derivee->nombre_point - 1] = (derivee->y)[derivee->nombre_point - 2];
  *(derivee->x) = source->x_min;
  *(derivee->y) = *(derivee->y + 1);
#endif

  GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* On cherche les bornes */
  return derivee;
}

/* ------------------------- Calcul de la drive seconde avec lissage par splines -----------------\
 |   Renvoie le graphe contenant la drive seconde                                                 |
 |   Suppose que le graphe est tri selon les X et qu'il est REEL                                   |
 |   Mthode : on a la SPLINE -> on drive l'expression, donne ce qu'il faut...                     |
 |                                                                                                  |
 | graphe  : le graphe dont on veut la drive                                                      |
 | nbr_pts : le nombre de points dans la drive [bornes : celles du graphe de dpart]              |
 | b, c, d : tableaux des coefficients du lissage par spline, dj calculs...                      |
 \------------------------------------------------------------------------------------------------- */

Graphe *DER_seconde(const Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d)
{
  Graphe *derivee;
  register double *x_der, *y_der;
  double x, pas;
  unsigned long i;

  if (graphe == NULL) return NULL;
  if ( (b == NULL) || (c == NULL) || (d == NULL) ) /* Coefficients non dfinis */
    return NULL;

  if (graphe->genre_donnees != GRF_FCT_REELLE) {
    EXF_ERREUR(GRAPHE_COMPLEXE);
    return NULL;
  }

  /* Cration du graphe qui contiendra la drive seconde */
  {
    char *titre;
    titre = TXT_genere_chaine( txt_titre_derivee2, graphe->nom );

    OPTION_A_FAIRE( "Scinder l'interface et le calcul" );
    
    derivee = GRF_nouveau( NULL/* graphe */, nbr_pts, GR_DERIVEE2, graphe->genre_donnees, titre);
    free( titre ); titre = NULL;
    if (derivee == NULL)
      return NULL;
  }

  /* Calcul effectif de la drive */
  x   = graphe->x_min;
  pas = (graphe->x_max - graphe->x_min) / (nbr_pts - 1);

  x_der = derivee->x; y_der = derivee->y;
  for (i = 0 ; i < nbr_pts ; i++, x_der++, y_der++) {
    unsigned long j;
    double delta_x;

    *x_der = x;

    /* On recherche le point en x le plus proche */
    if ( x <= graphe->x_min) {
      delta_x = x - graphe->x_min;
      j = 0;
    } else if (x >= graphe->x_max) {
      delta_x = x - graphe->x_max;
      j = graphe->nombre_point - 1;
    } else { /* Recherche du point le plus proche */
      register double *zone_x = graphe->x;

      j = 0;
      while (x > *zone_x) {
	zone_x++; j++;
      }
      /* Ici, zone_x pointe sur l'lment juste au-dessus de X et on veut celui d'avant... */

      delta_x = x - *(zone_x - 1);
      j--;
    }

    if (fabs(delta_x) > 0) {
      *y_der  = delta_x * d[j] * 3;
    } else {
      *y_der = 0;
    }
    *y_der += c[j]; /* drive seconde = 2*c + 2*3*d*DX */
    *y_der *= 2;

    x += pas; /* On passe au point suivant */
  }

  GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* On cherche les bornes */

  /* Log. dans l'historique */
  HIST_Titre(1, "D"e_AIGU"riv"e_AIGU"e seconde d'un graphe", "MATH");
  HIST_Texte(1, 0, "Graphe original : %s\n", graphe->nom);
  HIST_Texte(1, 0, "Graphe d"e_AIGU"riv"e_AIGU" : %s\n", derivee->nom);
  HIST_Texte(4, 0, "M"e_AIGU"thode de calcul : d"e_AIGU"rivation de la spline cubique d'interpolation\n");

  return derivee;
}
