/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996 - 2005, Emmanuel CURIS
   Cration du fichier : 5 mars 1999

   Oprations diverses sur les graphes, plus gnrales que l'EXAFS lui-mme
   (oprations faisant intervenir plus d'un graphe)
   (les oprations statistiques sont dans ../Stats)

   Interface pour le choix des oprations  raliser
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "serie.h"
#include "operations.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

struct
{
  unsigned short int *choisi; /* Un par graphe, VRAI si le graphe a t choisi */
  float *coef;                /* Coefficients de la C. Linraire, 1 par graphe */
  unsigned long prem_affiche;
  unsigned long num_choisi; /* ~0UL : aucun slectionn */
  unsigned long nbr_choisis;
} infos_op = {NULL, NULL, 0, ~0UL, 0};

typedef struct {
  Graphe *graphe; double coef;
} Infos_CL;

#include "op_interface.txt"

#define OP_AUCUNE      0
#define OP_MOYENNE     1
#define OP_CLINEAIRE   2
#define OP_COMPARAISON 3
#define OP_RECALAGE    4
#define OP_RECALE_MULT 5
#define OP_RECALE_LIN  6

#define LGMAX_TXT_NOM_REFERENCE 56

#define NOMBRE_OPERATIONS 7
#define LG_MAX_TXT_OPERATION 26

#define NBR_GRAPHES_VISIBLES (OPER_DERNIER_GRF - OPER_PREM_GRF + 1)
#define LGMAX_TXT_NOM_GRAPHE 42

static int op_demande = OP_AUCUNE;

static Graphe *reference = NULL; /* Quel graphe sert de rfrence ? */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int GER_operations    (unsigned int objet, int x, int y);
static void    prepare_operations(void);

static Graphe *realise_operations(Infos_CL *infos, unsigned long nbr_graphes);

static int aff_defileur(void);
static int lit_defileur(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Appel de la bote de dialogue et lancement des calculs suivant l'opration demande
     Renvoie le graphe rsultat obtenu (s'il y a lieu)
     Si ref_a_utiliser existe, il est pris comme graphe de rfrence par dfaut.
   -------- */

Graphe *op_diverses(Graphe *ref_a_utiliser)
{
  int reponse;
  Graphe *resultat = NULL;
  Infos_CL *infos = NULL;

  infos_op.choisi = (unsigned short int *) calloc(serie_courante->nbr_graphes, sizeof(unsigned short int));
  if (infos_op.choisi == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "Operations/interface.c", "op_diverses", "infos_op.choisi");
    return NULL;
  }
  infos_op.coef   = (float *) calloc(serie_courante->nbr_graphes, sizeof(float));
  if (infos_op.coef == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "Operations/interface.c", "op_diverses", "infos_op.coef");
    /* Seul effet : interdit les CL avances */
  }
  infos_op.num_choisi = ~0UL;
  infos_op.nbr_choisis = 0; /* Aucun choisi au dbut */

  reference = ref_a_utiliser; /* Un graphe de rfrence existe dj */

  if (arbres[GRF_OPERATIONS].drapeaux.prepare == FAUX)
    prepare_operations();

  if (reference) { /* Il y a un spectre de rfrence connu */
    met_chaine(GRF_OPERATIONS, OPER_REFERENCE, LGMAX_TXT_NOM_REFERENCE, reference->nom, TXT_COPIER);
  } else {
    met_chaine(GRF_OPERATIONS, OPER_REFERENCE, LGMAX_TXT_NOM_REFERENCE, txt_aucun_graphe, TXT_COPIER);
  }
  aff_defileur();

  /* On dmarre avec aucune opration car aucun spectre en plus de la rfrence choisi */
  OBJ_active(GRF_OPERATIONS, OD_TYPE_OPERE, FORCE_NON);

  do {
    reponse = ARB_Bloquant( GRF_OPERATIONS, titre_interface, principale, 0 );
    if (reponse == OD_ANNULER) {
      free(infos_op.choisi);
      free(infos_op.coef);
      reference = NULL;
      return NULL; /* Aucun calcul */
    }
  }
  while ( reponse != OPERE_LANCER);

  if (op_demande != OP_AUCUNE) { /* Vaut la peine de rserver */
    /* On fait la table des graphes utiliss, en plus de la rfrence
       Indispensable car les copies et crations de graphes induites par la suite
       risquent de modifier de faon indue la correspondance du tableau des graphes choisis */
    unsigned long i = 0, j = 0;

    infos = (Infos_CL *) malloc( sizeof(Infos_CL) * infos_op.nbr_choisis );
    if (infos == NULL) {
      SYS_ERREUR(MANQUE_MEMOIRE, "Operations/interface.c", "op_diverses", "infos (CL)");
      free(infos_op.choisi);
      free(infos_op.coef);
      reference = NULL;
      return NULL;
    }

    lit_defileur();
    for (resultat = serie_courante->premier_graphe ; resultat != NULL ; i++, resultat = resultat->suivant) {
      if (infos_op.choisi[i] == VRAI) {
	infos[j].graphe = resultat;
	infos[j].coef = infos_op.coef[i];
#ifdef _DEVELOPPE_
	printf("On rserve le graphe %s avec le coefficient %g\n", resultat->nom, infos[j].coef);
#endif
	j++;
	if (j > infos_op.nbr_choisis) {
	  fprintf(stderr, "Oprations diverses : incohrence sur les graphes choisis\n");
	  break;
	}
      }
    }
  } else
    infos = NULL;

  free(infos_op.choisi); /* Plus besoin d'eux */
  free(infos_op.coef);

  /* On commence le travail srieux */
  reference = GRF_Copier( reference, NULL, FAUX );

  if (reference == NULL) { /* Pas de rfrence : copie a chou par exemple */
    form_alert(1, txt_alertes[ERR_PAS_DE_REFERENCE]);
    free(infos);
    return NULL;
  }


  resultat = realise_operations(infos, infos_op.nbr_choisis);

  reference = NULL; /* Plus de rfrence */
  free(infos); /* On libre ce dont on n'a plus besoin */
  return resultat;
}

/* -------- On ralise de faon pratique les oprations demandes --- */

static Graphe *realise_operations(Infos_CL *infos, unsigned long nbr_graphes)
{
  Graphe *resultat = NULL;
  unsigned long i;

  switch (op_demande)
  {
  case OP_AUCUNE:
    return reference;

  case OP_CLINEAIRE: /* On ralise une combinaison linaire des spectres */
    resultat = GRF_Copier( reference, NULL, FAUX );
    if (resultat == NULL) return NULL;
    GRF_titre(resultat, txt_titre_grf_clin);

    HIST_Titre(1, "Combinaison lin"e_AIGU"aire de graphes", "MATH");
    HIST_Texte(1, 0, "R"e_AIGU"sultat dans : %s\n", resultat->nom);
    HIST_Texte(2, 0, "Graphe : %s", reference->nom);
    HIST_Texte(3, 2, "Coefficient : %.10g\n", 1.0, reference->nom);

    for (i = 0 ; i < nbr_graphes ; i++) {
#ifdef _DEVELOPPE_
      printf("C_lin : on lit %s avec comme coefficient %g\n", infos[i].graphe->nom, infos[i].coef);
#endif
      if ( infos[i].coef != 0.0 ) {
#ifdef _DEVELOPPE_
	printf("C_lin : on garde %s avec comme coefficient %g\n", infos[i].graphe->nom, infos[i].coef);
#endif
	if ( OBJ_choisit(GRF_OPERATIONS, OG_X_IGNORER, QUESTION) == VRAI ) { /* On se fiche des X */
	  GRF_clin(resultat, 1.0, infos[i].graphe, infos[i].coef, resultat);
	} else { /* On ne s'en fiche pas, on recale sur la rfrence */
	  GRF_Rclin(resultat, 1.0, infos[i].graphe, infos[i].coef);
	}

	HIST_Texte(2, 0, "Graphe : %s", infos[i].graphe->nom);
	HIST_Texte(3, 2, "Coefficient : %.10g\n", infos[i].coef);
      }
    }
    break;

  case OP_RECALAGE: /* On veut recaler les spectres / la rfrence */
    if ( OBJ_choisit(GRF_OPERATIONS, OG_X_IGNORER, QUESTION) == VRAI ) { /* On se fiche des X */
      double moyenne_y = 0.0;
      register double *y_ref = reference->y;

      /* On calcule la moyenne des Y de la rfrence */
      for (i = 0 ; i < reference->nombre_point ; i++, y_ref++) {
	moyenne_y += *y_ref;
      }
      moyenne_y /= (double) (reference->nombre_point);

      HIST_Titre(1, "Recaler des graphes sur une r"e_AIGU"f"e_AIGU"rence", "NORM");
      HIST_Texte(1, 0, "R"e_AIGU"f"e_AIGU"rence : %s\n", reference->nom);
      HIST_Texte(2, 0, "Mod"e_GRAVE"le : additif");
      HIST_Texte(3, 0, " (moyenne de recalage : %.20g)", moyenne_y);
      HIST_Texte(2, 0, "\n");

      /* On recale les graphes choisis /  cette rfrence */
      for (i = 0 ; i < nbr_graphes ; i++) {
	GRF_recale_y(infos[i].graphe, moyenne_y, VRAI, 0.0);
      }

      resultat = reference; /* On va sur la rfrence */
    }
    break;

  case OP_RECALE_MULT: /* Recalage multiplicatif */
    if ( OBJ_choisit(GRF_OPERATIONS, OG_X_IGNORER, QUESTION) == VRAI ) { /* On se fiche des X */
      double moyenne_y = 0.0;
      register double *y_ref = reference->y;

      /* On calcule la moyenne des Y de la rfrence */
      for (i = 0 ; i < reference->nombre_point ; i++, y_ref++) {
	moyenne_y += *y_ref;
      }
      moyenne_y /= (double) (reference->nombre_point);
      if (moyenne_y == 0.0) {
	EXF_ERREUR(DIV_ZERO, "Moyenne de r"e_AIGU"f"e_AIGU"rence nulle", txt_err_calc_annule);
	return reference;
      }

      HIST_Titre(1, "Recaler des graphes sur une r"e_AIGU"f"e_AIGU"rence", "NORM");
      HIST_Texte(1, 0, "R"e_AIGU"f"e_AIGU"rence : %s\n", reference->nom);
      HIST_Texte(2, 0, "Mod"e_GRAVE"le : multiplicatif");
      HIST_Texte(3, 0, " (moyenne de recalage : %.20g)", moyenne_y);
      HIST_Texte(2, 0, "\n");

      /* On recale les graphes choisis /  cette rfrence */
      for (i = 0 ; i < nbr_graphes ; i++) {
	GRF_mrecale_y(infos[i].graphe, moyenne_y, VRAI, 0.0);
      }

      resultat = reference; /* On va sur la rfrence */
    }
  }

  return resultat;
}

/* -------- Gestion de l'arbre pour faire les oprations ------------ */

static int GER_operations(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  Graphe *nv_graphe = NULL;
  unsigned long i;
  int nv_operation;

  if ( (objet >= OPER_PREM_GRF) && (objet <= OPER_DERNIER_GRF) ) { /* On active un graphe */
    unsigned int j = 0;

    i = infos_op.prem_affiche;

    /* On commence par dsactiver l'ancienne slection */
    if ( (infos_op.num_choisi >= infos_op.prem_affiche) && (infos_op.num_choisi < ~0U) ) {
      unsigned long k = i;

      while (k < infos_op.num_choisi) {
	k++;
	if (infos_op.choisi[k] == VRAI) j++;
      }
      /* L, on est sur le spectre actuellement choisi */
      if ( j < NBR_GRAPHES_VISIBLES )
	OBJ_choisit(GRF_OPERATIONS, OPER_PREM_GRF + j, FORCE_NON);
      j = 0;
    }

    /* On s'occupe de reprer la nouvelle slection */
    while (j < objet - OPER_PREM_GRF) {
      i++;
      if ( infos_op.choisi[i] == VRAI )
	j++;
    }
    /* L, on devrait tre sur le spectre cliqu */

    if (i == infos_op.num_choisi) { /* On dsactive en fait : plus aucun n'est choisi */
      OBJ_active(GRF_OPERATIONS, OPER_EFFACER, FORCE_NON);
      infos_op.num_choisi = ~0U; /* Suppose qu'on n'aura jamais 2^sizeof(UL) - 1 spectres en mmoire, c'est plausible... */
      return BI_EVENEMENT_TRAITE;
    }

    infos_op.num_choisi = i;
    OBJ_choisit(GRF_OPERATIONS, objet, FORCE_OUI);
    OBJ_active(GRF_OPERATIONS, OPER_EFFACER, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;
  }

  switch (objet)
  {
  case OPER_REFERENCE: /* Choisir le spectre de rfrence */
    OBJ_choisit(GRF_OPERATIONS, OPER_REFERENCE, FORCE_OUI);
    nv_graphe = GRF_choisir(GR_TOUS, FAUX, VRAI, NULL);
    if ( (nv_graphe != NULL) && (nv_graphe != reference) ) {
      reference = nv_graphe;
      met_chaine(GRF_OPERATIONS, OPER_REFERENCE, 56, reference->nom, TXT_COPIER);
    }
    OBJ_choisit(GRF_OPERATIONS, OPER_REFERENCE, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case OD_TYPE_OPERE: /* Choix du type d'opration  effectuer */
    nv_operation = SS_menu_texte(txt_operations, NOMBRE_OPERATIONS, titre_ssmenu_operations,
				 GRF_OPERATIONS, OD_TYPE_OPERE);
    if ( (nv_operation >= 0) && (nv_operation != op_demande) ) {
      if ( op_demande == OP_CLINEAIRE )	aff_defileur(); /* Pour mettre  jour les coefficients de la CL */
      op_demande = nv_operation;
      if ( op_demande == OP_CLINEAIRE )	aff_defileur(); /* Pour mettre  jour les coefficients de la CL */
      met_chaine(GRF_OPERATIONS, OD_TYPE_OPERE, LG_MAX_TXT_OPERATION,
		 txt_operations[nv_operation], TXT_COPIER);
    }
    return BI_EVENEMENT_TRAITE;

  case OPER_AJOUTER: /* Ajouter un graphe de type compatible avec la rfrence */
    if (reference == NULL) return VRAI; /* Pas de rfrence - rare */
    nv_graphe = GRF_choisir(reference->type, FAUX, VRAI, NULL);
    if ( nv_graphe != NULL ) {
      if (nv_graphe != reference) {
	Graphe *graphe = serie_courante->premier_graphe;
	unsigned long i = 0;

	while (graphe != nv_graphe) {
	  graphe = graphe->suivant;
	  i++;
	  if (graphe == NULL) {
	    EXF_ERREUR(ERR_STRUCT_GRF);
	    return BI_EVENEMENT_TRAITE;
	  }
	}

	if ( infos_op.choisi[i] == VRAI ) {
	  printf("Graphe dj slectionn\n");
	  /* Message "graphe dj dans la liste" */
	} else {
	  if (infos_op.nbr_choisis == 0) /* On choisit le premier */
	    infos_op.prem_affiche = i;
	  infos_op.nbr_choisis++;
	  infos_op.choisi[i] = VRAI;
	  aff_defileur();
	}
      } else {
	/* Message "c'est la rfrence" */
      }
    }
    return BI_EVENEMENT_TRAITE;

  case OPER_TT_AJOUTER: /* Ajouter tous les graphes compatibles avec la rfrence */
    if (reference == NULL) return VRAI; /* Pas de rfrence - rare */
    infos_op.prem_affiche = 0;
    infos_op.nbr_choisis = 0;
    for (nv_graphe = serie_courante->premier_graphe, i = 0 ; nv_graphe ; nv_graphe = nv_graphe->suivant, i++) {
      if (nv_graphe == reference) continue; /* On ne choisit pas la rfrence ! */

      if ( (nv_graphe->type == reference->type)  /* Mme type que la rfrence */
	&& ( (nv_graphe->nombre_point == reference->nombre_point) /* Mme nombre de points */
	  || ( OBJ_choisit(GRF_OPERATIONS, OG_X_INTERPOLER, QUESTION) == VRAI) ) ) {
	infos_op.choisi[i] = (unsigned short int) VRAI;
	infos_op.nbr_choisis++;
	if ( (infos_op.prem_affiche == 0) && (infos_op.choisi[0] == FAUX) )
	  infos_op.prem_affiche = i;
      } else
	infos_op.choisi[i] = (unsigned short int) FAUX;
    }
    if ( infos_op.nbr_choisis == 0 ) {
      if (reference->type == GR_SSTYPE)
	EXF_ERREUR(AUCUN_GRAPHE, "Tout graphe");
      else
	EXF_ERREUR( AUCUN_GRAPHE, GRF_TexteTypeGraphe( reference->type ) );
    }
    aff_defileur();
    return BI_EVENEMENT_TRAITE;

  case OPER_EFFACER: /* Effacer le graphe choisi dans la liste */
    if ( infos_op.num_choisi < ~0UL /* ~0 = aucun graphe choisi */ ) {
#ifdef _DEVELOPPE_
      printf("Graphe choisi : numro %ld\n", infos_op.num_choisi);
#endif
      infos_op.choisi[infos_op.num_choisi] = FAUX; /* Ce graphe n'est plus choisi */
      infos_op.nbr_choisis--;
      if (infos_op.nbr_choisis == 0) { /* Plus aucun choisi */
	infos_op.prem_affiche = 0;
	infos_op.num_choisi = ~0UL; /* Plus de spectre choisi */
	aff_defileur(); /* Donc on refait l'ascenseur */
	return BI_EVENEMENT_TRAITE;
      }
      if ( infos_op.num_choisi == infos_op.prem_affiche ) {
	Graphe *prem_graphe, *graphe;
	unsigned long i, nv_premier, nbr_grf;

	for (graphe = serie_courante->premier_graphe, i = 0 ; i < infos_op.prem_affiche ; i++, graphe = graphe->suivant);

	/* Le suivant devient premier affich */
	while ( infos_op.choisi[i] == FAUX ) {
	  graphe = graphe->suivant;
	  i++;
	}
	prem_graphe = graphe;
	if (prem_graphe == NULL) {
#ifdef _DEVELOPPE_
	  fprintf(stderr, "Op. diverses, interface : problme car prem_graphe = NULL\n");
#endif
	  return BI_EVENEMENT_TRAITE;
	}
	nv_premier = i;
	
	/* On compte combien de graphes aprs */
	for (nbr_grf = 0 ; graphe != NULL ; i++, graphe = graphe->suivant)
	  if ( infos_op.choisi[i] == VRAI ) nbr_grf++;

#ifdef _DEVELOPPE_
	printf("Il reste %lu graphes aprs le premier\n", nbr_grf);
#endif
	if (nbr_grf < NBR_GRAPHES_VISIBLES) { /* Il faut alors remonter d'un cran */
	  i = nv_premier - 1;
	  graphe = prem_graphe->precedent;

	  while ( (graphe != NULL) && ( infos_op.choisi[i] == FAUX ) ) {
	    graphe = graphe->precedent;
	    i--;
	  }
	  if (graphe != NULL) /* Il y en a bien un avant */
	    nv_premier = i;
	}

	infos_op.prem_affiche = nv_premier;
      }
      infos_op.num_choisi = ~0UL; /* Plus de spectre choisi */

      aff_defileur(); /* Donc on refait l'ascenseur */
    }
    return BI_EVENEMENT_TRAITE;

  case OPER_TT_EFFACER: /* ter tous les graphes de la liste */
    for (i = 0, nv_graphe = serie_courante->premier_graphe ; nv_graphe != NULL ; i++, nv_graphe = nv_graphe->suivant)
      infos_op.choisi[i] = FAUX;
    infos_op.nbr_choisis = 0;
    infos_op.num_choisi = ~0UL;
    aff_defileur(); /* Donc on refait l'ascenseur */
    return BI_EVENEMENT_TRAITE;

  case OPER_GRF_AVANT: /* Faire remonter d'un cran le dfileur */
    lit_defileur();
    /* On se place sur le graphe avant premier affich actuel */
    for (i = 0, nv_graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, nv_graphe = nv_graphe->suivant);
    do { /* Et on descend jusqu' trouver un affichable */
      i--;
      if ( (i == 0) && (infos_op.choisi[0] == FAUX) ) { /* On tait dj sur le premier */
	OBJ_active(GRF_OPERATIONS, OPER_GRF_AVANT, FORCE_NON); /* Aurait dj d l'tre */
	return VRAI;
      }
    } while ( infos_op.choisi[i] == FAUX );
    infos_op.prem_affiche = i;
    aff_defileur();
    return BI_EVENEMENT_TRAITE;

  case OPER_ASC_FOND:
    {
      int pos_bouton;
      int j = NBR_GRAPHES_VISIBLES - 1;

      OBJ_position(GRF_OPERATIONS, OPER_ASC_BOUTON, -1, -1, NULL, &pos_bouton);
      lit_defileur();
      /* On se place sur le graphe avant premier affich actuel */
      for (i = 0, nv_graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, nv_graphe = nv_graphe->suivant);

      do {
	i--;
	nv_graphe = (y < pos_bouton) ? (nv_graphe->precedent) : (nv_graphe->suivant);
	if (infos_op.choisi[i] == VRAI) {
	  infos_op.prem_affiche = i;
	  j--;
	}
      } while ( (j > 0) && (nv_graphe != NULL) );
    }
    return BI_EVENEMENT_TRAITE;

  case OPER_GRF_APRES: /* Faire descendre d'un cran le dfileur */
    lit_defileur();
    /* On se place sur le graphe avant premier affich actuel */
    for (i = 0, nv_graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, nv_graphe = nv_graphe->suivant);
    do { /* Et on descend jusqu' trouver un affichable */
      i++; nv_graphe = nv_graphe->suivant;
      if ( nv_graphe == NULL ) { /* On tait dj sur le dernier */
	OBJ_active(GRF_OPERATIONS, OPER_GRF_APRES, FORCE_NON); /* Aurait dj d l'tre */
	return BI_EVENEMENT_TRAITE;
      }
    } while ( infos_op.choisi[i] == FAUX );
    infos_op.prem_affiche = i;
    aff_defileur();
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Affichage des graphes (et de leurs coefs.) dans le dfileur -------- */

static int aff_defileur(void)
{
  boolean etat;
  unsigned long i, j = OPER_PREM_GRF;
  unsigned long nbr_grf_avant = 0, nbr_grf_apres = 0, nbr_vides = 0;
  Graphe *graphe;

  OBJ_active(GRF_OPERATIONS, OPER_EFFACER, (infos_op.num_choisi < ~0UL)); /* Actif seulement si un spectre est choisi */
  OBJ_active(GRF_OPERATIONS, OPER_TT_EFFACER, (infos_op.nbr_choisis > 0));

  etat = (infos_op.nbr_choisis <= NBR_GRAPHES_VISIBLES);
  OBJ_active(GRF_OPERATIONS, OPER_GRF_AVANT, !etat); /* Utilisable seulement si on peut dfiler ! */
  OBJ_active(GRF_OPERATIONS, OPER_GRF_APRES, !etat);
  OBJ_active(GRF_OPERATIONS, OPER_ASC_FOND, !etat);
  OBJ_active(GRF_OPERATIONS, OPER_ASC_BOUTON, !etat);

  if (infos_op.nbr_choisis == 0) { /* Aucun graphe n'est choisi */
    unsigned int haut_fond;

#ifdef _DEVELOPPE_
    printf("Aucun spectre choisi\n");
#endif
    OBJ_position(GRF_OPERATIONS, OPER_ASC_BOUTON, 0, 0, NULL, NULL);
    OBJ_taille(GRF_OPERATIONS, OPER_ASC_FOND, -1, -1, NULL, &haut_fond);
    OBJ_taille(GRF_OPERATIONS, OPER_ASC_BOUTON, -1, haut_fond, NULL, NULL);

    for (j = 0 ; j < NBR_GRAPHES_VISIBLES ; j++) {
      met_chaine(GRF_OPERATIONS, OPER_PREM_GRF + j, LGMAX_TXT_NOM_GRAPHE, "", TXT_COPIER);
      OBJ_active(GRF_OPERATIONS, OPER_PREM_GRF + j, FORCE_NON);
      OBJ_active(GRF_OPERATIONS, OPER_COEF_1 + j, FORCE_NON);
      OBJ_choisit(GRF_OPERATIONS, OPER_PREM_GRF + j, FORCE_NON);
    }

    /* Aucune opration possible sans autre graphe que la rfrence */
    op_demande = OP_AUCUNE;
    met_chaine(GRF_OPERATIONS, OD_TYPE_OPERE, LG_MAX_TXT_OPERATION, txt_operations[OP_AUCUNE], TXT_COPIER);
    OBJ_active(GRF_OPERATIONS, OD_TYPE_OPERE, FORCE_NON);
    OBJ_active(GRF_OPERATIONS, OG_X_IGNORER, FORCE_NON);
    OBJ_active(GRF_OPERATIONS, OG_X_INTERPOLER, FORCE_NON);
    return VRAI;
  } else {
    OBJ_active(GRF_OPERATIONS, OD_TYPE_OPERE, FORCE_OUI);
    OBJ_active(GRF_OPERATIONS, OG_X_IGNORER, FORCE_OUI);
    OBJ_active(GRF_OPERATIONS, OG_X_INTERPOLER, FORCE_OUI);
  }

  /* On se place sur le premier graphe  afficher */
  for (i = 0, graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, graphe = graphe->suivant)
    if ( infos_op.choisi[i] == VRAI ) nbr_grf_avant++; /* Graphe choisi avant le 1er affich */
  if ( (nbr_grf_avant > 0) && (infos_op.nbr_choisis <= NBR_GRAPHES_VISIBLES) ) { /* Bizarre ! */
    for (i = 0, graphe = serie_courante->premier_graphe ; infos_op.choisi[i] == FAUX ; i++, graphe = graphe->suivant);
    infos_op.prem_affiche = i;
    return aff_defileur();
  }

  for ( ; (j <= OPER_DERNIER_GRF) && (graphe != NULL) ; i++, graphe = graphe->suivant) {
    if (infos_op.choisi[i] == VRAI) { /* Le graphe a t choisi */
      met_chaine(GRF_OPERATIONS, j, LGMAX_TXT_NOM_GRAPHE, graphe->nom, TXT_COPIER);
      OBJ_active(GRF_OPERATIONS, j, FORCE_OUI);
      OBJ_active(GRF_OPERATIONS, j - OPER_PREM_GRF + OPER_COEF_1, op_demande == OP_CLINEAIRE);
      OBJ_choisit(GRF_OPERATIONS, j, i == infos_op.num_choisi);
      OBJ_MettreNombre( GRF_OPERATIONS, j - OPER_PREM_GRF + OPER_COEF_1, infos_op.coef[i], 0.0 );
      j++;
    }
  }
  for ( ; j <= OPER_DERNIER_GRF ; j++) {
    met_chaine(GRF_OPERATIONS, j, LGMAX_TXT_NOM_GRAPHE, "", TXT_COPIER);
    OBJ_active(GRF_OPERATIONS, j, FORCE_NON);
    OBJ_active(GRF_OPERATIONS, j - OPER_PREM_GRF + OPER_COEF_1, FORCE_NON);
    OBJ_choisit(GRF_OPERATIONS, j, FORCE_NON);
    nbr_vides++; /* Nombre d'emplacements vides */
  }
  if ( (nbr_vides > 0) && (infos_op.nbr_choisis > NBR_GRAPHES_VISIBLES) ) { /* Il y a des places vides, autant viter ! */
  /* On se replace sur le premier graphe */
    for (i = 0, graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, graphe = graphe->suivant);
    do {
      i--; graphe = graphe->precedent;
      if (infos_op.choisi[i] == VRAI) {
	infos_op.prem_affiche = i;
	nbr_vides--; /* Une place libre de moins */
      }
    } while ( (nbr_vides > 0) && (graphe != NULL) );
    aff_defileur();
    return VRAI; /* Inutile de faire la suite puisque l'on s'est rappel soi-mme */ 
  }

  for ( ; graphe != NULL ; i++, graphe = graphe->suivant) /* Pour que i contienne le nombre total de graphes */
   if ( infos_op.choisi[i] == VRAI ) nbr_grf_apres++; /* Graphe choisi apres le dernier affich */

  if (etat) { /* Pas besoin de l'ascenseur */
    unsigned int haut_bouton;

    OBJ_taille(GRF_OPERATIONS, OPER_ASC_FOND, -1, -1, NULL, &haut_bouton);
    OBJ_taille(GRF_OPERATIONS, OPER_ASC_BOUTON, -1, haut_bouton, NULL, NULL);
    OBJ_position(GRF_OPERATIONS, OPER_ASC_BOUTON, 0, 0, NULL, NULL);
  } else { /* On s'occupe de l'ascenseur */
    unsigned int haut_fond, haut_bouton;
    float pos_asc;

    OBJ_active(GRF_OPERATIONS, OPER_GRF_AVANT, nbr_grf_avant > 0);
    OBJ_active(GRF_OPERATIONS, OPER_GRF_APRES, nbr_grf_apres > 0);

    OBJ_taille(GRF_OPERATIONS, OPER_ASC_FOND, -1, -1, NULL, &haut_fond);
    pos_asc  = (float) NBR_GRAPHES_VISIBLES / (float) (infos_op.nbr_choisis); /* i = nombre total de grf */
    pos_asc *= (float) haut_fond;
    if (pos_asc < 5.0) pos_asc = 5.0;
    OBJ_taille(GRF_OPERATIONS, OPER_ASC_BOUTON, -1, (int) pos_asc, NULL, &haut_bouton);

    pos_asc  = (float) (haut_fond - haut_bouton);
    pos_asc *= (float) (nbr_grf_avant);
    pos_asc /= (float) (infos_op.nbr_choisis - NBR_GRAPHES_VISIBLES);
    OBJ_position(GRF_OPERATIONS, OPER_ASC_BOUTON, -1, (int) pos_asc, NULL, NULL);
  }

  return VRAI; /* Tout s'est bien pass */
}

/* -------- Lit les valeurs des coefficients dans le dfileur ------- */

static int lit_defileur(void)
{
  unsigned long i;
  unsigned int j;
  Graphe *graphe;

  /* On se place sur le premier spectre affich */
  for (i = 0, graphe = serie_courante->premier_graphe ; i < infos_op.prem_affiche ; i++, graphe = graphe->suivant);

  /* Et on lit quand a colle... */
  for (j = OPER_PREM_GRF ; (j <= OPER_DERNIER_GRF) && (graphe != NULL) ; i++, graphe = graphe->suivant) {
    if (infos_op.choisi[i] == VRAI) { /* Le graphe a t choisi */
      infos_op.coef[i] = OBJ_LireNombre(GRF_OPERATIONS, j - OPER_PREM_GRF + OPER_COEF_1, NULL );
      j++;
    }
  }

  return 0;
}

/* -------- prparation de l'arbre pour faire les oprations -------- */

static void prepare_operations(void)
{
  OBJ_deplace(GRF_OPERATIONS, OPER_EFFACER   , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(GRF_OPERATIONS, OPER_TT_EFFACER, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(GRF_OPERATIONS, OPER_TT_AJOUTER, 0, +1, DEPLACE_PIXELS);

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte( GRF_OPERATIONS, 3, txt_trad_reference );
  met_chaine(GRF_OPERATIONS, OPER_AJOUTER   , 0, txt_trad_ajouter    , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OPER_TT_AJOUTER, 0, txt_trad_ttajouter  , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OPER_EFFACER   , 0, txt_trad_effacer    , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OPER_TT_EFFACER, 0, txt_trad_tteffacer  , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OG_X_IGNORER   , 0, txt_trad_ignorer    , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OG_X_INTERPOLER, 0, txt_trad_interpoler , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OPERE_LANCER   , 0, txt_trad_lancer     , TXT_REMPLACE);
  met_chaine(GRF_OPERATIONS, OD_AVEC_ERREURS, 0, txt_trad_avecerreurs, TXT_REMPLACE);
#endif

  met_chaine(GRF_OPERATIONS, OD_TYPE_OPERE, LG_MAX_TXT_OPERATION, txt_operations[op_demande], TXT_COPIER);

  arbres[GRF_OPERATIONS].drapeaux.prepare = VRAI;
  arbres[GRF_OPERATIONS].fonction.arbre = GER_operations;
}
