/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 2005, Emmanuel CURIS
   Cration : 6 mars 2005

   Oprations diverses sur les graphes, pas spcifiques de l'exafs

   Transformer, de faon "autonome", un graphe --- avec transformations prdfinies possibles !
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "serie.h"

#include "c_formel.h"
#include "operations.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

Graphe *GRF_Transformer_R(Graphe *graphe, struct expr *tr_X, struct expr *tr_sX, struct expr *tr_Y, struct expr *tr_sY)
{
  Graphe *resultat;
  unsigned long i;

  /* Contrles prliminaires */
  if ( NULL == graphe ) return NULL;

  /* Le rsultat : graphe de type "autre", mme nombre de points,... */
  resultat = GRF_nouveau( graphe, graphe->nombre_point, GR_AUTRETYPE, graphe->genre_donnees, "Graphe transform" );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer le graphe rsultat !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Initialisation des variables */
  Var_X.type = VAR_CONSTANTE;
  Var_Y.type = VAR_CONSTANTE;
  Var_sX.type = VAR_CONSTANTE;
  Var_sY.type = VAR_CONSTANTE;

  /* Les transformations demandes */
  /* a. Transformation en X */
  if ( tr_X != NULL ) {
    if ( ( NULL == graphe->x ) || ( NULL == resultat->x ) ) {
      (void) fprintf(stderr, "%s, %s l.%u : Transformation demande en X, pas de X vrai dans le graphe !\n",
		     __FILE__, __FUNCTION__, __LINE__);
    } else {
      double x_transforme;
      boolean calcul_reussi;

      for ( i = 0 ; i < graphe->nombre_point ; i++ ) {
	/* Initialisation des variables */
	Var_X.valeur_initiale = graphe->x[ i ];
	Var_Y.valeur_initiale = graphe->y[ i ];

	/* Calcul de l'arbre */
	calcul_reussi = calcule_arbre( tr_X, &x_transforme );
	if ( VRAI == calcul_reussi ) {
	  resultat->x[ i ] = x_transforme;
	} else {
	  resultat->x[ i ] = 0.0;
	  (void) fprintf(stderr, "%s, %s l.%u : Erreur de calcul pour X=%g, Y=%g\n",
			 __FILE__, __FUNCTION__, __LINE__, graphe->x[ i ], graphe->y[ i ]);
	}
      }

      if ( tr_sX != NULL ) {
	Graphe *erreurs;

	/* On cre des erreurs pour le graphe rsultat */
	erreurs = GRF_creer_erreurs( resultat, AXE_X );
	if ( erreurs != NULL ) {
	}

	OPTION_A_FAIRE( "Transformer les erreurs en X !" );
      }
    }
  } else {
    /* X inchangs ! */
    if ( graphe->x != NULL ) {
      if ( NULL == resultat->x ) { /* Graphe de type srie -> table des X commune ! */
	resultat->x = graphe->x;
      } else {
	memcpy( resultat->x, graphe->x, sizeof( double ) * ( graphe->nombre_point ) );
      }

      if ( graphe->erreur_x != NULL ) {
	OPTION_A_FAIRE( "Copier les erreurs en X !" );
      }
    }
  }

  /* b. Transformation en Y */
  if ( tr_Y != NULL ) {
    double y_transforme;
    boolean calcul_reussi;

    for ( i = 0 ; i < graphe->nombre_point ; i++ ) {
      /* Initialisation des variables */
      Var_X.valeur_initiale = graphe->x[ i ];
      Var_Y.valeur_initiale = graphe->y[ i ];

      /* Calcul de l'arbre */
      calcul_reussi = calcule_arbre( tr_Y, &y_transforme );
      if ( VRAI == calcul_reussi ) {
	resultat->y[ i ] = y_transforme;
      } else {
	resultat->y[ i ] = 0.0;
	(void) fprintf(stderr, "%s, %s l.%u : Erreur de calcul pour X=%g, Y=%g\n",
		       __FILE__, __FUNCTION__, __LINE__, graphe->x[ i ], graphe->y[ i ]);
      }
    }

    if ( tr_sY != NULL ) {
      OPTION_A_FAIRE( "Transformer les erreurs en Y !" );
    }
  } else {
    /* Y inchangs ! */
    memcpy( resultat->y, graphe->y, sizeof( double ) * ( graphe->nombre_point ) );

    if ( graphe->erreur_y != NULL ) {
      OPTION_A_FAIRE( "Copier les erreurs en Y !" );
    }
  }

  /* Mise  jour des bornes */
  GRF_bornes( resultat, DESORDONNE, DESORDONNE );

  /* Transformation termine */
  return resultat;
}
