/* LASE - Logiciel d'Analyse des Spectres Expérimentaux
   (C) 1996 - 2006, Emmanuel CURIS
   Création : 11 juillet 2006

   Calculs de probabilité divers

   Inégalités probabilistes
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "proba.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* -------- Inégalité de Bonferroni ----------
   Soit Ai un ensemble d'événements [hypothèses Ho de stat...], alors p( union{Ai} ) <= p(A1) + p(A2) + ...

   Application statistique fréquente : soit a_i le risque de se tromper en rejettant H0i, a le risque global de se tromper
    alors a <= a_1 + a_2 + ...
   ==> pour contrôler a et avec les a_i tous égaux, il suffit de prendre a_i = 1/n a, n : nombre de comparaisons
 */

double PROBA_INEG_Bonferroni(double p_global, unsigned long n_evenements)
{
  if ( 0 == n_evenements ) return 0.0;  /* Aucun sens de toute façon... */

  return p_global / (double) n_evenements;
}

/* -------- Inégalité de Sidak --------

   Application statistique fréquente : soit a_i le risque de se tromper en rejettant H0i, a le risque global de se tromper
    alors a <= 1 - (1 - a_1) * (1 - a_2) * ...
   ==> pour contrôler a et avec les a_i tous égaux, il suffit de prendre a_i = 1 - (1 - a)^(1/n), n : nombre de comparaisons
 */

double PROBA_INEG_Sidak(double p_global, unsigned long n_evenements)
{
  if ( 0 == n_evenements ) return 0.0;  /* Aucun sens de toute façon... */

  return 1.0 - pow( 1.0 - p_global, 1.0 / (double) n_evenements );
}
