/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2003, Emmanuel CURIS

   Oprations sur les graphes contenant une srie de valeurs (chantillon)
   Calcul de la moyenne et des paramtres descriptifs drivs
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "stats_utils.h"

#include "graphes.h"

#include "ser_moyenne.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static double CarresDesEcarts(double *table, unsigned long nbr_valeurs, double moyenne);

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

int GRF_Serie_Parametres_Moyenne(Graphe *graphe)
{
  double moyenne, variance, ecart_absolu = 0.0;
  int retour;
  unsigned long i;

  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun graphe/chantillon transmis\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
  if ( NULL == graphe->y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de donnes en Y, calcul impossible\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  if ( graphe->genre_donnees != GRF_SERIE ) {
    AVERTISSEMENT( "Le graphe %s n'est pas not comme un chantillon. Rsultat  prendre avec prcautions", graphe->nom );
  }

  /* Calcul de la moyenne : simple ! */
  retour = STAT_Moyenne_D( graphe->y, graphe->nombre_point, &moyenne );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le calcul de la moyenne [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }

  /* On en dduit la variance, en commanant par la somme des carrs des carts  la moyenne */
  variance = CarresDesEcarts( graphe->y, graphe->nombre_point, moyenne );

  /* Moyenne des carts absolus  la moyenne */
  for ( i = 0 ; i < graphe->nombre_point ; i++ ) {
    ecart_absolu += fabs( graphe->y[ i ] - moyenne );
  }
  ecart_absolu /= (double) ( graphe->nombre_point );

  /* Sauvegarde dans l'historique */
  HIST_Titre( 1, "Param"e_GRAVE"tres d'un "e_AIGU"chantillon", "Stat" );
  HIST_Texte( 1, 0, "Graphe "e_AIGU"chantillon : %s\n", graphe->nom );
  HIST_Texte( 3, 0, "Nombre de valeurs : N = %lu\n", graphe->nombre_point );
  HIST_Texte( 3, 0, "Moyenne           : m = %g [%g / N ]\n", moyenne, moyenne * (double) graphe->nombre_point );
  HIST_Texte( 3, 0, E_AIGU"cart-type (ch.) : s = %g [%g / N ] ==> s = %g\n", variance / (double) graphe->nombre_point, variance,
	      sqrt( variance / (double) graphe->nombre_point ) );
  if ( graphe->nombre_point > 1 ) {
    HIST_Texte( 3, 0, "cart-type (pop.) : s* = %g [%g / (N-1) ] ==> s* = %g\n", variance / (double) (graphe->nombre_point - 1), variance,
		sqrt( variance / (double) ( graphe->nombre_point - 1 ) ) );
  }
  HIST_Texte( 3, 0, E_AIGU"cart moyen absolu : %g [%g / N ] ==> s = %g\n", ecart_absolu, ecart_absolu * (double) graphe->nombre_point );

  /* Affichage des principaux rsultats */
  variance /= (double) ( graphe->nombre_point - 1 );
  retour = BI_ERR_alerte( NULL, &MSG_Parametres, VRAI, moyenne, variance, sqrt( variance ), ecart_absolu );
  if ( 2 == retour ) {
    return 0;
  }

  OPTION_A_FAIRE( "Intervalles de confiance sur les paramtres" );

  /* Tout s'est bien pass */
  return 0;
}

static double CarresDesEcarts(double *table, unsigned long nbr_valeurs, double moyenne)
{
  unsigned long i;
  double somme = 0.0, ecart;

  for ( i = 0 ; i < nbr_valeurs ; i++, table++ ) {
    ecart = *table - moyenne;
    somme += ecart * ecart;
  }

  return somme;
}
