/* LASE - Logiciel d'Analyse des Spectres EXAFS
   Copyright (C) 1996 - 1998 Emmanuel CURIS
   Cration du fichier : 6 novembre 1998

   Module de calculs statistiques

   Calcul des moyennes  simples  entre diffrents graphes
   Dans l'tat actuel, suppose que les spectres choisis soient de type rel uniquement
                                   les X soient comparables (aucun contrle)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "serie.h"

#include "statistiques.h"
#include "tri.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

typedef struct {
  unsigned long index_dep;
  boolean *a_fondre;
  Graphe *prem_graphe;
} inf_asc;

static unsigned long nombre_graphes = 0;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void affiche_defileur(inf_asc *infos);

/* Pas static car utilises aussi pour la fusion, mais dclares en local
   car ne devraient pas servir ailleurs ! */

int gere_arbre_fusion(unsigned int objet, int x, int y);
void    gere_asc_fusion  (inf_asc *infos, int objet, int y);
int     init_asc_fusion  (inf_asc *infos);
void    prep_arbre_fusion(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Moyennage de diffrents blocs --------------------------------------------------\
 |   Renvoie le nombre de points effectivement moyenns                                     |
 |   Ne modifie pas le contenu des TRI_infos                                                |
 |                                                                                          |
 | Principe : On fait la somme des X_k sur tous les blocs de X, on divise par le nombre     |
 |                   de blocs = fait le point X_moyen_k                                     |
 |            Mme chose pour les Y                                                         |
 |                                                                                          |
 | Suppose donc qu'il y ait cohrence des X lors de l'appel                                 |
 |                                                                                          |
 | x_moyen       = adresse o crire les X moyenns                                         |
 | y_moyen       = adresse o crire les Y moyenns                                         |
 | infos_moyenne = toutes les infos ncessaire au calcul                                    |
 | nbr_blocs     = nombre de blocs  moyenner                                               |
 | total_pts     = nombre total de points au maximum dans x_moyen/y_moyen                   |
 \----------------------------------------------------------------------------------------- */

unsigned long MOY_blocs(double *x_moyen, double *y_moyen,
			TRI_infos *infos_moyenne, unsigned long nbr_blocs, unsigned long total_pts)
{
  unsigned long i, num_bloc;
  unsigned long reel_nbr_blocs; /* Nombre rel de blocs dans la moyenne
				   utilis si certains blocs finissent plus tt */
  unsigned long vx_blocs;
  double x, y;

  vx_blocs = nbr_blocs; /* Nombre attendu de blocs */
  for (i = 0 ; i < total_pts ; i++, x_moyen++, y_moyen++) {
    x = 0; y = 0;
    reel_nbr_blocs = 0; /* Aucun bloc ne convient pour l'instant */
    for (num_bloc = 0 ; num_bloc < nbr_blocs ; num_bloc++) {
      if ( i < infos_moyenne[num_bloc].nbr_points ) { /* Ce bloc convient encore */
	x += ( infos_moyenne[ num_bloc ].x )[i];
	y += ( infos_moyenne[ num_bloc ].y )[i];
	reel_nbr_blocs++;
      }
    }

    if ( reel_nbr_blocs == 0 ) { /* Plus aucun bloc ne convient - ne devrais JAMAIS arriver */
      EXF_ERREUR(DIV_ZERO, "Plus de blocs !", txt_err_calc_annule);
#ifdef _DEVELOPPE_
      /* Ne devrait jamais arriver -> on sort ce qui se passe pour essayer de voir pourquoi ! */
      fprintf(stderr, "*** Division par zro dans MOY_blocs ! ***\n");
      fprintf(stderr, "Nombre de points demands : %lu\n", total_pts);
      fprintf(stderr, "Index de point actuel : %lu [commence  zro]\n", i);
      for (i = 0 ; i < nbr_blocs ; i++)
	fprintf(stderr, "Nombre de points dans le bloc %lu : %lu\n", i, infos_moyenne[i].nbr_points);
#endif
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return i; /* On s'arrte ici... */
    } else {
      if (vx_blocs != reel_nbr_blocs) {
	if ( STAT_ERREUR(NBR_PTS_DIFFERENTS) == 2) /* On arrte ici */
	  return i;
      }
      *x_moyen = x / reel_nbr_blocs;
      *y_moyen = y / reel_nbr_blocs;
      vx_blocs = reel_nbr_blocs;
    }
  }

  return total_pts; /* On les a tous utiliss */
}

/* ~~~~~~~~ Interfaage des routines ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Moyenne : procdure d'appel, avec arbre GEM et tout et tout ----------\
 |   Renvoie le graphe contenant la moyenne de tous les graphes choisis           |
 |                                                                                |
 | Cette fonction permet  l'utilisateur de crer une liste de graphes  moyenner |
 | Il y a quelques contrles sur la cohrence des graphes choisis, mais ce n'est  |
 | pas encore parfait ! Il faudrait amliorer a...                               |
 \------------------------------------------------------------------------------- */

Graphe *moyenner(void)
{
  TRI_infos *infos_tri;
  Graphe *resultat = NULL;
  int renvoi, type;
  unsigned long total_points = 0;
  unsigned long i,compteur;
  unsigned long nbr_a_moyenner = 0; /* Nombre de graphes  moyenner */
  inf_asc infos_ascenseur;
  char titre[100]; /* Devrait suffire puisque titre ne reprend aucun ancien titre */
  const char *titre_dialogue;

  /* Prparation de l'ascenseur (aucun graphe n'est slectionn) */
  if (init_asc_fusion(&infos_ascenseur) != VRAI)
    return NULL;

#ifdef _ENGLISH_
  titre_dialogue = "Choose the spectra to average";
  met_chaine(FUSION_GRAPHES, FUSION_OK, 0, "Average", TXT_REMPLACE);
#else
  titre_dialogue = "Choisir les spectres "a_GRAVE" moyenner";
  met_chaine(FUSION_GRAPHES, FUSION_OK, 0, "Moyenner", TXT_REMPLACE);
#endif

  OBJ_cache(FUSION_GRAPHES, FUS_DOUBLE_NOM , FORCE_OUI);
  OBJ_cache(FUSION_GRAPHES, FUS_DOUBLE_FOND, FORCE_OUI);

  do {
    renvoi = ARB_Bloquant( FUSION_GRAPHES, titre_dialogue, principale, 0 );
    if (renvoi == FUSION_ANNULE) { /* On annule la fusion */
      free(infos_ascenseur.a_fondre); /* On libre la mmoire */
      return NULL;
    }
  } while (renvoi != FUSION_OK);

  for (i = 0 ; i < serie_courante->nbr_graphes ; i++) {
    if ( (infos_ascenseur.a_fondre)[i] == VRAI)
      nbr_a_moyenner++;
  }

  if (nbr_a_moyenner < 2) {
    STAT_ERREUR(MOY_TROP_PEU_GRAPHES);
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  infos_tri = (TRI_infos *) calloc( nbr_a_moyenner, sizeof(TRI_infos) );
  if (infos_tri == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "moyennes.c", "moyenner", "infos_tri");
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  /* On prpare les blocs  moyenner */
  resultat = serie_courante->premier_graphe;
  compteur = 0; /* Les graphes slectionns */
  i = 0;
  while ( (infos_ascenseur.a_fondre)[i] == FAUX) { /* Recherche du premier graphe  moyenner */
    i++;
    resultat = resultat->suivant;
    if (resultat == NULL) { /* Dernier graphe - normalement, ne devrait jamais arriver
			       avec le test prcdent, mais on ne sait jamais... */
      STAT_ERREUR(MOY_TROP_PEU_GRAPHES);
      free(infos_ascenseur.a_fondre); /* On libre la mmoire */
      free(infos_tri);
      return NULL;
    }
  }
  type = resultat->type; /* Juste vrification des types... */
  total_points = resultat->nombre_point;

  while ( i < serie_courante->nbr_graphes ) {
    if ( (infos_ascenseur.a_fondre)[i] == VRAI) {
      if (resultat->nombre_point > total_points)
	total_points = resultat->nombre_point; /* Nombre de points du total */
      
      infos_tri[ compteur ].x = resultat->x;
      infos_tri[ compteur ].y = resultat->y;
      infos_tri[compteur].position   = 0;
      infos_tri[compteur].nbr_points = resultat->nombre_point;
      compteur++;

      if ( ( type != resultat->type ) && (resultat->type != -1) ) {
	if (type == -1) 
	  type = resultat->type;
	else {
	  if (STAT_ERREUR(MOY_TYPES_DIFFERENTS) == 2) {
	    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	    free(infos_tri);
	    return NULL;
	  }
	}
      }
    }
    resultat = resultat->suivant;
    i++;
  }

  free(infos_ascenseur.a_fondre); /* On libre la mmoire */

#ifdef _ENGLISH_
  sprintf(titre, "Average of %lu graphs", compteur);
#else
  sprintf(titre, "Moyenne de %lu graphes", compteur);
#endif

  resultat = GRF_nouveau( NULL, total_points, type, GR_NORMAL, titre );
  if (resultat == NULL) {
    free(infos_tri);
    return NULL;
  }

  /* On fait la moyenne effective */
  i = MOY_blocs(resultat->x, resultat->y, infos_tri, compteur, resultat->nombre_point);
  if (i > 0)
    resultat->nombre_point = i; /* Nombre de points rel */
  GRF_bornes(resultat, DESORDONNE, DESORDONNE);

  free(infos_tri);
  return resultat;
}

/* -------- Gestion de l'arbre de paramtrage des moyennes ------------ */

int gere_arbre_fusion(unsigned int objet, int x, int y)
{
  inf_asc *infos_ascenseur;
  unsigned long i;
  unsigned int haut_fond, haut_bouton;
  Graphe *graphe;
  
  infos_ascenseur = (inf_asc *) ARB_LireComplement( FUSION_GRAPHES );
  
  if ( (objet >= FUSION_GR1) && (objet <= FUSION_GR11) ) {
    i = infos_ascenseur->index_dep + objet - FUSION_GR1;
    
    (infos_ascenseur->a_fondre)[i] = OBJ_choisit(FUSION_GRAPHES, objet, ECHANGE);
    return BI_EVENEMENT_TRAITE; /* Traitement termin */
  }

  OBJ_taille(FUSION_GRAPHES, FUSION_FOND  , -1, -1, NULL, &haut_fond);
  OBJ_taille(FUSION_GRAPHES, FUSION_BOUTON, -1, -1, NULL, &haut_bouton);

  switch (objet)
  {
  case FUSION_AVANT:
  case FUSION_APRES:
  case FUSION_FOND: /* Fond de l'ascenseur */
  case FUSION_BOUTON: /* On fait glisser l'ascenseur */
    infos_ascenseur->index_dep = ARB_ascenseur( FUSION_GRAPHES, FUSION_AVANT, FUSION_FOND, FUSION_BOUTON, FUSION_APRES, ASC_VERTICAL,
						serie_courante->nbr_graphes, 11, infos_ascenseur->index_dep, objet, x, y, ECHANGE );
    /* On recherche le nouveau premier affichable */
    for (i = 0, graphe = serie_courante->premier_graphe ; i < infos_ascenseur->index_dep ; i++, graphe = graphe->suivant);
    infos_ascenseur->prem_graphe = graphe;
    affiche_defileur(infos_ascenseur);
    return BI_EVENEMENT_TRAITE;

  case FUSION_AUCUN: /* On dslectionne tout */
    for (i = 0 ; i < serie_courante->nbr_graphes ; i++)
      infos_ascenseur->a_fondre[i] = FAUX; /* Il n'est pas choisi */
    for (i = FUSION_GR1 ; i <= FUSION_GR11 ; i++)
      OBJ_choisit(FUSION_GRAPHES, i, FORCE_NON);
    return BI_EVENEMENT_TRAITE;

  case FUSION_TOUS: /* On slectionne tout */
    for (i = 0 ; i < serie_courante->nbr_graphes ; i++)
      infos_ascenseur->a_fondre[i] = VRAI; /* Il est choisi */

    if ( serie_courante->nbr_graphes <= (FUSION_GR11 - FUSION_GR1) ) /* Moins de graphes que d'affichables */
      for (i = 0 ; i < serie_courante->nbr_graphes ; i++)
	OBJ_choisit(FUSION_GRAPHES, FUSION_GR1 + i, FORCE_OUI);
      /* Note : moins de 12 graphes -> parcourir deux fois la boucle n'est pas gnant... */
    else
      for (i = FUSION_GR1 ; i <= FUSION_GR11 ; i++)
	OBJ_choisit(FUSION_GRAPHES, i, FORCE_OUI);
    return BI_EVENEMENT_TRAITE;

  case FUSION_EXAFS: /* Slectionner tous les graphes de type EXAFS */
    for (i = 0, graphe = serie_courante->premier_graphe ; i < serie_courante->nbr_graphes ; i++, graphe = graphe->suivant)
      infos_ascenseur->a_fondre[i] = graphe->drapeaux.exafs; /* Choisi seulement si de type exafs */

    if ( serie_courante->nbr_graphes <= (FUSION_GR11 - FUSION_GR1) ) /* Moins de graphes que d'affichables */
      for (i = 0 ; i < serie_courante->nbr_graphes ; i++)
	OBJ_choisit(FUSION_GRAPHES, FUSION_GR1 + i, infos_ascenseur->a_fondre[i]);
      /* Note : moins de 12 graphes -> parcourir deux fois la boucle n'est pas gnant... */
    else
      for (i = 0 ; i <= FUSION_GR11 - FUSION_GR1 ; i++)
	OBJ_choisit(FUSION_GRAPHES, FUSION_GR1 + i,
		    infos_ascenseur->a_fondre[i + infos_ascenseur->index_dep]);
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Affichage de l'ascenseur et du dfileur  partir des donnes -------\
 | Cette fonction n'est appele QUE s'il y a plus de graphes que d'entres dans |
 | le slecteur. Elle ne fait aucune vrification  ce sujet...                 |
 \----------------------------------------------------------------------------- */

static void affiche_defileur(inf_asc *infos)
{
  Graphe *graphe;
  short int i;

#ifdef _DEVELOPPE_
  if (nombre_graphes <= 11)
    printf("affiche_defileur : appel avec moins de graphes que d'entres !\n");

  if (infos == NULL)
    printf("affiche_defileur : appel sans informations !\n");
#endif

  for (i = 0, graphe = infos->prem_graphe ; i < 11 ; i++, graphe = graphe->suivant) {
#ifdef _DEVELOPPE_
    if ( graphe == NULL )
      printf("affiche_defileur : appel avec un numro de dpart laissant\n"
	     "des entres libres  la fin du slecteur\n");
#endif

    met_chaine (FUSION_GRAPHES, FUSION_GR1 + i, 50, graphe->nom, TXT_COPIER);
    OBJ_choisit(FUSION_GRAPHES, FUSION_GR1 + i, infos->a_fondre[infos->index_dep + i]);
  }

  OBJ_active(FUSION_GRAPHES, FUSION_APRES, (graphe != NULL)); /* On est sur le dernier graphe */
  OBJ_active(FUSION_GRAPHES, FUSION_AVANT, infos->prem_graphe != serie_courante->premier_graphe);
}

/* -------- Initialisation du slecteur de graphes de l'arbre ----------\
 |    Renvoie <= 0 si problme, sinon renvoie VRAI                      |
 |                                                                      |
 | Rserve infos->a_fondre avec calloc, ce qui fait qu'aucun graphe     |
 | n'est dj slectionn lors de l'appel (tout  0 = FAUX)             |
 \--------------------------------------------------------------------- */

int init_asc_fusion(inf_asc *infos)
{
  Graphe *graphe;
  short int i;

  /* On compte les graphes en mmoire */
  for (nombre_graphes = 1, graphe = serie_courante->premier_graphe ; graphe ; nombre_graphes++, graphe = graphe->suivant);
#ifdef _DEVELOPPE_
  printf("Moyenne/Fusion : on travaille sur %lu graphes\n", nombre_graphes);
#endif

  infos->a_fondre = (boolean *) calloc( nombre_graphes, sizeof(boolean) );
  if (infos->a_fondre == NULL) {
    SYS_ERREUR(MANQUE_MEMOIRE, "moyennes.c", "init_asc_fusion", "infos->a_fondre");
    return -1; /* Aucun graphe cr */
  }
  infos->index_dep   = 0; /* On commence au graphe 1 */
  infos->prem_graphe = serie_courante->premier_graphe;

  if ( arbres[FUSION_GRAPHES].drapeaux.prepare == FAUX ) /* Pas encore prpar */
    prep_arbre_fusion();

  /* On initialise l'arbre : textes du slecteur = noms des premiers graphes */
  for (i = 0, graphe = serie_courante->premier_graphe ; i < 11 ; i++) {
    if (graphe) { /* Tant que ce n'est pas le dernier... */
      met_chaine(FUSION_GRAPHES, FUSION_GR1 + i, 50, graphe->nom, TXT_COPIER);
      OBJ_active(FUSION_GRAPHES, FUSION_GR1 + i, FORCE_OUI);
      graphe = graphe->suivant;
    } else {
      met_chaine(FUSION_GRAPHES, FUSION_GR1 + i, 50, "", TXT_COPIER);
      OBJ_active(FUSION_GRAPHES, FUSION_GR1 + i, FORCE_NON);
    }
    OBJ_choisit(FUSION_GRAPHES, FUSION_GR1 + i, FORCE_NON); /* Non choisi par dfaut */
  }

  ARB_ascenseur(FUSION_GRAPHES, FUSION_AVANT, FUSION_FOND, FUSION_BOUTON, FUSION_APRES, ASC_VERTICAL,
		serie_courante->nbr_graphes, 11, 0, ROOT, 0, 0, METTRE);

  ARB_MettreComplement( FUSION_GRAPHES, infos );

  return VRAI; /* Pas de problme */
}

/* -------- Prparation de l'arbre de paramtrage des moyennes -------- */

void prep_arbre_fusion(void)
{
  ARB_EnjoliverDefileur( FUSION_GRAPHES, ROOT, FUSION_GR1, FUSION_GR11,
			 FUSION_AVANT, FUSION_FOND, FUSION_BOUTON, FUSION_APRES, ARB_ASCENSEUR_DROITE );

#ifndef _DEVELOPPE_
  OBJ_active(FUSION_GRAPHES, FUSION_TOUT_TYPE, FORCE_NON);
  OBJ_active(FUSION_GRAPHES, FUSION_TYPE, FORCE_NON);
#endif

#ifdef _ENGLISH_ /* Traductions anglaises des boutons */
#endif

  arbres[FUSION_GRAPHES].fonction.arbre = gere_arbre_fusion;
  arbres[FUSION_GRAPHES].drapeaux.prepare = VRAI;
}

