/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2006, Emmanuel CURIS
   Création : 3 aoûtt 2006

   Analyses statistiques

   Analyses post-ANVA (comparaisons multiples ; même si l'ANVA n'est pas requise avant...)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "graphes.h"
#include "proba.h"
#include "statistiques.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Réalisation pratique ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Comparaisons au témoin ~-~-~-~-~-~-~-~-~ */

/* -------- Comparaison à un témoin, ANVA à un facteur --------
   Renvoie 0 si tout s'est bien passé
           != 0 en cas d'erreur (>0 : simple avertissement)

 infos_ANVA     : l'analyse de variance considérée
 methode        : la méthode utilisée pour faire le test [Dunnett par défaut]
 methode_calcul : la méthode utilisée pour calculer le fractile de Dunnett
 sens           : unilatéral [<0 si i-témoin>0, >0 si i-témoin<0] ou bilatéral [0]
 alpha_global   : le risque d'erreur GLOBAL pour toutes les comparaisons
   -------- */

int ANVA_1F_ComparerAuTemoin(struct infos_ANVA_1F *infos_ANVA, unsigned int methode, unsigned int methode_calcul, int sens, double alpha_global)
{
  boolean est_equilibree = FAUX;
  int retour = -1;

  /* Contrôles préliminaires */
  if ( NULL == infos_ANVA ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucune ANVA 1 facteur indiquée !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }
  if ( infos_ANVA->modalite_controle < 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de témoin défini !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  if ( infos_ANVA->comparaisons != NULL ) {
    (void) fprintf(stderr, "%s, %s l.%u : Il y a déjà des comparaisons d'effectuées !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -10;
  }

  /* On prépare la liste des comparaisons */
  infos_ANVA->comparaisons = STAT_Init_ComparaisonsAUnTemoin( infos_ANVA->facteur.nbr_modalites, infos_ANVA->facteur.modalites, infos_ANVA->modalite_controle, sens );
  if ( NULL == infos_ANVA->comparaisons ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de créer la table des comparaisons\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }
  infos_ANVA->nombre_comparaisons = infos_ANVA->facteur.nbr_modalites - 1;

  /* On fait les calculs en utilisant la méthode indiquée */
  /* a. Utilise-t-on un plan équilibré ? */
  est_equilibree = ANVA_1F_EstEquilibree( infos_ANVA );

  /* b. Appel de la bonne routine, en fonction de la méthode */
  switch ( methode )
  {
  case ANVA_CMP_TEMOIN_DUNETT:
    retour = STAT_ComparerAUnTemoin_Dunnett( infos_ANVA->nombre_comparaisons, infos_ANVA->comparaisons, sens, alpha_global, methode_calcul,
					     infos_ANVA->SR, infos_ANVA->ddl_R, est_equilibree );
    break;

  case ANVA_CMP_TEMOIN_NMPG:
    OPTION_A_FAIRE( "Méthode NMPG" );
    break;

  case ANVA_CMP_TEMOIN_DT:
    OPTION_A_FAIRE( "Méthode DT" );
    break;

  case ANVA_CMP_TEMOIN_STEEL:
    OPTION_A_FAIRE( "Méthode de Steel" );
    break;

  default:
    AVERTISSEMENT( "Méthode de comparaison au témoin inconnue : %u", methode );
    retour = -42;
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur survenue dans l'analyse [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    if ( retour < 0 ) {
      /* Erreur grave : effacer les comparaisons */
      STAT_EffacerComparaisons( infos_ANVA->nombre_comparaisons, infos_ANVA->comparaisons );
      infos_ANVA->comparaisons = NULL;
      infos_ANVA->nombre_comparaisons = 0;
      return -30;
    }
  }

  /* Terminé correctement */
  return 0;
}
