/* LASE - Logiciel d'Analyse des Spectres EXAFS
   LASE - (C) 1998-2003, Emmanuel CURIS

   Construction d'une description de configuration expérimentale
   Gestion des composants élémentaires d'un échantillon
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "echantillon.h"
#include "graphes.h"
#include "interface.h"
#include "lissage.h"

#include "th_composition.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* Version pour le sélecteur universel */
static int COMPOSITION_Charger_I (void);
static int COMPOSITION_Copier_I  (unsigned long n_original);
static int COMPOSITION_Effacer_I (unsigned long n_original);
static int COMPOSITION_Modifier_I(unsigned long n_original);
static int COMPOSITION_Nouveau_I (unsigned long n_avant);
static int COMPOSITION_Sauver_I  (unsigned long n_composition);

static const char     *COMPOSITION_TrouverNom(unsigned int arbre, unsigned long objet);

static double Matiere_CalculerMu_1p(struct matiere *composition, double energie);

/* ~~~~~~~~ Réalisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Manipulation des structures ~-~-~-~-~-~-~-~-~ */

struct matiere *Matiere_CreerComposition(struct matiere *avant, unsigned int nbr_composants)
{
  struct matiere *composition;

  /* Réservation de la structure */
  composition = (struct matiere *) calloc( 1, sizeof( struct matiere ) );
  if ( NULL == composition ) {
    (void) fprintf(stderr, "%s,%s l.%u : mémoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Initialisation complémentaire */
  composition->densite = 1.0;

  /* Chaînage */
  if ( NULL == premiere_composition ) {
    premiere_composition = composition;
  } else if ( NULL == avant ) {
    composition->suivante = premiere_composition;
    premiere_composition = composition;
  } else {
    composition->suivante = avant->suivante;
    avant->suivante = composition;
  }

  /* Réservation des deux tables, si nécessaire */
  if ( nbr_composants > 0 ) {
    composition->fraction_massique = ( float * ) calloc( nbr_composants, sizeof( float ) );
    if ( NULL == composition->fraction_massique ) {
      (void) fprintf(stderr, "%s,%s l.%u : Mémoire insuffisante pour %u composants\n",
		     __FILE__, __FUNCTION__, __LINE__, nbr_composants);
    } else {
      composition->composants = ( struct composant ** ) calloc( nbr_composants, sizeof( struct composant * ) );
      if ( NULL == composition->fraction_massique ) {
	(void) fprintf(stderr, "%s,%s l.%u : Mémoire insuffisante pour %u composants\n",
		       __FILE__, __FUNCTION__, __LINE__, nbr_composants);
	free( composition->fraction_massique );
	composition->fraction_massique = NULL;
      } else {
	composition->nbr_composants = nbr_composants;
      }
    }
  }

  /* Code couleur : à déterminer => BLANC [trou, par défaut] */
  composition->couleur = BLANC;

  /* Construction réussie */
  return composition;
}

int Matiere_EffacerComposition(struct matiere *composition, boolean forcer)
{
  if ( NULL == composition ) return -1;

  OPTION_A_FAIRE( "Effacer une composition" );

  if ( FAUX == forcer ) {
  }

  /* Tout s'est bien passé */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Sélecteur de compositions ~-~-~-~-~-~-~-~-~ */

struct matiere *Matiere_ChoisirComposition(unsigned int arbre, unsigned int objet, unsigned long lg_max, struct matiere *premiere)
{
  InfosSelecteur infos_selecteur;
  struct matiere *choix = NULL;
  int retour;

  if ( objet != ROOT )
    OBJ_choisit( arbre, objet, FORCE_OUI );

  if ( NULL == premiere ) premiere = premiere_composition;

  /* Initialisation du sélecteur */
  infos_selecteur.nouveau  = COMPOSITION_Nouveau_I;
  infos_selecteur.copier   = COMPOSITION_Copier_I;
  infos_selecteur.modifier = COMPOSITION_Modifier_I;
  infos_selecteur.sauver   = COMPOSITION_Sauver_I;
  infos_selecteur.charger  = COMPOSITION_Charger_I;
  infos_selecteur.effacer  = COMPOSITION_Effacer_I;
  infos_selecteur.compter  = COMPOSITION_Compter;
  infos_selecteur.objet_v_texte = COMPOSITION_TrouverNom;

  retour = I_SelecteurUniversel( "Composition", &infos_selecteur, ARB_QuelleFenetre( arbres + arbre, FAUX ) );
  if ( retour == 0 ) {
    choix = COMPOSITION_N_Vers_C( NULL, infos_selecteur.obj_choisi );
  }
  if ( objet != ROOT ) {
    if ( NULL == choix ) {
      met_chaine( arbre, objet, lg_max, "Pas de composition choisie", TXT_COPIER );
    } else {
      met_chaine( arbre, objet, lg_max, choix->nom, TXT_COPIER );
    }
    OBJ_choisit( arbre, objet, FORCE_NON );
  }

  return choix;
}

/* -------- Trouver la composition demandée -------- */

inline unsigned long COMPOSITION_C_Vers_N(struct matiere *premiere, struct matiere *composition)
{
  unsigned long i;
  struct matiere *idx_c;

  if ( NULL == premiere ) premiere = premiere_composition;
  if ( NULL == composition ) return 0;

  for ( i = 0, idx_c = premiere ; idx_c != composition ; i++, idx_c = idx_c->suivante ) {
    if ( NULL == idx_c ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de trouver la composition %s\n",
		     __FILE__, __FUNCTION__, __LINE__, composition->nom);
      return 0;
    }
  }

  return i;
}

struct matiere *COMPOSITION_N_Vers_C(struct matiere *premiere, unsigned long n)
{
  struct matiere *composition;
  unsigned long i;

  if ( NULL == premiere ) premiere = premiere_composition;
  if ( 0 == n ) return premiere;

  for ( i = 0, composition = premiere ; i < n ; i++, composition = composition->suivante ) {
    if ( NULL == composition ) {
      (void) fprintf(stderr, "%s,%s l.%u : n=%lu, seulement %lu compositions\n",
		     __FILE__, __FUNCTION__, __LINE__, n + 1, COMPOSITION_Compter() );
      return NULL;
    }
  }

  /* On a trouvé */
  return composition;
}

int COMPOSITION_Charger_I(void)
{
  (void) fprintf(stderr, "%s,%s l.%u : À FAIRE\n",
		 __FILE__, __FUNCTION__, __LINE__);

  /* Tout s'est bien passé */
  return 0;
}

int COMPOSITION_Copier_I(unsigned long n_original)
{
  struct matiere *original;

  original = COMPOSITION_N_Vers_C( premiere_composition, n_original );
  if ( NULL == original ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans COMPOSITION_N_Vers_C()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  (void) fprintf(stderr, "%s,%s l.%u : À FAIRE\n",
		 __FILE__, __FUNCTION__, __LINE__);

  return 0;
}

int COMPOSITION_Effacer_I(unsigned long n_original)
{
  struct matiere *original;

  original = COMPOSITION_N_Vers_C( premiere_composition, n_original );
  if ( NULL == original ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans COMPOSITION_N_Vers_C\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  Matiere_EffacerComposition( original, FAUX );

  return 0;
}

int COMPOSITION_Modifier_I(unsigned long n_original)
{
  struct matiere *original;

  original = COMPOSITION_N_Vers_C( premiere_composition, n_original );
  if ( NULL == original ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans COMPOSITION_N_Vers_C\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  Matiere_ModifierComposition( original );

  return 0;
}

int COMPOSITION_Nouveau_I(unsigned long n_avant)
{
  struct matiere *composition, *avant;

  /* On cherche où la placer dans la liste */
  if ( NULL == premiere_composition ) {
    avant = NULL;
  } else {
    avant = COMPOSITION_N_Vers_C( premiere_composition, n_avant );
  }

  /* On la crée */
  composition = Matiere_CreerComposition( avant, 0 );
  if ( NULL == composition ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans Matiere_CreerComposition()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* Et on la paramètre */
  Matiere_ModifierComposition( composition );

  return 0;
}

int COMPOSITION_Sauver_I(unsigned long n_composition)
{
  (void) fprintf(stderr, "%s,%s l.%u : À FAIRE [%lu]\n",
		 __FILE__, __FUNCTION__, __LINE__, n_composition);

  /* Tout s'est bien passé */
  return 0;
}

unsigned long COMPOSITION_Compter(void)
{
  unsigned long i;
  struct matiere *d;

  for ( d = premiere_composition, i = 0 ; d != NULL ; d = d->suivante, i++);
  return i;
}

const char *COMPOSITION_TrouverNom(unsigned int arbre __attribute__((__unused__)), unsigned long objet)
{
  struct matiere *d;

  d = COMPOSITION_N_Vers_C( premiere_composition, objet );
  if ( NULL == d ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans COMPOSITION_N_Vers_D\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  return d->nom;
}

/* ~-~-~-~-~-~-~-~-~ Gestion du contenu ~-~-~-~-~-~-~-~-~ */

/* -------- Ajouter un composant dans une composition --------
   Les composants forment un tableau dynamique dans la composition,
   mais sont extraits d'une liste simplement chainée [seuls les pointeurs sont stockés]

   On insère donc le composant indiqué dans la table ; si on passe NULL, on le crée avant.
 */

struct composant *Matiere_AjouterComposant( struct matiere *composition, struct composant *composant )
{
  struct composant **table;
  float *fractions;

  ENTREE_FONCTION("Matiere_AjouterComposant", "composition=%p composant=%p", composition, composant);

  if ( NULL == composition ) {
    SORTIE_FONCTION("Matiere_AjouterComposant", "%p ERREUR [composition=NULL]", NULL);
    return NULL;
  }

  /* Vérifier que l'on ne le met pas en double */
  if ( composition->composants != NULL ) {
    unsigned int i;

    for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
      if ( composition->composants[ i ] == composant ) {
	(void) BI_ERR_alerte( NULL, &MSG_ConstituantPresent, VRAI, composant->nom );

	SORTIE_FONCTION("Matiere_AjouterComposant", "%p [déjà présent]", composition->composants[ i ]);
	return composition->composants[ i ];
      }
    }
  }

  /* On augmente la taille du tableau dynamique des constituants */
  table = (struct composant **) realloc( composition->composants, sizeof( struct composant * ) * ( composition->nbr_composants + 1 ) );
  if ( NULL == table ) {
    (void) fprintf(stderr, "Matiere_AjouterComposant : Impossible de réallouer la table des composants\n");

    SORTIE_FONCTION("Matiere_AjouterComposant", "%p [table des c.]", NULL);
    return NULL;
  }
  composition->composants = table;

  /* Idem pour les fractions massiques */
  fractions = (float *) realloc( composition->fraction_massique, sizeof( float ) * ( composition->nbr_composants + 1 ) );
  if ( NULL == fractions ) {
    (void) fprintf(stderr, "Matiere_AjouterComposant : Impossible de réallouer la table des fractions massiques\n");

    SORTIE_FONCTION("Matiere_AjouterComposant", "%p [table des f. m.]", NULL);
    return NULL;
  }
  composition->fraction_massique = fractions;

  /* Ici seulement, car si la réallocation des tables a échoué, évite d'avoir un constituant créé pour rien */
  if ( NULL == composant ) {
    DANS_FONCTION("Créer un nouveau composant", 0);

    composant = CreerComposant( NULL );
    if ( NULL == composant ) {
      (void) fprintf(stderr, "Matiere_AjouterComposant : ERREUR dans CreerComposant()\n");
      SORTIE_FONCTION("Matiere_AjouterComposant", "%p ERREUR", NULL);
      return NULL;
    }
  }

  /* On ajoute le nouveau composant à la fin de la table */
  composition->composants[ composition->nbr_composants ] = composant;
  composition->fraction_massique[ composition->nbr_composants ] = 1.0; /* Par défaut... */

  /* Un composant de plus... */
  composition->nbr_composants++;

  /* On a fini, tout s'est bien passé */
  SORTIE_FONCTION("Matiere_AjouterComposant", "%p", composant);
  return composant;
}

/* -------- Contrôler qu'une composition indiquée est valide -------- */

int Matiere_ControlerComposition(struct matiere *composition)
{
  unsigned int i;
  double f_totale = 0.0;

  /* A-t-on les informations nécessaires ? */
  if ( NULL == composition ) {
    (void) fprintf(stderr, "Matiere_ControlerComposition : composition absente\n");
    return -2;
  }

  if ( ( NULL == composition->composants ) || ( NULL == composition->fraction_massique ) ) {
    (void) fprintf(stderr, "Matiere_ControlerComposition : composition non-définie\n");
    return -3;
  }

  /* Les informations sont-elles cohérentes ? */
  if ( composition->densite <= 0.0 ) {
    (void) fprintf(stderr, "Matiere_ControlerComposition : densité incorrecte [%g]\n", composition->densite);
    return -4;
  }

  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    double f_massique;

    f_massique = composition->fraction_massique[ i ];
    if ( ( f_massique < 0.0 ) || ( f_massique > 1.0 ) ) {
      (void) fprintf(stderr, "Matiere_ControlerComposition : fraction massique incorrecte [%g, composant %u/%u « %s »]\n",
		     f_massique, i + 1, composition->nbr_composants, composition->composants[ i ]->nom );
      return -1;
    }
    f_totale += f_massique;
  }
  if ( f_totale != 1.0 ) {
#ifdef _DEVELOPPE_
    (void) printf("Matiere_ControlerComposition : La somme des f. massiques ne vaut pas 1.0 [différence : %g]\n", 1.0 - f_totale);
#endif
    if ( fabs( f_totale - 1.0 ) > 1.0e-4 ) { /* Au-delà des erreurs d'arrondi */
      (void) fprintf(stderr, "Matiere_ControlerComposition : La somme des f. massiques ne vaut pas 1.0 [%g]\n", f_totale);
      return -6;
    }
  }

  /* Contrôle terminé, pas de problème trouvé */
  return 0;
}

/* -------- Estimer la densité à partir de la composition -------- */

int Matiere_EstimerDensite(struct matiere *composition)
{
  double somme = 0.0, densite = 0.0;
  unsigned int i;

  if ( NULL == composition ) return -1;
  if ( 0 == composition->nbr_composants ) {
    composition->densite = 0.0; /* Vide ! */
    return 0;
  }

  /* Moyenne pondérée des densités de chaque constituant de la composition */
  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    somme   += composition->fraction_massique[ i ];
    densite += composition->fraction_massique[ i ] * composition->composants[ i ]->densite;
  }
  if ( somme <= 0.0 ) { /* Somme des fractions massiques absurde */
    (void) fprintf(stderr, "%s,%s l.%u : Somme des fractions massiques absurde [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, somme);
    return -2;
  }

  /* Densité (idéalement, la somme des fractions massique devrait valoir 1.0,
     mais avec les erreurs d'arrondi... */
  densite /= somme;

  /* On mémorise le résultat */
  composition->densite = densite;

  /* Terminé, sans erreur */
  return 0;
}

/* -------- Changer le nom de la composition -------- */

int Matiere_Nommer(struct matiere *composition, const char *nom)
{
  if ( NULL == composition ) return -1;
  if ( NULL == nom ) return +1;

  free( composition->nom );

  composition->nom = strdup( nom );
  if ( NULL == composition->nom ) {
    (void) fprintf(stderr, "%s,%s l.%u : Mémoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +2;
  }

  return 0;
}

/* -------- Calcul de la section efficace de diffusion -------- */

int Matiere_CalculerTau( struct matiere *composition, Graphe *graphe, boolean elastique, int methode, boolean ajouter )
{
  int retour;
  unsigned long i;

  if ( ( NULL == composition ) || ( NULL == graphe ) ) {
    return -1;
  }
  if ( NULL == composition->composants ) {
    (void) fprintf(stderr, "Matiere_CalculerTau : Composition non définie\n");
    return -2;
  }

  if ( FAUX == ajouter ) {
    /* On commence par mettre à zéro le graphe */
    for ( i = 0 ; i < graphe->nombre_point ; i++ ) {
      graphe->y[ i ] = 0.0;
    }
  }

  /* On ajoute la contribution de chaque constituant */
  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    retour = Composant_CalculerTau( composition->composants[ i ], graphe, composition->fraction_massique[ i ], elastique, methode );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "Matiere_CalculerTau : ERREUR dans Composant_CalculerTau [code %d]\n", retour);
      if ( retour < 0 ) { /* Erreur grave */
	return -5;
      }
    }
  }

  /* Calcul des bornes [si le graphe est affiché...] */
  GRF_bornes( graphe, NE_PAS_FAIRE, DESORDONNE );

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Calcul du spectre d'absorption [photo-électrique] pour une composition donnée --------
 * Renvoie le graphe ISOLÉ ainsi obtenu, NULL en cas d'erreur
 */

Graphe *Matiere_CalculerMu( struct matiere *composition, double *table, unsigned long taille, double E_min, double E_max, double pas )
{
  Graphe *mu_total;
  int retour;
  unsigned int i;

  if ( NULL == composition ) {
    return NULL;
  }

  /* Controle : le spectre n'a-t-il pas déjà été calculé ? */
  if ( composition->mu_total != NULL ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "Matiere_CalculerMu : Spectre déjà calculé\n");
#endif
    return composition->mu_total;
  }

  if ( NULL == composition->composants ) {
    (void) fprintf(stderr, "Matiere_CalculerMu : Composition non définie\n");
    return NULL;
  }

  /* On prépare le graphe */
  if ( ( NULL == table ) || ( 0 == taille ) ) {
    mu_total = GRF_nv_GrilleXP( NULL, GR_MUMATRICE, GRF_FCT_REELLE, NULL, VRAI, E_min, E_max, pas );
  } else { /* X précalculés */
    mu_total = GRF_nv_TableX( NULL, GR_MUMATRICE, GRF_SERIE, NULL, VRAI /* FAUX */, table, taille, VRAI/*FAUX*/ );
  }
  if ( NULL == mu_total ) {
    (void) fprintf(stderr, "Matiere_CalculerMu : ERREUR dans GRF_nv_GrilleXP()\n");
    return NULL;
  }
/*   exporter( mu_total, "creation.asc" ); */

  /* On y ajoute la contribution de chaque constituant */
  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    retour = Composant_CalculerMu( composition->composants[ i ], mu_total, composition->fraction_massique[ i ] );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "Matiere_CalculerMu : ERREUR dans Composant_CalculerMu [code %d]\n", retour);
      if ( retour < 0 ) { /* Erreur grave */
	GRF_eff_isole( mu_total );
	return NULL;
      }
    }
  }
/*   exporter( mu_total, "fin_calcul.asc" ); */

  /* Les bornes, si jamais utilisées ailleurs [affichage p. ex.] */
  GRF_bornes( mu_total, NE_PAS_FAIRE, DESORDONNE );
/*   exporter( mu_total, "bornes_faites.asc" ); */

  /* Tout s'est bien passé */
  return mu_total;
}

/* -------- Calcul du coefficient massique d'absorption [photo-électrique] pour une composition donnée --------
 * Renvoie la valeur obtenue, en cm^2/g, pour l'énergie indiqué [en eV]
 */

static double Matiere_CalculerMu_1p(struct matiere *composition, double energie)
{
  unsigned int i;
  double mu = 0.0;

  /* On suppose une relation additive des (mu/rho) : vrai sans interaction chimique entre les composants */
  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    mu += Composant_CalculerMu_1p( composition->composants[ i ], composition->fraction_massique[ i ], energie );
  }

#ifdef _DEVELOPPE_
  if ( ( mu < 0.0 ) || ( isinf( mu ) == VRAI ) || ( isnan( mu ) == VRAI ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans le calcul de (mu/rho) [%g]\n",
		   __FILE__, __FUNCTION__, __LINE__, mu);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return 0.0;
  }
#endif
  return mu;
}

/* -------- Calcul du spectre de fluorescence idéal -------- */

int Matiere_CalculerFluo( struct matiere *composition, double energie, boolean zero )
{
  unsigned long i;
  int retour;
  double mu;

  if ( NULL == composition ) {
    return -1;
  }

  if ( NULL == composition->composants ) {
    (void) fprintf(stderr, "Matiere_CalculerFluo : Composition non définie\n");
    return -2;
  }
  if ( NULL == composition->fluo ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de spectre de fluo préparé [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, composition->nom);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -3;
  }

  /* Calcul du coefficient d'absorption pour la composition indiquée */
  if ( NULL == composition->mu_total ) { /* Non calculé : arrive si énergie incidente unique ! */
    mu = Matiere_CalculerMu_1p( composition, energie );
  } else {			/* Précalculé dans un graphe pour les diverses énergies incidentes */
    mu = GRF_LireValeur( composition->mu_total, energie, INTERPOLE_LINEAIRE );
  }

  /* On remet le graphe à zéro */
  if ( VRAI == zero ) {
    for ( i = 0 ; i < composition->fluo->nombre_point ; i++ ) {
      composition->fluo->y[ i ] = 0.0;
    }
  }

  /* On calcule le spectre de fluorescence : contribution de chaque constituant */
  for ( i = 0 ; i < composition->nbr_composants ; i++ ) {
    retour = Composant_CalculerFluo( composition->composants[ i ], composition->fluo, composition->fraction_massique[ i ], energie, mu );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "Matiere_CalculerMu : ERREUR dans Composant_CalculerFluo [code %d]\n", retour);
      if ( retour < 0 ) { /* Erreur grave */
	return -4;
      }
    }
  }

  /* Les bornes, si jamais utilisées ailleurs [affichage p. ex.] */
  GRF_bornes( composition->fluo, NE_PAS_FAIRE, DESORDONNE );

  /* Tout s'est bien passé */
  return 0;
}
