/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2002, Emmanuel CURIS

   Conversion numro atomique -> diverses donnes sur les atomes
*/

#define UTILITAIRES_CONTIENT_DONNEES_ATOMES

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "chimie_utils.h"

/* Tableau regroupant toutes les donnes sur les atomes connus
   lectrongativits selon Allred-Rochow, calle sur Pauling
   Rayons exprims en pm (1 pm = 10^-12 m)
   Masse volumique exprime en g/cm^3 */

ATM_infos ATM_donnees[URANIUM] = {
/* Symbole   Nom         M           EN     rc  rm   rho */
  {"H" , "Hydrogne"   ,  1.00794  , 2.20,  37, 120, 0.000089},
  {"He", "Hlium"      ,  4.002602 , 5.50,   0, 160, 0.000179},
  {"Li", "Lithium"     ,  6.941    , 0.97, 134, 181, 0.534},
  {"Be", "Bryllium"   ,  9.012182 , 1.47,  90,   0, 1.848},
  {"B" , "Bore"        , 10.811    , 2.01,  86,   0, 2.34},
  {"C" , "Carbone"     , 12.0107   , 2.5 ,  77, 169, 2.26},
  {"N" , "Azote"       , 14.00674  , 3.07,  75, 155, 0.0125},
  {"O" , "Oxygne"     , 15.9994   , 3.5 ,  73, 151, 0.001429},
  {"F" , "Fluor"       , 18.9984032, 4.1 ,  71, 147, 1.108},
  {"Ne", "Non"        , 20.1797   , 4.84,   0, 160, 0.0009},
  {"Na", "Sodium"      , 22.9989770, 1.01, 154, 227, 0.97},
  {"Mg", "Magnsium"   , 24.3050   , 1.23, 130, 171, 1.74},
  {"Al", "Aluminium"   , 26.981538 , 1.47, 118, 205, 2.72},
  {"Si", "Silicium"    , 28.0855   , 1.74, 117, 210, 2.33},
  {"P" , "Phosphore"   , 30.973761 , 2.06, 106, 182, 1.82},
  {"S" , "Soufre"      , 32.066    , 2.44, 102, 180, 2.},
  {"Cl", "Chlore"      , 35.4527   , 2.83,  99, 177, 1.56},
  {"Ar", "Argon"       , 39.948    , 3.20,   0, 189, 0.001784},
  {"K" , "Potassium"   , 39.0983   , 0.91, 196, 275, 0.862},
  {"Ca", "Calcium"     , 40.078    , 1.04, 174,   0, 1.55},
  {"Sc", "Scandium"    , 44.955910 , 1.20, 144,   0, 2.992},
  {"Ti", "Titane"      , 47.867    , 1.32, 132,   0, 4.54},
  {"V" , "Vanadium"    , 50.9415   , 1.45, 125,   0, 6.11},
  {"Cr", "Chrome"      , 51.9961   , 1.56, 127,   0, 7.19},
  {"Mn", "Manganse"   , 54.93804  , 1.60, 143,   0, 7.42},
  {"Fe", "Fer"         , 55.845    , 1.64, 122,   0, 7.86},
  {"Co", "Cobalt"      , 53.933200 , 1.7 , 126,   0, 8.9},
  {"Ni", "Nickel"      , 58.6934   , 1.75, 120, 161, 8.9},
  {"Cu", "Cuivre"      , 63.546    , 1.75, 138, 141, 8.94},
  {"Zn", "Zinc"        , 65.39     , 1.66, 125, 140, 7.14},
  {"Ga", "Gallium"     , 69.723    , 1.82, 121, 188, 5.903},
  {"Ge", "Germanium"   , 72.61     , 2.02, 122,   0, 5.323},
  {"As", "Arsenic"     , 74.92160  , 2.82, 120, 185, 5.73},
  {"Se", "Slnium"    , 78.96     , 2.48, 117, 190, 4.79},
  {"Br", "Brome"       , 79.904    , 2.74, 114, 185, 3.12},
  {"Kr", "Krypton"     , 83.80     , 2.94, 115, 201, 3.74},
  {"Rb", "Rubidium"    , 85.4678   , 0.89, 211, 242, 1.532},
  {"Sr", "Strontium"   , 87.62     , 0.97, 192,   0, 2.54},
  {"Y" , "Yttrium"     , 88.90585  , 1.11, 162,   0, 4.405},
  {"Zr", "Zirconium"   , 91.224    , 1.22, 145,   0, 6.53},
  {"Nb", "Niobium"     , 92.90638  , 1.23, 134,   0, 8.57},
  {"Mo", "Molybdne"   , 95.94     , 1.30, 129,   0, 10.22},
  {"Tc", "Techntium"  , 98.       , 1.36,   0,   0, 11.5},
  {"Ru", "Ruthnium"   , 101.07    , 1.42, 124,   0, 12.41},
  {"Rh", "Rhodium"     , 102.90550 , 1.45, 125,   0, 12.44},
  {"Pd", "Palladium"   , 106.42    , 1.35, 128, 161, 12.16},
  {"Ag", "Argent"      , 107.8682  , 1.42, 134, 171, 10.5},
  {"Cd", "Cadmium"     , 112.411   , 1.46, 141, 159, 8.65},
  {"In", "Indium"      , 114.818   , 1.49, 144, 191, 7.28},
  {"Sn", "tain"       , 118.710   , 1.72, 141, 218, 5.31}, /* 50 */
  {"Sb", "Antimoine"   , 121.760   , 1.82, 140, 220, 6.691},
  {"Te", "Tellure"     , 127.60    , 2.01, 136, 208, 6.24},
  {"I" , "Iode"        , 126.90447 , 2.21, 133, 200, 4.94},
  {"Xe", "Xnon"       , 131.29    , 2.40, 128, 218, 0.59e-2},
  {"Cs", "Csium"      , 132.90545 , 0.86, 230, 260, 1.873},
  {"Ba", "Baryum"      , 137.327   , 0.97, 198,   0, 3.5},
  {"La", "Lanthane"    , 138.9055  , 1.08, 169,   0, 6.15},
  {"Ce", "Crium"      , 140.116   , 1.08, 165,   0, 6.67},
  {"Pr", "Prasodyme"  , 140.90765 , 1.07, 165,   0, 6.769},
  {"Nd", "Nodyme"     , 144.24    , 1.07, 164,   0, 6.96},
  {"Pm", "Promthium"  , 145.      , 1.07,   0,   0, 6.782},
  {"Sm", "Samarium"    , 150.36    , 1.07, 166,   0, 7.536},
  {"Eu", "Europium"    , 151.964   , 1.01, 185,   0, 5.259},
  {"Gd", "Gadolinium"  , 157.25    , 1.11, 161,   0, 7.95},
  {"Tb", "Terbium"     , 158.92534 , 1.10, 159,   0, 8.272},
  {"Dy", "Dysprosium"  , 162.50    , 1.10, 159,   0, 8.536},
  {"Ho", "Holmium"     , 164.93032 , 1.10, 158,   0, 8.803},
  {"Er", "Erbium"      , 167.26    , 1.11, 157,   0, 9.051},
  {"Tm", "Thulium"     , 168.93421 , 1.11, 156,   0, 9.332},
  {"Yb", "Ytterbium"   , 173.04    , 1.06, 170,   0, 6.57}, /* Web lements */
  {"Lu", "Luttium"    , 174.967   , 1.14, 156,   0, 9.842},
  {"Hf", "Hafnium"     , 178.49    , 1.23, 146,   0, 13.3},
  {"Ta", "Tantale"     , 180.9479  , 1.33, 136,   0, 16.6},
  {"W" , "Tungstne"   , 183.84    , 1.40, 138,   0, 19.3},
  {"Re", "Rhnium"     , 186.207   , 1.46, 128,   0, 21.02},
  {"Os", "Osmium"      , 190.23    , 1.52, 126,   0, 22.5},
  {"Ir", "Iridium"     , 192.217   , 1.55, 126,   0, 22.42},
  {"Pt", "Platine"     , 195.078   , 1.44, 128, 172, 21.37},
  {"Au", "Or"          , 196.96655 , 1.41, 136, 168, 19.37},
  {"Hg", "Mercure"     , 200.59    , 1.44, 144, 152, 13.546},
  {"Tl", "Thallium"    , 204.3833  , 1.44, 153, 198, 11.86},
  {"Pb", "Plomb"       , 207.2     , 1.5 , 154, 201, 11.34},
  {"Bi", "Bismuth"     , 208.98038 , 1.67, 152, 240, 9.8},
  {"Po", "Polonium"    , 209.      , 1.76, 153,   0, -1.},
  {"At", "Astate"      , 210.      , 1.90, 145,   0, -1.},
  {"Rn", "Radon"       , 222.      , 2.06,   0,   0, 0.973e-2},
  {"Fr", "Francium"    , 223.02    , 0.86,   0,   0, -1},
  {"Ra", "Radium"      , 226.03    , 0.97,   0,   0, -1},
  {"Ac", "Actinium"    , 227.03    , 1.00,   0,   0, -1},
  {"Th", "Thorium"     , 232.0381  , 1.11,   0,   0, 11.7},
  {"Pa", "Protactinium", 231.03588 , 1.14,   0,   0, -1},
  {"U" , "Uranium"     , 238.02891 , 1.22, 175, 186, 19.05}, /* Plutonium, rho = 19.7 g/cm^3 */
};

/* -------- Fonctions permettant de rcuprer les informations -------- */

const char *ATM_Z_vers_symbole(int Z)
{
  if ( (Z <= 0) || (Z > URANIUM) ) {
    return "??"; /* Numro atomique inconnu */
  }

  return ATM_donnees[Z - 1].symbole;
}

const char *ATM_Z_vers_nom(int Z)
{
  if ( (Z <= 0) || (Z > URANIUM) ) {
    return "lment inconnu"; /* Numro atomique inconnu */
  }

  return ATM_donnees[Z - 1].nom;
}

/* -------- Renvoie le rayon, en pm -------- */

float ATM_Z_vers_rayon(int Z, int type_rayon)
{
  if ( (Z <= 0) || (Z > URANIUM) ) {
    return 0.1; /* Numro atomique inconnu ==> rayon absurde mais sans risque de problme (0 / <0 risquerait...) */
  }

  switch (type_rayon)
  {
  case ATM_RAYON_VANDERWAALS:
    return ATM_donnees[Z - 1].rayon_vdw;

  case ATM_RAYON_METALLIQUE:
    break;
  case ATM_RAYON_IONIQUE:
    break;

  default: /* Rayon covalent par dfaut */
    return ATM_donnees[Z - 1].rayon_covalent;
  }

  return 0.1; /* Type de rayon encore inconnu */
}

/* -------- Renvoie la masse demande, en units normales -------- */

float ATM_Z_vers_masse(int Z, int type_masse)
{
  if ( (Z <= 0) || (Z > URANIUM) ) {
    return 0.0; /* Numro atomique inconnu ==> masse absurde */
  }

  switch ( type_masse )
  {
  case ATM_MASSE_ATOMIQUE:
    return ATM_donnees[Z - 1].m_molaire / AVOGADRO;

  case ATM_MASSE_VOLUMIQUE:
    return ATM_donnees[Z - 1].m_volumique;

  case ATM_MASSE_MOLAIRE:
  default:
    return ATM_donnees[Z - 1].m_molaire;
  }
}
