/* LASE - Logicie d'Analyse des Spectres Exprimentaux
   Logs divers pour faciliter le dboguage
   (c) 2002 - Emmanuel CURIS
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#ifndef AVEC_DB_MALLOC_
# define DEBOGUE_MALLOC_ 1 /* Dboguage de la gestion mmoire */
#else
# undef DEBOGUE_MALLOC_
#endif
/*#define DEBOGUE_FONCTIONS_ 1*/  /* Dboguage des appels de fonctions [log dans un fichier]*/
/*#define DEBOGUE_FONCTIONS_ 0*/  /* Dboguage des appels de fonctions [affichage]*/

#if defined(_WINDOWS_) && defined(_DEVELOPPE_)
# define DEBOGUE_FONCTIONS_ 1	/* Sous windows, difficile sans log des fonctions ! */
#endif

#undef ENTREE_FONCTION
#undef DANS_FONCTION
#undef SORTIE_FONCTION

#include "deboguage.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#ifdef DEBOGUE_MALLOC_
static FILE *Malloc_log_ = NULL;
#endif

#ifdef DEBOGUE_FONCTIONS_
static FILE *Fonction_log_ = NULL;
static unsigned long niveau_fonctions = 0;
#endif

/* ~~~~~~~~ Initialisation (renvoie FAUX en cas de problme) ~~~~~~~~ */

int DEBOGUE_init_logs(void)
{
#ifdef DEBOGUE_MALLOC_
  if ( Malloc_log_ != NULL ) {
    (void) fprintf(stderr, "%s, %s l.%u : Log des malloc dj lanc : annul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 1;
  }
  Malloc_log_ = fopen("malloc.log", "w");
  if ( NULL == Malloc_log_ ) {
    (void) fprintf(stderr, "DEBOGUE_init_logs : Impossible de crer malloc.log\n");
    return FAUX;
  }

  (void) fprintf(Malloc_log_, "******** Logs des rservations et librations mmoire ********\n\n");
#endif

#ifdef DEBOGUE_FONCTIONS_
# if DEBOGUE_FONCTIONS_ == 1
  if ( Fonction_log_ != NULL ) {
    (void) fprintf(stderr, "%s, %s l.%u : Log des fonctions dj lanc : annul\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 1;
  }

  Fonction_log_ = fopen("fonctions.log", "w");
  if ( NULL == Fonction_log_ ) {
    (void) fprintf(stderr, "DEBOGUE_init_logs : Impossible de crer fonctions.log\n");
    return FAUX;
  }
  (void) fprintf(Fonction_log_, "******** Logs des appels de fonctions ********\n\n");

# elif DEBOGUE_FONCTIONS_ == 0
  Fonction_log_ = stdout;
  (void) fprintf(Fonction_log_, "******** Logs des appels de fonctions ********\n\n");
# endif
#endif

  return VRAI;
}

void DEBOGUE_arrete_logs(void)
{
#ifdef DEBOGUE_MALLOC_
  if ( Malloc_log_ != NULL ) {
    (void) fprintf( Malloc_log_, " ******** Arret des logs de malloc ********\n" );
    (void) fclose( Malloc_log_ );
    Malloc_log_= NULL;
  }
#endif

#ifdef DEBOGUE_FONCTIONS_
  if ( ( Fonction_log_ != NULL ) && ( Fonction_log_ != stdout ) ) {
    (void) fprintf( Fonction_log_, " ******** Arret des logs de fonctions ********\n" );
    (void) fclose( Fonction_log_ );
    Fonction_log_= NULL;
  }
#endif
}

/* ~~~~~~~~~~~~~~~~ Logs pour la mmoire ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#ifdef DEBOGUE_MALLOC_
# undef malloc
# undef calloc
# undef realloc
# undef strdup
# undef free

void *debogue_malloc(size_t taille)
{
  void *resultat;

  ENTREE_FONCTION("malloc", "taille=%lu", taille);

  if (NULL != Malloc_log_) { /* Log dmarr */
    fprintf( Malloc_log_, "MALLOC : taille demande %lu... ", (unsigned long) taille);
  }
  resultat = malloc( taille );
  if (NULL != Malloc_log_) { /* Log dmarr */
    if (NULL == resultat) {
      fprintf( Malloc_log_, "chec\n");
    } else {
      fprintf( Malloc_log_, "rserve en %p\n", resultat);
    }
    (void) fflush( Malloc_log_ );
  }

  SORTIE_FONCTION("malloc", "%p", resultat, 0);
  return resultat;
}

void *debogue_calloc(size_t nombre, size_t taille)
{
  void *resultat;

  if (NULL != Malloc_log_) { /* Log dmarr */
    fprintf( Malloc_log_, "CALLOC : taille demande %lu (%lu objets de taille unitaire %lu)... ",
	     (unsigned long) nombre * taille, (unsigned long) nombre, (unsigned long) taille);
  }
  resultat = calloc( nombre, taille );
  if (NULL != Malloc_log_) { /* Log dmarr */
    if (NULL == resultat) {
      fprintf( Malloc_log_, "chec\n");
    } else {
      fprintf( Malloc_log_, "rserve en %p\n", resultat);
    }
    (void) fflush( Malloc_log_ );
  }
  return resultat;
}

void *debogue_realloc(void *pointeur, size_t taille)
{
  void *resultat;

  if (NULL != Malloc_log_) { /* Log dmarr */
    fprintf( Malloc_log_, "REALLOC : on redimensionne %p, nouvelle taille demande %lu... ",
	     pointeur, (unsigned long) taille);
  }
  resultat = realloc( pointeur, taille );
  if (NULL != Malloc_log_) { /* Log dmarr */
    if (NULL == resultat) {
      fprintf( Malloc_log_, "chec\n");
    } else {
      fprintf( Malloc_log_, "rserve en %p\n", resultat);
    }
    (void) fflush( Malloc_log_ );
  }
  return resultat;
}

void debogue_free(void *pointeur)
{
  if (NULL == pointeur) return;

  ENTREE_FONCTION("free", "pointeur=%p", pointeur);

  if (NULL != Malloc_log_) { /* Log dmarr */
    fprintf( Malloc_log_, "FREE   : on libre la mmoire en %p\n", pointeur);
    (void) fflush( Malloc_log_ );
  }
  free(pointeur);

  SORTIE_FONCTION("free", NULL, 0 );
}

char *debogue_strdup(const char *chaine)
{
  char *resultat;

  if (NULL != Malloc_log_) { /* Log dmarr */
    fprintf( Malloc_log_, "STRDUP : on copie la chane en %p [%s]... ", chaine, chaine);
  }
  resultat = strdup( chaine );
  if (NULL != Malloc_log_) { /* Log dmarr */
    if (NULL == resultat) {
      fprintf( Malloc_log_, "chec\n");
    } else {
      fprintf( Malloc_log_, "rserve en %p\n", resultat);
    }
    (void) fflush( Malloc_log_ );
  }
  return resultat;
}

void Malloc_Logue(char *masque, ...)
{
  va_list params;

  if (NULL == Malloc_log_) return; /* Log non dmarr */
  va_start(params, masque);
  vfprintf(Malloc_log_, masque, params);
  va_end(params);

  fflush( Malloc_log_ );
}
#endif

/* ~~~~~~~~~~~~~~~~ Logs pour les fonctions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#if defined(DEBOGUE_FONCTIONS_) && defined(_DEVELOPPE_)
void ENTREE_FONCTION(const char *nom_fonction, const char *params, ...)
{
#if DEBOGUE_FONCTIONS_ != -1
  unsigned long i;

  if ( NULL == Fonction_log_ ) return;  /* Log des fonctions pas enclench ! */

  /* On incrmente */
  for ( i = 0 ; i < niveau_fonctions ; i++ ) {
    (void) fputc(' ', Fonction_log_);
  }

  /* Nom de la fonction */
  (void) fprintf( Fonction_log_, "%s(", nom_fonction );
  if ( params != NULL ) { /* Paramtres */
    va_list parametres;

    va_start( parametres, params );
    (void) vfprintf( Fonction_log_, params, parametres );
    va_end( parametres );
  } else {
    (void) fprintf( Fonction_log_, "void" );
  }
  fputc(')', Fonction_log_);
  fputc('\n', Fonction_log_);

  /* On augmente d'un cran le niveau d'appel */
  niveau_fonctions++;

  /* On s'assure que tout est bien dans le fichier */
  fflush( Fonction_log_ );
#endif
}

void DANS_FONCTION(const char *texte, ...)
{
#if DEBOGUE_FONCTIONS_ != -1
  unsigned long i;
  va_list parametres;

  if ( NULL == Fonction_log_ ) return;  /* Log des fonctions pas enclench ! */

  /* On incrmente */
  for ( i = 0 ; i < niveau_fonctions ; i++ ) {
    (void) fputc(' ', Fonction_log_);
  }

  /* En-tete */
  fputc('*', Fonction_log_);
  fputc(' ', Fonction_log_);

  /* Texte souhait */
  va_start( parametres, texte );
  (void) vfprintf( Fonction_log_, texte, parametres );
  va_end( parametres );
  fputc('\n', Fonction_log_);

  /* On s'assure que tout est bien dans le fichier */
  fflush( Fonction_log_ );
#endif
}

void SORTIE_FONCTION(const char *nom_fonction, const char *renvoi, ...)
{
#if DEBOGUE_FONCTIONS_ != -1
  unsigned long i;

  if ( NULL == Fonction_log_ ) return;  /* Log des fonctions pas enclench ! */

  /* On descend d'un cran */
  if ( 0 == niveau_fonctions ) {
    (void) fprintf(stderr, "SORTIE_FONCTION : ERREUR, plus de sorties de fonctions que d'entres !\n");
  } else {
    niveau_fonctions--;
  }

  /* On incrmente */
  for ( i = 0 ; i < niveau_fonctions ; i++ ) {
    (void) fputc(' ', Fonction_log_);
  }

  /* On sort le nom */
  (void) fprintf( Fonction_log_, "Sortie de %s", nom_fonction );
  if ( renvoi != NULL ) {
    va_list parametres;

    (void) fputc( ' ', Fonction_log_ );
    (void) fputc( '-', Fonction_log_ );
    (void) fputc( ' ', Fonction_log_ );

    va_start( parametres, renvoi );
    (void) vfprintf( Fonction_log_, renvoi, parametres );
    va_end( parametres );
  }

  /* On s'assure que tout est bien dans le fichier */
  fputc('\n', Fonction_log_);
  fflush( Fonction_log_ );
#endif
}
#endif
