/* Routines utilitaires pour LASE et autres
   (C) 2002, Emmanuel CURIS

   Gestion d'un historique : sauvegarde dans un fichier du dtail des oprations
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>

#include "historique.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static FILE *Histoire = NULL;
static unsigned int HIST_niveau = 0;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Dmarrage de l'historique ----------------\
 |  Renvoie VRAI si l'historique a pu tre ouvert     |
 |                                                    |
 | nom = le nom du fichier qui contient l'historique  |
 \--------------------------------------------------- */

int HIST_LancerHistorique(const char *nom, unsigned int niveau)
{
  time_t heure;
  struct tm *date;
  char txt_date[128]; /* Il y a peu de chances que la date soit plus longue... */
  const char *utilisateur = "Non dterminable";

  if (NULL == nom) nom = "Historique.txt";

  if ( Histoire != NULL ) { /* Historique dj ouvert */
    (void) fprintf(stderr, "Historique dj ouvert... Il faut le fermer d'abord\n");
    return FAUX;
  }

  Histoire = fopen(nom, "a");
  if ( NULL == Histoire ) {
    return FAUX;
  }

  /* Introduction : sance de travail, date,... */
  (void) fprintf(Histoire, "-H-> dbut de l'historique <--\n");
  time( &heure );
  date = localtime( &heure ); /* Structure interne lib C, pas besoin de la librer */
  strftime(txt_date, 127, "%A %d %B %Y", date);
  (void) fprintf(Histoire, " ************ Sance de travail du %s ************\n", txt_date);

#ifdef _UNIX_
  {
    struct passwd *infos;

    infos = getpwuid( getuid() ); /* Infos sur l'utilisateur du logiciel */
    if ( NULL == infos ) {
      utilisateur = "Utilisateur inconnu";
    } else {
      utilisateur = infos->pw_gecos; /* Nom rel d'utilisateur */
    }
  }
#endif

  (void) fprintf(Histoire, " Utilisateur : %s\n\n", utilisateur);
  (void) fflush(Histoire);

  HIST_niveau = niveau; /* Niveau au dmarrage */
  return VRAI;
}

/* -------- Couper l'historique -------- */

void HIST_ArreterHistorique(void)
{
  if (NULL == Histoire) return; /* Dj arrt... */

  (void) fprintf(Histoire, "-H-> fin de l'historique <--\n");
  (void) fclose(Histoire);
  Histoire = NULL;
}

/* -------- Changer le niveau de prolixit de l'historique -------- */

int HIST_ChangerNiveau(unsigned int nv_niveau)
{
  if ( NULL == Histoire ) return HIST_niveau;

  if ( nv_niveau > HIST_niveau )
    (void) fprintf(Histoire, "\n -H-> Verbosite augmente, maintenant %d <--\n\n", nv_niveau);
  else if ( nv_niveau < HIST_niveau )
    (void) fprintf(Histoire, "\n -H-> Verbosite diminue, maintenant %d <--\n\n", nv_niveau);
  (void) fflush(Histoire);

  HIST_niveau = nv_niveau;
  return nv_niveau;
}

/* -~-~-~-~-~-~-~ criture de textes dans l'historique -~-~-~-~-~-~-~ */

void HIST_Erreur(unsigned int niveau, const char *action, const char *raison)
{
  if ( NULL == Histoire ) return; /* Historique inactif */
  if ( niveau > HIST_niveau ) return; /* Historique pas assez dtaill */

  (void) fprintf(Histoire, "ERREUR : %s ==> %s\n", raison, action);
  (void) fflush( Histoire );
}

void HIST_Titre(int niveau, const char *titre, const char *code)
{
  if ( NULL == Histoire ) return; /* Historique inactif */
  if ( ( (unsigned) abs(niveau) ) > HIST_niveau ) return; /* Historique pas assez dtaill */

  if (niveau == 0) { /* Titre fondamental */
    (void) fprintf(Histoire, " *** %s ******** %s ***********\n", code, titre);
  } else if (niveau == 1) { /* Titre important : on le distingue bien */
    (void) fprintf(Histoire, "\n==> %s --- %s <==\n", code, titre);
  } else if (niveau == 2) {
    (void) fprintf(Histoire, "\n -- %s --- %s --\n", code, titre);
  } else if (niveau == -1) { /* Message d'erreur */
    (void) fprintf(Histoire, "\n ==> ERREUR : %s ! <== %s ==\n", titre, code);
  } else if (niveau < 0) { /* Pseudo titre pour oprations isoles */
    (void) fprintf(Histoire, "\n[-- %s -- %s --]\n", code, titre);
  } else {
    unsigned int i;

    fputc('\n', Histoire);
    for (i = 3 ; i < (unsigned) abs(niveau) ; i++) fputc('.', Histoire); /* Un dcalage */
    (void) fprintf(Histoire, " %s / %s\n", code, titre);
  }

  (void) fflush( Histoire );
}

void HIST_Texte(unsigned int niveau, int decalage, const char *masque, ...)
{
  unsigned int i;
  va_list params;

  if ( NULL == Histoire ) return; /* Historique inactif */
  if ( niveau > HIST_niveau ) return; /* Historique pas assez dtaill */

  if ( VRAI == decalage )
    for (i = 1 ; i < niveau ; i++) fputc(' ', Histoire); /* Un dcalage */

  va_start(params, masque);
  (void) vfprintf(Histoire, masque, params);
  va_end(params);

  (void) fflush(Histoire);
}

/* ------ criture des diffrentes valeurs d'une table de nombres ------ */

void HIST_Table(unsigned int niveau, int decalage, double *table, unsigned long nbr_valeurs, int premier_code)
{
  unsigned long i;

  if ( NULL == Histoire ) return; /* Historique inactif */
  if ( niveau > HIST_niveau ) return; /* Historique pas assez dtaill */
  if ( NULL == table ) return;
  if ( 0 == nbr_valeurs) return;

  for (i = 0 ; i < nbr_valeurs ; i++) {
    if ( VRAI == decalage )
      for (i = 1 ; i < niveau ; i++) fputc(' ', Histoire); /* Un dcalage */

    if (premier_code > ' ') {
      (void) fprintf(Histoire, "%c = %.20g\n", premier_code, table[i]);
      premier_code++;
    } else {
      (void) fprintf(Histoire, "Valeur %lu : %.20g\n", i, table[i]);
    }
  }

  (void) fflush(Histoire);
}

/* -------- Polynme avec ses coefficients -------- */

void HIST_Polynome(unsigned int niveau_formule, unsigned int niveau_coefficients, int decalage,
		   double *table, unsigned long degre, int premier_code)
{
  unsigned long i;
  char coefficient;

  if ( NULL == Histoire ) return; /* Historique inactif */
  if ( niveau_formule > HIST_niveau ) return; /* Historique pas assez dtaill */

  if (premier_code <= ' ') premier_code = 'a';

  fputc(premier_code, Histoire); /* a */
  coefficient = premier_code + 1;
  if (degre > 0)
    (void) fprintf(Histoire, " + %c X", coefficient++);
  if (degre > 1)
    (void) fprintf(Histoire, " + %c X", coefficient++);
  if (degre > 2)
    (void) fprintf(Histoire, " + %c X", coefficient++);
  for (i = 4 ; i <= degre ; i++)
    (void) fprintf(Histoire, " + %c X^%lu", coefficient++, i);
  fputc('\n', Histoire);

  HIST_Table(niveau_coefficients, decalage, table, degre + 1, premier_code);
}
