/* Calcul de la fonction d'autocorrlation d'un signal alatoire
   (C) Emmanuel CURIS, 2000

   Prend en entre un fichier 2 colonnes (i, f(i))
   Donne en sortie un fichier
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "fichiers.h"

/* -------- Variables locales -------- */

unsigned int i;
unsigned long nbr_pts = 0;

FILE *sortie = NULL;

double *les_y = NULL;

/* -------- Fonctions locales -------- */

double calc_correl(Graphe *graphe, unsigned long delta);
void   centrer    (Graphe *graphe);

/* -------- Ralisation pratique ----- */

int main(int n_args, char *args[])
{
  Graphe *autocorrel = NULL;
  char *nom_i = NULL, *nom_o = NULL;
  register double *x, *y;
  unsigned int nbr_colonnes = 2, colonne_y = 2, colonne_x = 1;
  double variance;

  if (n_args > 1) /* Il y a des paramtres */
  {
    for (i = 1 ; i < n_args ; i++)
    {
      if ( *args[i] == '-' )
      {
	switch ( args[i][1] )
	{
	case 'i': /* Fichier en entre */
	  if (i+1 < n_args)
	    nom_i = args[i + 1];
	  else
	    fprintf(stderr, "-i doit tre suivi d'un nom de fichier\n");
	  break;

	case 'N': /* Nombre _total_ de colonnes */
	  if (i+1 < n_args)
	    nbr_colonnes = (unsigned int) atoi(args[i + 1]);
	  else
	    fprintf(stderr, "-N doit tre suivi du nombre total de colonnes !\n");
	  break;

	case 'o': /* Fichier en sortie */
	  if (i+1 < n_args)
	    nom_o = args[i + 1];
	  else
	    fprintf(stderr, "-o doit tre suivi d'un nom de fichier\n");
	  break;

	case 'x': /* Colonne des X, valeurs sans intrt en fait... */
	  if (i+1 < n_args)
	  {
	    colonne_x = (unsigned int) atoi(args[i + 1]);
	    fprintf(stderr, "Les X se trouvent dans la colonne %u\n", colonne_x);
	  }
	  else
	    fprintf(stderr, "-x doit tre suivi du numro de la colonne des X !\n");
	  break;

	case 'y': /* Colonne des Y, valeurs intressantes */
	  if (i+1 < n_args)
	  {
	    colonne_y = (unsigned int) atoi(args[i + 1]);
	    fprintf(stderr, "Les Y se trouvent dans la colonne %u\n", colonne_y);
	  }
	  else
	    fprintf(stderr, "-y doit tre suivi du numro de la colonne des Y !\n");
	  break;
	}
      }
    }
  }

  if (nom_i == NULL)
  {
    fprintf(stderr, "Pas de fichier indiqu en entre !\n");
    return -1;
  }

  if (nom_o != NULL)
  {
    sortie = fopen(nom_o, "w");
    if (sortie == NULL)
    {
      fprintf(stderr, "Impossible d'ouvrir %s en criture\n", nom_o);
      return -1;
    }
  }
  else
    sortie = stdout;

  if ( import_ascii(nom_i, nbr_colonnes, colonne_x, colonne_y, 0, '#') == FAUX )
  {
    fprintf(stderr, "Impossible d'ouvrir %s en lecture\n", nom_i);
    return -1;
  }

  nbr_pts = graphe_courant->nombre_point; /* Il n'y a normalement qu'un graphe, donc le courant... */
  if (nbr_pts < 10)
  {
    fprintf(stderr, "Trop peu de points... 10 minimum\n");
    if (nom_o != NULL)
      (void) fclose(sortie);
    GRF_efface(graphe_courant, VRAI, VRAI);
    return -5;
  }
  else
    printf("Graphe avec %ld points : on peut calculer %ld coefficients\n", nbr_pts, nbr_pts >> 2);

  centrer(graphe_courant);

  /* Calcul proprement dit de la fonction d'autocorrlation */

  autocorrel = GRF_creer(nbr_pts >> 2, GR_SSTYPE, GR_NORMAL,
			 "Fonction d'autocorrlation", "t", "f(t)", SS_UNITE, SS_UNITE);
  if (autocorrel == NULL)
  {
    if (nom_o != NULL)
      (void) fclose(sortie);
    GRF_efface(graphe_courant, VRAI, VRAI);
    return -10;
  }

  for (i = 0, x = autocorrel->x, y = autocorrel->y ; i < autocorrel->nombre_point ; i++, x++, y++)
  {
    *x = (double) i;
    *y = calc_correl(graphe_courant, i);
  }

  /* On divise par la valeur en zro pour avoir la corrlation, et non la covariance */

  if ( *(autocorrel->y) == 0.0 )
  {
    fprintf(stderr, "GROS PROBLME : on trouve une variance nulle !\n");
    GRF_efface(graphe_courant, VRAI, VRAI);
    GRF_efface(autocorrel, VRAI, VRAI);
    if (nom_o != NULL)
      (void) fclose(sortie);

    return -15;
  }

  variance = *(autocorrel->y);
  for (i = 0, y = autocorrel->y ; i < autocorrel->nombre_point ; i++, y++)
  {
    *y /= variance;
  }

  GRF_bornes(autocorrel, DESORDONNE, DESORDONNE);

  /* On sauve le rsultat, en ASCII 2 colonnes */

  for (i = 0, x = autocorrel->x, y = autocorrel->y ; i < autocorrel->nombre_point ; i++, x++, y++)
  {
    fprintf(sortie, "%f %g\n", *x, *y);
  }

  /* On libre tout */

  GRF_efface(graphe_courant, VRAI, VRAI);
  GRF_efface(autocorrel, VRAI, VRAI);
  if (nom_o != NULL)
    (void) fclose(sortie);

  return 0; /* Tout s'est bien pass */
}

/* Fonction de corrlation : dfinie par [pour un processus CENTR]

           / + infini
          |
   g(t) = |           f(x) f(x + t) dx
          |
         / - infini

	 Dans le cas d'une fonction  temps discret, a de sens seulement si t = k(dx) et on a

          N - k - 1
   g(k) =   Somme   f_i f_(i + k)
            i = 0
*/

double calc_correl(Graphe *graphe, unsigned long delta)
{
  double somme = 0.0;
  unsigned long i;
  register double *y;

  for (i = 0, y = graphe->y ; i < graphe->nombre_point - delta ; i++, y++)
  {
    somme += *y * *(y + delta);
  }
  somme /= graphe->nombre_point;

  if (delta == 0) /* On calcule en fait la variance */
    printf("Variance trouve : %.20f soit un cart-type de %.20f\n", somme, sqrt(somme));
  else if (delta == 1) /* On calcule la covariance avec dcalage d'un cran */
    printf("Covariance : %.20f\n", somme);

  return somme;
}

void centrer(Graphe *graphe)
{
  double moyenne = 0.0;
  unsigned long i;
  register double *y;

  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
  {
    moyenne += *y;
  }
  moyenne /= (double) (graphe->nombre_point);
  fprintf(stderr, "Moyenne trouve : %f\n", moyenne);

  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
    *y -= moyenne;
}
