/* Calcul de diffrentes moyennes sur un chantillon
   (C) Emmanuel Curis, fvrier 2000
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "fichiers.h"

static Graphe *graphe = NULL;
extern Graphe *graphe_courant = NULL, *premier_graphe = NULL, *dernier_graphe = NULL;
unsigned long nbr_graphes, num_graphe;
char vrai_message[250];

static double moy_arithmetique(Graphe *graphe);
static double moy_bevington   (Graphe *graphe);
static double moy_variance    (Graphe *graphe);

int main(int n_args, char *args[])
{
  double m_a, m_b, m_v;

  if (n_args < 2)
  {
    fprintf(stderr, "moy_diverses : indiquez un fichier !\n");
    return -1;
  }

  graphe = ch_interne(args[1]);
  if (graphe == NULL)
  {
    fprintf(stderr, "moy_diverses : le fichier %s n'a pas pu tre charg !\n", args[1]);
    return -1;
  }

  if ( Graphe_Complexe(graphe->drapeaux.genre) )
  {
    fprintf(stderr, "moy_diverses : le graphe %s est  valeurs complexes !\n", graphe->nom);
    GRF_efface(graphe, VRAI, VRAI);
    return -1;
  }

  m_a = moy_arithmetique(graphe);
  m_b = moy_bevington(graphe);
  m_v = moy_variance(graphe);

  printf("Moyennes : classique = %g \t Bevington = %g \t variance = %g\n", m_a, m_b, m_v);

  GRF_efface(graphe, VRAI, VRAI);
  return 0;
}

/* -------- Calcul de la moyenne arithmtique, classique -------- */

double moy_arithmetique(Graphe *graphe)
{
  unsigned long i;
  double moyenne = 0.0;
  register double *y;

  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
  {
    moyenne += *y;
  }

  return moyenne / (double) (graphe->nombre_point);
}

/* -------- Calcul de la moyenne de Bevington, soit sqrt(1/<(1/y)>) -------- */

double moy_bevington(Graphe *graphe)
{
  unsigned long i;
  double moyenne = 0.0;
  register double *y;

  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
  {
    if (*y != 0.0)
    {
      moyenne += 1 / (*y * *y);
    }
  }
  moyenne /= (double) (graphe->nombre_point);

  return sqrt (1 / moyenne);
}

/* -------- Calcul de la moyenne de variances, soit sqrt(<y>) -------- */

double moy_variance(Graphe *graphe)
{
  unsigned long i;
  double moyenne = 0.0;
  register double *y;

  for (i = 0, y = graphe->y ; i < graphe->nombre_point ; i++, y++)
  {
    moyenne += *y * *y;
  }

  return sqrt (moyenne / (double) (graphe->nombre_point));
}
