/* LASE - Logiciel d'Analyse des Spectres EXAFS
   Dclaration de toutes les variables globales, regroupes ici
   (c) Emmanuel CURIS 1996 - 2003

	tat : essayer de comprendre pourquoi init de RCT_courant marche pas !

   26 & 27 novembre, 9 dcembre 1997
    * Modifi initialisation de RCT_courant : plus propre...
    * Modifi paramtres par dfaut pour le graphe courant : grille pointille, grise, visible
*/

#define VARIABLES_GENERALES_ /* Empche la redfinition avec le extern */

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "serie.h" /* Inclue graphes */
#include "cristaux.h"
#include "echantillon.h" /* Pour la liste des compositions */
#include "experience.h" /* Pour l'exprience modlise */
#include "images.h"
#include "tracer.h"
#include "couches.h"
#include "statistiques.h" /* Pour le nombre de lois disponibles */

#include "variables.txt"

int points[256]; /* Pour tracer les graphes */

/* ~~~~~~~~~~~ Confort de l'utilisateur, donc choses diverses ~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Interface -------- */

Fenetre *fenetre_onglets = NULL; /* Pour la fentre avec les onglets : liste des graphes, modles,... */

/* Les chemins prdfinis */
char *chemin_calculs_feff = NULL;

unsigned int nombre_chemins_chargement = 0; char **chemins_chargement = NULL;
unsigned int nombre_chemins_modeles    = 0; char **chemins_modeles    = NULL;
unsigned int nombre_chemins_sauvegarde = 0; char **chemins_sauvegarde = NULL;
unsigned int nombre_chemins_experience = 0; char **chemins_experience = NULL;

/* ~~~~~~~~~~~ Fentre principale (affichage du graphe courant,...) ~~~~~~~~~~~~~~~~ */

Fenetre *principale = NULL; /* Structure de fentre de la fentre principale */

const char *extensions[NOMBRE_TYPES_GRAPHES] = /* Extension associe  chaque type de graphe */
{
  "*.brt", "*.exp", "*.der", "*.de2", "*"     , "*"    , "*.exf", "*.tf" , "*.tfi", "*.amp",
  "*.pha", "*.aj" , "*.mu0", "*.cmp", "*.rdf" , "*.re" , "*.im" , "*.abs", "*.arg", "*.i0" ,
  "*.i1" , "*.lpm", "*.fen", "*.sig", "*.s02" , "*.his", "*.dp" , "*.idp", "*.exp", "*.mum",
  "*.prf", "*.nrj", "*.flx", "*.cin", "*.conc", "*.exf"
};

/* Nb : idp = "intgrale de la dp" = fontion de rpartition... */

Graphe *graphe_courant = NULL; /* Graphe actuellement utilis (dans la fentre principale) */

Serie_Graphes *premiere_serie = NULL, *serie_courante = NULL;

/* ~~~~~~~~~~~~~~~~ Superposition de plusieurs graphes ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

Fenetre *trace_fenetre = NULL; /* Fentre des tracs multiples */

/* ~~~~~~~~~~~~~~~~ Gestion de << macro-commandes >> ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean TT_xanes = FAUX; /* VRAI -> on veut faire toutes les tapes du   XANES  la suite */
boolean TT_exafs = FAUX; /* VRAI -> on veut faire toutes les tapes de l'EXAFS  la suite */

char vrai_message[200]; /* Le message d'alerte complet, une fois les parties variables modifies
			   1 code d'icne            =   1 + 2
			   5 lignes de 30 caractres = 150 + 6
			   3 boutons de 8 caractres =  24 + 4
			   -----------------------------------
                           total                       175 + 12 = 187 caractres -> 200 par scurit
			*/

/* ~~~~~~~~ Donnes sur les cristaux de monochromateurs ~~~~~~~~ */

/* Noms dans nom_cristaux, dans variables.uk/fr/... */

const Cristal_Mono cristaux[] = 
{
  { 3.1355, 160.103007, 60.0 }, /* Silicium  : 111 - 220 - 311 - 331 - 422 */
  { 1.92, 160.103007, 0.0 },
  { 1.637206601, 160.103007, 0.0 },
  { 1.25, 160.103007, 41.0 },
  { 1.1085, 160.103007, 0.0 },
  { 3.266, 181.321496, 0.0 }, /* Germanium : 111 - 220 - 400 - 422 */
  { 2.0, 181.321496, 154 },
  { 1.41, 181.321496, 0.0 },
  { 1.155, 181.321496, 0.0 },
  { 4.26, 0.0, 0.0 }, /* Quartz */
  { 3.3435, 0.0, 0.0 },
  { 2.282, 0.0, 0.0 },
  { 1.818, 0.0, 0.0 },
  { 1.541, 0.0, 0.0 },
  { 1.3745, 0.0, 0.0 },
  { 0.812, 0.0, 0.0 },
  { 2.0135, 0.0, 0.0 }, /* Fluorure de lithium */
  { 1.424, 0.0, 0.0 },
  { 0.9, 0.0, 0.0 },
  { 0.826, 0.0, 0.0},
  { 4.187, 0.0, 0.0}, /* Topaze */
  { 1.3975, 0.0, 0.0 },
  { 2.32, 0.0, 0.0 },
  { 1.356, 0.0, 0.0 },
  { 9.92, 0.0, 0.0 }, /* Muscovite (mica) */
  { 4.51, 0.0, 0.0 },
  { 2.555, 0.0, 0.0 },
  { 2.58, 0.0, 0.0 },
  { 1.5, 0.0, 0.0 },
  { 3.15233247 , 162.771336, 0.0 }, /* Fluorine : 111 - 220 - 222 */
  { 1.930401513, 162.771336, 0.0 },
  { 1.576166235, 162.771336, 0.0 },
  { 2.338, 0.0, 0.0 }, /* Aluminium */
  { 2.024, 0.0, 0.0 },
  { 2.495, 0.0, 0.0 },  /* Gypse */
  { 7.595, 0.0, 0.0 },
  { 2.8205, 171.879616, 0.0 }, /* Sel gemme */
  { 3.0355, 0.0, 0.0 }, /* Calcite */
  { 0.0, 0.0, 0.0 },
  { 5.32, 0.0, 0.0 }, /* ADP = dihydrognophosphate d'ammonium */
  { 3.07, 0.0, 0.0 },
  { 3.75, 0.0, 0.0 },
  { 3.354, 35.09969786, 0.0 }, /* Graphite */
  { 4.371, 0.0, 0.0 }, /* PET = pentarythrol */
  { 4.404, 0.0, 0.0 }, /* EDDT = thylnediaminedextrotartrate */
  { 6.99, 0.0, 0.0 },  /* SHA = hexaactate de sorbitol */
  { 2.087, 46.656, 0.0 }, /* Cuivre */
  { 3.7405, 0.0, 0.0 }, /* InSb */
  { 12.8815, 0.0, 0.0 }, /* TAP = paracarboxybenzoate de thallium [Ph(COOH)(COO)Tl] (para ?) */
};

/* ~~~~~~~~ Variables pour les modles statistiques ~~~~~~~~ */

const char *txt_loi_probas_continues[DP_NOMBRE_LOIS] = 
{
  "Gaussienne (1D) (normale)",
  "Uniforme",
  "Cauchy (Lorentz)",
  "Exponentielle",
  "Exponentielle sym"e_AIGU"trique",
  "Vecteur gaussien (multinormale)",
  "Beta, type I",
  "Beta, type II",
  "Fisher-Snedecor (F)",
  "Gamma",
  "Khi-deux",
  "Gumbel",
  "Student (T)",
  "Pearson",
  "log-normale",
  "Weibull"
};

/* ~~~~~~~~ Variables ncessaires pour l'ajustement par un modle en couches ~~~~~~~~ */

/* ------ Variables = paramtres gnraux de l'ajustement ------ */

Couche *couche_editee = NULL;     /* Pas statique pour des questions d'interface graphique... */

/* ---- Affichage d'images pour les cartos en fluo X (par exemple) ---- */

Fenetre      *fen_images = NULL; /* Fentre de l'affichage */
Serie_Images *IMG_premiere_serie = NULL; /* Dbut de la liste chane des sries d'images */
Serie_Images *IMG_serie_courante = NULL; /* Srie d'images actuellement affiche */
Image        *image_courante = NULL; /* Image actuellement affiche */

/* ~~~~~~~~ Modlisations thoriques ~~~~~~~~ */

struct composant *premier_composant = NULL;
struct matiere   *premiere_composition = NULL;
struct faisceau  *premier_faisceau = NULL;
struct experience *experience_courante = NULL;
