/* Bibliothèque X-Window pour les interfaces
            Routines de gestion du clavier
	    Compatible avec gem_gene.c (version GEM : référence)
   (C) 1998, Emmanuel CURIS - Création le 29 mai 1998
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h> /* Pour toupper */

#define _LIBRAIRIE_X_ /* Pour accéder aux fonctions internes aussi */

#include <X11/DECkeysym.h> /* Pour les touches mortes DEC */

#include "bib_interface.h"
#include "deboguage.h"

#include "txt_utils.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LONGUEUR_TAMPON_LOOKUPSTRING 10 /* Taille du tampon pour XLookupString, Xutf8LookupString... */

#define TM_ACCENT_AUCUN          0
#define TM_ACCENT_CIRCONFLEXE    1
#define TM_ACCENT_TREMA          2
#define TM_ACCENT_AIGU           3
#define TM_ACCENT_GRAVE          4
#define TM_ACCENT_ROND           5 /* Le rond sur le a de ansgtrom, par exemple */
#define TM_ACCENT_TILDE          6
#define TM_ACCENT_CEDILLE        7
#define TM_ACCENT_TREMA_LONG     8 /* hongrois uniquement... */
#define TM_ACCENT_POINT_DESSOUS  9
#define TM_ACCENT_BREVE         10
#define TM_ACCENT_MACRON        11
#define TM_ACCENT_POINT_DESSUS  12
#define TM_ACCENT_CARON         13

static short int mode_accent = TM_ACCENT_AUCUN;

static boolean mode_insertion = FAUX; /* VRAI = on insère le caractère, FAUX = on le remplace
					 dans les champs éditables uniquement */

/* ~~~~~~~~ Fonctions internes ~~~~~~~~ */

static int CLAV_Arbre        (Arbre *arbre, KeySym touche, const char *chaine);
static int CLAV_ObjetEditable(Arbre *arbre, KeySym touche, const char *chaine);

static int BI_ClavierAscenseurFenetre( Fenetre *fenetre, KeySym touche, int BI_etat );
static int BI_ClavierGeneriqueFenetre( Fenetre *fenetre, KeySym touche, int BI_etat );
static int BI_ClavierGeneriqueArbre  ( Arbre   *arbre  , KeySym touche, int BI_etat );
static int BI_ClavierTexteEditable   ( Arbre   *arbre  , const char *chaine );
static int BI_ClavierTexteInfini     ( Arbre   *arbre  , const char *chaine );
static int BI_ClavierTexteNombre     ( Arbre   *arbre  , const char *chaine );

static boolean BI_ValiderCaractere( int code, int filtre );

static int CLAV_TraiterToucheMorte (KeySym touche, char *chaine, unsigned long lgmax_chaine);
static int CLAV_VerifierToucheMorte(KeySym touche);

static char *OBJ_TrouverCurseur(const char *masque, unsigned int position);

/* ********* Traitement automatique d'un événement clavier ******** */

int EVNT_clavier(Fenetre *fenetre, XKeyEvent *clavier, boolean inclure_menu)
{
  KeySym touche = NoSymbol;
  char chaine[ LONGUEUR_TAMPON_LOOKUPSTRING + 1 ]; /* +1 = place pour un 0 final */
  int retour = FAUX, nbr_octets, i;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p, clavier=%p, inclure_menu=%d", fenetre, clavier, inclure_menu );

  /* On s'assure que la chaîne finisse toujours par 0 */
  for ( i = 0 ; i <= LONGUEUR_TAMPON_LOOKUPSTRING ; i++ )
    chaine[ i ] = 0;

  /* On vérifie l'état des modificateurs */
  BIBX_verifie_modificateurs(clavier->state);

  /* On obtient le symbole de touche X */
  if ( fenetre->xic != NULL ) {
    Status resultat;

#ifdef X_HAVE_UTF8_STRING
    nbr_octets = Xutf8LookupString( fenetre->xic, clavier, chaine, LONGUEUR_TAMPON_LOOKUPSTRING, &touche, &resultat );
#else
    nbr_octets = XmbLookupString  ( fenetre->xic, clavier, chaine, LONGUEUR_TAMPON_LOOKUPSTRING, &touche, &resultat );
#endif
    if ( XBufferOverflow == resultat ) {
      (void) fprintf( stderr, "%s,%s l.%u : Tampon insuffisant, il faudrait %d octets [%d disponibles]\n",
		      __FILE__, __FUNCTION__, __LINE__, nbr_octets, LONGUEUR_TAMPON_LOOKUPSTRING );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      SORTIE_FONCTION( __FUNCTION__, "%d [XBufferOverflow]", BI_EVENEMENT_ERREUR );
      return BI_EVENEMENT_ERREUR;
    }
    if ( XLookupNone == resultat ) {
      /* Pas de résultat exploitable pour l'instant... */
      SORTIE_FONCTION( __FUNCTION__, "%d [XLookupNone]", BI_EVENEMENT_TRAITE );
      return BI_EVENEMENT_TRAITE;
    }
    /* Autres résultats possibles : XLookupChars  -> rien dans touche, chaîne renvoyée
                                    XLookupKeySym -> touche sans caractère, nbr_octets = 0
				    XLookupBoth   -> touche + caractère
     */

#ifdef _DEVELOPPE_
    /* if ( nbr_octets > 0 ) { */
    /*   (void) fprintf( stderr, "%s,%s l.%u : Chaîne décodée : %s [%d caractère(s), 1er = %hhX]\n", */
    /* 		      __FILE__, __FUNCTION__, __LINE__, chaine, nbr_octets, *chaine ); */
    /* } else { */
    /*   (void) fprintf( stderr, "%s,%s l.%u : Code sans caractère [%lu = %s]\n", */
    /*   		      __FILE__, __FUNCTION__, __LINE__, touche, XKeysymToString( touche ) ); */
    /* } */
#endif
  } else {
    /* Pas de méthode d'entrée : on reste en Latin-1... */
    (void) fprintf( stderr, "%s,%s l.%u : Pas de méthode d'entrée pour la fenêtre, entrée en latin-1\n",
		    __FILE__, __FUNCTION__, __LINE__ );
   nbr_octets = XLookupString( clavier, chaine, LONGUEUR_TAMPON_LOOKUPSTRING, &touche, NULL );
    if ( NoSymbol == touche ) {
      AVERTISSEMENT( "Touche appuyée sans KeySym [Keycode = %u]\n", clavier->keycode );
      SORTIE_FONCTION( __FUNCTION__, "%d [NoSymbol]", BI_EVENEMENT_TRAITE );
      return BI_EVENEMENT_TRAITE; /* Touche non reconnue par X-Window : inutilisable ! */
    }
  }

  if ( 0 == nbr_octets ) {
    /* Touches de fonction : pas de caractère associé
        ==> on les traite en priorité, car au besoin peut donner une chaîne de caractères qui sera traitée...
     */

    /* a. Tous les modificateurs d'état, gérés par le système ==> ignorés */
    if ( XK_Control_L        == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Control_R        == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Shift_L          == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Shift_R          == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Alt_L            == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Alt_R            == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Meta_L           == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Meta_R           == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Multi_key        == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Mode_switch      == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Num_Lock         == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Scroll_Lock      == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Caps_Lock        == touche ) return BI_EVENEMENT_TRAITE;
    if ( XK_Super_L          == touche ) return BI_EVENEMENT_TRAITE; /* Mageia 5 : touche « windows » de droite */
    if ( XK_ISO_Level3_Shift == touche ) return BI_EVENEMENT_TRAITE; /* Mageia 5 : touche Alt Gr */

    /* b. Touche « Menu » : ouvrir le 1er menu, comme si on était avec Windows... */ 
    if ( touche == XK_Menu ) {
      if ( VRAI == inclure_menu ) {
	return CLIC_menu( BI_Menu, 0 );
      } else {
	XBell( BI_Display, 100 );
	return BI_EVENEMENT_TRAITE; /* Touche « traitée » */
      }
    }

    /* c. Touches du pavé numérique : prendre l'équivalent sur le clavier principal */
    if ( XK_KP_Home   == touche ) touche = XK_Home;
    if ( XK_KP_Up     == touche ) touche = XK_Up;
    if ( XK_KP_Prior  == touche ) touche = XK_Prior;
    if ( XK_KP_Left   == touche ) touche = XK_Left;
    if ( XK_KP_Begin  == touche ) touche = XK_Begin;
    if ( XK_KP_Right  == touche ) touche = XK_Right;
    if ( XK_KP_End    == touche ) touche = XK_End;
    if ( XK_KP_Down   == touche ) touche = XK_Down;
    if ( XK_KP_Next   == touche ) touche = XK_Next;
    if ( XK_KP_Insert == touche ) touche = XK_Insert;
    if ( XK_KP_Delete == touche ) touche = XK_Delete;

#ifdef _DEVELOPPE_
    /* Pas de message, car touches fléchées pas encore gérées et sans caractère associé */
    /* AVERTISSEMENT( "Touche sans caractère associé, non gérée par LASE : KeySym = %s", XKeysymToString( touche ) ); */
#endif
  }

  /* S'agit-il d'une touche morte ? */
  retour = CLAV_VerifierToucheMorte( touche );
  if ( retour != FAUX ) { /* Oui */
    SORTIE_FONCTION( __FUNCTION__, "%d [touche morte]", BI_EVENEMENT_TRAITE );
    return BI_EVENEMENT_TRAITE;
  }

  /* S'il y a une touche morte en cours de traitement : on la traite ! */
  if ( mode_accent != TM_ACCENT_AUCUN ) {
    int retour;
    char nv_chaine[ 10 ];

    retour = CLAV_TraiterToucheMorte( touche, nv_chaine, 10 );
    if ( retour >  0 ) {
      int i;

      nbr_octets = retour;
      for ( i = 0 ; i <= retour ; i++ ) { /* On copie aussi le 0 final..; */
	chaine[ i ] = nv_chaine[ i ];
      }
    }
  }

  /* Avec control : on récupère la lettre même */
  if ( clavier->state & ControlMask ) {
    if ( clavier->state & ShiftMask ) { /* avec Control, on ignore le Shift pour détecter la touche */
    }

    if ( ( touche >= ' ' ) && ( touche <= 127 ) ) { /* ASCII pur */
      nbr_octets = 1;
      chaine[ 0 ] = touche;
      chaine[ 1 ] = 0;
    } else if ( ( touche >= XK_KP_0 ) && ( touche <= XK_KP_9 ) ) { /* Pavé numérique simple */
      nbr_octets = 1;
      chaine[ 0 ] = '0' + ( touche - XK_KP_0 );
      chaine[ 1 ] = 0;
    } else if ( ( touche >= XK_nobreakspace ) && ( touche <= XK_ydiaeresis ) ) { /* Latin 1*/
      nbr_octets = 1;
      chaine[ 0 ] = (touche - XK_nobreakspace);
      chaine[ 1 ] = 0;
#ifdef _DEVELOPPE_
    } else if ( ( XK_Left == touche ) || ( XK_Right == touche ) ||
		( XK_Up   == touche ) || ( XK_Down  == touche ) ) { /* Ctrl-flèches */
      /* Touche contient déjà la bonne chose : rien à faire ! */
    } else {
      AVERTISSEMENT("Ctrl-%s, pas de caractère trouvé...", XKeysymToString( touche ) );
#endif
    }
  }

  /* Peut-il s'agir d'une fenêtre ? */
  if ( fenetre != NULL ) {
    Fenetre *racine;

    /* On commence par l'arbre, s'il y en a un... */
    if ( FEN_ContientArbre( fenetre->type ) ) {
      if ( NULL == fenetre->arbre ) {
#ifdef _DEVELOPPE_
	AVERTISSEMENT( "Fenêtre avec arbre [type %d] sans arbre associé...", fenetre->type );
	BI_AfficherFenetre( fenetre );
#endif
	retour = BI_EVENEMENT_NON_TRAITE;
      } else {
	retour = CLAV_Arbre( fenetre->arbre, touche, chaine );
      }
/*       printf("%s : retour CLAV_Arbre() = %d\n", __FUNCTION__, retour); */
      if ( retour != BI_EVENEMENT_NON_TRAITE ) {/* La touche a été gérée... */
	SORTIE_FONCTION( __FUNCTION__, "%d [traitée dans l'arbre]", retour );
	return retour;
      }
    }

    /* On essaye de récupérer la fenêtre "racine"
       En effet, une sous-fenêtre et on n'a pas forcément les fonctions de gestion ou les bonnes infos !
    */
    racine = FEN_FenetreMere( fenetre );

    /* Ensuite, existe-t-il une fonction spéciale pour la fenêtre ? */
    if ( ( int (*)(Fenetre *, unsigned int, int ) ) NULL != fenetre->clavier ) {
      if ( nbr_octets > 0 ) {
	retour = fenetre->clavier( racine, (unsigned int) chaine[ 0 ], BI_etat );
      } else {
	retour = fenetre->clavier( racine, (unsigned int) touche, BI_etat );
      }
      if ( retour != BI_EVENEMENT_NON_TRAITE ) {
#ifdef _DEVELOPPE_
  	(void) fprintf( stderr, "%s,%s l.%u : Touche traitée dans la fenêtre [%d = %c]\n", 
			__FILE__, __FUNCTION__, __LINE__, (unsigned int) touche, chaine[ 0 ] );
#endif

	SORTIE_FONCTION( __FUNCTION__, "%d [traitée dans la fenêtre]", retour );
	return retour; /* La touche a été gérée... */
      }
    }

    /* Si non : existe-t-il une fonction spéciale pour la racine ? */
    if ( ( int (*)(Fenetre *, unsigned int, int ) ) NULL != racine->clavier ) {
      if ( nbr_octets > 0 ) {
	retour = racine->clavier( racine, (unsigned int) chaine[ 0 ], BI_etat );
      } else {
	retour = racine->clavier( racine, (unsigned int) touche, BI_etat );
      }
      if ( retour != BI_EVENEMENT_NON_TRAITE ) {
#ifdef _DEVELOPPE_
  	(void) fprintf( stderr, "%s,%s l.%u : Touche traitée dans la fenêtre [%d = %c]\n", 
			__FILE__, __FUNCTION__, __LINE__, (unsigned int) touche, chaine[ 0 ] );
#endif

	SORTIE_FONCTION( __FUNCTION__, "%d [traitée dans la fenêtre, via la racine]", retour );
	return retour; /* La touche a été gérée... */
      }
    }

    /* Ensuite, gestion générique pour toutes les fenêtres... */
    retour = BI_ClavierGeneriqueFenetre( fenetre, touche, BI_etat );
    if ( retour != BI_EVENEMENT_NON_TRAITE ) {
#ifdef _DEVELOPPE_
/*       (void) fprintf( stderr, "%s,%s l.%u : Touche traitée génériquement dans la fenêtre [%d = %c]\n",  */
/*  		     __FILE__, __FUNCTION__, __LINE__, (unsigned int) touche, chaine[ 0 ] );  */
#endif
      SORTIE_FONCTION( __FUNCTION__, "%d [traitée génériquement dans la fenêtre]", retour );
      return retour;
    }
  }

  /* Il ne s'agit pas d'une fenêtre, ou alors la touche n'a pas été gérée */
  if ( FAUX == inclure_menu ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [hors-menu]", BI_EVENEMENT_NON_TRAITE );
    return BI_EVENEMENT_NON_TRAITE; /* On ignore le menu... */
  }

  /* On cherche y compris dans les menus... */
  retour = CLAV_menu( BI_Menu, clavier ); /* Raccourci pour le menu ? */
#ifdef _DEVELOPPE_
  if ( BI_EVENEMENT_NON_TRAITE == retour ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche non-traitée. KeySym = %s, %d cars, chaine[ 0 ] = %c (%d), fenetre=%p\n",
		    __FILE__, __FUNCTION__, __LINE__,
		    XKeysymToString( touche ), nbr_octets, chaine[ 0 ], chaine[ 0 ], (void *) fenetre );
  }
#endif
  SORTIE_FONCTION( __FUNCTION__, "%d [menu]", retour );
  return retour;
}

/* -------- Gestion générique du clavier : cas d'une fenêtre --------
 *  Raccourcis clavier par défaut dans une fenêtre

  Espace : fermeture normale
  Escape : fermeture "annuler"
 */

static int BI_ClavierGeneriqueFenetre( Fenetre *fenetre, KeySym touche, int BI_etat )
{
  int retour = BI_EVENEMENT_NON_TRAITE;
  Fenetre *mere;

  /* Touches servant de raccourcis pour les ascenseurs */
  if ( fenetre->ascenseur != ( void (*)(struct fenetre *, int, int, int ) ) NULL ) {
    retour = BI_ClavierAscenseurFenetre( fenetre, touche, BI_etat );
    if ( retour != BI_EVENEMENT_NON_TRAITE )
      return retour;
  }

  /* Touches : raccourcis pour la fermeture
     La gestion de fermeture se fait forcément en passant par la mère !
  */
  mere = FEN_FenetreMere( fenetre );
  if ( NULL == mere ) {
    AVERTISSEMENT( "Fenêtre %p [%s] avec mère NULL !", (void *) fenetre, fenetre->titre );
    mere = fenetre;
  }
  if ( mere->attributs & CLOSER ) {
    Message message;

    message.type = WM_CLOSED;
    message.fenetre.application = 0;
    message.fenetre.en_plus = 0;
    message.fenetre.handle = 0; /*quel_handle(fenetre);*/
    message.fenetre.x = 0; message.fenetre.y = 0;
    message.fenetre.larg = 0; message.fenetre.haut = 0; /* Dans le cas où != 0 <=> fermeture forcée... */

    if ( XK_space == touche ) { /* Espace : fermeture « normale » */
      int final;

      if ( mere->message != ( int (*)(Fenetre *, Message *) ) NULL )
	final = (mere->message)( mere, &message );
      else {
	FEN_Detruire( mere );
	final = BI_EVENEMENT_FEN_FERMEE;
      }

      return final;
    }

    if ( XK_Escape == touche ) { /* Esc : fermeture forcée, en annulant */
      int final;

      if ( mere->message != ( int (*)(Fenetre *, Message *) ) NULL ) {
	message.fenetre.haut = -1;
	final = (mere->message)( mere, &message );
      } else {
	FEN_Detruire( mere );
	final = BI_EVENEMENT_FEN_FERMEE;
      }

      return final;
    }
  }

  /* La touche n'a pas pu être traitée ! */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion du clavier pour un ascenseur de fenêtre --------
  Sans Alt : ascenseur vertical   )
  Avec Alt : ascenseur horizontal ) pour Orig., Fin, Page précédente, Page suivante

  Aller au début : Orig. [Clr/Home] ; Ctrl  + flèche [H,G]
  Aller 10 avant : Page précédente  ; Shift + flèche [H,G]
  Aller  1 avant : flèche [H,G]
  Aller  1 après : flèche [B,D]
  Aller 10 après : Page suivante  ; Shift + flèche [B,D]
  Aller à la fin : Fin ; Shift + Orig. ; Ctrl + flèche [B,D]
 */

#define ASC_MOUVEMENT_DEBUT     0 /* Aller au début */
#define ASC_MOUVEMENT_PRECEDENT 1 /* Aller page précédente (-10) */
#define ASC_MOUVEMENT_AVANT     2 /* Aller au précédent    (-1)  */
#define ASC_MOUVEMENT_APRES     3 /* Aller au suivant      (+1)  */
#define ASC_MOUVEMENT_SUIVANT   4 /* Aller page suivante   (+10) */
#define ASC_MOUVEMENT_FIN       5 /* Aller à la fin */

static int BI_ClavierAscenseurFenetre(Fenetre *fenetre, KeySym touche, int BI_etat)
{
  int mouvement = -1, cible = -1; /* -1 : indéterminé */

  /* On détermine la cible ! */
  if ( BI_etat & K_ALT )
    cible = ASC_HORIZONTAL;
  else
    cible = ASC_VERTICAL;

  /* On détermine le mouvement */
  if ( XK_Home == touche ) {
    if ( BI_etat & ( K_LSHIFT | K_RSHIFT ) ) {
      mouvement = ASC_MOUVEMENT_FIN;
    } else if ( 0 == ( BI_etat & ~K_ALT ) ) {
      mouvement = ASC_MOUVEMENT_DEBUT;
    }
  }

  if ( XK_Begin == touche ) {
    if ( 0 == ( BI_etat & ~K_ALT ) ) {
      mouvement = ASC_MOUVEMENT_DEBUT;
    }
  }

  if ( XK_Page_Up == touche ) { /* aussi appelée XK_Prior */
    if ( 0 == ( BI_etat & ~K_ALT ) ) {
      mouvement = ASC_MOUVEMENT_PRECEDENT;
    }
  }

  if ( ( XK_Left == touche ) || ( XK_Up == touche ) ) {
    if ( XK_Left == touche ) {
      cible = ASC_HORIZONTAL;	/* <- : touche nécessairement un ascenseur horizontal ! */
    } else {
      cible = ASC_VERTICAL;	/* ^| : touche nécessairement un ascenseur vertical ! */
    }

    if ( K_CTRL & BI_etat ) {
      mouvement = ASC_MOUVEMENT_DEBUT;
    } else if ( ( K_LSHIFT | K_RSHIFT ) & BI_etat ) {
      mouvement = ASC_MOUVEMENT_PRECEDENT;
    } else {
      mouvement = ASC_MOUVEMENT_AVANT;
    }
  }

  if ( ( XK_Right == touche ) || ( XK_Down == touche ) ) {
    if ( XK_Right == touche ) {
      cible = ASC_HORIZONTAL;	/* -> : touche nécessairement un ascenseur horizontal ! */
    } else {
      cible = ASC_VERTICAL;	/* |v : touche nécessairement un ascenseur vertical ! */
    }

    if ( K_CTRL & BI_etat ) {
      mouvement = ASC_MOUVEMENT_FIN;
    } else if ( ( K_LSHIFT | K_RSHIFT ) & BI_etat ) {
      mouvement = ASC_MOUVEMENT_SUIVANT;
    } else {
      mouvement = ASC_MOUVEMENT_APRES;
    }
  }

  if ( XK_Page_Down == touche ) { /* aussi appelée XK_Next */
    if ( 0 == ( BI_etat & ~K_ALT ) ) {
      mouvement = ASC_MOUVEMENT_SUIVANT;
    }
  }
    
  if ( XK_End == touche ) {
    if ( 0 == ( BI_etat & ~K_ALT ) ) {
      mouvement = ASC_MOUVEMENT_FIN;
    }
  }

  /* On convertit en événement fenêtre, si nécessaire ! */
  if ( -1 == mouvement ) return BI_EVENEMENT_NON_TRAITE; /* Touche non utilisée pour gérer les ascenseurs ! */

  /* Cas d'un ascenseur vertical */
  if ( ASC_VERTICAL == cible ) {
    if ( 0 == ( fenetre->attributs & ASC_VERTICAL ) ) {
      /* Pas d'ascenseur vertical : touches sans fonction
	 Rq. : si l'utilisateur veut en faire quelque chose, il peut la capturer avant dans fenetre->clavier !*/
      return BI_EVENEMENT_TRAITE;
    }

    if ( ( void (*)(Fenetre *, int, int, int) ) NULL == fenetre->ascenseur ) {
      OPTION_A_FAIRE( "Fenêtre avec ascenseur vertical, sans routine de gestion" );
      return BI_EVENEMENT_TRAITE;
    }

    switch ( mouvement )
    {
    case ASC_MOUVEMENT_DEBUT:
      (fenetre->ascenseur)( fenetre, WM_VSLID, 0, 0 );
      break;

    case ASC_MOUVEMENT_PRECEDENT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_UPPAGE, 0 );
      break;

    case ASC_MOUVEMENT_AVANT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_UPLINE, 0 );
      break;

    case ASC_MOUVEMENT_APRES:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_DNLINE, 0 );
      break;

    case ASC_MOUVEMENT_SUIVANT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_DNPAGE, 0 );
      break;

    case ASC_MOUVEMENT_FIN:
      (fenetre->ascenseur)( fenetre, WM_VSLID, 0, 1000 );
      break;
    }

    return BI_EVENEMENT_TRAITE;
  }

  if ( ASC_HORIZONTAL == cible ) {
    if ( 0 == ( fenetre->attributs & ASC_HORIZONTAL ) ) {
      /* Pas d'ascenseur horizontal : touches sans fonction
	 Rq. : si l'utilisateur veut en faire quelque chose, il peut la capturer avant dans fenetre->clavier !*/
      return BI_EVENEMENT_TRAITE;
    }

    if ( ( void (*)(Fenetre *, int, int, int) ) NULL == fenetre->ascenseur ) {
      OPTION_A_FAIRE( "Fenêtre avec ascenseur horizontal, sans routine de gestion" );
      return BI_EVENEMENT_TRAITE;
    }

    switch ( mouvement )
    {
    case ASC_MOUVEMENT_DEBUT:
      (fenetre->ascenseur)( fenetre, WM_HSLID, 0, 0 );
      break;

    case ASC_MOUVEMENT_PRECEDENT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_LFPAGE, 0 );
      break;

    case ASC_MOUVEMENT_AVANT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_LFLINE, 0 );
      break;

    case ASC_MOUVEMENT_APRES:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_RTLINE, 0 );
      break;

    case ASC_MOUVEMENT_SUIVANT:
      (fenetre->ascenseur)( fenetre, WM_ARROWED, WA_RTPAGE, 0 );
      break;

    case ASC_MOUVEMENT_FIN:
      (fenetre->ascenseur)( fenetre, WM_HSLID, 0, 1000 );
      break;
    }

    return BI_EVENEMENT_TRAITE;
  }

  /* Si on est là, a priori il y a un problème, mais bon... */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------------- Gestion du clavier pour un arbre ------------------------ */

static int CLAV_Arbre(Arbre *arbre, KeySym touche, const char *chaine)
{
  int retour;

  if ( NULL == arbre ) {
    (void) fprintf( stderr, "%s,%s l.%u : arbre passé NULL !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  /* 1. S'il existe une fonction spécifique associée, on l'utilise */
  if ( (int (*)(unsigned int, int, int *) ) NULL != arbre->clavier ) {
    int renvoi;

    retour = arbre->clavier( (unsigned int) touche, BI_etat, &renvoi );
    if ( retour != BI_EVENEMENT_NON_TRAITE ) {/* Touche traitée ! */
/*       printf( "%s : retour fonction utilisateur = %d\n", __FUNCTION__, retour ); */
      return retour;
    }
  }

  /* 2. Touches spéciales */
  /* Inser : Bascule insertion/remplacement */
  if ( XK_Insert == touche ) {
    if ( VRAI == mode_insertion )
      mode_insertion = FAUX;
    else
      mode_insertion = VRAI;

    BI_DessinerCurseur( arbre );

    /* Touche traitée, pas de sortie */
    return BI_EVENEMENT_TRAITE;
  }

  /* Enter, Return : sortie avec l'objet par défaut, s'il existe */
  if ( ( XK_Return == touche ) || ( XK_KP_Enter == touche ) ) {
    unsigned int obj;

    for ( obj = ROOT ; obj < arbre->nbr_objets ; obj++ ) {
      if ( OB_FLAGS( arbre->adresse, obj ) & DEFAULT ) {
	if ( OB_STATE( arbre->adresse, obj ) & DISABLED )
	  break; /* Comme il n'y a qu'un seul objet par défaut possible... */

	if ( NULL != arbre->fonction.arbre ) { /* Il y a une fonction de gestion */
	  retour = (arbre->fonction.arbre)( obj, 0, 0 );
/* 	  printf( "%s : retour fonction utilisateur, bouton %d = %d\n", __FUNCTION__, obj, retour ); */
	  if ( BI_EVENEMENT_FERMER_ARBRE == retour ) {
	    /* On a demandé de fermer l'arbre... On précise alors quel objet a servi à cela !*/
	    return (BI_EVENEMENT_CODE_OBJET + obj);
	  } else if ( retour != BI_EVENEMENT_NON_TRAITE ) {
	    /* Autre cas de figure : traité, donc rien d'autre à faire, on renvoie tel quel le code ! */
	    return retour;
	  }
	}

	if ( ( OB_FLAGS( arbre->adresse, obj ) & EXIT      ) ||
	     ( OB_FLAGS( arbre->adresse, obj ) & TOUCHEXIT ) ) {
	  return ( BI_EVENEMENT_CODE_OBJET + obj ); /* On renvoie le numéro d'objet de sortie */
	}
      }
    }
  }
  if ( ( touche == XK_Undo ) || ( touche == XK_Cancel ) ) { /* Undo, Cancel : fermeture de l'arbre en annulant */
    if ( arbre->objet_annuler != ROOT ) {
      if ( NULL != arbre->fonction.arbre ) { /* Il y a une fonction de gestion */
	retour = (arbre->fonction.arbre)( arbre->objet_annuler, 0, 0 );

	if ( BI_EVENEMENT_FERMER_ARBRE == retour ) {
	  /* On a demandé de fermer l'arbre... On précise alors quel objet a servi à cela !*/
	  return (BI_EVENEMENT_CODE_OBJET + arbre->objet_annuler);
	} else if ( retour != BI_EVENEMENT_NON_TRAITE ) {
	  /* Autre cas de figure : traité, donc rien d'autre à faire, on renvoie tel quel le code ! */
	  return retour;
	}
      }

      /* Si on est là : objet existe, mais rien de spécial n'y est associé ! */
      return BI_EVENEMENT_CODE_OBJET + arbre->objet_annuler;
    }

    /* On ferme l'arbre de toute façon ! */
    return BI_EVENEMENT_FERMER_ARBRE;
  }
  if ( ( XK_Help  == touche ) || /* Help : appeler l'aide en ligne */
       ( XK_Print == touche ) || ( XK_Pause == touche ) ) { /* Print, Pause : aussi, car souvent pas de touche Help sur les claviers... */
    (void) ARB_AppelerAide( arbre->URL_aide );
    return BI_EVENEMENT_TRAITE;
  }

  /* 3. Entrée de texte dans un objet éditable --- s'il y en a */
  retour = CLAV_ObjetEditable( arbre, touche, chaine );
  if ( retour != BI_EVENEMENT_NON_TRAITE )
    return retour;

  /* 4. Gestion générique du clavier pour un arbre */
  return BI_ClavierGeneriqueArbre( arbre, touche, BI_etat );
}

static int BI_ClavierGeneriqueArbre(Arbre *arbre, KeySym touche, int BI_etat)
{
#ifdef _DEVELOPPE_
  (void) fprintf( stderr, "%s,%s l.%u : arbre = %p, touche = %s, BI_etat = %d\n",
		  __FILE__, __FUNCTION__, __LINE__,
		  (void *) arbre, XKeysymToString( touche ), BI_etat );
#endif

  if ( XK_Escape == touche ) { /* Esc : fermeture de l'arbre en annulant */
    if ( arbre->objet_annuler != ROOT ) {
      if ( NULL != arbre->fonction.arbre ) { /* Il y a une fonction de gestion */
	int retour;

	retour = (arbre->fonction.arbre)( arbre->objet_annuler, 0, 0);

	if ( BI_EVENEMENT_FERMER_ARBRE == retour ) {
	  /* On a demandé de fermer l'arbre... On précise alors quel objet a servi à cela !*/
	  return BI_EVENEMENT_CODE_OBJET + arbre->objet_annuler;
	} else if ( retour != BI_EVENEMENT_NON_TRAITE ) {
	  /* Autre cas de figure : traité, donc rien d'autre à faire, on renvoie tel quel le code ! */
	  return retour;
	}
      }

      /* Si on est là : objet existe, mais rien de spécial n'y est associé ! */
      return BI_EVENEMENT_CODE_OBJET + arbre->objet_annuler;
    }

    /* Si on est là, on ferme de toute façon */
    return BI_EVENEMENT_FERMER_ARBRE;
  }

  /* Touche non-traitée */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion du clavier, pour un arbre avec des objets ditables -------- */

static int CLAV_ObjetEditable(Arbre *arbre, KeySym touche, const char *chaine)
{
  int type, retour = BI_EVENEMENT_NON_TRAITE;
  unsigned int nv_editable, lg_texte = 0;
  char *texte = NULL;

  /* Vérifications : y a-t-il un objet éditable ? */
  if ( ROOT == arbre->premier_edit ) return BI_EVENEMENT_NON_TRAITE; /* Pas d'éditable => rien à traiter ici ! */
  if ( ROOT == arbre->dernier_edit ) {
    (void) fprintf( stderr, "%s,%s l.%u : 1er objet éditable existe [%u], pas le dernier !\n",
		    __FILE__, __FUNCTION__, __LINE__, arbre->premier_edit );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return BI_EVENEMENT_ERREUR;
  }
  if ( ROOT == arbre->edit_courant ) {
    (void) fprintf( stderr, "%s,%s l.%u : objets éditables existent [premier : %u, dernier : %u], pas le courant !\n",
		    __FILE__, __FUNCTION__, __LINE__, arbre->premier_edit, arbre->dernier_edit );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return BI_EVENEMENT_ERREUR;
  }

  /* 1. Déplacements d'un objet éditable au suivant */
  nv_editable = arbre->edit_courant;

  /* Tabulation... */
  if ( XK_Tab == touche ) {
    if ( BI_etat & ( K_RSHIFT | K_LSHIFT ) )
      touche = XK_Up;
    else if ( 0 == BI_etat )
      touche = XK_Down;
  }

  switch ( touche )
  {
  case XK_Up: /* Flèche vers le haut */
    if ( BI_etat & K_CTRL ) { /* Ctrl-flèche haute : on va au premier éditable */
      nv_editable = arbre->premier_edit;
    } else {
      nv_editable = ARB_EditablePrecedent( arbre - arbres, arbre->edit_courant, VRAI );
      if ( ROOT == nv_editable ) {
	(void) fprintf( stderr, "%s,%s l.%u : erreur dans ARB_EditablePrecedent()\n",
			__FILE__, __FUNCTION__, __LINE__ );
	nv_editable = arbre->edit_courant;
      }
    }
    (void) BI_ChangerEditable( arbre, nv_editable );
    return BI_EVENEMENT_TRAITE;

  case XK_Down: /* Flèche vers le bas */
    if ( BI_etat & K_CTRL ) { /* Ctrl-flèche basse : on va au dernier éditable */
      nv_editable = arbre->dernier_edit;
    } else {
      nv_editable = ARB_EditableSuivant( arbre - arbres, arbre->edit_courant, VRAI );
      if ( ROOT == nv_editable ) {
	(void) fprintf( stderr, "%s,%s l.%u : erreur dans ARB_EditableSuivant()\n",
			__FILE__, __FUNCTION__, __LINE__ );
	nv_editable = arbre->edit_courant;
      }
    }
    (void) BI_ChangerEditable( arbre, nv_editable );
    return BI_EVENEMENT_TRAITE;

/*   case XK_Home: */	/* Clr/Home => aller au premier champ éditable [NON : utilisé ensuite !]*/
/*     nv_editable = arbre->premier_edit; */
/*     (void) BI_ChangerEditable( arbre, nv_editable ); */
/*     return BI_EVENEMENT_TRAITE; */
  }

  /* On "construit" le texte et calcule sa longueur */
  type = arbre->adresse[ arbre->edit_courant ].ob_type;

  if ( G_TXT_INFINI == type ) {
    texte = arbre->adresse[ arbre->edit_courant ].ob_spec.txt_infini->texte;
    if ( NULL == texte ) {
      lg_texte = 0;
    } else {
      lg_texte = strlen( texte );
    }
  } else if ( G_NOMBRE == type ) {
    texte = arbre->adresse[ arbre->edit_courant ].ob_spec.nombre->texte.texte;
    if ( NULL == texte ) {
      lg_texte = 0;
    } else {
      if ( arbre->adresse[ arbre->edit_courant ].ob_spec.nombre->proprietes & BI_EDITABLE_PROBLEME ) {
	lg_texte = 0; /* Affichage d'un message d'erreur ! */
      } else {
	lg_texte = strlen( texte );
      }
    }
  } else if ( ( G_FTEXT == type ) || ( G_FBOXTEXT == type ) ) {
    TEDINFO *tedinfo;

    tedinfo = arbre->adresse[ arbre->edit_courant ].ob_spec.tedinfo;
    texte = tedinfo->te_ptext.editable;
#ifdef _DEVELOPPE_
    for ( lg_texte = 0 ; lg_texte < tedinfo->te_txtlen ; lg_texte++ ) {
      if ( 0 == texte[ lg_texte ] ) /* Chaîne plus courte... */
	break;
    }
    if ( lg_texte == tedinfo->te_txtlen ) {
      AVERTISSEMENT( "Le 0 final à disparu [objet %u, texte %s]\n", arbre->edit_courant, texte );
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
      lg_texte--;
    }
#endif
    /* Normalement, la chaîne contient toujours la place pour un 0 final */
    lg_texte = strlen( texte );
  }

  /* 2. déplacements au sein d'un objet éditable */
  if ( XK_Home == touche ) { /* Home : début / fin de champ */
    if ( 0 == BI_etat )
      touche = XK_Begin;
    else if ( BI_etat & ( K_RSHIFT | K_LSHIFT ) )
      touche = XK_End;
  }

  switch ( touche )
  {
  case XK_Begin: /* Begin : début de champ -> octet 0 de la chaîne, pas d'ambiguité */
    BI_PlacerCurseur( arbre, 0U );
    return BI_EVENEMENT_TRAITE;

  case XK_End: /* End : fin de champ -> 0 final, pas d'ambiguité */
    BI_PlacerCurseur( arbre, lg_texte );
    return BI_EVENEMENT_TRAITE;

  case XK_Left: /* flèche gauche : vers le début */
    if ( arbre->pos_curseur > 0 ) {
      int retour;

      retour = BI_ReculerCurseur( arbre, 1 );
      if ( retour != 0 ) {
	(void) fprintf( stderr, "%s,%s l.%u : Problème en avançant le curseur [code %d]\n",
			__FILE__, __FUNCTION__, __LINE__, retour );
      }
    } else
      XBell( BI_Display, 100 );
    return BI_EVENEMENT_TRAITE;

  case XK_Right: /* flèche droite : avancer d'un _caractère_ vers la droite */
    if ( arbre->pos_curseur < lg_texte ) {
      int retour;

      retour = BI_AvancerCurseur( arbre, 1 );
      if ( retour != 0 ) {
	(void) fprintf( stderr, "%s,%s l.%u : Problème en avançant le curseur [code %d]\n",
			__FILE__, __FUNCTION__, __LINE__, retour );
      }
    } else
      XBell( BI_Display, 100 );
    return BI_EVENEMENT_TRAITE;
  }

  /* 3. Effacements de caractères */
  switch ( touche )
  {
  case XK_BackSpace: /* Backspace : effacer le caractère avant le curseur */
    if ( 0 == arbre->pos_curseur ) {
      XBell( BI_Display, 100 );
      return BI_EVENEMENT_TRAITE;
    } else {
      int retour;

      /* On recule d'un caractère */
      retour = BI_ReculerCurseur( arbre, 1 );
      if ( retour != 0 ) {
	(void) fprintf( stderr, "%s,%s l.%u : Problème en reculant le curseur [code %d]\n",
			__FILE__, __FUNCTION__, __LINE__, retour );
	return BI_EVENEMENT_TRAITE;
      }
    }

    /* La suite est alors comme « delete »... */

  case XK_Delete: /* Suppr (Del) : effacer le caractère sous/après le curseur */
    if ( arbre->pos_curseur >= lg_texte ) { /* On est sur le dernier caractère */
      XBell( BI_Display, 100 );
    } else {
      unsigned int i;
      int lg_caractere;

      /* Combien d'octets occupe le caractère à la position du curseur ? */
      lg_caractere = TXT_LongueurCaractere( texte + arbre->pos_curseur );
      if ( lg_caractere < 0 ) lg_caractere = -lg_caractere;
      if ( 0 == lg_caractere ) {
	/* Rien à faire, et touche traitée */
#ifdef _DEVELOPPE_
	(void) fprintf( stderr, "%s,%s l.%u : Caractère de longueur nulle sous le curseur ?\n",
			__FILE__, __FUNCTION__, __LINE__ );
#endif
	return BI_EVENEMENT_TRAITE;
      }

/*       (void) fprintf(stderr, "%s,%s l.%u : Delete, texte=%s lg=%u pos=%u\n", */
/* 		     __FILE__, __FUNCTION__, __LINE__, texte, lg_texte, arbre->pos_curseur); */

      /* On recopie la fin de la chaîne */
      for ( i = arbre->pos_curseur + lg_caractere ; i < lg_texte ; i++ ) {
	texte[ i - lg_caractere ] = texte[ i ];
      }

      /* On marque la fin de la chaîne */
      texte[ lg_texte - 1 ] = 0;

      /* On redessine */
      OBJ_dessine( arbre->objets[ arbre->edit_courant ], arbre->adresse + arbre->edit_courant );
      BI_PlacerCurseur( arbre, arbre->pos_curseur );
    }
    return BI_EVENEMENT_TRAITE;

  case XK_Escape: /* Echap (Esc) : On vide le champ */
    memset( texte, 0, lg_texte );
    BI_PlacerCurseur( arbre, 0 );
    return BI_EVENEMENT_TRAITE; /* Traité */
  }

  /* 4. Caractère normal : dépend de la nature de l'objet... */
  if ( BI_etat & K_CTRL ) return BI_EVENEMENT_NON_TRAITE; /* Touches avec control : pas des caractères éditables ! */
  if ( NULL == chaine ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : touche sans caractère associé [%lu, %s]\n",
		    __FILE__, __FUNCTION__, __LINE__, touche, XKeysymToString( touche ) );
#endif
    return BI_EVENEMENT_NON_TRAITE;
  }
  if ( ( 0 <= *chaine ) && ( ' ' > *chaine ) ) {
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : code de contrôle [touche %lu = %s ; code %hhX, %s]\n",
		    __FILE__, __FUNCTION__, __LINE__, touche, XKeysymToString( touche ), *chaine, chaine );
#endif
    /* Touche sans caractère associé */
    return BI_EVENEMENT_NON_TRAITE;
  }

  /* On ajoute le texte indiqué */
  switch( type ) {
  case G_TXT_INFINI:
    retour = BI_ClavierTexteInfini( arbre, chaine );
    break;

  case G_NOMBRE:
    retour = BI_ClavierTexteNombre( arbre, chaine );
    break;

  case G_FTEXT:
  case G_FBOXTEXT:
    retour = BI_ClavierTexteEditable( arbre, chaine );
    break;

  default:
    retour = BI_EVENEMENT_NON_TRAITE;
  }

  if ( BI_EVENEMENT_ERREUR == retour ) {
    (void) fprintf( stderr, "%s,%s l.%u : Erreur survenue au cours du traitement de la chaine [%s]\n",
		    __FILE__, __FUNCTION__, __LINE__, chaine );
  }

  /* Terminé sur ce que l'on pouvait faire automatiquement */
  return retour;
}

/* -------- Ajouter un caractère dans un texte éditable -------- */

static int BI_ClavierTexteEditable(Arbre *arbre, const char *chaine)
{
  TEDINFO *tedinfo;
  unsigned int position;
  int code;
  char *texte;
  const char *validation, *c;

  /* On récupère les informations d'édition */
  tedinfo = arbre->adresse[ arbre->edit_courant ].ob_spec.tedinfo;

  /* Quelques contrôles de cohérence */
  if ( NULL == tedinfo ) {
    (void) fprintf( stderr, "%s,%s l.%u : TEDINFO == NULL, ERREUR GRAVE\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }
  texte = tedinfo->te_ptext.editable;
  if ( NULL == texte ) {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de texte !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }
  validation = tedinfo->te_pvalid;
  if ( NULL == validation ) {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de chaîne de validation !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  /* Position du curseur (dans la partie éditable !) */
  position = arbre->pos_curseur;

  /* 1. On valide le code à partir du masque */
  for ( c = chaine ; *c != 0 ; c++ ) {
    code = *c;

    if ( FAUX == BI_ValiderCaractere( code, validation[ position ] ) ) {
      const char *masque, *pos_curseur = NULL, *pos_suivant;

      if ( '_' == code ) { /* Ne peut servir de séparateur, puisque signale les caractères éditables ! */
	XBell( BI_Display, 100 );
	return BI_EVENEMENT_TRAITE;
      }

      /* On cherche si, après la position courante, se trouve dans le masque un caractère identique */
      masque = tedinfo->te_ptmplt;
      if ( NULL == masque ) {
	(void) fprintf( stderr, "%s,%s l.%u : Pas de masque !\n",
			__FILE__, __FUNCTION__, __LINE__ );
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return BI_EVENEMENT_ERREUR;
      }

      /* Où est le curseur dans le masque ? */
      pos_curseur = OBJ_TrouverCurseur( masque, position );
      if ( NULL == pos_curseur ) {
	(void) fprintf( stderr, "%s,%s l.%u : ERREUR dans OBJ_TrouverCurseur() ! Traitement annulé\n",
			__FILE__, __FUNCTION__, __LINE__ );
	return BI_EVENEMENT_ERREUR;
      }

      /* Trouve-t-on le caractère demandé après ce caractère éditable ? */
      pos_suivant = strchr( pos_curseur, code );
      if ( NULL == pos_suivant ) {
	/* Cas spécial : séparateur décimal, aussi bien , que . */
	if ( ',' == code ) {
	  pos_suivant = strchr( pos_curseur, '.' );
	} else if ( '.' == code ) {
	  pos_suivant = strchr( pos_curseur, ',' );
	}
      }
      if ( NULL == pos_suivant ) {
#ifdef _DEVELOPPE_
	(void) fprintf( stderr, "%s,%s l.%u : code invalide, %d = %c\n",
			__FILE__, __FUNCTION__, __LINE__, (int) code, (int) code );
#endif
	XBell( BI_Display, 100 );
	return BI_EVENEMENT_TRAITE;
      }

    /* On a trouvé : mais existe-t-il des éditables après ? */
      if ( NULL == strchr( pos_suivant, '_' ) ) {
	XBell( BI_Display, 100 );
	return BI_EVENEMENT_TRAITE;
      }

    /* Oui : on se place donc dessus */
      do {
	if ( '_' == *pos_curseur ) { /* On complète par des espaces qui REMPLACENT toujours */
	  texte[ position ] = ' ';
	  position++;
	}
	pos_curseur++;
      } while ( pos_curseur < pos_suivant );
    }
    BI_PlacerCurseur( arbre, position );

    /* La touche a été traitée correctement */
    return BI_EVENEMENT_TRAITE;

    /* 2. Puisque le caractère est valide, on l'insère */
    if ( VRAI == mode_insertion ) { /* On déplace la fin de la chaîne */
      unsigned int i;

      for ( i = tedinfo->te_txtlen - 1 ; i > position ; i-- )
	texte[ i ] = texte[ i - 1 ];
    }
    texte[ position ] = code;
  }

  /* 3. On se place sur le caractère suivant --- redessine au passage */
  BI_PlacerCurseur( arbre, position + 1 );

  /* Touche traitée */
  return BI_EVENEMENT_TRAITE;
}

/* ---- Ajout du caractère dans un objet de type G_TXT_INFINI ---- */

static int BI_ClavierTexteInfini(Arbre *arbre, const char *chaine)
{
  int retour;
  Texte_Infini *texte_infini;

  /* Le texte à modifier */
  texte_infini = arbre->adresse[ arbre->edit_courant ].ob_spec.txt_infini;
  if ( NULL == texte_infini ) {
    (void) fprintf( stderr, "%s,%s l.%u : Texte infini NULL [arbre %p]\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) arbre );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  /* On ajoute le caractère dans la structure */
  retour = BI_AjouterCaractere_TexteInfini( texte_infini, arbre->pos_curseur, chaine, mode_insertion );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : Impossible d'ajouter le caractère\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return BI_EVENEMENT_ERREUR;
  }

  /* On place le curseur un cran après */
  BI_AvancerCurseur( arbre, 1 );

  return BI_EVENEMENT_TRAITE; /* Touche traitée */
}

static int BI_ClavierTexteNombre(Arbre *arbre, const char *chaine)
{
  int retour;
  struct infos_editable *infos;
  boolean est_valide;

  infos = arbre->adresse[ arbre->edit_courant ].ob_spec.nombre;
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'informations !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return BI_EVENEMENT_ERREUR;
  }

  /* On contrôle que le caractère est acceptable */
  est_valide = BI_ValiderCaractere_Nombre( infos, arbre->pos_curseur, chaine[ 0 ], mode_insertion );
  if ( FAUX == est_valide ) { /* Caractère non acceptable */
    XBell( BI_Display, 100 );
    return BI_EVENEMENT_TRAITE;
  }

  /* Caractère valide : on insère ou remplace */
  retour = BI_AjouterCaractere_TexteInfini( &(infos->texte), arbre->pos_curseur, chaine, mode_insertion );
  if ( retour != 0 ) {
    (void) fprintf( stderr, "%s, %s l.%u : Impossible d'ajouter le caractère [%hhX, %s]\n",
		    __FILE__, __FUNCTION__, __LINE__, *chaine, chaine );
    return BI_EVENEMENT_ERREUR;
  }

  /* On place le curseur un cran après */
  BI_PlacerCurseur( arbre, arbre->pos_curseur + 1 );

  return BI_EVENEMENT_TRAITE;  /* Touche traitée */
}

/* ~-~-~-~-~-~-~-~- Routines utilitaires ~-~-~-~-~-~-~-~- */

/* -------- Dessin réel du curseur dans l'arbre -------- */

int BI_DessinerCurseur(Arbre *arbre)
{
  XGCValues params;
  unsigned int obj_editable, position;
  int x_curseur = 0, car = 0;

  if ( NULL == arbre ) {
    (void) fprintf( stderr, "%s,%s l.%u : Appel avec un arbre NULL\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );(void) fprintf( stderr, "%s,%s l.%u : \n",
						      __FILE__, __FUNCTION__, __LINE__ );
    return -5;
  }

  obj_editable = arbre->edit_courant;
  if ( ROOT == obj_editable ) { /* Pas d'objet éditable ! */
#ifdef _DEVELOPPE_
    (void) fprintf( stderr, "%s,%s l.%u : appel pour un arbre sans objet éditable ! [%p]\n",
		    __FILE__, __FUNCTION__, __LINE__, (void *) arbre );
#endif
    return -1;
  }
  position = arbre->pos_curseur;

  /* Où est le curseur ? */
  if ( ( G_TXT_INFINI == arbre->adresse[ obj_editable ].ob_type ) ||
       ( G_NOMBRE     == arbre->adresse[ obj_editable ].ob_type ) ) {
    Texte_Infini *texte;

    if ( G_TXT_INFINI == arbre->adresse[ obj_editable ].ob_type ) {
      texte = arbre->adresse[ obj_editable ].ob_spec.txt_infini;
    } else {
      texte = &( arbre->adresse[ obj_editable ].ob_spec.nombre->texte );
    }
    if ( position > texte->prem_car ) {
      if ( NULL != BI_FonteUTF8 ) {
	XRectangle dimensions;
	int retour __attribute__((__unused__));

#ifdef X_HAVE_UTF8_STRING
	retour = Xutf8TextExtents( BI_FonteUTF8,
				   texte->texte + texte->prem_car, position - texte->prem_car,
				   NULL, &dimensions );
#else
	retour = XmbTextExtents( BI_FonteUTF8,
				 texte->texte + texte->prem_car, position - texte->prem_car,
				 NULL, &dimensions );
#endif
	x_curseur = dimensions.width;
      } else {
	/* Fonte non-UTF8 : un octet = un caractère... */
	x_curseur = XTextWidth( BI_Fonte, texte->texte + texte->prem_car, position - texte->prem_car );
      }
    }

    /* On ajoute 2 pixels de marge... */
    x_curseur += 2;

    if ( NULL == texte->texte ) {
      car = '$';
    } else
      car = *(texte->texte + position);
  } else if ( ( G_FTEXT    == arbre->adresse[ obj_editable ].ob_type ) ||
	      ( G_FBOXTEXT == arbre->adresse[ obj_editable ].ob_type ) ) {
    char *texte;
    const char *masque, *pos_curseur;
    unsigned int lg_texte = 0;
    TEDINFO *tedinfo;

    tedinfo = arbre->adresse[ obj_editable ].ob_spec.tedinfo;
    if ( NULL == tedinfo ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas de TEDINFO\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -20;
    }
    masque = tedinfo->te_ptmplt; /* Le masque */

    /* Où est le curseur dans le masque ? */
    pos_curseur = OBJ_TrouverCurseur( masque, arbre->pos_curseur );
    if ( NULL == pos_curseur ) {
      (void) fprintf(stderr, "BI_DessinerCurseur : ERREUR dans OBJ_TrouverCurseur !\n");
      return -2;
    }
    lg_texte = (pos_curseur - masque) + 1;

    /* On compte le nombre réel de caractères affichés */
    texte = faire_chaine( tedinfo ); /* On fait le texte réel */
    if (NULL == texte) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "BI_DessinerCurseur : ERREUR dans faire_chaine()\n");
#endif
      return -2;
    }

    if ( lg_texte > 1 ) /* On n'est pas sur le 1er caractre du masque */
      x_curseur = XTextWidth( BI_Fonte, texte, lg_texte - 1);
    if (tedinfo->te_just == TE_CNTR) { /* Texte centr */
      unsigned int larg_texte;

      larg_texte = XTextWidth(BI_Fonte, texte, (signed) strlen(texte));
      x_curseur += (arbre->adresse[ obj_editable ].ob_width - larg_texte ) >> 1;
    } else {
      x_curseur += 2;
    }

/*     printf("Position = %u, texte = %s / %u cars soit %s\n", position, texte, lg_texte, texte + position ); */

    /* Plus besoin du texte rel */
    free( texte );
  }

  /* Dessin proprement dit */
  params.function = GXinvert; /* Mode inversion vido */
  XChangeGC(BI_Display, BI_GC_objets, GCFunction, &params);

  XSetForeground(BI_Display, BI_GC_objets, BI_Couleurs[ NOIR ].pixel); /* Curseur noir */
  if (mode_insertion == VRAI) { /* Curseur = un trait vertical */
    XDrawLine(BI_Display, arbre->objets[ obj_editable ], BI_GC_objets,
	      x_curseur, 0, x_curseur, arbre->adresse[ obj_editable ].ob_height);
  } else { /* Curseur = pav noir par-dessus le caractre */
    unsigned int larg_curseur;

    if ( BI_Fonte->per_char == NULL ) { /* Pas d'information par caractre */
      AVERTISSEMENT( "car = %d [%c] mal gr\n", car, car);
      larg_curseur = BI_Fonte->max_bounds.width;
    } else if ( car < 0 ) {
      int nv_car;

      nv_car = (int) (unsigned char) (char) car;
      AVERTISSEMENT( "car = %d [%c] devient %d [%c]\n", car, car, nv_car, nv_car );
      larg_curseur = BI_Fonte->per_char[ nv_car ].width;
    } else {
      larg_curseur = BI_Fonte->per_char[ car ].width;
    }

    XFillRectangle(BI_Display, arbre->objets[ obj_editable ], BI_GC_objets,
		   x_curseur, 0, larg_curseur, arbre->adresse[ obj_editable ].ob_height );
  }

  /* On repasse en mode superposition */
  params.function = GXcopy; /* Mode recouvrement */
  XChangeGC(BI_Display, BI_GC_objets, GCFunction, &params);

  /* Tout s'est bien passé */
  return 0;
}

/* -------- Validation d'un caractère entré en fonction du masque tedinfo -------- */

static boolean BI_ValiderCaractere( int code, int filtre )
{
  switch ( filtre )
  {
  case 'a': /* Lettre + espace */
    if ( ' ' == code ) return VRAI;
    if ( isalpha( code ) ) return VRAI;
    break;

  case 'F': /* Nom de fichier + * ? */
    if ( isalpha( code ) ) return VRAI;
    if ( ( '*' == code ) || ( '?' == code ) ) return VRAI;
    break;

  case 'n': /* Chiffre, lettre, espace */
    if ( ' ' == code ) return VRAI;
    if ( isalpha( code ) ) return VRAI;
    if ( isdigit( code ) ) return VRAI;
    break;

  case 'N': /* n, mais majuscules seules */
    if ( ' ' == code ) return VRAI;
    if ( isupper( code ) ) return VRAI;
    if ( isdigit( code ) ) return VRAI;
    break;

  case 'p': /* Chemin */
    break;

  case 'P': /* Chemin + * ? */
    if ( ( '*' == code ) || ( '?' == code ) ) return VRAI;
    break;

  case 'x': /* N'importe quoi, mais en majuscules */
  case 'X': /* N'importe quoi */
    return VRAI;

  case 's': /* Signe, sens strict - PAS DE L'ATARI ORIGINAL */
    if ( ' ' == code ) return VRAI;
    if ( ( code == '+' ) || ( code == '-' ) ) return VRAI;
  case '9': /* Un chiffre, quelconque */
    if ( isdigit( code ) ) return VRAI;
    break;

  default:  /* Masque inconnu -> n'accepte que si identiquement gal */
    if ( filtre == code ) return VRAI;
  }

  return FAUX;
}

/* ******** quelques « macros » générales de gestion du clavier **************** */

/* ***
 *  Renvoie un code de touche au format atari à partir d'un code X-Windows
 *    Ce n'est qu'un pseudo-code : code_clavier = 0 sf si pas de code ascii...
 *
 * si renvoie 0  : la touche a été traitée en interne
 * si renvoie 255 comme code clavier : touche X à fonction interne définie
 * *** */

unsigned int XToucheVersGEM(XKeyEvent *clavier, int mode, KeySym *touche_X)
{
  KeySym touche;
  char tampon[ 128 ];
  unsigned int touche_gem = 0;

  XLookupString(clavier, tampon, 128, &touche, (XComposeStatus *) NULL);

  if ( touche_X != NULL ) /* Utilisateur veut aussi le code X */
    *touche_X = touche;

  if ( (touche == XK_Control_L) || (touche == XK_Control_R) ||
       (touche == XK_Shift_L  ) || (touche == XK_Shift_R  ) ||
       (touche == XK_Alt_L    ) || (touche == XK_Alt_R    ) )
    return TOUCHE_TRAITEE; /* Rien à faire pour le traitement */

  if ( mode != TOUCHE_PRESSEE ) /* On relâche la touche -> on s'en fiche, en fait... */
    return TOUCHE_TRAITEE;

  /* On s'arrange pour les touches mortes, si nécessaire */

  /* Touches de fonction */
  if ( (touche >= XK_F1) && (touche <= XK_F35) ) {
    return touche;
  }

  /* Touches normales pour la suite */
  switch (touche)
  {
  case XK_Up: /* Flèche vers le haut */
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Fleche_Haut;
    break;

  case XK_Down: /* Flèche vers le bas */
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Fleche_Bas;
    break;

  case XK_Right:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Fleche_Droite;
    break;

  case XK_Left:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Fleche_Gauche;
    break;

  case XK_Page_Up:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Page_Avant;
    break;

  case XK_Page_Down:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Page_Apres;
    break;

  case XK_Home:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Debut_Page;
    break;

  case XK_End:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = Fin_Page;
    break;

  case XK_Return:
  case XK_KP_Enter:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = 13;
    break;

  case XK_Delete:
    mode_accent = TM_ACCENT_AUCUN;
    touche_gem = 127;
    break;

  default: /* En gros, les lettres normales */
    if ( BI_etat & K_CTRL ) {
      if ( (touche >= XK_KP_0) && (touche <= XK_KP_9) ) /* Chiffres du pavé numérique */
	touche_gem = '0' + (touche - XK_KP_0);
      else {
	/*      printf("Control - %d (%c) utilisé...\n", touche, touche); */
	touche_gem = (unsigned int) touche;
      }
    } else {
      touche_gem = (unsigned int) touche;
    }
  }

  return touche_gem;
}

/* -------- Le traitement des touches mortes, si nécessaire -------- 
   Codes en XK_...  : touches X définies par défaut
            DXK_... : touches X spécifiques DEC, dans DECkeysym.h
*/

static int CLAV_VerifierToucheMorte(KeySym touche)
{
  if ( mode_accent != TM_ACCENT_AUCUN ) {
    /* Quand une touche morte est active, impossible d'en changer ! */
    return FAUX;
  }

 /* L'accent grave */
  if ( ( XK_dead_grave == touche ) || ( DXK_grave_accent == touche ) ) {
    mode_accent = TM_ACCENT_GRAVE;
    return VRAI;
  }

  /* L'accent aigu */
  if ( ( XK_dead_acute == touche ) || ( DXK_acute_accent == touche ) ) {
    mode_accent = TM_ACCENT_AIGU;
    return VRAI;
  }

  /* L'accent circonflexe */
  if ( ( XK_dead_circumflex == touche ) || ( DXK_circumflex_accent == touche ) ) {
    mode_accent = TM_ACCENT_CIRCONFLEXE;
    return VRAI;
  }

  /* Le tilde */
  if ( ( XK_dead_tilde == touche ) || ( DXK_tilde == touche ) ) {
    mode_accent = TM_ACCENT_TILDE;
    return VRAI;
  }

  /* Le "macron" */
  if ( XK_dead_macron == touche ) {
    mode_accent = TM_ACCENT_MACRON;
    return VRAI;
  }

  /* La brève */
  if ( XK_dead_breve == touche ) {
    mode_accent = TM_ACCENT_BREVE;
    return VRAI;
  }

  /* Le point suscrit */
  if ( XK_dead_abovedot == touche ) {
    mode_accent = TM_ACCENT_POINT_DESSUS;
    return VRAI;
  }

  /* Le tréma classique */
  if ( ( XK_dead_diaeresis == touche ) || ( DXK_diaeresis == touche ) ) {
    mode_accent = TM_ACCENT_TREMA;
    return VRAI;
  }

  /* Le rond suscrit */
  if ( ( XK_dead_abovering == touche ) || ( DXK_ring_accent == touche ) ) {
    mode_accent = TM_ACCENT_ROND;
    return VRAI;
  }

  /* Le tréma long hongrois */
  if ( XK_dead_doubleacute == touche ) {
    mode_accent = TM_ACCENT_TREMA_LONG;
    return VRAI;
  }

  /* Le caron */
  if ( XK_dead_caron == touche ) {
    mode_accent = TM_ACCENT_CARON;
    return VRAI;
  }

  /* La cédille */
  if ( ( XK_dead_cedilla == touche ) || ( DXK_cedilla_accent == touche ) ) {
    mode_accent = TM_ACCENT_CEDILLE;
    return VRAI;
  }

  /* L'ogonek */
  if ( XK_dead_ogonek == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- Ogonek\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }


  /* L'iota souscrit */
  if ( XK_dead_iota == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- Iota souscrit\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  /* Le son vocalisé */
  if ( XK_dead_voiced_sound == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- « Voiced sound »\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  /* Le son mi-vocalisé */
  if ( XK_dead_semivoiced_sound == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- « Semi-voiced sound »\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  /* Le point souscrit */
  if ( XK_dead_belowdot == touche ) {
    mode_accent = TM_ACCENT_POINT_DESSOUS;
    return VRAI;
  }

  /* Le crochet */
  if ( XK_dead_hook == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- Crochet\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  /* "Horn" */
  if ( XK_dead_horn == touche ) {
    (void) fprintf( stderr, "%s,%s l.%u : touche morte non reconnue -- « Horn »\n",
		    __FILE__, __FUNCTION__, __LINE__ );
  }

  return FAUX; /* Ce n'est pas une touche morte */
}

/* -------- Traitement des touches mortes... --------
   Renvoie le nombre d'_octets_ de la chaîne créée
            (= nombre de caractères en latin-1, mais <= si UTF-7 ou similaire)

   touche       = le symbole de la touche appuyée
   chaine       = là où stocker la chaîne de caractères construite
   lgmax_chaine = nombre maximal de caractères que l'on peut mettre dans cette chaîne
 */

static int CLAV_TraiterToucheMorte(KeySym touche, char *chaine, unsigned long lgmax_chaine)
{
  const char *caractere = NULL, *accent = NULL;

  if ( ( NULL == chaine ) || ( 0 == lgmax_chaine ) ) {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de chaîne passée pour indiquer le résultat !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return 0;
  }

  /* Dans toute la suite, part du principe que KeySym = code ASCII ou latin-1 */
  switch ( mode_accent )
  {
  case TM_ACCENT_CIRCONFLEXE:	/* âĉêĝĥîĵôŝûŵŷẑ ⁰¹²³⁴⁵⁶⁷⁸⁹⁺⁻⁽⁾⁼ ếềầ */
    accent = "^";

    switch ( touche )
    {
    case '0': caractere = ZERO_EXPOSANT  ; break;
    case '1': caractere = UN_EXPOSANT    ; break;
    case '2': caractere = DEUX_EXPOSANT  ; break;
    case '3': caractere = TROIS_EXPOSANT ; break;
    case '4': caractere = QUATRE_EXPOSANT; break;
    case '5': caractere = CINQ_EXPOSANT  ; break;
    case '6': caractere = SIX_EXPOSANT   ; break;
    case '7': caractere = SEPT_EXPOSANT  ; break;
    case '8': caractere = HUIT_EXPOSANT  ; break;
    case '9': caractere = NEUF_EXPOSANT  ; break;
    case '=': caractere = EGAL_EXPOSANT  ; break;
    case '+': caractere = PLUS_EXPOSANT  ; break;
    case '-': caractere = MOINS_EXPOSANT ; break;
    case '(': caractere = PARENTHESE_OUVRANTE_EXPOSANT; break;
    case ')': caractere = PARENTHESE_FERMANTE_EXPOSANT; break;

    case 'a': caractere = a_CIRC; break; case 'A': caractere = A_CIRC; break;
    case 'c': caractere = c_CIRC; break; case 'C': caractere = C_CIRC; break;
    case 'e': caractere = e_CIRC; break; case 'E': caractere = E_CIRC; break;
    case 'g': caractere = g_CIRC; break; case 'G': caractere = G_CIRC; break;
    case 'h': caractere = h_CIRC; break; case 'H': caractere = H_CIRC; break;
    case 'i': caractere = i_CIRC; break; case 'I': caractere = I_CIRC; break;
    case 'j': caractere = j_CIRC; break; case 'J': caractere = J_CIRC; break;
    case 'o': caractere = o_CIRC; break; case 'O': caractere = O_CIRC; break;
    case 's': caractere = s_CIRC; break; case 'S': caractere = S_CIRC; break;
    case 'u': caractere = u_CIRC; break; case 'U': caractere = U_CIRC; break;
    case 'w': caractere = w_CIRC; break; case 'W': caractere = W_CIRC; break;
    case 'y': caractere = y_CIRC; break; case 'Y': caractere = Y_CIRC; break;
    case 'z': caractere = z_CIRC; break; case 'Z': caractere = Z_CIRC; break;

    case XK_dead_circumflex:
    case DXK_circumflex_accent:
      caractere = accent;
      break;
    }
    break;

  case TM_ACCENT_TREMA:		/* äëḧïöẗüẅẍÿ */
    accent = TREMA;

    switch ( touche )
    {
    case 'a': caractere = a_TREMA; break; case 'A': caractere = A_TREMA; break;
    case 'e': caractere = e_TREMA; break; case 'E': caractere = E_TREMA; break;
    case 'h': caractere = h_TREMA; break; case 'H': caractere = H_TREMA; break;
    case 'i': caractere = i_TREMA; break; case 'I': caractere = I_TREMA; break;
    case 'o': caractere = o_TREMA; break; case 'O': caractere = O_TREMA; break;
    case 't': caractere = t_TREMA; break; case 'T': caractere = T_TREMA; break;
    case 'u': caractere = u_TREMA; break; case 'U': caractere = U_TREMA; break;
    case 'w': caractere = w_TREMA; break; case 'W': caractere = W_TREMA; break;
    case 'x': caractere = x_TREMA; break; case 'X': caractere = X_TREMA; break;
    case 'y': caractere = y_TREMA; break; case 'Y': caractere = Y_TREMA; break;

    case XK_dead_diaeresis:
    case DXK_diaeresis:
      caractere = accent;
      break;
    }
    break;

  case TM_ACCENT_AIGU:		/* áćéǵíḱĺḿńóṕŕśúẃýź ǽ ǘ */
    accent = ACCENT_AIGU;

    switch ( touche )
    {
    case 'a': caractere = a_AIGU; break; case 'A': caractere = A_AIGU; break;
    case 'c': caractere = c_AIGU; break; case 'C': caractere = C_AIGU; break;
    case 'e': caractere = e_AIGU; break; case 'E': caractere = E_AIGU; break;
    case 'g': caractere = g_AIGU; break; case 'G': caractere = G_AIGU; break;
    case 'i': caractere = i_AIGU; break; case 'I': caractere = I_AIGU; break;
    case 'k': caractere = k_AIGU; break; case 'K': caractere = K_AIGU; break;
    case 'l': caractere = l_AIGU; break; case 'L': caractere = L_AIGU; break;
    case 'm': caractere = m_AIGU; break; case 'M': caractere = M_AIGU; break;
    case 'n': caractere = n_AIGU; break; case 'N': caractere = N_AIGU; break;
    case 'o': caractere = o_AIGU; break; case 'O': caractere = O_AIGU; break;
    case 'p': caractere = p_AIGU; break; case 'P': caractere = P_AIGU; break;
    case 'r': caractere = r_AIGU; break; case 'R': caractere = R_AIGU; break;
    case 's': caractere = s_AIGU; break; case 'S': caractere = S_AIGU; break;
    case 'u': caractere = u_AIGU; break; case 'U': caractere = U_AIGU; break;
    case 'w': caractere = w_AIGU; break; case 'W': caractere = W_AIGU; break;
    case 'y': caractere = y_AIGU; break; case 'Y': caractere = Y_AIGU; break;
    case 'z': caractere = z_AIGU; break; case 'Z': caractere = Z_AIGU; break;

    case XK_dead_acute:
    case DXK_acute_accent:
      caractere = accent;
      break;
    }
    break;

  case TM_ACCENT_GRAVE:
    accent = ACCENT_GRAVE;

    switch ( touche )
    {
    case 'a': caractere = a_GRAVE; break; case 'A': caractere = A_GRAVE; break;
    case 'e': caractere = e_GRAVE; break; case 'E': caractere = E_GRAVE; break;
    case 'i': caractere = i_GRAVE; break; case 'I': caractere = I_GRAVE; break;
    case 'o': caractere = o_GRAVE; break; case 'O': caractere = O_GRAVE; break;
    case 'u': caractere = u_GRAVE; break; case 'U': caractere = U_GRAVE; break;

    case XK_dead_grave:
    case DXK_grave_accent:
      caractere = accent;
      break;
    }
    break;

  case TM_ACCENT_TILDE:
    accent = "~";

    switch ( touche )
    {
    case 'a': caractere = a_TILDE; break; case 'A': caractere = A_TILDE; break;
    case 'n': caractere = n_TILDE; break; case 'N': caractere = N_TILDE; break;
    case 'o': caractere = o_TILDE; break; case 'O': caractere = O_TILDE; break;

    case XK_dead_tilde:
    case DXK_tilde:
      caractere = accent;
      break;
    }
    break;

  default:
    AVERTISSEMENT( "Touche morte inconnue : %d\n", mode_accent );
    accent = "?";
  }

  /* Plus besoin de gérer la touche morte ! */
  mode_accent = TM_ACCENT_AUCUN;

  /* On copie le caractère dans la chaîne fournie */
  if ( caractere != NULL ) {
    (void) strncpy( chaine, caractere, lgmax_chaine );
  } else {
    snprintf( chaine, lgmax_chaine, "%s%c", accent, (char) touche );
  }

  return strlen( chaine );
}

/* -------- Vérifie l'état des modificateurs et adapte en conséquence -------- */

void BIBX_verifie_modificateurs(unsigned int etat)
{
  if (etat & ShiftMask) /* Appuie sur shift */
    BI_etat |=  (K_LSHIFT | K_RSHIFT);
  else
    BI_etat &= ~(K_LSHIFT | K_RSHIFT);

  if (etat & ControlMask) /* Appuie sur control */
    BI_etat |= K_CTRL;
  else
    BI_etat &= ~K_CTRL;

  if (etat & Mod1Mask) { /* Premier modificateur : alt */
/*    printf("Modificateur 1 actif\n"); */
    BI_etat |= K_ALT;
/*    printf("On a donc BI_etat = %d\n", BI_etat); */
  } else
    BI_etat &= ~K_ALT;

  /* Deuxime modificateur = numlock, ignor */

  if (etat & Mod3Mask) { /* Troisime modificateur */
    printf("Modificateur 3 actif\n");
/*    BI_etat |= K_ALT; */
  }
/*  else
    BI_etat &= ~K_ALT; */

  if (etat & Mod4Mask) { /* Quatrime modificateur */
#ifndef _DEC_UNIX_ /* Mod4 = pav numrique actif sous Digital Unix ! */
    printf("Modificateur 4 actif\n");
#endif
/*    BI_etat |= K_ALT; */
  }
/*  else
    BI_etat &= ~K_ALT; */

  if (etat & Mod5Mask) { /* Cinquime modificateur */
    printf("Modificateur 5 actif\n");
/*    BI_etat |= K_ALT; */
  }
/*  else
    BI_etat &= ~K_ALT; */
}

/* ------------- Fabrication de la chaîne à afficher d'un objet éditable ---------------- */

char *faire_chaine(TEDINFO *tedinfo)
{
  char *resultat;
  unsigned int i_chaine, i_texte = 0;

  resultat = (char *) malloc( (unsigned) (tedinfo->te_tmplen + 1));
  if ( NULL == resultat ) {
    (void) fprintf( stderr, "%s,%s l.%u : mémoire insuffisante -- chaîne résultat\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  for ( i_chaine = 0; i_chaine < tedinfo->te_tmplen ; i_chaine++ ) {
    if ( (tedinfo->te_ptmplt)[ i_chaine ] == '_' ) { /* Caractère éditable */
      if ( (tedinfo->te_ptext.editable)[ i_texte ] == 0) /* Fin du texte */
	resultat[ i_chaine ] = '_';
      else {
	resultat[ i_chaine ] = (tedinfo->te_ptext.editable)[ i_texte ];
	i_texte++; /* Caractère suivant de l'entrée */
      }
    } else
      resultat[i_chaine] = (tedinfo->te_ptmplt)[i_chaine];
  }

  return resultat;
}

/* -------- Renvoie un pointeur sur le caractère contenant le curseur _dans le masque_ -------- */

static char *OBJ_TrouverCurseur(const char *masque, unsigned int position)
{
  unsigned int i = 0;
  char *pos_curseur;

  if ( NULL == masque ) {
    (void) fprintf( stderr, "%s,%s l.%u : Pas de masque !\n",
		    __FILE__, __FUNCTION__, __LINE__ );
    return NULL;
  }

  /* Premier caractère éditable */
  pos_curseur = strchr(masque, '_' );
  if ( NULL == pos_curseur ) {
    (void) fprintf( stderr, "%s,%s l.%u : aucun caractère éditable dans le masque [%s]\n",
		    __FILE__, __FUNCTION__, __LINE__, masque );
    return NULL;
  }

  /* Tant que l'on n'est pas sur le bon caractère éditable */
  while ( i < position ) {
    pos_curseur= strchr( pos_curseur + 1, '_' );
    if ( NULL == pos_curseur ) {
      AVERTISSEMENT( "Impossible de trouver le curseur [i = %u, position = %u / %s]",
		     i, position, masque );
      return NULL;
    }
    i++;
  }

  /* On a trouvé, tout va bien... */
  return pos_curseur;
}

/* Donne le nombre de caractères dans le texte modifiable
   On a donc te_ptext[cette valeur] = 0 si moins que te_txtlen */

unsigned int OBJ_der_caractere(TEDINFO *tedinfo)
{
  unsigned int position;

  for ( position = 0; position < tedinfo->te_txtlen ; position++ )
    if ( (tedinfo->te_ptext.editable)[position] == 0 ) /* Chaîne s'interrompt avant */
      return position;

  return tedinfo->te_txtlen; /* Sinon, chaîne complète */
}
